(() => {
    'use strict';

    angular.module('noc.components').directive('nocActionRequiredNotice', (actionRequiredCanShowIfOffFilter, $timeout) => {
        return {
            restrict: 'E',
            templateUrl: 'assets/templates/components/action-required-notice.html',
            replace: true,
            scope: {},
            link: function link($scope) {
                let showAnalyzing = false;
                const scope = $scope.$parent;

                function hasOntapUpgradeVersion() {
                    return scope.selectedWorkingEnvironment.ontapClusterProperties && scope.selectedWorkingEnvironment.ontapClusterProperties.upgradeVersions &&
                        scope.selectedWorkingEnvironment.ontapClusterProperties.upgradeVersions.length > 0;
                }

                function hasMediatorUpgradeVersion() {
                    return _.get(scope.selectedWorkingEnvironment.haProperties, 'mediatorVersionsToUpdate') && scope.selectedWorkingEnvironment.haProperties.mediatorVersionsToUpdate.length > 0;
                }

                $scope.upgradeRequired = () => {
                    return hasOntapUpgradeVersion() && scope.selectedWorkingEnvironment._allowedOperations['change-ontap-image'];
                };

                $scope.upgradeMediatorRequired = () => {
                    return hasMediatorUpgradeVersion() && scope.selectedWorkingEnvironment._allowedOperations['upgrade-mediator'] && !$scope.upgradeRequired();
                };

                $scope.getNumOfActionsRequired = () => {
                    const numOfActionsRequired = scope.actionsRequired[scope.selectedWorkingEnvironment.publicId].length;

                    return numOfActionsRequired > 1 ? numOfActionsRequired + ' Actions required' : numOfActionsRequired + ' Action required';
                };

                $scope.actionRequiredState = () => {
                    if (showAnalyzing) return 'analyzing';
                    if (scope.selectedWorkingEnvironment.workingEnvironmentType === 'ON_PREM' && scope.actionsRequired[scope.selectedWorkingEnvironment.publicId]) {
                        return 'action-required';
                    }
                    if (!scope.vsaResponse) {
                        //Special case: working environment is off and has action required.
                        //NOC-5416 only show action required in case of IPA warning/error
                        if (scope.selectedWorkingEnvironment.status.status === 'off' && $scope.canShowIfOff(scope.actionsRequired[scope.selectedWorkingEnvironment.publicId])) {
                            return 'action-required';
                        } else {
                            return '';
                        }
                    }
                    if (scope.actionsRequired[scope.selectedWorkingEnvironment.publicId] && _.isEmpty(scope.vsaResponse.activeActions) && !scope.vsaComputingCapacity) {
                        return 'action-required';
                    } else if (!_.isEmpty(scope.vsaResponse.activeActions) || scope.vsaComputingCapacity) {
                        showAnalyzing = true;
                        $timeout(() => {
                            showAnalyzing = false;
                        }, 2000);

                        return 'analyzing';
                    }

                    return '';
                };

                $scope.goToActionRequired = $scope.$parent.goToActionRequired;

                $scope.canShowIfOff = (actionsRequired) => {
                    if (!actionsRequired) return false;
                    else return actionRequiredCanShowIfOffFilter(actionsRequired).length > 0;
                };

                $scope.getActionsRequiredHighestLevel = () => {
                    return $scope.$parent.actionsRequiredHighestLevel[$scope.$parent.selectedWorkingEnvironment.publicId];
                };
            }
        };
    });
})();
