angular.module('noc').component('costAnnualComponent', {
    bindings: {
        annualData: '='
    },
    templateUrl: 'assets/templates/components/cost/cost-annual-component.html',
    controller: function ($timeout, $filter) {
        this.costAnnualBarHighchart = function(data, numberFormatter){
            //addPatternToCurrentMonth(data);

            /* This function actually wait half of the duration
               and run the animation linearly the other half of duration */
            this.easeAnimation = function (pos) {
                if ((pos) < 0.5) {
                    return (0);
                }
                return (2 * (pos - 0.5));
            };

            return new Highcharts.Chart({
                chart: {
                    type: 'column',
                    backgroundColor: 'transparent',
                    marginBottom: 70,
                    spacingBottom: 0,
                    marginTop: 10,
                    marginLeft: 70,
                    renderTo: "costAnnualBarHighchart",
                    style:{
                        fontFamily: '"Open Sans", sans-serif'
                    }
                },
                colors: ['#4ed3ba'],
                credits: false,
                title: {text: ''},
                xAxis: {
                    gridLineColor: 'transparent',
                    categories: data.categories,
                    labels: {
                        style: {
                            color: "#979797",
                            'font-weight': 'normal',
                            'font-size': "12px",
                            'line-height': "16px"
                        },
                        y: 27,
                        useHTML: true,
                        formatter: function () {
                            let curDate = new Date();
                            if (this.value.month() === curDate.getMonth() && this.value.year() === curDate.getFullYear()) {
                                return this.value.format('MMM</br>YYYY') + "<div class='current-month-arrow'></div>";
                            }

                            return this.value.format('MMM</br>YYYY');
                        }
                    },
                    lineWidth: 0,
                    minorGridLineWidth: 0,
                    lineColor: "transparent",
                    tickWidth: 0


/*                    tickWidth: 0,
                    lineWidth: 4,
                    offset: 17*/
                },
                yAxis: {
                    min: 0,
                    offset: 50,
                    gridLineColor: 'transparent',
                    title: {text: null},
                    labels: {
                        enabled: true,
                        align: 'left',
                        style: {
                            color: "#979797",
                            'font-weight': 'normal',
                            'font-size': "13px"
                        },
                        // eslint-disable-next-line no-template-curly-in-string
                        /*format: " ${value}"*/
                        formatter: function formatter() {
                            return "$" + numberFormatter(this.value);
                        }
                    },
                    stackLabels: {
                        enabled: false,
                        y: -36,
                        formatter: function() {
                            var isToDate = this.x === data.storageValues.length - 1;
                            var toDateText = isToDate ? "<div class='to-date'> to date</div>" : "";
                            var toDateClass = isToDate ? "to-date-box" : '';
                            var storageSubDiv = "<div class='storage sub-total'><div class='circle'></div>$" + numberFormatter(this.points['0,' + this.x + ',0'][1] - this.points['0,' + this.x + ',0'][0], 2) + "</div>";
                            //var computeSubDiv = "<div class='compute sub-total'><div class='circle'></div>$" + numberFormatter(this.points['1,' + this.x + ',0'][1] - this.points['1,' + this.x + ',0'][0], 2) + "</div>";
                            return "<div class='stack-label " + toDateClass +"'><div class='inner'><div class='total'>$"+ numberFormatter(this.total, 2)+"</div>"+toDateText+ "<div>" + storageSubDiv  + "</div></div><div class='chupchik'></div></div>";
                        },
                        useHTML: true
                    }
                },
                legend: {
                    enabled: false,
                    align: 'center',
                    verticalAlign: 'bottom',
                    y: -5,
                    floating: true,
                    borderColor: null,
                    borderWidth: 0,
                    shadow: false,
                    symbolRadius: 5,
                    symbolHeight: 10,
                    symbolWidth: 10,
                    itemStyle: {
                        color: "#747474",
                        'font-size': "13px",
                        'font-weight': "normal",
                        cursor: 'default'
                    },
                    itemHoverStyle: {
                        color: '#747474'
                    }
                },
                tooltip: {
                    outside: true,
                    backgroundColor: "#0c1558",
                    borderColor: "#0c1558",
                    padding: 0,
                    style : {
                        color : "#ffffff",
                        fontSize : '15px'
                    },
                    headerFormat: '',
                    formatter: function formatter() {
                        return "$" + numberFormatter(this.y);
                    },
                    positioner: function(labelWidth, labelHeight,  point) {
                        //let elem = document.getElementById("costAnnualBarHighchart");cost-compo
                        let elemPos = $("#costAnnualBarHighchart").position();
                        return {x: point.plotX + elemPos.left + this.chart.plotLeft - (labelWidth/2) - 8, y: point.plotY + elemPos.top - labelHeight - 15};
                    },
                    useHTML: true
                },
                plotOptions: {
                    column: {
                        stacking: 'normal',
                        borderWidth: 0,
                        pointWidth: 12,
                        dataLabels: {
                            enabled: false
                        },
                        events: {
                            legendItemClick: function(e){
                                return !e.target.visible || _.reduce(this.chart.series, function(sum, serie) {
                                    return sum + (serie.visible ? 1 : 0);
                                }, 0) > 1;
                            }
                        },
                        point: {

                            /*events: {
                                mouseOver: function(){
                                    $(this.series.yAxis.stacks.column[this.x].label.element).find(".stack-label").addClass("expanded");
                                },
                                mouseOut: function(){
                                    $(this.series.yAxis.stacks.column[this.x].label.element).find(".stack-label").removeClass("expanded");
                                }
                            }*/
                        }
                    }
                },
                series: [
                    {
                        name: 'Total Costs',
                        data: data.totalCosts,
                        legendIndex: 0,
                        animation: {
                            duration: 1500

                            /*easing: this.easeAnimation *//* This function actually wait half of the duration
                                                           and run the animation linearly the other half of duration */
                        }
                    }
                ]
            });
        };


        this.handleNewCostCharts = function() {
            //put a mocked cost in case the cost is empty and show it dimmed
            if (!this.annualData || this.annualData.length === 0) {
                // var year = new Date().getFullYear();
                this.annualData = [
                    {
                        "start": "2018-1-1",
                        "end": "2018-1-31",
                        "estimated": false,
                        "costAndUsage": [
                            {
                                "name": "Compute",
                                "cost": {
                                    "amount": 16.1,
                                    "currency": "USD"
                                },
                                "detailedMonthlyCostAndUsage": [
                                    {
                                        "name": "m4.2xlarge",
                                        "cost": {
                                            "amount": 10.5,
                                            "currency": "USD"
                                        },
                                        "usage": {
                                            "amount": 24,
                                            "units": "Hrs"
                                        }
                                    },
                                    {
                                        "name": "m4.xlarge",
                                        "cost": {
                                            "amount": 5.5,
                                            "currency": "USD"
                                        },
                                        "usage": {
                                            "amount": 24,
                                            "units": "Hrs"
                                        }
                                    },
                                    {
                                        "name": "t2.micro",
                                        "cost": {
                                            "amount": 0.1,
                                            "currency": "USD"
                                        },
                                        "usage": {
                                            "amount": 24,
                                            "units": "Hrs"
                                        }
                                    }
                                ]
                            },
                            {
                                "name": "EBS Storage",
                                "cost": {
                                    "amount": 21,
                                    "currency": "USD"
                                },
                                "detailedMonthlyCostAndUsage": [
                                    {
                                        "name": "GP2",
                                        "cost": {
                                            "amount": 15,
                                            "currency": "USD"
                                        },
                                        "usage": {
                                            "amount": 24,
                                            "units": "GB-Month"
                                        }
                                    },
                                    {
                                        "name": "ST1",
                                        "cost": {
                                            "amount": 5,
                                            "currency": "USD"
                                        },
                                        "usage": {
                                            "amount": 24,
                                            "units": "GB-Month"
                                        }
                                    },
                                    {
                                        "name": "Snapshots",
                                        "cost": {
                                            "amount": 1,
                                            "currency": "USD"
                                        },
                                        "usage": {
                                            "amount": 24,
                                            "units": "GB-Month"
                                        }
                                    }
                                ]
                            },
                            {
                                "name": "S3",
                                "cost": {
                                    "amount": 3.3,
                                    "currency": "USD"
                                },
                                "detailedMonthlyCostAndUsage": [
                                    {
                                        "name": "Storage",
                                        "cost": {
                                            "amount": 3,
                                            "currency": "USD"
                                        },
                                        "usage": {
                                            "amount": 24,
                                            "units": "GB-Month"
                                        }
                                    },
                                    {
                                        "name": "Tier1 requests",
                                        "cost": {
                                            "amount": 1,
                                            "currency": "USD"
                                        },
                                        "usage": {
                                            "amount": 100,
                                            "units": "Requests"
                                        }
                                    },
                                    {
                                        "name": "Tier2 requests",
                                        "cost": {
                                            "amount": 0.2,
                                            "currency": "USD"
                                        },
                                        "usage": {
                                            "amount": 2000,
                                            "units": "Requests"
                                        }
                                    }
                                ]
                            },
                            {
                                "name": "Data Transfer",
                                "cost": {
                                    "amount": 3.3,
                                    "currency": "USD"
                                },
                                "detailedMonthlyCostAndUsage": [
                                    {
                                        "name": "EC2 In",
                                        "cost": {
                                            "amount": 0,
                                            "currency": "USD"
                                        },
                                        "usage": {
                                            "amount": 0,
                                            "units": "GB"
                                        }
                                    },
                                    {
                                        "name": "EC2 Out",
                                        "cost": {
                                            "amount": 0.1,
                                            "currency": "USD"
                                        },
                                        "usage": {
                                            "amount": 0.5,
                                            "units": "GB"
                                        }
                                    },
                                    {
                                        "name": "S3 In",
                                        "cost": {
                                            "amount": 0,
                                            "currency": "USD"
                                        },
                                        "usage": {
                                            "amount": 0,
                                            "units": "GB"
                                        }
                                    },
                                    {
                                        "name": "S3 Out",
                                        "cost": {
                                            "amount": 0.2,
                                            "currency": "USD"
                                        },
                                        "usage": {
                                            "amount": 1,
                                            "units": "GB"
                                        }
                                    }
                                ]
                            }
                        ]
                    }
                ];
                //$scope.costResponse.empty = true;
            }

            var barData = this.constructAnnualBarData(this.annualData);
            this.barChart = this.costAnnualBarHighchart(barData, $filter('number'));
        };

        this.constructAnnualBarData = function(annualData){
            const sortedAnnualArray = _.sortBy(annualData, (s) => s.start);
            var categories = _.map(sortedAnnualArray, function(cost){
                //return moment({year: cost.year, month: cost.month - 1}).format('MMMM YYYY');
                let date = _.split(cost.start, '-');
                return moment({year: date[0], month: date[1] - 1});
            });

            var totalCosts = _.map(sortedAnnualArray, function(cost){
                return {y:_.round(cost.total.amount) };
            });

            return {totalCosts: totalCosts, categories: categories};
        };

        this.$onInit = function () {
            this.handleNewCostCharts();
        };
    }
});

