angular.module('noc.components').directive('nocDataFabricLayout', ($filter, $timeout, workingEnvironmentStatuses, $log, nocUtil,
                                                                   workingEnvironmentResource, Constants, nocRegionsService) => {
    'use strict';

    var isInside = function(x, y, box) {
        return ((x > box.left) && (x < box.right) && (y > box.top) && (y < box.bottom));
    };

    var pathCoords = 'M158.14,21.04c-1.376,0-2.736,0.066-4.079,0.187 C149.925,9.311,136.695,0.589,121,0.589c-12.129,0-22.785,5.211-28.913,13.077C84.5,5.619,73.72,0.589,61.757,0.589 c-22.963,0-41.578,18.523-41.578,41.372c0,3.169,0.359,6.255,1.038,9.221C9.505,55.455,1.138,66.73,1.138,79.969 c0,16.911,13.649,30.62,30.486,30.62h123.89v-0.078c0.869,0.05,1.744,0.078,2.626,0.078c24.851,0,44.998-20.046,44.998-44.775 C203.138,41.085,182.991,21.04,158.14,21.04z';

    var createCloudPath = function(path) {
        return path.attr('stroke-miterlimit', 10).attr('d', pathCoords);
    };
    var createGrayCloudPath = function(path) {
        return createCloudPath(path).attr('fill', '#EBEBEB');
    };

    return{
        restrict: "A",
        scope: true,
        templateUrl: "assets/templates/tabs/working-environments/data-fabric-layout.html",
        link: function (scope, elem) {
            var d3 = window.d3;
            var workingEnvironments = _.cloneDeep(scope.workingEnvironments);
            var cloudImgWidth = 200, cloudImgHeight = 111;

//======================================================= events handling and util functions =======================================================================

            //This function is used to put the dragged node in the front of all other nodes
            d3.selection.prototype.moveToFront = function() {
                return this.each(function(){
                    this.parentNode.appendChild(this);
                });
            };

            function packageHierarchy(workingEnvironments) {
                var root = {name: "", parent: null, children: _.clone(workingEnvironments)};

                _.forEach(root.children,function(we){
                    we.parent = root;
                    we.children = null;
                });

                return root;
            }

            // Build links after building hierarchy
            function packageImports(nodes) {
                var map = {},
                    imports = [];
                if (!nodes[0].children) return [];
                nodes = nodes[0].children;
                map = _.keyBy(nodes, 'publicId');

                // For each import, construct a link from the source to target node.
                imports = _.flatMap(nodes, function (node) {
                    return _.map(node.peers, function (peer) {
                        return {source: map[peer], target: node};
                    });
                });

                return imports;
            }

            function rotateNode(d) {
                // -1 for left side 1 for right side, 0 for none
                // theAngle 0 is 3 o'clock and goes up clockwise
                var isLeftRightHandSide = function(angle) {
                    if (angle === 90 || angle === 270){
                        return 0;
                    } else {
                        return (theAngle > 90 && theAngle < 270 ? -1 : 1);
                    }
                };

                // returns a value from the range [0-1] that reflects how much the element is far from 12 and 6 oclock
                var horizontalizm = function(angle) {
                    // the angle 0 should be 12 oclock so adding 90 deg
                    var inRadians = Math.PI * (angle + 90) / 180;
                    return Math.abs(Math.sin(inRadians));
                };

                // theAngle 0 is 3 o'clock and goes up clockwise
                var theAngle = (d.x - 90) % 360;
                if (d.x && d.y) {
                    return    "rotate(" + (theAngle) + ")" +
                        "translate(" + (d.y - 15) + "," + ((-1)*cloudImgHeight/2) + ")" +
                        "rotate(" + (-1 * theAngle) + "," + cloudImgWidth/2 + "," + cloudImgHeight/2 + ")" +
                        "translate(" + (isLeftRightHandSide(theAngle)*15) + ",0)" +
                        "translate(" + (isLeftRightHandSide(theAngle)*30*horizontalizm(theAngle)) + ",0)";
                }
                return "";
            }

            function onMouseOver(d) {
                if (dragManager.dragStarted && dragManager.dragCandidate !== d) {
                    node.classed("targeted", function(target) {
                        return target === d;
                    });
                }
            }

            function onMouseEnter(d) {
                link
                    .classed("link--target", function (l) {
                        return l[0].publicId === d.publicId;
                    })
                    .classed("link--source", function (l) {
                        return l[2].publicId === d.publicId;
                    });

                link
                    .style("marker-start", function(l) {
                        return l[2].publicId === d.publicId || l[0].publicId === d.publicId ? "url(#markerCircleHover)" : "url(#markerCircle)";
                    })
                    .style("marker-end", function(l) {
                        return l[2].publicId === d.publicId || l[0].publicId === d.publicId ? "url(#markerCircleHover)" : "url(#markerCircle)";
                    });

                sourceArrow.classed("hover", function(l){
                    return l[0].publicId === d.publicId || l[2].publicId === d.publicId;
                });

                targetArrow.classed("hover", function(l){
                    return l[0].publicId === d.publicId || l[2].publicId === d.publicId;
                });

                link.sort(function(l) {
                    return l[0] === d || l[2] === d;
                });

                sourceArrow.sort(function(l) {
                    return l[0] === d || l[2] === d;
                });

                targetArrow.sort(function(l) {
                    return l[0] === d || l[2] === d;
                });
            }

            function onMouseOut() {
                node.classed("targeted", false);
            }

            function onMouseLeave() {
                link
                    .classed("link--target", false)
                    .classed("link--source", false)
                    .style("marker-start", "url(#markerCircle)")
                    .style("marker-end", "url(#markerCircle)");

                node
                    .classed("node--both", false)
                    .classed("node--target", false)
                    .classed("node--source", false);

                targetArrow.classed("hover", false);
                sourceArrow.classed("hover", false);
            }

            var DBL_CLICK_TIMEOUT = 300, dblClickTimeout,
                mousedownRecently = false, recentMouseDownTarget;

            function onMouseDown(d) {
                $log.debug('mouse down');
                if (mousedownRecently && recentMouseDownTarget === d && _.get(d, 'status.status') !== 'failed-to-create') {
                    $log.debug('double click detected');
                    scope.showMonitoring(d);
                } else {
                    scope.onClick(d);
                }
                recentMouseDownTarget = d;
                mousedownRecently = true;
                $timeout.cancel(dblClickTimeout);
                dblClickTimeout = $timeout(() => {
                    mousedownRecently = false;
                }, DBL_CLICK_TIMEOUT);
            }

            var dragManager = {
                dragStarted: false,
                dragCandidate: null,
                dropCandidate: null,
                moveWithMouse: false,
                startMovingTimeout: null
            };

            var grayCloud;

            function dropHandler(d) {
                $log.debug("drag drop");
                $timeout.cancel(dragManager.startMovingTimeout);
                dragManager.moveWithMouse = false;
                if (d.originX && d.originY) {
                    d3.select(this)
                        .attr("transform", "translate(" + d.originX + "," + d.originY + ")")
                        .attr("transform", rotateNode);
                }
                if (dragManager.dropCandidate && dragManager.dropCandidate.publicId !== d.publicId) {
                    scope.onDrop(d, dragManager.dropCandidate);
                }
                dragManager.dropCandidate = null;
                dragManager.dragStarted = false;
                dragManager.dragCandidate = null;
                //can't be moved to the start of drag, prevents the click from working!!!

                node.classed('not-applicable', false);
                node.classed('targeted', false);
                var draggedNode = d3.select(this);
                //Set pointer events to be available
                draggedNode.attr('pointer-events','').classed('dragged', false);

                // remove gray cloud
                if (grayCloud) grayCloud.remove();
            }

            function dragMove(d) {
                $log.debug("drag move");
                const draggedNode = d3.select(this);
                draggedNode.moveToFront();
                // d3.select(this).moveToFront();
                if (!dragManager.dragStarted) {
                    dragManager.dragStarted = true;
                    dragManager.dragCandidate = d;
                    d.originX = d.x;
                    d.originY = d.y;
                }

                var x = d3.event.x - (cloudImgWidth/2);
                var y = d3.event.y - (cloudImgHeight/2);
                if (dragManager.moveWithMouse) draggedNode.attr("transform", "translate(" + x + "," + y + ")");
                node.classed('not-applicable', (target) => {
                    return !(d === target || workingEnvironmentStatuses.isReplicationAllowed(d, target));
                });
                var tempDropCandidate = null;
                let evX, evY;
                if (d3.event.sourceEvent.type === 'touchmove') {
                    evX = d3.event.sourceEvent.touches[0].clientX;
                    evY = d3.event.sourceEvent.touches[0].clientY;
                } else {
                    evX = d3.event.sourceEvent.clientX;
                    evY = d3.event.sourceEvent.clientY;
                }
                node.each(function(d) {
                    if (d !== dragManager.dragCandidate && isInside(evX, evY, this.getBoundingClientRect())) {
                        tempDropCandidate = d;
                    }
                });
                dragManager.dropCandidate = tempDropCandidate;
            }

            var drag = d3.behavior.drag()
                .on("dragstart", function() {
                    $log.debug("drag start");
                    dragManager.startMovingTimeout = $timeout(function() {
                        dragManager.moveWithMouse = true;
                    }, 50);

                    var domNode = d3.select(this);
                    //Set no pointer events on the dragged object
                    domNode.attr('pointer-events', 'none').classed('dragged', true);
                    grayCloud = createGrayCloudPath(g.insert("path",":first-child")).attr("transform", domNode.attr("transform"));
                })
                .on("drag", dragMove)
                .on("dragend", dropHandler);

            var noDrag = d3.behavior.drag()
                .on("dragstart", _.noop)
                .on("drag", _.noop)
                .on("dragend", _.noop);

            var cluster = d3.layout.cluster();

            var bundle = d3.layout.bundle();

            var line = d3.svg.line.radial()
                .interpolate("bundle")
                .tension(0.15)
                .radius(function (d) {
                    return d.y;
                })
                .angle(function (d) {
                    return d.x / 180 * Math.PI;
                });

            var rawSvg = elem.find('svg')[0];
            var svg = d3.select(rawSvg);

            var g = svg.select("g.container");
            var linksContainer = g.append('g').attr('class', 'links-container');
            var sourceArrowsContainer = g.append('g').attr('class', 'source-arrows-container');
            var targetArrowsContainer = g.append('g').attr('class', 'target-arrows-container');

            var nodes, links, link, sourceArrow, targetArrow, node, nodeGraphics;

            var diameter, radius, innerRadius;

            var fitToScreen = function(container) {
                var htmlContainer = $('[noc-data-fabric-layout]'),
                    footerTop = $('.main-footer').offset().top,
                    graphWidth = container.node().getBoundingClientRect().width,
                    graphHeight = container.node().getBoundingClientRect().height + 35, // adding extra space otherwise clouds get cut
                    jqSvgEl = $(rawSvg),
                    translateY = 0,
                    scrollX = htmlContainer.scrollLeft(), scrollY = htmlContainer.scrollTop();

                // make the html container touch the footer 166 is the htmlContainer top offset, -12 for margin-top of canvas container for not covering add-env
                htmlContainer.height(footerTop - 166 - 10 - 12);
                jqSvgEl.height('100%');
                jqSvgEl.width('100%');

                jqSvgEl.height(jqSvgEl.height() < graphHeight ? graphHeight : "98%");
                jqSvgEl.width(jqSvgEl.width() < graphWidth ? graphWidth + 10 : "100%");
                translateY = workingEnvironments.length === 1 ? (jqSvgEl.height() / 2 - radius / 2) : (jqSvgEl.height() / 2);
                container.attr("transform", "translate(" + (jqSvgEl.width() / 2)+ "," + (translateY - 7) + ")");
                htmlContainer.addClass('visible');

                htmlContainer.scrollTop(scrollY);
                htmlContainer.scrollLeft(scrollX);
            };

            var calcClusterDimensions = function() {
                var defaultDiameter = 550, maxDiameter = 870, startExpandingThreshold = 7, numWE = workingEnvironments.length;
                diameter = numWE < startExpandingThreshold ? defaultDiameter : (maxDiameter * numWE / 10);
                radius = diameter / 2;
                innerRadius = radius - 175;

            };
            var genArrowsTranform = function(point) {
                return function(d, i) {
                    var thePath = link[0][i];
                    var p1 = thePath.getPointAtLength(thePath.getTotalLength() * point);
                    var p2 = thePath.getPointAtLength(thePath.getTotalLength() * (point > 0.5 ? point + 0.02 : point - 0.02));
                    var deg = 90 + (Math.atan2(p2.y - p1.y, p2.x - p1.x) * (180 / Math.PI));
                    return "translate(" + p1.x + "," + p1.y + ") rotate(" + deg + ")";
                };
            };

            var update = function() {
                calcClusterDimensions();
                //update cluster size
                cluster.size([360, innerRadius]);
                // data
                nodes = cluster.nodes(packageHierarchy(workingEnvironments));
                links = packageImports(nodes);

                // DOM
                link = linksContainer.selectAll(".link").data(bundle(links), function(d) {
                    return d[0].publicId + d[2].publicId;
                });
                sourceArrow = sourceArrowsContainer.selectAll(".source-arrow").data(bundle(links), function(d) {
                    return d[0].publicId + d[2].publicId;
                });
                targetArrow = targetArrowsContainer.selectAll(".target-arrow").data(bundle(links), function(d) {
                    return d[0].publicId + d[2].publicId;
                });

                node = g.selectAll('.node').data(
                    //eslint-disable-next-line lodash/prefer-lodash-method
                    nodes.filter((node) => node.name && !node.children),
                    (node) => node.publicId
                );

                // link enter
                link.enter()
                    .append("path")
                    .attr("class", "link")
                    .style("marker-start", "url(#markerCircle)")
                    .style("marker-end", "url(#markerCircle)");

                // link update
                link.attr("d", line)
                    .attr("sourceid", function(d) {
                        return d[0].name + d[0].workingEnvironmentType;
                    })
                    .attr("targetid", function(d) {
                        return d[2].name + d[2].workingEnvironmentType;
                    });
                //link exit
                link.exit().remove();

                sourceArrow.enter().append("polygon")
                    .attr("points", "0,-5 8,5 -8,5");

                targetArrow.enter().append("polygon")
                    .attr("points", "0,-5 8,5 -8,5");

                sourceArrow
                    .classed({'source-arrow': true, source: function(d){
                            return _.includes(d[2].replicationTargets, d[0].publicId);
                        }})
                    .attr("transform", genArrowsTranform(0.2));

                targetArrow
                    .classed({'target-arrow': true, target: function(d){
                            return _.includes(d[0].replicationTargets, d[2].publicId);
                        }})
                    .attr("transform", genArrowsTranform(0.8));

                sourceArrow.exit().remove();
                targetArrow.exit().remove();

                // node enter
                var enterNode = node.enter();
                nodeGraphics = enterNode.append("g")
                    .attr("class", "node")
                    .attr("id", function (d) {
                        return "nodeId" + d.publicId;
                    })
                    .on("mouseover", onMouseOver)
                    .on("mouseenter", onMouseEnter)
                    .on("mouseout", onMouseOut)
                    .on("mouseleave", onMouseLeave)
                    .on("mousedown", onMouseDown);

                createCloudPath(nodeGraphics.append("path")).attr("class", "cloud-image second-cloud").attr("transform", function(d) {
                    return d.isHA ? "translate(20,-20)" : "translate(0,0)";
                });
                createCloudPath(nodeGraphics.append("path")).attr("class", "cloud-image");

                nodeGraphics.append("text").attr("class", "node-title").attr("x", cloudImgWidth/2).attr("y", cloudImgHeight / 2 + 10).attr("text-anchor", "middle");
                nodeGraphics.append("text").attr("class", "node-subtitle").attr("x", cloudImgWidth/2).attr("y", cloudImgHeight / 2 + 27).attr("text-anchor", "middle");

                var nodeGraphicsRoot = nodeGraphics;
                nodeGraphics = nodeGraphicsRoot.append('g').attr("class", "cloud-container");

                var providerNameBoxWidth = function(d) {
                    return d._workingEnvironmentLocation === "ON_PREM" ? 65 : 55;
                };

                var providerDisplayName = function (d) {
                    return Constants.dataONTAPLocations[_.toUpper(d._workingEnvironmentLocation || d._cloudProviderName)].shortName;
                };

                //Adds a colored rectangle on which the cloud provider name will appear.
                nodeGraphics.append("rect").attr("width", providerNameBoxWidth)
                    .attr("height", 20).attr("y", "25").attr("transform", "translate(-15,0)")
                    .attr("class", function (d) {
                        return _.toLower(providerDisplayName(d)) + ' cloud-provider-background';
                    });

                //Adds the cloud provider name.
                nodeGraphics.append("text")
                    .text(function (d) {
                        return providerDisplayName(d);
                    }).attr("width", providerNameBoxWidth).attr("height", 20).attr("y", "25")
                    .attr("transform", function (d) {
                        //puts the text in the middle of the rectangle
                        return "translate(" + ((providerNameBoxWidth(d) / 2) - 15) + ",15)";
                    }).attr("text-anchor", "middle")
                    .attr("class", function (d) {
                        return _.toLower(providerDisplayName(d)) + ' provider-name';
                    });

                //Adds Preview Mode rectangle
                /*nodeGraphics.append('rect')
                    .attr('width', 100).attr('height', 19)
                    .attr('y', '117').attr('x', '60').attr('transform', 'translate(-15,0)')
                    .attr('fill', '#006dc9').attr('rx', '3').attr('ry', '3')
                    .attr('display', (d) => {
                        if (d._cloudProviderName !== 'Azure' || !d.isHA) return 'none';
                    });

                //Adds Preview Mode text
                nodeGraphics.append('text')
                    .text('Preview Mode').attr('width', 120).attr('height', 20).attr('y', '116')
                    .attr('x', '83').attr('font-size', 12).attr('fill', 'white')
                    .attr('transform', 'translate(12,15)')
                    .attr('text-anchor', 'middle')
                    .attr('style', 'fill: white')
                    .attr('display', (d) => {
                        if (d._cloudProviderName !== 'Azure' || !d.isHA) return 'none';
                    });*/

                var InnerNodeGraphics = nodeGraphics.append("g").attr("transform", "translate(35,40)");
                var cloudHeaderContainer = InnerNodeGraphics.append("g").attr("class", "node-header");

                cloudHeaderContainer.append("image").attr("class", "node-ar");

                var nameRegionText = cloudHeaderContainer.append("g").attr("class", "node-name-region");

                nameRegionText.append("text")
                    .attr("class", "node-text");

                nameRegionText.append("text")
                    .attr("class", "lock-icon");

                nameRegionText.append("text")
                    .attr("class", "region-text")
                    .attr("y", "15");

                InnerNodeGraphics.append("image").attr("class", "node-type");
                var capacityGroup = InnerNodeGraphics.append("g").attr("transform", "translate(50,40)").attr("class", "capacityGroup");
                var capacityText = capacityGroup.append('text').attr("class", "capacity-text");
                capacityText.append('tspan').attr('class', 'number');
                capacityText.append('tspan').attr('class', 'unit');

                capacityGroup.append('text').attr("class", "allocated-capacity").attr('y','15');

                nodeGraphics = nodeGraphicsRoot;

                nodeGraphics.append("image").attr("xlink:href", "assets/images/tabs/working-environments/cloud-grey.png")
                    .attr("width", 0)
                    .attr("height", 112)
                    .attr("class", "pending-overlay")
                    .attr("x","-2")
                    .attr("y","-2");

                createCloudPath(nodeGraphics.append("path")).attr("class", "cloud-image cloud-image-stroke").attr('fill', 'transparent').attr('display', 'none');

                nodeGraphics.append("image").attr("class", "ha-icon")
                    .attr("width", function(d) {return d.isHA ? 40 : 0;}).attr("height", 40).attr("x", 170).attr("y", 20);

                nodeGraphics.append("image").attr("class", "balls-container").attr("x", cloudImgWidth/2 - 30).attr("y", cloudImgHeight - 20).attr("width",62)
                // NOC-5321: firefox halts the animation of this gif when its coming from cache so forcing it to come from the server.
                    .attr("height", 0).attr("xlink:href", "assets/images/loader.gif?rand" + new Date().getTime());

                //Adds a cloud sync indication
                var cloudSyncIndication = nodeGraphics.append("g")
                    .attr("width", 70)
                    .attr("height", 20)
                    .attr("y", "110")
                    .attr("class", "cloud-sync-indication");

                cloudSyncIndication.append("image")
                    .attr("width", 20)
                    .attr("height", 15)
                    .attr("background-color", 'red')
                    .attr("class", "cloud-sync-indication-icon");

                cloudSyncIndication.append("text")
                    .attr("class", "cloud-sync-indication-text")
                    .attr('x', 30)
                    .attr('y', 12);

                updateCloudSyncIndication();

                //node update;
                node.attr("transform", rotateNode);

                updateState();
                updateActionRequired();

                node.select('.node-text').text((d) => {
                    const maxLength = (d._encryptionType === 'AWS' || d._encryptionType === 'Cloud Volumes ONTAP' || d._encryptionType === 'Azure' || (scope.actionsRequired && scope.actionsRequired[d.publicId])) ? 11 : 14; //To allow space for the lock icon and action required icon

                    return nocUtil.trimText(d.name, maxLength);
                })
                    .append('title')
                    .text((d) => d.name);

                updateEncryption();

                node.select('.region-text')
                    .text(function (d) {
                        if (d.workingEnvironmentType === 'VSA') {
                            if (d._cloudProviderName === 'AWS') {
                                return ($filter('availabilityZonesFormat')(d.availabilityZones));
                            } else if (d.regionName && d._cloudProviderName === 'GCP') {
                                return d.providerProperties.zoneName;
                            } else if (d.regionName && d._cloudProviderName === 'Azure') {
                                return nocRegionsService.azureRegions()[d.regionName].name;
                            }
                        }
                        //for ON_Prem
                        return "";
                    });

                node.select('.node-type')
                    .attr("xlink:href",function(d) {
                        return d.workingEnvironmentType === "VSA" ? (d.isHA ? (d.status.status === 'degraded' ? "assets/images/tabs/data-fabric/degraded-cloud-inner.png" : "assets/images/tabs/data-fabric/ha-cloud-inner.png") : "assets/images/tabs/data-fabric/cot-cloud-inner.png") : "assets/images/tabs/data-fabric/fas-cloud-inner.png";
                    })
                    .attr("width", "42")
                    .attr("height", "27")
                    .attr("y", "30").attr("x", "-4");

                // node remove
                node.exit().remove();

                fitToScreen(g);
            };

            var updateActionRequired = function() {
                node.each(function(d) {
                    var self = d3.select(this),
                        actionRequiredSym = self.select('.node-ar'),
                        headerContainer = self.select('.node-name-region');

                    if (scope.actionsRequired && scope.actionsRequired[d.publicId]) {
                        actionRequiredSym
                            .attr("xlink:href", scope.actionsRequiredHighestLevel[d.publicId] === "warning" ? "assets/images/tabs/data-fabric/warning-info.png" : "assets/images/tabs/data-fabric/error-info.png")
                            .attr('width', 30).attr('height', 20)
                            .attr('x', d.workingEnvironmentType === 'ON_PREM' ? '15' : '10')
                            .attr('y', -14)
                            .attr('opacity', 1);
                        headerContainer.attr("transform", "translate(45, 0)");
                    } else {
                        headerContainer.attr("transform", "translate(20, 0)");
                        actionRequiredSym.attr("height", 0);
                    }

                    // classes to add the node, to determine the cloud stroke color
                    self.classed('warning', function(d) {
                        return scope.actionsRequiredHighestLevel[d.publicId] === 'warning';
                    });
                    self.classed('error', function(d) {
                        return scope.actionsRequiredHighestLevel[d.publicId] === 'error';
                    });
                });
            };

            var updateEncryption = function() {
                node.each(function() {
                    var self = d3.select(this),
                        encryptionSym = self.select('.lock-icon'),
                        nodeName = self.select('.node-text');

                    var nameWidth = nodeName.node().getBBox().width;
                    encryptionSym
                        .text(function(d) {
                            if (d._encryptionType === 'AWS' || d._encryptionType === 'Cloud Volumes ONTAP' || d._encryptionType === 'Azure' || d._encryptionType === 'GCP') {
                                return "\uf023";
                            }
                        })
                        .attr("style", function (d) {
                            return d._encryptionType === 'AWS' || d._encryptionType === 'Azure' || d._encryptionType === 'GCP' ? "font-family: FontAwesome; fill: #747474;" : "font-family: FontAwesome; fill: #006dc9;";
                        })
                        .attr("width", "12")
                        .attr("height", "12")
                        .attr("y", "0").attr("x", nameWidth + 3);
                });
            };

            var updateCloudSyncIndication = function () {
                node.each(function () {
                    var self = d3.select(this);
                    self.select(".cloud-sync-indication")
                        .attr("transform", "translate(40,115)")
                        .attr("display", function (d) {
                            if (!d._allowedOperations['sync-to-s3-indication-in-gallery']) return 'none';
                        });

                    self.select(".cloud-sync-indication")
                        .attr('class', function (d) {
                            if (_.get(d, 'cloudSyncProperties.status.status') === 'FAILED' || _.get(d, 'cloudSyncProperties._displayAsFailed')) return 'cloud-sync-indication failed';
                            return 'cloud-sync-indication';
                        });

                    self.select(".cloud-sync-indication-icon")
                        .attr("xlink:href", function (d) {
                            if (!d.cloudSyncProperties || !d.cloudSyncProperties.isSynced) return;
                            if (d.cloudSyncProperties.status.status === 'FAILED' || _.get(d, 'cloudSyncProperties._displayAsFailed')) return "assets/images/icons/sync-red.svg";
                            if (d.cloudSyncProperties.status.status === 'COMPLETE') return "assets/images/icons/sync-blue.svg";
                        });

                    self.select(".cloud-sync-indication-text")
                        .text(function (d) {
                            if (!d.cloudSyncProperties || !d.cloudSyncProperties.isSynced) return;
                            if (d.cloudSyncProperties.status.status === 'FAILED' ||
                                d.cloudSyncProperties.status.status === 'COMPLETE') {
                                return 'Synced to S3';
                            }
                        });
                });
            };

            var updateState = function() {
                var showAs = function(elem, we, pending, failed) {
                    var pendingOverlay = elem.select('.pending-overlay'),
                        cloudImageStroke = elem.select('.cloud-image-stroke'),
                        secondCloud = elem.select('.second-cloud'),
                        ballsContainer = elem.select('.balls-container'),
                        capacityNumber = elem.select('.number'),
                        capacityGroup = elem.select('.capacityGroup'),
                        capacityUnit = elem.select('.unit'),
                        allocatedCapacity = elem.select('.allocated-capacity'),
                        title = elem.select('.node-title'),
                        subtitle = elem.select('.node-subtitle');

                    elem.attr("class", 'node ' + we.status.status);
                    ballsContainer.attr("height", pending ? 12 : 0);
                    pendingOverlay.attr("width", pending ? 205 : 0);
                    cloudImageStroke.attr('display', pending ? "block" : "none");

                    if (pending) {
                        secondCloud.style("fill", "#E2E3E3");
                        secondCloud.style("fill-opacity", "0.5");
                    }

                    if(we.status.status === 'degraded') {
                        secondCloud.style("fill", "#E2E3E3");
                        secondCloud.style("stroke-width", "0");
                        secondCloud.style("fill-opacity", "1");
                        secondCloud.attr("xlink:href", "assets/images/tabs/working-environments/cloud-grey.png");
                    } else if(!pending) {
                        secondCloud.style("fill", "#FFFFFF");
                        secondCloud.style("stroke-width", "1px");
                        secondCloud.attr("xlink:href", "assets/images/tabs/working-environments/cloud-default.png");
                    }
                    capacityNumber.text((we.status.status !== "on" && we.status.status !== "degraded") ? (we.status.status === "off" ? we._stateInfo.title : '') : $filter('galleryCapacity')(we._normalizedReservedSizeTB.size));
                    //If we is off, vertically align the text "Turned off" to the center of the icon.
                    if (we.status.status === "off"){
                        capacityGroup.attr("transform", "translate(48, 48)");
                    }else {
                        capacityGroup.attr("transform", "translate(45, 40)");
                    }
                    capacityUnit.text((we.status.status !== "on" && we.status.status !== "degraded") ? "" : " " + we._normalizedReservedSizeTB.unit);
                    allocatedCapacity.text((we.status.status !== "on" && we.status.status !== "degraded") ? (we.status.status === "off" ? "" : we._stateInfo.title) : 'Allocated Capacity');

                    title.text(pending || failed ? we._stateInfo.title : "");
                    var subTitle = (we.isHA && we._stateInfo.haSubTitle) ? we._stateInfo.haSubTitle : we._stateInfo.subTitle;
                    subtitle.text(pending || failed ? subTitle : "");

                    elem.call(workingEnvironments.length < 2 || (we.status.status !== "on" && we.status.status !== "degraded") ? noDrag : drag);
                };

                node.each(function(d) {
                    var self = d3.select(this);
                    if (d.status.status !== "on" && d.status.status !== "off" && d.status.status !== "degraded") {
                        if (d.status.status === "unknown" || d.status.status === "initializing" || d.status.status === "deleting" || d.status.status === "starting" || d.status.status === "stopping" || d.status.status === "updating") {
                            showAs(self, d, true, false);
                        } else {
                            showAs(self, d, false, true);
                        }
                    } else {
                        showAs(self, d, false, false);
                    }
                });
            };

            var resizeFunction =  function() {
                fitToScreen(g);
            };

            $(window).on('resize',resizeFunction);

            scope.$on('$destroy', function() {
                $(window).off('resize', resizeFunction);
            });

            scope.$watchCollection('workingEnvironments', function () {
                workingEnvironments = _.cloneDeep(scope.workingEnvironments);
                update();
            });

            scope.$watchCollection('actionsRequired', function() {
                updateActionRequired();
            });

            scope.$watch('selectedWorkingEnvironment', function(selectedWE) {
                node.classed('picked', function(d) {
                    return selectedWE && selectedWE.publicId === d.publicId;
                });
            });

            scope.$on(workingEnvironmentResource.WORKING_ENVIRONMENT_STATUS_UPDATE, function() {
                workingEnvironments = _.cloneDeep(scope.workingEnvironments);
                update();
            });
        }
    };
});
