// Grid menu killer is responsible for closing the open menu when there is a click anywhere but on the menu
angular.module('noc.components').directive('gridMenuKiller', ($document) => {
    'use strict';

    return {
        controller: function controller() {
            let lastOpenedMenu = null;
            this.setLastOpenedMenu = (el, ctrl) => {
                if (lastOpenedMenu) lastOpenedMenu.ctrl.close();
                lastOpenedMenu = {element: el, ctrl};
            };

            this.getLastOpenedMenu = () => {
                return lastOpenedMenu;
            };
        },
        restrict: 'A',
        require: 'gridMenuKiller',
        link: (scope, element, attrs, ctrl) => {
            const body = $document.find('body');
            const clickKiller = (event) => {
                const lastOpenedMenu = ctrl.getLastOpenedMenu();
                if (!lastOpenedMenu) {
                    return;
                }
                //Close menu if clicked outside it's area:
                if ($(event.target).closest('.grid-menu')[0] !== $(lastOpenedMenu.element)[0]) {
                    lastOpenedMenu.ctrl.close();
                }
            };

            body.on('click', clickKiller);

            scope.$on('$destroy', () => {
                body.off('click', clickKiller);
            });
        }
    };
});

angular.module('noc.components').directive('gridMenu', () => {
    'use strict';

    return {
        controller: function ($scope) {
            $scope.opened = false;
            this.close = () => {
                $scope.opened = false;
                // $scope.opened = !$scope.opened;
                $scope.$applyAsync(); //Fix an issue where the DOM is not immediately updated with the added class
            };
        },
        restrict: 'E',
        templateUrl: 'assets/templates/components/grid/grid-menu.html',
        replace: true,
        require: ['gridMenu', '^gridMenuKiller'],
        link: ($scope, element, attrs, ctrls) => {
            const menuCtrl = ctrls[0];
            const killerCtrl = ctrls[1];
            $scope.gridName = attrs.gridName;

            $scope.toggleOpen = () => {
                // Register as the last opened menu upon opening
                if (!$scope.opened) {
                    killerCtrl.setLastOpenedMenu(element, menuCtrl);
                }
                $scope.opened = !$scope.opened;
            };
        }
    };
});
