angular.module('noc.components').directive('nocGridStickyHeader', ($document) => {
    'use strict';

    return {
        restrict: 'A',
        require: 'uiGrid',
        compile: () => {
            return {
                pre: function pre(scope, element, attributes, ctrl) {
                    const scrollThreshold = parseInt(attributes.nocGridStickyHeader) || 0;
                    let normalZIndex, normalPosition, origHeaders;
                    let headerClonedFillers = [];

                    ctrl.grid.api.core.on.renderingComplete(scope, () => {
                        const onScroll = () => {
                            const scrollOffset = $document.scrollTop();

                            if (!origHeaders || !origHeaders.length) {
                                origHeaders = element.find('.ui-grid-header'); //Find them only once.
                                if (!origHeaders.length) return; // Grid has not been drawn yet
                            }

                            normalPosition = normalPosition || origHeaders.css('position'); //Save the original position
                            normalZIndex = normalZIndex || origHeaders.css('z-index'); //Save the original z-index

                            if (scrollOffset > scrollThreshold) {
                                //When changing original headers position to fixed, they are removed from their original
                                //place in the DOM, so the grid below them moved up (and the first line of the grid is
                                // hidden behind the fixed header). To solve this we create placeholders:
                                if (_.isEmpty(headerClonedFillers)) {
                                    origHeaders.each((index, element) => {
                                        const headerClone = $(element).clone();
                                        $(element).parent()
                                            .prepend(headerClone);
                                        headerClonedFillers.push(headerClone);
                                    });
                                }
                                origHeaders.css('position', 'fixed');
                                origHeaders.css('z-index', 2);
                                origHeaders.css('top', scrollOffset - scrollThreshold);
                            } else {
                                //Remove the placeholders:
                                if (headerClonedFillers.length) {
                                    _.forEach(headerClonedFillers, (element) => {
                                        element.remove();
                                    });
                                    headerClonedFillers = [];
                                }
                                origHeaders.css('position', normalPosition);
                                origHeaders.css('z-index', normalZIndex);
                            }
                        };

                        const document = $($document);
                        document.on('scroll', onScroll);

                        scope.$on('$destroy', () => {
                            document.off('scroll', onScroll);
                        });
                    });
                }
            };
        }
    };
});
