/* When the main-header appears it should trigger the calculation of its filler height */
angular.module("noc").directive("mainHeader", function($rootScope, $timeout, $log) {
    "use strict";
    return {
        restrict: "C",
        link: function() {
            $timeout(function () {
                $log.debug("broadcast header-change-height");
                $rootScope.$broadcast("header-change-height");
            });
        }
    };
});

/*
 This directive is used by the layouts to be able to dynamically set the height of the top fill
 e.g. if alerts are added, or special widgets (like the credentials widget) are added/removed to
 the header, then the top fills size should be changed accordingly

 This directive should not be used for any other purpose!
 */
angular.module("noc.components").directive('nocMatchHeaderHeight', function ($document, $timeout, $rootScope, $log, Constants) {
    "use strict";

    return {
        restrict: 'A',
        link: function (scope, element) {
            var mainHeaderHeight, layoutHeader, virginWizard;

            function setHeight() {
                //set the height of the top fill to be the height of the layouts header + main header
                //occurs in a timeout so that we can measure the new height after dom changes have been made after the current digest cycle finishes

                $timeout(function(){
                    //recalculate -- can be changed due to blue panel hide
                    mainHeaderHeight = $document.find("body").find(".main-header").height();
                    layoutHeader = element.parents(".layout").children(".header");
                    virginWizard = $document.find("body").find(".virgin-header");
                    $log.debug("setting the height mainHeaderHeight " + mainHeaderHeight + " layoutHeaderHeight " + layoutHeader.height());

                    element.css("height", layoutHeader.height() + mainHeaderHeight + (virginWizard.length ? 60 : 0));
                    $rootScope.$broadcast(Constants.HEADER_HEIGHT_CHANGED_EVENT);
                }, 100);
            }

            scope.$watch("alertMessages", function(){
                $log.debug("nocMatchHeaderHeight calculating height due to alert messages change");
                setHeight();
            });

            scope.$on("header-change-height", function(){
                $log.debug("nocMatchHeaderHeight calculating height due to header-change-height");
                setHeight();
            });
        }
    };
});

angular.module('noc.components').directive('nocPrependHttps', () => {
    'use strict';

    return {
        restrict: 'A',
        require: 'ngModel',
        link: function link(scope, element, attributes, controller) {
            let doneOnce = false;
            function ensureHttpPrefix(value) {
                if (!value) {
                    doneOnce = false;

                    return value;
                } else if (!doneOnce && !(/^(https?):\/\//i).test(value) && (!_.includes('https://', value) && !_.includes('http://', value))) {
                    controller.$setViewValue('https://' + value);
                    controller.$render();
                    doneOnce = true;

                    return 'https://' + value;
                } else {
                    return value;
                }
            }
            controller.$formatters.push(ensureHttpPrefix);
            controller.$parsers.splice(0, 0, ensureHttpPrefix);
        }
    };
});
