(() => {
    'use strict';

    angular.module('noc.components').directive('nocAutocomplete', () => {
        let index = -1;

        return {
            restrict: 'E',
            scope: {
                searchParam: '=ngModel',
                suggestions: '=data',
                onType: '=onType',
                onSelect: '=onSelect',
                autocompleteRequired: '@',
                tagValidation: '='
            },
            transclude: true,
            controller: ['$scope', function($scope){
                // the index of the suggestions that's currently selected

                $scope.selectedIndex = -1;

                $scope.initLock = true;

                // set new index
                $scope.setIndex = function(i){
                    $scope.selectedIndex = parseInt(i);
                };

                this.setIndex = function(i){
                    $scope.setIndex(i);
                    $scope.$apply();
                };

                $scope.getIndex = function(){
                    return $scope.selectedIndex;
                };

                // watches if the parameter filter should be changed
                var watching = true;

                // autocompleting drop down on/off
                $scope.completing = false;

                // starts autocompleting on typing in something
                $scope.$watch('searchParam', function(newValue, oldValue){

                    if (oldValue === newValue || (!oldValue && $scope.initLock)) {
                        return;
                    }

                    if(watching && !_.isUndefined($scope.searchParam) && $scope.searchParam !== null) {
                        $scope.completing = true;
                        $scope.searchFilter = $scope.searchParam;
                        $scope.selectedIndex = -1;
                    }

                    //Function that is passed to on-type attribute gets executed
                    if ($scope.onType) $scope.onType($scope.searchParam);
                });

                // for hovering over suggestions
                this.preSelect = function(){

                    watching = false;

                    // this line determines if it is shown
                    // in the input field before it's selected:
                    //$scope.searchParam = suggestion;

                    $scope.$apply();
                    watching = true;

                };

                $scope.preSelect = this.preSelect;

                this.preSelectOff = function(){
                    watching = true;
                };

                $scope.preSelectOff = this.preSelectOff;

                //Selecting a suggestion with RIGHT ARROW or ENTER
                $scope.select = (suggestion) => {
                    if (suggestion) {
                        $scope.searchParam = suggestion;
                        $scope.searchFilter = suggestion;
                        if ($scope.onSelect) {
                            $scope.onSelect(suggestion);
                        }
                    }
                    watching = false;
                    $scope.completing = false;
                    setTimeout(() => {
                        watching = true;
                    }, 1000);
                    $scope.setIndex(-1);
                };
            }],
            link: function(scope, element, attrs){

                setTimeout(function() {
                    scope.initLock = false;
                    scope.$apply();
                }, 250);

                var attr = '';

                // Default atts
                scope.attrs = {
                    "placeholder": "start typing...",
                    "class": "",
                    "id": "",
                    "inputclass": "",
                    "inputid": ""
                };

                for (var a in attrs) {
                    attr = _(a).toLower().replace('attr', '');
                    // add attribute overriding defaults
                    // and preventing duplication
                    if (_.startsWith(a, 'attr')) {
                        scope.attrs[attr] = attrs[a];
                    }
                }

                if (attrs.clickActivation) {
                    element[0].onclick = function(){
                        if(!scope.searchParam){
                            setTimeout(function() {
                                scope.completing = true;
                                scope.$apply();
                            }, 200);
                        }
                    };
                }

                const key = {left: 37, up: 38, right: 39, down: 40, enter: 13, esc: 27, tab: 9};

                document.addEventListener('keydown', (e) => {
                    const keyCode = e.keyCode || e.which;

                    if (keyCode === key.esc) {
                        //Disable suggestions on escape
                        scope.select();
                        scope.setIndex(-1);
                        scope.$apply();
                        e.preventDefault();
                    }
                }, true);

                document.addEventListener("blur", function(){
                    // disable suggestions on blur
                    // we do a timeout to prevent hiding it before a click event is registered
                    setTimeout(function() {
                        scope.select();
                        scope.setIndex(-1);
                        scope.$apply();
                    }, 150);
                }, true);

                element[0].addEventListener("keydown",function (e){
                    var keycode = e.keyCode || e.which;

                    var l = angular.element(this).find('li').length;

                    //This allows submitting forms by pressing Enter in the autocompleted field
                    if (!scope.completing || l === 0) return;

                    // implementation of the up and down movement in the list of suggestions
                    switch (keycode){
                        case key.up:

                            index = scope.getIndex()-1;
                            if(index<-1){
                                index = l-1;
                            } else if (index >= l ){
                                index = -1;
                                scope.setIndex(index);
                                scope.preSelectOff();
                                break;
                            }
                            scope.setIndex(index);

                            if (index !== -1) {
                                scope.preSelect(angular.element(angular.element(this).find('li')[index]).text());
                            }

                            scope.$apply();

                            break;
                        case key.down:
                            index = scope.getIndex()+1;
                            if (index < -1) {
                                index = l - 1;
                            } else if (index >= l) {
                                index = -1;
                                scope.setIndex(index);
                                scope.preSelectOff();
                                scope.$apply();
                                break;
                            }
                            scope.setIndex(index);

                            if (index !== -1) {
                                scope.preSelect(angular.element(angular.element(this).find('li')[index]).text());
                            }

                            break;
                        case key.left:
                            break;
                        case key.right:
                        case key.enter:
                        case key.tab:

                            index = scope.getIndex();
                            if (index !== -1) {
                                scope.select(angular.element(angular.element(this).find('li')[index]).text());
                                if (keycode === key.enter) {
                                    e.preventDefault();
                                }
                            } else if (keycode === key.enter) {
                                scope.select();
                            }
                            scope.setIndex(-1);
                            scope.$apply();

                            break;
                        case key.esc:
                            // disable suggestions on escape
                            scope.select();
                            scope.setIndex(-1);
                            scope.$apply();
                            e.preventDefault();
                            break;
                        default:
                    }
                });
            },
            templateUrl:'assets/templates/components/noc-autocomplete.html'
        };
    });

    angular.module("noc.components").directive('nocSuggestion', function(){
        return {
            restrict: 'A',
            require: '^nocAutocomplete', // ^look for controller on parents element
            link: function(scope, element, attrs, autoCtrl){
                element.bind('mouseenter', function() {
                    autoCtrl.preSelect(attrs.val);
                    autoCtrl.setIndex(attrs.index);
                });

                element.bind('mouseleave', function() {
                    autoCtrl.preSelectOff();
                });
            }
        };
    });
})();
