angular.module('noc').component('pieChartComponent', {
    bindings: {
        mainTitle: '<',
        subtitle: '@',
        chartData: '<',
        colorsArray: '<',
        diameterSize: '<',
        showPercentageInTooltip: '<'
    },
    controller: function ($element) {

        /* This function actually wait half of the duration
               and run the animation linearly the other half of duration */
        this.easeAnimation = (pos) => {
            if ((pos) < 0.5) {
                return (0);
            }

            return (2 * (pos - 0.5));
        };

        this.getChartOptions = (element) => {
            return {
                chart: {
                    renderTo: element,
                    type: 'pie',
                    events: {
                        redraw: updateTextsLocation
                    },
                    height: this.diameterSize + 20,
                    width: this.diameterSize + 20,
                    marginBottom: 0,
                    marginTop: 0,
                    marginLeft: 0,
                    marginRight: 0
                },
                title: {
                    floating: true,
                    text: ''
                },
                credits: {
                    enabled: false
                },
                tooltip: {
                    pointFormat: this.showPercentageInTooltip ? '{point.percentage:.1f}%' : ''
                },
                plotOptions: {
                    pie: {
                        innerSize: '87%',
                        dataLabels: {
                            enabled: false
                        },
                        colors: this.colorsArray,
                        borderColor: null,
                        borderWidth: 0
                    }
                },
                series: [{
                    data: this.chartData,
                    animation: {
                        duration: 1500

                       /* easing: this.easeAnimation*/ /* This function actually wait half of the duration
                                                           and run the animation linearly the other half of duration */
                    }
                }]
            };
        };

        let pieHighchart = null;
        let mainTitleTextObj = null;
        let subtitleTextObj = null;

        const updateChartData = (chart, mainTitleTextObj, subtitleTextObj, mainTitle, subtitle, chartData) => {
            mainTitleTextObj.textSetter(mainTitle);
            subtitleTextObj.textSetter(subtitle);
            chart.series[0].setData(chartData, true);
        };

        const updateTextsLocation = () => {
            let x;
            let y;
            let textBoundingBox;
            if (pieHighchart && mainTitleTextObj && subtitleTextObj) {
                textBoundingBox = mainTitleTextObj.getBBox();
                x = pieHighchart.plotLeft + 0.5 * (pieHighchart.plotWidth - textBoundingBox.width);
                y = pieHighchart.plotTop + (pieHighchart.plotHeight * 0.48);
                mainTitleTextObj.attr({x, y});

                textBoundingBox = subtitleTextObj.getBBox();
                x = pieHighchart.plotLeft + 0.5 * (pieHighchart.plotWidth - textBoundingBox.width);
                y = pieHighchart.plotTop + (pieHighchart.plotHeight * 0.6);
                subtitleTextObj.attr({x, y});
            }
        };

        this.$onDestroy = () => {
            if (pieHighchart) {
                pieHighchart.destroy();
            }
        };

        this.$onChanges = () => {
            if (pieHighchart) {
                updateChartData(pieHighchart, mainTitleTextObj, subtitleTextObj, this.mainTitle, this.subtitle, this.chartData);
            }
        };

        this.$onInit = () => {
            const $chartContainer = $element.append('<div></div>');
            this.chartOptions = this.getChartOptions($chartContainer[0]);
            pieHighchart = new Highcharts.Chart(this.chartOptions);

            //Create the objects for the texts in the middle of the pie chart:
            mainTitleTextObj = pieHighchart.renderer.text().add();
            subtitleTextObj = pieHighchart.renderer.text().add();
            mainTitleTextObj.css({
                fontSize: '22px',
                fontFamily: 'Open sans',
                fontWeight: '600',
                color: '#01225e'
            });
            subtitleTextObj.css({
                fontSize: '13px',
                fontFamily: 'Open sans',
                fontWeight: '600',
                color: '#01225e'
            });

            updateChartData(pieHighchart, mainTitleTextObj, subtitleTextObj, this.mainTitle, this.subtitle, this.chartData);
        };
    }
});
