(function () {
    "use strict";

    angular.module('noc').directive('radialProgress', function () {
        function polarToCartesian(centerX, centerY, radius, angleInDegrees) {
            var angleInRadians = (angleInDegrees-90) * Math.PI / 180.0;

            return {
                x: centerX + (radius * Math.cos(angleInRadians)),
                y: centerY + (radius * Math.sin(angleInRadians))
            };
        }

        function describeArc(x, y, radius, startAngle, endAngle){

            var start = polarToCartesian(x, y, radius, endAngle);
            var end = polarToCartesian(x, y, radius, startAngle);

            var arcSweep = endAngle - startAngle <= 180 ? "0" : "1";

           return [
                "M", start.x, start.y,
                "A", radius, radius, 0, arcSweep, 0, end.x, end.y
            ].join(" ");
        }

        return {
            restrict: "E",
            templateUrl: "assets/templates/components/radial-progress/radial-progress.html",
            link: function (scope, element, attrs) {
                scope.r = 28;
                scope.x = 30;
                scope.y = 30;
                scope.strokeWidth = 6;

                var arc = element.find("path");

                attrs.$observe('rpPercentage', function(value) {
                    arc.attr('d', describeArc(scope.x, scope.y, scope.r - scope.strokeWidth/2, 0, 360 * value || 1));
                    scope.text = (value > 0 && value < 0.01 ? "<1" : Math.round(value*100)) + "%";
                });
            }

        };
    });
})();