angular.module('noc.components').directive('nocRangeVerify', () => {
    'use strict';

    return {
        require: '?ngModel',
        scope: {
            nocPasswordVerify: '='
        },
        link: function link(scope, element, attributes, ctrl) {
            const allowedRange = _.split(attributes.nocRangeVerify, '-');
            const minAllowedRange = parseInt(allowedRange[0]);
            const maxAllowedRange = parseInt(allowedRange[1]);
            const rangeSize = attributes.nocRangeVerifySize ? parseInt(attributes.nocRangeVerifySize) : maxAllowedRange - minAllowedRange;

            let keepViewValue;

            const validate = (viewValue) => {
                let valid = true;
                keepViewValue = viewValue;

                const match = viewValue.match(/^([0-9]|[1-9][0-9]|[1-9][0-9][0-9]|[1-3][0-9][0-9][0-9]|[4][0][0-8][0-9]|[4][0][9][1-5])-([0-9]|[1-9][0-9]|[1-9][0-9][0-9]|[1-3][0-9][0-9][0-9]|[4][0][0-8][0-9]|[4][0][9][1-6])$/);
                if (match) {
                    const minViewRange = parseInt(match[1]);
                    const maxViewRange = parseInt(match[2]);

                    valid = minViewRange >= minAllowedRange && maxViewRange <= maxAllowedRange && maxViewRange > minViewRange && maxViewRange - minViewRange <= rangeSize;

                    // It's valid if model is in range
                    ctrl.$setValidity('rangeVerify', valid);
                } else {
                    ctrl.$setValidity('rangeVerify', false);
                }

                return viewValue;
            };

            ctrl.$parsers.unshift(validate);
            ctrl.$formatters.push(validate);

            scope.$watch(() => {
                if (keepViewValue !== ctrl.$viewValue) {
                    keepViewValue = ctrl.$viewValue;
                }

                return keepViewValue;
            }, (value) => {
                validate(value);
            });
        }
    };
});
