(() => {
    'use strict';

    angular.module('noc').controller('AccountSettingsController', ($scope, CloudProviderAccountResource, nocHttp,
                                                                   $window, $timeout, nocDialog, nocUtil) => {
        $scope.initialize = () => {
            $scope.accountMenuAvailable = true;
            CloudProviderAccountResource.getFormattedAccounts(nocHttp)
                .then((accounts) => {
                    $scope.accounts = accounts;
                    const instanceProfileOrManagedIdentityAccountIndex = _.findIndex($scope.accounts, (account) => {
                        return (account.accountType === 'INSTANCE_PROFILE') || (account.accountType === 'SERVICE_IDENTITY');
                    });
                    if (instanceProfileOrManagedIdentityAccountIndex > 0) {
                        const instanceProfileOrManagedIdentityAccount = $scope.accounts.splice(instanceProfileOrManagedIdentityAccountIndex, 1);
                        $scope.accounts.unshift(instanceProfileOrManagedIdentityAccount[0]);
                    }
                    $scope.filteredAccounts = $scope.accounts;
                });
        };

        $scope.tabs = [
            {tabName: 'working-environments', title: 'Working Environments'},
            {tabName: 'replication-status', title: 'Replication Status'},
            {tabName: 'k8s-clusters', title: 'Kubernetes Clusters'},
            {tabName: 'timeline', title: 'Timeline'}
        ];

        $scope.onTabClick = (tab) => {
            $scope.checkAndSetState('main.' + tab.tabName + (tab.tabName !== 'timeline' ? '.index' : ''));
        };

        const doFiltering = nocUtil.getNgFilterFunc($scope, 'accounts', 'filteredAccounts', 'searchAccounts',
            ['accountName', '_prettyAccountType', 'accessKey', 'accountId', 'occmRole', 'applicationId', 'tenantId', '_roleName']);

        $window.onclick = () => {
            $timeout(() => {
                if (_.isNumber($scope.selectedCardIndex)) {
                    $scope.selectedCardIndex = null;
                }
            }); //Timeout actives digest cycle so that account menu instantly closes
        };

        $scope.toggleCardMenu = (index, event) => {
            $scope.selectedCardIndex = index;
            if (event) event.stopPropagation();
        };

        $scope.goToEditAccount = (account) => {
            $scope.checkAndSetState('cloud-provider-accounts.add-' + account._provider + '-account', {
                accountPublicId: account.publicId
            });
        };

        $scope.deleteAccount = (account) => {
            return nocDialog.confirm('', 'assets/templates/cloud-provider-accounts/delete-account.html', {
                width: 650,
                accountName: account.accountName
            }).then(() => {
                CloudProviderAccountResource.deleteAccount(nocHttp, account.publicId)
                    .then(() => {
                        CloudProviderAccountResource.getFormattedAccounts(nocHttp)
                            .then((accounts) => {
                                if (accounts.length > 0) {
                                    $scope.initialize();
                                } else {
                                    $scope.goToAddCloudProviderAccount();
                                }
                            });
                    });
            });
        };

        $scope.$watch('searchAccounts', () => {
            doFiltering();
        });

        $scope.initialize();
    });
})();
