(() => {
    'use strict';

    angular.module('noc').controller('AddAwsAccountController', ($scope, nocDialog, CloudProviderAccountResource,
                                                                 nocHttp, nocUI, $rootScope, $state) => {
        $scope.initialize = () => {
            $scope.awsAccountRequest = {};
            $scope.accountType = {};
            const instancePlacement = $rootScope.instancePlacement;
            $scope.hasInstanceProfile = !!(instancePlacement && instancePlacement.role && instancePlacement.providerRoleType === 'AWS');
            $scope.editMode = Boolean($state.params.accountPublicId);
            if ($scope.editMode) {
                CloudProviderAccountResource.getFormattedAccounts(nocHttp)
                    .then((accounts) => {
                        $scope.selectedAccount = _.find(accounts, {publicId: $state.params.accountPublicId});
                        $scope.awsAccountRequest.accountName = $scope.selectedAccount.accountName;
                        if ($scope.selectedAccount.accountType === 'AWS_KEYS') {
                            $scope.accountType.name = 'keys';
                            $scope.awsAccountRequest.providerKeys = {awsAccessKeys: {accessKey: $scope.selectedAccount.accessKey}};
                        } else {
                            $scope.accountType.name = 'assume-role';
                            $scope.awsAccountRequest.providerKeys = {assumeRoleArn: $scope.selectedAccount.assumeRoleArn};
                        }
                    });
            } else {
                $scope.accountType.name = 'keys';
            }
        };

        $scope.openLearnMoreDialog = () => {
            nocDialog.confirm('', 'assets/templates/cloud-provider-accounts/aws-learn-more.html', {
                width: 890,
                fromTop: '20%'
            });
        };

        $scope.goBack = () => {
            if ($scope.editMode) {
                $scope.goToCloudProviderAccountSettings();
            } else {
                $scope.goToAddCloudProviderAccount();
            }
        };

        const onError = (error) => {
            if (_.endsWith(error.message, 'Account name is already in use.')) {
                $scope.nameErrorMessage = 'Account name already exists';
            } else {
                $scope.errorMessage = error.message;
            }
        };

        $scope.createOrUpdateAwsAccount = () => {
            $scope.errorMessage = '';
            $scope.nameErrorMessage = '';
            if ($scope.addAwsAccountForm.$valid) {
                if ($scope.accountType.name === 'assume-role') {
                    $scope.awsAccountRequest.providerKeys.awsAccessKeys = null;
                } else {
                    $scope.awsAccountRequest.providerKeys.assumeRoleArn = null;
                }

                if ($scope.editMode) {
                    return CloudProviderAccountResource.editAccount(nocHttp, 'aws', $scope.selectedAccount.publicId, $scope.awsAccountRequest.accountName, $scope.awsAccountRequest.providerKeys)
                        .then($scope.goToCloudProviderAccountSettings)
                        .catch((error) => {
                            onError(error);
                        });
                } else {
                    return CloudProviderAccountResource.createAccount(nocHttp, 'aws', $scope.awsAccountRequest.accountName, $scope.awsAccountRequest.providerKeys, null, true)
                        .then($scope.goToCloudProviderAccountSettings)
                        .catch((error) => {
                            onError(error);
                        });
                }
            } else {
                nocUI.handleFormValidity();
            }
        };

        $scope.initialize();
    });
})();
