(function () {
    'use strict';

    angular.module('noc').factory('providerDiskTypes', function () {
        const diskTypesAws = [
            {
                name: 'gp2',
                shortName: 'gp2',
                diskName: 'gp2',
                display: 'General Purpose SSD (gp2)',
                description: 'General purpose SSD volume that balances price and performance.',
                supportedInHa: true,
                supportedCapacityTiers: ['S3'],
                'default': true
            },
            {
                name: 'standard',
                shortName: 'std',
                diskName: 'standard',
                display: 'Previous generation HDD (standard)',
                description: 'Lower cost and performance. Performance may be significantly lower for some workloads.',
                supportedInHa: true
            },
            {
                name: 'st1',
                shortName: 'st1',
                diskName: 'st1',
                display: 'Throughput Optimized HDD (st1)',
                description: 'Low cost HDD volume designed for frequently accessed, throughput-intensive workloads.',
                supportedInHa: true,
                supportedCapacityTiers: ['S3']
            },
            {
                name: 'io1',
                shortName: 'io1',
                diskName: 'io1',
                display: 'Provisioned IOPS SSD (io1)',
                description: 'For critical applications that require the highest performance at a higher cost.',
                supportedInHa: true,
                supportedCapacityTiers: ['S3']
            },
            {
                name: 'sc1',
                shortName: 'sc1',
                diskName: 'sc1',
                display: 'Cold HDD (sc1)',
                description: 'Lowest cost HDD volume designed for less frequently accessed workloads.',
                supportedInHa: false
            }
        ];

        const diskTypesAzure = [
            {
                name: 'Premium_LRS',
                shortName: 'Prm',
                diskName: 'Premium_LRS',
                display: 'Premium SSD',
                description: 'Higher performance at a higher cost.',
                supportedCapacityTiers: ['Blob'],
                'default': true
            },
            {
                name: 'StandardSSD_LRS',
                shortName: 'Ssd',
                diskName: 'StandardSSD_LRS',
                display: 'Standard SSD',
                description: 'For production workloads when you need predictable IOPS performance with lower latency than HDD.',
                supportedCapacityTiers: ['Blob']
            },
            {
                name: 'Standard_LRS',
                shortName: 'Hdd',
                diskName: 'Standard_LRS',
                display: 'Standard HDD',
                description: 'Lower cost and performance. Performance may be significantly lower for some workloads.',
                supportedCapacityTiers: ['Blob']
            }
        ];

        const diskTypesGoogle = [
            {
                name: 'pd-ssd',
                shortName: 'Ssd',
                diskName: 'pd-ssd',
                display: 'SSD',
                description: 'Higher performance at a higher cost.',
                supportedCapacityTiers: ['cloudStorage'],
                'default': true
            },
            {
                name: 'pd-standard',
                shortName: 'Std',
                diskName: 'pd-standard',
                display: 'Standard',
                description: 'Lower cost and performance. Performance may be significantly lower for some workloads.',
                supportedCapacityTiers: ['cloudStorage']
            }
        ];
        //Sanity:
        /* istanbul ignore next */
        if (_.filter(diskTypesAws, {'default': true}).length !== 1) {
            throw new Error('number of default AWS disks is different than one');
        }
        /* istanbul ignore next */
        if (_.filter(diskTypesAzure, {'default': true}).length !== 1) {
            throw new Error('number of default Azure disks is different than one');
        }

        return {
            'AWS': _.keyBy(diskTypesAws, 'name'),
            'Amazon': _.keyBy(diskTypesAws, 'name'),
            'Azure': _.keyBy(diskTypesAzure, 'name'),
            'GCP': _.keyBy(diskTypesGoogle, 'name')
        };
    });
})();
