(() => {
    'use strict';

    angular.module('noc').factory('workingEnvironmentStatuses', (versionUtil) => {
        const workingEnvironmentStatuses = {};

        workingEnvironmentStatuses.stateInfoMap = {
            'unknown': {title: 'Loading data...'},
            'initializing': {title: 'Initializing...', subTitle: 'This could take about 25 minutes', haSubTitle: 'This could take about 30 minutes'},
            'starting': {title: 'Starting...'},
            'on': {},
            'off': {title: 'Turned off'},
            'stopping': {title: 'Stopping...'},
            'failed': {title: 'Failed'},
            'failed-to-create': {title: 'Failed to create'},
            'deleting': {title: 'Deleting...', subTitle: 'This could take about 5 minutes'},
            'not-found': {title: 'Not Found'},
            'deleted': {},
            'updating': {title: 'Updating...', subTitle: 'This could take about 25 minutes', haSubTitle: 'This could take about 90 minutes'},
            'degraded': {}
        };

        const workingEnvironmentOperationsToAllowedStatusesAndTypes = {
            'access-timeline-from-gallery': {
                statuses: ['failed-to-create'],
                types: ['VSA_AWS', 'VSA_Azure', 'VSA_GCP']
            },
            'add-capacity': {
                statuses: ['on', 'degraded'],
                types: ['VSA_AWS', 'VSA_Azure', 'ON_PREM', 'VSA_GCP']
            },
            'aggregate-add-capacity': {
                statuses: ['on'],
                types: ['VSA_AWS', 'VSA_Azure', 'VSA_GCP']
            },
            'aggregate-add-disks': {
                statuses: ['on'],
                types: ['VSA_AWS', 'VSA_Azure', 'VSA_GCP']
            },
            'aggregate-delete': {
                statuses: ['on'],
                types: ['VSA_AWS', 'VSA_Azure', 'VSA_GCP']
            },
            'aggregate-info': {
                statuses: ['on'],
                types: ['VSA_AWS', 'VSA_Azure', 'VSA_GCP']
            },
            'change-instance-type': {
                statuses: ['on', 'off'],
                types: ['VSA_AWS', 'VSA_Azure', 'VSA_GCP']
            },
            'change-license': {
                statuses: ['on', 'off'],
                types: ['VSA_AWS', 'VSA_Azure', 'VSA_GCP']
            },
            'change-network-utilization': {
                statuses: ['on', 'degraded'],
                types: ['VSA_AWS']
            },
            'change-ontap-image': {
                statuses: ['on'],
                types: ['VSA_AWS', 'VSA_Azure', 'VSA_GCP']
            },
            'change-tier': {
                statuses: ['on'],
                types: ['VSA_AWS', 'VSA_Azure', 'VSA_GCP']
            },
            'cifs-setup': {
                statuses: ['on'],
                types: ['VSA_AWS', 'VSA_Azure', 'VSA_GCP']
            },
            'clone-volume': {
                statuses: ['on', 'degraded'],
                types: ['VSA_AWS', 'VSA_Azure', 'ON_PREM', 'VSA_GCP']
            },
            'configuration-backups': {
                statuses: ['on'],
                types: ['VSA_AWS', 'VSA_Azure', 'VSA_GCP']
            },
            'configure-k8s': {
                statuses: ['on', 'degraded'],
                types: ['VSA_AWS', 'VSA_Azure', 'VSA_GCP']
            },
            'cost': {
                statuses: ['on', 'degraded', 'off', 'starting', 'stopping', 'updating'],
                types: ['VSA_AWS']
            },
            'delete': {
                statuses: ['on', 'off', 'starting', 'stopping', 'updating', 'failed', 'degraded'],
                types: ['VSA_AWS', 'VSA_Azure', 'VSA_GCP']
            },
            'delete-volume': {
                statuses: ['on', 'degraded'],
                types: ['VSA_AWS', 'VSA_Azure', 'ON_PREM', 'VSA_GCP']
            },
            'edit-tags': {
                statuses: ['on', 'degraded', 'off'],
                types: ['VSA_AWS', 'VSA_Azure', 'VSA_GCP']
            },
            'edit-volume': {
                statuses: ['on', 'degraded'],
                types: ['VSA_AWS', 'VSA_Azure', 'ON_PREM', 'VSA_GCP']
            },
            'enable-capacity-tiering': {
                statuses: ['on', 'degraded'],
                types: ['VSA_AWS', 'VSA_Azure', 'VSA_GCP']
            },
            'launch-system-manager': {
                statuses: ['on', 'degraded'],
                types: ['VSA_AWS', 'VSA_Azure', 'ON_PREM', 'VSA_GCP']
            },
            'monitor-ha': {
                statuses: ['on', 'degraded'],
                types: ['VSA_AWS', 'VSA_Azure']
            },
            'no-actions-in-gallery': {
                statuses: ['initializing', 'starting', 'stopping', 'deleting', 'updating', 'unknown'],
                types: ['VSA_AWS', 'VSA_Azure', 'ON_PREM', 'VSA_GCP']
            },
            'no-quick-navigation-in-gallery': {
                statuses: ['unknown'],
                types: ['VSA_AWS', 'VSA_Azure', 'ON_PREM', 'VSA_GCP']
            },
            'ransomware-protection': {
                statuses: ['on', 'degraded'],
                types: ['VSA_AWS', 'VSA_Azure', 'VSA_GCP']
            },
            'refresh': {
                statuses: ['on', 'degraded', 'failed'],
                types: ['VSA_AWS', 'VSA_Azure', 'ON_PREM', 'VSA_GCP']
            },
            'register-support': {
                statuses: ['on'],
                types: ['VSA_AWS', 'VSA_Azure', 'VSA_GCP']
            },
            'replicate': {
                statuses: ['on'],
                types: ['VSA_AWS', 'VSA_Azure', 'ON_PREM', 'VSA_GCP']
            },
            'replication-status': {
                statuses: ['on', 'degraded'],
                types: ['VSA_AWS', 'VSA_Azure', 'ON_PREM', 'VSA_GCP']
            },
            'resources': {
                statuses: ['on', 'off', 'starting', 'stopping', 'deleting', 'initializing', 'updating', 'failed', 'not-found', 'degraded'],
                types: ['VSA_AWS', 'VSA_Azure', 'ON_PREM', 'VSA_GCP']
            },
            'resources-menu': {
                statuses: ['on', 'off', 'degraded', 'not-found', 'failed'],
                types: ['VSA_AWS', 'VSA_Azure', 'ON_PREM', 'VSA_GCP']
            },
            'return-to-wizard': {
                statuses: ['failed-to-create'],
                types: ['VSA_AWS', 'VSA_Azure', 'VSA_GCP']
            },
            'set-password': {
                statuses: ['on'],
                types: ['VSA_AWS', 'VSA_Azure', 'VSA_GCP']
            },
            'show-failure-message': {
                statuses: ['failed', 'not-found', 'failed-to-create'],
                types: ['VSA_AWS', 'VSA_Azure', 'ON_PREM', 'VSA_GCP']
            },
            'shutdown-vsa': {
                statuses: ['on', 'off', 'degraded'],
                types: ['VSA_AWS', 'VSA_Azure', 'VSA_GCP']
            },
            'snapshot-volume': {
                statuses: ['on', 'degraded'],
                types: ['VSA_AWS', 'VSA_Azure', 'ON_PREM', 'VSA_GCP']
            },
            'sync-to-s3': {
                statuses: ['on', 'degraded'],
                types: ['VSA_AWS', 'ON_PREM']
            },
            'sync-to-s3-indication-in-gallery': {
                statuses: ['on', 'degraded'],
                types: ['VSA_AWS', 'ON_PREM']
            },
            'tiering-level': {
                statuses: ['on', 'degraded'],
                types: ['VSA_AWS', 'VSA_Azure']
            },
            'turn-off': {
                statuses: ['on', 'degraded'],
                types: ['VSA_AWS', 'VSA_Azure', 'VSA_GCP']
            },
            'turn-on': {
                statuses: ['off'],
                types: ['VSA_AWS', 'VSA_Azure', 'VSA_GCP']
            },
            'update-credentials-from-gallery': {
                statuses: ['failed'],
                types: ['VSA_AWS', 'ON_PREM', 'VSA_Azure', 'VSA_GCP']
            },
            'upgrade-mediator': {
                statuses: ['on', 'degraded'],
                types: ['VSA_AWS', 'VSA_Azure']
            },
            'view-aggregates': {
                statuses: ['on'],
                types: ['VSA_AWS', 'VSA_Azure', 'VSA_GCP']
            },
            'view-information': {
                statuses: ['on', 'degraded'],
                types: ['VSA_AWS', 'VSA_Azure', 'ON_PREM', 'VSA_GCP']
            },
            'view-instances': {
                statuses: ['on', 'off', 'starting', 'stopping', 'initializing', 'deleting', 'failed', 'not-found', 'deleted', 'updating', 'degraded'],
                types: ['VSA_AWS']
            },
            'view-instances-data': {
                statuses: ['on', 'off', 'starting', 'stopping', 'initializing', 'updating', 'degraded'],
                types: ['VSA_AWS']
            },
            'view-svm': {
                // todo: check if google is relevant
                statuses: ['on', 'degraded'],
                types: ['VSA_AWS', 'VSA_Azure', 'ON_PREM', 'VSA_GCP']
            },
            'writing-speed': {
                statuses: ['on'],
                types: ['VSA_AWS', 'VSA_Azure', 'VSA_GCP']
            }
        };

        workingEnvironmentStatuses.isRemovedAllowed = function isRemovedAllowed(typeAndProvider, status) {
            const map = {
                'VSA_AWS': ['not-found', 'failed-to-create'],
                'VSA_Azure': ['not-found', 'failed-to-create'],
                'VSA_GCP': ['not-found', 'failed-to-create'],
                'ON_PREM': ['on', 'failed']
            };

            return _.includes(map[typeAndProvider], status);
        };

        workingEnvironmentStatuses.getAllowedOperations = function getAllowedOperations(typeAndProvider, status) {
            const allowedOperations = _.mapValues(workingEnvironmentOperationsToAllowedStatusesAndTypes, (operation) => {
                return _.includes(operation.statuses, status) && _.includes(operation.types, typeAndProvider);
            });
            allowedOperations.remove = workingEnvironmentStatuses.isRemovedAllowed(typeAndProvider, status);

            return allowedOperations;
        };

        workingEnvironmentStatuses.relationshipActionsSourceAndTargetToWorkingEnvironmentStates = {
            'break-relationship': {
                source: ['on', 'degraded', 'off'],
                target: ['on', 'degraded'],
                policyType: ['async_mirror', 'mirror_vault']
            },
            'resync-relationship': {
                source: ['on', 'degraded'],
                target: ['on', 'degraded'],
                policyType: ['async_mirror', 'mirror_vault']
            },
            'reverse-relationship': {source: ['on'], target: ['on'], policyType: ['async_mirror', 'mirror_vault']},
            'edit-relationship': {
                source: ['on', 'degraded'],
                target: ['on', 'degraded'],
                policyType: ['async_mirror', 'mirror_vault', 'vault']
            },
            'delete-relationship': {
                source: ['on', 'degraded'],
                target: ['on', 'degraded'],
                policyType: ['async_mirror', 'mirror_vault', 'vault']
            },
            'initialize-relationship': {
                source: ['on', 'degraded'],
                target: ['on', 'degraded'],
                policyType: ['async_mirror', 'mirror_vault', 'vault']
            },
            'policy-info': {
                source: ['on', 'degraded'],
                target: ['on', 'degraded'],
                policyType: ['async_mirror', 'mirror_vault', 'vault']
            },
            'update-snapmirror': {
                source: ['on', 'degraded'],
                target: ['on', 'degraded'],
                policyType: ['async_mirror', 'mirror_vault', 'vault']
            }
        };

        workingEnvironmentStatuses.isRelationshipOperationAllowed = function isRelationshipOperationAllowed(operation, sourceWorkingEnvironmentStatus, targetWorkingEnvironmentStatus, targetPolicyType) {
            return _.includes(workingEnvironmentStatuses.relationshipActionsSourceAndTargetToWorkingEnvironmentStates[operation].source, _.toLower(sourceWorkingEnvironmentStatus)) &&
                _.includes(workingEnvironmentStatuses.relationshipActionsSourceAndTargetToWorkingEnvironmentStates[operation].target, _.toLower(targetWorkingEnvironmentStatus)) &&
                _.includes(workingEnvironmentStatuses.relationshipActionsSourceAndTargetToWorkingEnvironmentStates[operation].policyType, targetPolicyType);
        };

        workingEnvironmentStatuses.isReplicationAllowed = function isReplicationAllowed(sourceWorkingEnvironment, targetWorkingEnvironment) {
            const replicationMap = {
                'VSA_AWS': ['VSA_AWS', 'VSA_Azure', 'ON_PREM', 'VSA_GCP'],
                'VSA_Azure': ['VSA_AWS', 'VSA_Azure', 'ON_PREM', 'VSA_GCP'],
                'VSA_GCP': ['VSA_AWS', 'VSA_Azure', 'ON_PREM', 'VSA_GCP'],
                'ON_PREM': ['VSA_AWS', 'VSA_Azure', 'ON_PREM', 'VSA_GCP']
            };

            return (sourceWorkingEnvironment.status.status === 'on' || sourceWorkingEnvironment.status.status === 'degraded') &&
                (targetWorkingEnvironment.status.status === 'on') &&
                versionUtil.compare(targetWorkingEnvironment.ontapClusterProperties.ontapVersion, '8.3') >= 0 &&
                _.includes(replicationMap[sourceWorkingEnvironment._typeAndProvider], targetWorkingEnvironment._typeAndProvider);
        };

        return workingEnvironmentStatuses;
    });
})();
