(() => {
    'use strict';

    angular.module('noc').controller('UpdateEvalLicenseDialogController', ($scope, workingEnvironmentResource, nocHttp, nocCurrentUserService, CloudProviderAccountResource, $log, $q) => {

        $scope.isValid = () => {
            return $scope.evalForm.$valid;
        };

        $scope.initialize = () => {
            $scope.licenseNocHttp = nocHttp;

            $q.all([
                workingEnvironmentResource.getWorkingEnvironmentByPublicId(nocHttp, $scope.selectedWorkingEnvironment.publicId, $scope.selectedWorkingEnvironment._pathPrefix, 'supportRegistrationInformation'),
                CloudProviderAccountResource.getFormattedAccounts(nocHttp)
            ])
                .then(_.spread((vsaResponse, accounts) => {
                    $scope.existingNssAccountId = vsaResponse.supportRegistrationInformation[0].nssAccountId;
                    $scope.existingNssAccounts = _.filter(accounts, {'_provider': 'nss'});
                    if ($scope.existingNssAccounts.length > 0) {
                        $scope.selectedNssAccount = $scope.existingNssAccounts[0];
                    }
                    $scope.licenseMode = $scope.selectedNssAccount ? 'serial-number' : 'upload-file';
                }))
                .catch($log.log);
        };

        $scope.isAllowed = (action) => {
            return nocCurrentUserService.isAllowed(action);
        };


        $scope.goToAccountsSettings = () => {
            $scope.checkAndSetState('cloud-provider-accounts.account-settings');
            $scope.modalCancel();
        };

        $scope.updateEvalLicense = () => {
            if ($scope.licenseMode === 'serial-number') {
                $scope.updateSerialNumber();
            } else {
                $scope.uploadLicenseFile();
            }
        };

        $scope.uploadLicenseFile = () => {
            let nodeOneLicense, nodeTwoLicense;
            if ($scope.selectedWorkingEnvironment.isHA) {
                nodeOneLicense = $scope.providedLicenseNode1;
                nodeTwoLicense = $scope.providedLicenseNode2;
            } else {
                nodeOneLicense = $scope.providedLicense;
            }
            workingEnvironmentResource.uploadLicenseFile(nocHttp, $scope.selectedWorkingEnvironment, nodeOneLicense, nodeTwoLicense)
                .then(() => {
                    workingEnvironmentResource.updateWorkingEnvironmentStatus($scope.selectedWorkingEnvironment, 'updating');
                    $scope.modalOk();
                });
        };

        $scope.updateNssAccount = () => {
            if (!$scope.existingNssAccountId) {
                return CloudProviderAccountResource.registerVsaToNssAccount(nocHttp, $scope.selectedNssAccount.publicId, $scope.selectedWorkingEnvironment.publicId);
            } else return $q.resolve();
        };

        $scope.updateSerialNumber = () => {
            if ($scope.serialNumberForm.$valid) {
                $scope.updateNssAccount()
                    .then(() => {
                        let nodeOneSerial, nodeTwoSerial;
                        if ($scope.selectedWorkingEnvironment.isHA) {
                            nodeOneSerial = $scope.platformSerialNumberNode1;
                            nodeTwoSerial = $scope.platformSerialNumberNode2;
                        } else {
                            nodeOneSerial = $scope.platformSerialNumber;
                        }
                        workingEnvironmentResource.changeSerial(nocHttp, $scope.selectedWorkingEnvironment, nodeOneSerial, nodeTwoSerial)
                            .then(() => {
                                workingEnvironmentResource.updateWorkingEnvironmentStatus($scope.selectedWorkingEnvironment, 'updating');
                                $scope.modalOk();
                            });
                    })
                    .catch($log.log);
            }
        };

        $scope.initialize();
    });
})();
