(() => {
    'use strict';

    angular.module('noc').controller('UsersAddController', function ($scope, nocUI, workingEnvironmentResource, usersResource, nocTenantsService, nocHttp) {
        this.selectedTenantWE = [];
        this.tenants = nocTenantsService.getTenants();
        const weNumberThreshold = 12;

        const initialize = () => {
            this.userToAdd = {grantedMap: {}, role: $scope.roles[0], tenant: {}, authenticationMode: 'Cloud Manager', notificationsOptIn: true};
        };

        const buildGrantPermission = (user) => {
            //When adding user, there is only need to add WE permissions (nothing to remove)
            return {
                addWorkingEnvironments: _.keys(_.pickBy(user.grantedMap, (isChecked) => !!(isChecked))),
                removeWorkingEnvironments: []
            };
        };

        const emptyGrantedMapFromGivenUser = (user) => {
            user.grantedMap = {};
        };

        this.onChangeRoleSelection = (roleId) => {
            delete this.userToAdd.tenant;
            //If the role is not WE Admin, the list of WE should be emptied to support the design (buttons shouldn't be fixed to footer)
            if (roleId !== 'Role-3') {
                this.selectedTenantWE = [];
            }
            if (roleId !== 'Role-1' && this.tenants.length === 1) {
                this.userToAdd.tenant = this.tenants[0];
                //Call onChangeTenantSelection in order to populated list of WE
                this.onChangeTenantSelection(this.userToAdd);
            }
        };

        this.onChangeTenantSelection = (userToAdd) => {
            emptyGrantedMapFromGivenUser(userToAdd);
            if (userToAdd.tenant && userToAdd.role.publicId === 'Role-3') {
                workingEnvironmentResource.getWorkingEnvironments(nocHttp, userToAdd.tenant.publicId, true)
                    .then((data) => {
                        this.selectedTenantWE = data;
                        this.fixButtonsToFooter = this.selectedTenantWE.length > weNumberThreshold;
                    });
            }
        };

        this.isValid = () => {
            return $scope.addUserForm.$valid;
        };

        this.backToTable = () => {
            $scope.checkAndSetState('^.index');
        };

        this.addUser = () => {
            if (this.isValid()) {
                usersResource.addUser(nocHttp, this.userToAdd)
                    .then((user) => {
                        const grantPermission = buildGrantPermission(this.userToAdd);
                        if (!_.isEmpty(grantPermission.addWorkingEnvironments)) {
                            usersResource.grantRevokeWePermissions(nocHttp, user.publicId, grantPermission)
                                .then(() => {
                                    this.backToTable();
                                });
                        } else {
                            this.backToTable();
                        }
                    });
            } else {
                nocUI.handleFormValidity(this.addUserForm, $scope);
            }
        };

        initialize();
    });
})();
