(() => {
    'use strict';

    angular.module('noc').controller('BackupRestoreController', ($scope, pidgeon, nocHttp, nocUI, nocDialog, nocPoller, Constants, $timeout) => {
        const extensionMatch = /^.+\.(gz|7z|tgz)$/;
        const backupNameMatch = /^occm-backup-.*$/;

        $scope.download = () => {
            nocHttp.download('/occm/system/backup', 'downloading backup', nocHttp.BLOCK)
                .then(() => {
                    pidgeon.success('Backup was successfully downloaded');
                });
        };

        $scope.handleRestoreFileSelected = (e) => {
            pidgeon.clear('restore');
            const file = e.files[0];
            $(e).parent()
                .get(0)
                .reset();

            if (!file) return;

            pidgeon.clear('restore');

            if (!extensionMatch.test(file.name)) {
                $scope.$apply(() => {
                    pidgeon.error('File must have extension gz, 7z or tgz', null, 'restore');
                });

                return;
            }

            const uploadFunction = () => {
                $scope.uploadFile(file);
            };

            if (!backupNameMatch.test(file.name)) {
                $scope.showRestoreFromAsupWarningDialog(uploadFunction);
            } else {
                uploadFunction();
            }
        };

        $scope.uploadFile = function (file) {
            var fd = new FormData();
            fd.append("filename", file.name);
            fd.append("backup", file);

            nocPoller.pause();
            nocHttp.post({
                url:"/occm/system/restore",
                data:fd,
                headers: {"Content-Type": undefined, "filename": file.name},
                transformRequest: angular.identity
            }, "restoring the database", nocHttp.BLOCK)
                .then(function () {
                    pidgeon.success("Database was successfully restored");
                    nocUI.logout(2000);
                })
                .catch(function () {
                    nocPoller.unpause();
                });
        };

        $scope.showRestoreFromAsupWarningDialog = function(proceedFunction) {
            $timeout(function() {
                nocDialog
                    .confirm("Warning! You are about to use an ASUP file as a source for database restore. Passwords are scrubbed from the ASUP files, and restoring from an ASUP file may result in some Cloud Manager system instability. Do you wish to proceed?")
                    .then(proceedFunction);
            }, 500);
        };

        var RESTORE_DIALOG_ID = "RESTORE_DIALOG_1";
        $scope.showRestoreDialog = function () {
            nocDialog
                .confirm("Restoring from backup will erase all your current data. Are you sure you would like to continue?", undefined, undefined, RESTORE_DIALOG_ID);
        };

        $scope.$on(Constants.broadcastEventNames.MODAL_OK_CLICKED, function(event, id) {
            if (id === RESTORE_DIALOG_ID) $("#hiddenBackupAndRestoreInputButton").click();
        });
    });
})();
