(() => {
    'use strict';
    angular.module('noc').factory('configService', ($q, NocErrorResponse, $http) => {
        const configService = {};

        configService.getConfig = (nocHttp, requestedParams, noLoading) => {
            let getConfigPromise;
            if (nocHttp) {
                if (noLoading) {
                    getConfigPromise = nocHttp.get('/occm/config', 'retrieving configuration values from server');
                } else {
                    getConfigPromise = nocHttp.get('/occm/config', 'retrieving configuration values from server', nocHttp.LOADING);
                }
            } else {
                getConfigPromise = $http.get('/occm/config', 'retrieving configuration values from server');
            }

            return getConfigPromise
                .then((paramsFromServer) => {
                    const serverParams = nocHttp ? paramsFromServer : paramsFromServer.data;
                    const response = _.pick(serverParams, requestedParams);
                    if (_.isEqual(_.keys(response), requestedParams)) {
                        return response;
                    } else {
                        return $q.reject(new NocErrorResponse({message: 'Some of the parameters \'' + requestedParams.join(', ') + '\' do not exist in the server response: ' + JSON.stringify(serverParams)}));
                    }
                });
        };

        configService.updateConfig = (nocHttp, configParams, newValues, originalValues) => {
            // Holds needed API calls to perform
            const confObj = {};
            _.forEach(configParams, (key) => {
                if (!_.isEqual(newValues[key], originalValues[key])) {
                    confObj[key] = newValues[key];
                }
            });

            if (!_.isEmpty(confObj)) {
                return nocHttp ? nocHttp.put({
                    url: '/occm/config',
                    data: confObj
                }, 'saving configuration', nocHttp.BLOCK) : $http.put('/occm/config', confObj);
            } else {
                return $q.resolve();
            }
        };

        return configService;
    });
})();
