(() => {
    'use strict';

    angular.module('noc').controller('UpdateController', ($scope, pidgeon, nocUI, nocHttp, nocDialog, nocUploader,
                                                          nocPoller, $q, $rootScope, Constants, ServerUpgradeService) => {
        $scope.extensionMatch = /^.+\.(zip)$/;

        nocUploader.call($scope, nocHttp);

        $scope.whatNewUrl = $rootScope.externalConfig.occm_release_notes_url;
        $scope.initialize = () => {
            nocHttp.get('/occm/system/available-update-versions',
                'checking for available Cloud Manager software updates',
                nocHttp.LOADING)
                .then((updates) => {
                    $scope.versions = updates;
                    $scope.needsUpdate = $scope.versions.length > 0;
                });
        };

        function logout() {
            if ($rootScope._isPortalAuthenticationMode) {
                nocUI.reloadSilently(true);
            } else {
                nocUI.logout(1000);
            }
        }

        $scope.versionNumber = (versionName) => {
            const parsedVersion = (/^occm-(.+)/).exec(versionName);

            return parsedVersion ? parsedVersion[1] : versionName;
        };

        $scope.downloadBackup = () => {
            return nocHttp.download('/occm/system/backup', 'downloading backup', nocHttp.BLOCK)
                .then(() => {
                    pidgeon.success('Backup was successfully downloaded');
                });
        };

        $scope.updateToVersion = function updateToVersion(version) {
            nocDialog.confirm('', 'assets/templates/dialogs/update-occm-confirm.html', {
                version: $scope.versionNumber(version.name),
                releaseNumber: version.releaseNumber,
                eulaUrl: 'https://s3-us-west-1.amazonaws.com/cloud.support.netapp.com/occm/docs/eula_license.rtf',
                width: 500
            })
                .then(() => {
                    nocPoller.pause();

                    nocHttp.post({
                        url: '/occm/system/update/',
                        data: version
                    }, 'updating Cloud Manager software to ' + version.name, nocHttp.BLOCK)
                        .then(() => {
                            pidgeon.inProgress('Updating Cloud Manager to ' + version.name + '...', false, pidgeon.DURATION_LONG);
                            ServerUpgradeService.pollForServerUpgrade(logout);
                        })
                        .catch(() => {
                            nocPoller.unpause();
                        });
                });
        };

        $scope.showAll = function () {
            $scope.showAllUpdates = true;
        };


        const UPLOAD_DIALOG_ID = 'UPLOAD_DIALOG';
        $scope.showPatchDialog = function () {
            pidgeon.clear('upload');

            const eulaUrl = $rootScope.about.beta ? 'https://s3-us-west-1.amazonaws.com/cloud.support.netapp.com/occm/docs/eula_license_beta.pdf' : 'https://s3-us-west-1.amazonaws.com/cloud.support.netapp.com/occm/docs/eula_license.rtf';
            nocDialog
                .confirm('', 'assets/templates/dialogs/update-occm-confirm.html',
                    {
                        eulaUrl,
                        width: 500
                    },
                    UPLOAD_DIALOG_ID)
        };

        $scope.$on(Constants.broadcastEventNames.MODAL_OK_CLICKED, (event, id) => {
            if (id === UPLOAD_DIALOG_ID) $('#hiddenPatchInputButton').click();
        });

        $scope.handlePatchFileSelected = function handlePatchFileSelected(hiddenPatchInputButton) {
            $q.when(null)
                .then(() => {
                    pidgeon.clear('upload');
                    const file = hiddenPatchInputButton.files[0];
                    $(hiddenPatchInputButton).parent()
                        .get(0)
                        .reset();

                    if (!file) return;

                    pidgeon.clear('upload');

                    if (!$scope.extensionMatch.test(file.name)) {
                        pidgeon.error('File must have .zip extension', null, 'upload');

                        return;
                    }

                    const descriptors = [{
                        fieldName: 'filename',
                        file,
                        uploadFormField: 'patch'
                    }];
                    const uploadUrl = '/occm/system/patch';

                    $scope.upload('updating Cloud Manager', descriptors, uploadUrl)
                        .then(() => {
                            pidgeon.inProgress('Updating Cloud Manager...', false, pidgeon.DURATION_LONG);
                            ServerUpgradeService.pollForServerUpgrade(logout);
                        });
                });
        };

        $scope.$on('$destroy', () => {
            $rootScope.expectNetworkFailures = false;
        });

        $scope.initialize();
    });
})();
