(() => {
    'use strict';

    angular.module('noc').controller('DelegateCaController', ($scope, nocDialog, nocHttp, Constants, $timeout, nocEncryptionResourcesService) => {
        $scope.getCertificate = function () {
            nocEncryptionResourcesService.getOccmCertificate(nocHttp)
                .then(function (certificate) {
                    $scope.certificate = certificate;
                    $scope.certificateLoaded = true;
                });
        };

        $scope.getCsr = function () {
            nocHttp.get('/occm/encryption/csr','retrieving CSR',nocHttp.LOADING)
                .then(function (results) {
                    $scope.csr = results ? results.csr : null;
                    $scope.occmCertificate = null;
                })
                .catch(function () {
                    $scope.csr = null;
                });
        };

        $scope.initialize = function () {
            $scope.getCertificate();
            $scope.getCsr();
        };

        $scope.viewCertificate = function () {
            nocDialog.confirm("Your Cloud Manager Certificate", "assets/templates/tabs/encryption/show-certificate.html", {
                certificate: $scope.certificate.certificate,
                width: Constants.dialogWidthForDisplayingCertificate,
                callback: $scope.selectCertificateText
            });
        };

        $scope.generateCsr = () => {
            nocHttp.post('/occm/encryption/csr', 'generating CSR', nocHttp.LOADING)
                .then((results) => {
                    $scope.csr = results.csr;
                    $timeout(() => {
                        document.getElementById('csrTextArea').select(); //Select the CSR in the text area for easy copying.
                    });
                    $scope.occmCertificate = '';
                })
                .catch(() => {
                    //If this API fails, we call getCsr again to make sure we display the correct CSR to the user.
                    $scope.getCsr();
                });
        };

        $scope.regenerateCsr = function () {
            nocDialog.confirm("Regenerate CSR?", "assets/templates/tabs/encryption/delete-current-csr-dialog.html")
                .then($scope.generateCsr);
        };

        $scope.installOccmCertificate = function () {
            nocHttp.post({url:'/occm/encryption/certificate', data:{"occmCertificate": $scope.occmCertificate}}, "installing certificate", nocHttp.LOADING)
                .then(function (certificate) {
                    $scope.certificate = certificate;
                    nocEncryptionResourcesService.formatCertificate($scope.certificate);
                    $scope.csr = null;
                    $scope.occmCertificate = null;
                    $scope.renew = false;
                    nocDialog.dialog("Certificate installed successfully", "assets/templates/tabs/encryption/occm-certificate-installed.html");
                });
        };

        $scope.initialize();
    });
})();
