(() => {
    'use strict';

    angular.module('noc').controller('UserSettingsController', ($scope, $rootScope, nocUI, nocHttp, $stateParams, nocTenantsService, usersResource, nocCurrentUserService) => {
        $scope.saasUrl = $rootScope.supportServices.portalService.portalInformation.edit_user_url;
        $scope.selectedTenantWE = [];
        $scope.tenants = nocTenantsService.getTenants();
        $scope.allowNotifications = _.get($rootScope, 'supportServices.kinesisEnabled');

        $scope.initializeEditor = (userId) => {
            $scope.currentUserRolePublicId = nocCurrentUserService.getCurrentUserRolePublicId();
            const user = _.find($scope.users, {'publicId': userId});
            $scope.usersMode = 'settings';
            $scope.originalSelectedUser = user;
            user.grantedMap = $scope.generateGrantedWorkingEnvironmentsMap(user);

            $scope.selectedUser = _.cloneDeep($scope.originalSelectedUser);
            $scope.selectedRolePublicId = $scope.selectedUser.roleId;

            if ($scope.selectedUser.tenant) {
                $scope.selectedUserTenantId = $scope.selectedUser.tenant.publicId;
            }

            $scope.selectedUser.authenticationMode = $scope.selectedUser.isLdap ? 'Active Directory' : 'Cloud Manager';

            usersResource.getUser(nocHttp, userId).then((user) => {
                $scope.selectedUser.accountId = user.accountId;
            });

            $scope.createdWorkingEnvironments = [];
            $scope.createdWorkingEnvironments = _.reject($scope.selectedUser.createdWorkingEnvironments, {workingEnvironmentType: 'ON_PREM'});
            $scope._createdWorkingEnvironmentsNames = $scope.createdWorkingEnvironments.length > 2 ? _.map($scope.createdWorkingEnvironments, 'name').join(', ') : '';

        };

        $scope.updateSettingsUser = () => {
            $scope.originalSelectedUser = _.cloneDeep($scope.selectedUser);
        };

        $scope.isPropertyNotChanged = (prop, ignoreEmpty) => {
            //ComparisonFuncion is used to deal with special comparison cases, e.g. null and "" to return true.
            const comparisonFuncion = (value1, value2) => {
                if (value1 === value2) return true; //The simple case
                else if (!value1 && !value2) return true; //Comparing null to "" or boolean. In case a prop doesn't exist in originalSelectedUser (null) and the user writes something and then deletes it ("") (e.g. s3Bucket).
                else if ((!value1 || _.isEmpty(value1)) && _.isObject(value2) && !ignoreEmpty) return !_.some(value2); //In case value1 is null or an empty object, and value2 is an object with all falsy values like "" (e.g. azureCredentials).
                // All other cases will be handled by _.isEqual method (e.g. deep objects values comparison)
            };

            return _.isEqualWith(_.get($scope.originalSelectedUser, prop), _.get($scope.selectedUser, prop), comparisonFuncion);
        };

        $scope.isGrantedMapNotChanged = () => {
            const grantedMapDelta = $scope.buildGrantPermission($scope.selectedUser, $scope.originalSelectedUser);

            return (_.isEmpty(grantedMapDelta.addWorkingEnvironments) && _.isEmpty(grantedMapDelta.removeWorkingEnvironments));
        };

        $scope.isDirty = () => {
            return $scope.usersSettingsForm.$dirty;
        };

        $scope.buildGrantPermission = (clonedUser, originalSelectedUser) => {
            const grantPermission = {};
            const clonedUserGrantedMap = _.get(clonedUser, 'grantedMap');
            const originalSelectedUserGrantedMap = _.get(originalSelectedUser, 'grantedMap');
            const selectedWorkingEnvironments = _.keys(_.pickBy(clonedUserGrantedMap));
            const originalWorkingEnvironments = _.keys(_.pickBy(originalSelectedUserGrantedMap));

            const intersect = _.intersection(originalWorkingEnvironments, selectedWorkingEnvironments);

            grantPermission.addWorkingEnvironments = _.difference(selectedWorkingEnvironments, intersect);
            grantPermission.removeWorkingEnvironments = _.difference(originalWorkingEnvironments, intersect);

            return grantPermission;
        };

        $scope.saveSettingsUser = () => {
            const clonedUser = _.cloneDeep($scope.selectedUser);

            if (_.get(clonedUser, 'role.publicId') === 'Role-1') {
                delete clonedUser.tenant;
            }

            const grantPermission = $scope.buildGrantPermission(clonedUser, $scope.originalSelectedUser);
            const url = '/users/' + clonedUser.publicId;
            const method = 'put';

            const grantFunction = (publicId) => {
                if (!_.isEmpty(grantPermission.addWorkingEnvironments) || !_.isEmpty(grantPermission.removeWorkingEnvironments)) {
                    nocHttp.post({
                        url: '/users/' + publicId + '/working-environments-grant-revoke',
                        data: grantPermission
                    }, 'changing working environment permissions', nocHttp.BLOCK)
                        .then(() => {
                            $scope.updateSettingsUser();
                            $scope.toggleUsersRowInformation();
                        });
                } else {
                    //So is dirty is false, to avoid confirm on navigate after save
                    $scope.updateSettingsUser();
                    $scope.toggleUsersRowInformation();
                }
            };

            /*NOC-3502 WE admin role can't change his own e-mail
             allow updating the current user, or any other user which is not we admin*/
            if ($scope.selectedUser.currentUser || nocCurrentUserService.getCurrentUserRolePublicId() !== 'Role-3') {
                nocHttp[method]({url, data: clonedUser},
                    'updating user',
                    nocHttp.BLOCK)
                    .then((user) => {
                        grantFunction(user.publicId);
                    });
            } else {
                grantFunction($scope.selectedUser.publicId);
            }

            //If updating the current user, update the name of scope.currentUser, to update the display.
            if ($scope.selectedUser.currentUser) {
                $scope.currentUser.firstName = $scope.selectedUser.firstName;
                $scope.currentUser.lastName = $scope.selectedUser.lastName;
            }
        };

        $scope.toggleUsersRowInformation = (section) => {
            nocUI.clearFormValidity();
            $scope.selectedUser = _.cloneDeep($scope.originalSelectedUser);
            if ($scope.currentOpenSection === section) {
                $scope.currentOpenSection = '';
            } else {
                $scope.currentOpenSection = section;
            }
        };

        $scope.shortInfoNotifications = () => {
            return $scope.selectedUser ? ($scope.selectedUser.notificationsOptIn ? 'ON' : 'OFF') : '';
        };

        $scope.shortInfoUserRole = () => {
            return $scope.selectedUser ? ($scope.selectedUser.tenant ? $scope.selectedUser.role.name + ' (' + $scope.selectedUser.tenant.name + ')' : $scope.selectedUser.role.name) : '';
        };

        $scope.shortInfoWorkingEnv = () => {
            if ($scope.selectedUser) {
                const selectedWorkingEnvironments = _.keys(_.pickBy($scope.selectedUser.grantedMap));
                if (selectedWorkingEnvironments.length === 0) {
                    return 'No Working Environments';
                } else if (selectedWorkingEnvironments.length === 1) {
                    return '1 Working Environment';
                }

                return selectedWorkingEnvironments.length + ' Working Environments';
            } else {
                return '';
            }
        };

        if (!$scope.users) {
            $scope.initializeUsers().then(() => {
                $scope.initializeEditor($stateParams.userId);
            });
        } else {
            $scope.initializeEditor($stateParams.userId);
        }

        $scope.$watch('selectedUser.role', (role) => {
            if (role) {
                $scope.selectedUser.tenant = _.find($scope.tenants, {publicId: $scope.selectedUserTenantId});
            }
        });

        $scope.$watch('selectedUser.tenant', (tenant) => {
            $scope.selectedTenantWE = tenant ? _.filter($scope.workingEnvironments, {tenantId: tenant.publicId}) : [];
        });
    });
})();
