(function () {
    "use strict";

    angular.module("noc").controller("VolumeViewAddVolumeFloatingIpsController", function ($scope, nocHttp, nocUI, nocDialog, MetadataResource, pidgeon) {

        //todo:svmFloatingIP
        $scope.validateRouteTables = function validateRouteTables() {
            return MetadataResource.validateFloatingIPAddresses(
                nocHttp,
                $scope.createVolumeSelectObjects.region.code,
                $scope.createVolumeSelectObjects.vpc.vpcId,
                $scope.wizardController.volumeRequest.haParams.routeTableIds,
                {
                    clusterFloatingIP: $scope.wizardController.volumeRequest.haParams.clusterFloatingIP,
                    dataFloatingIP: $scope.wizardController.volumeRequest.haParams.dataFloatingIP,
                    dataFloatingIP2: $scope.wizardController.volumeRequest.haParams.dataFloatingIP2,
                    svmFloatingIP: $scope.wizardController.volumeRequest.haParams.svmFloatingIP
                }
            );
        };

        $scope.finishStep = function finishStep() {
            if (!$scope.isValid()) {
                nocUI.handleFormValidity($scope.volumeViewAddVolumeForm, $scope);
            } else {
                $scope.validateRouteTables()
                    .then(function () {
                        return $scope.createVolume($scope.wizardController.placementInfo);
                    })
                    .then($scope.backToVolumesPage);
            }
        };

        $scope.showRouteTablesDialog = function showRouteTablesDialog() {
            var preDialogRouteTablesSelection = _.cloneDeep($scope.wizardController.routeTables);
            if (!$scope.isValid()) {
                nocUI.handleFormValidity($scope.volumeViewAddVolumeForm, $scope);
            } else {
                nocDialog.confirm('Route Tables', "assets/templates/tabs/volume_view/dialogs/route-tables-dialog.html", {
                    wizardController: $scope.wizardController,
                    routeTables: $scope.wizardController.routeTables,
                    regionCode: $scope.createVolumeSelectObjects.region.code,
                    vpcId: $scope.createVolumeSelectObjects.vpc.vpcId,
                    floatingIps: [
                        $scope.wizardController.volumeRequest.haParams.dataFloatingIP,
                        $scope.wizardController.volumeRequest.haParams.dataFloatingIP2,
                        $scope.wizardController.volumeRequest.haParams.clusterFloatingIP,
                        $scope.wizardController.volumeRequest.haParams.svmFloatingIP
                    ],
                    width: 925,
                    dialogOptions: {
                        confirmBtnText: "Save",
                        cancelBtnText: "Cancel"
                    }
                })
                    .then(function (dialogScope) {
                        $scope.selectedRouteTablesString = MetadataResource.stringifySelectedRouteTables(dialogScope.routeTables);
                        $scope.numberOfSelectedRouteTables = _.filter(dialogScope.routeTables, 'selected').length;
                        $scope.wizardController.volumeRequest.haParams.routeTableIds = _(dialogScope.routeTables).filter("selected").map("id").value();
                    })
                    .catch(function () {
                        $scope.wizardController.routeTables = preDialogRouteTablesSelection;
                    });
            }
        };

        $scope.getKeyPairs = function () {
            MetadataResource.getKeyPairs(nocHttp, pidgeon, $scope.createVolumeSelectObjects.region.code)
                .then(function (keyPairs) {
                    $scope.wizardController.keyPairs = keyPairs;
                    $scope.wizardController.volumeRequest.haParams.mediatorKeyPairName = $scope.wizardController.keyPairs[0];
                });
        };

        $scope.getRouteTables = function getRouteTables() {
            var region = $scope.wizardController.volumeRequest.region;
            var vpcId = $scope.wizardController.volumeRequest.vpcId;

            MetadataResource.getRouteTables(nocHttp, region, vpcId)
                .then(function(response) {
                    $scope.wizardController.routeTables = _(response)
                        .map(function (row) {
                            row._subnets = _(row.subnets).map(function (subnetId) {
                                return _.find($scope.createVolumeSelectObjects.vpc.subnets, {subnetId: subnetId});
                            }).sortBy('cidr').value();
                            // select the main route table and tables that are associated with the nodes subnets
                            row.selected = row.main ||
                                _.some(row._subnets, function (s) {
                                    return s.subnetId === $scope.wizardController.volumeRequest.haParams.node1SubnetId || s.subnetId === $scope.wizardController.volumeRequest.haParams.node2SubnetId;
                                });
                            return row;
                        })
                        .orderBy('selected', 'desc').value();

                    $scope.wizardController.volumeRequest.haParams.routeTableIds = _($scope.wizardController.routeTables).filter("selected").map("id").value();
                    $scope.selectedRouteTablesString = MetadataResource.stringifySelectedRouteTables($scope.wizardController.routeTables);
                    $scope.wizardController.hasMultipleRouteTables = response.length > 1;
                    $scope.numberOfSelectedRouteTables = _.filter($scope.wizardController.routeTables, "selected").length;
                });
        };

        $scope.init = function init() {
            $scope.getKeyPairs();
            $scope.getRouteTables();
        };

        $scope.init();

    });
}());
