(function () {
    "use strict";

    angular.module("noc").controller("VolumeViewAddVolumeNodesAndMediatorController", function ($scope, nocUI) {

        $scope.selectDefaultAvailabilityZones = function selectDefaultAvailabilityZones() {
            var availabilityZoneCount = $scope.wizardController.availabilityZones.length;

            $scope.wizardController.node1AvailabilityZone = $scope.wizardController.availabilityZones[0];
            $scope.onAvailabilityZoneChange('node1');
            $scope.selectDefaultSubnet('node1');

            $scope.wizardController.node2AvailabilityZone = $scope.wizardController.availabilityZones[availabilityZoneCount > 1 ? 1 : 0];
            $scope.onAvailabilityZoneChange('node2');
            $scope.selectDefaultSubnet('node2');

            $scope.wizardController.mediatorAvailabilityZone = $scope.wizardController.availabilityZones[availabilityZoneCount > 2 ? 2 : 0];
            $scope.onAvailabilityZoneChange('mediator');
            $scope.selectDefaultSubnet('mediator');
        };

        $scope.selectDefaultSubnet = function selectDefaultSubnet(nodeName) {
            $scope.wizardController.volumeRequest.haParams[nodeName + 'SubnetId'] = $scope.filteredSubnets[nodeName][0].subnetId;
        };

        $scope.onAvailabilityZoneChange = function onAvailabilityZoneChange(nodeName) {
            $scope.wizardController.volumeRequest.haParams[nodeName + 'SubnetId'] = null;
            $scope.filteredSubnets[nodeName] = _.filter($scope.createVolumeSelectObjects.vpc.subnets, {availabilityZone: $scope.wizardController[nodeName + 'AvailabilityZone']});
        };

        $scope.switchEditMode = function switchEditMode() {
            if ($scope.editMode) {
                $scope.selectDefaultAvailabilityZones();
            }
            $scope.editMode = !$scope.editMode;
            if ($scope.currentStepPath !== $scope.wizardController.path['nodes-and-mediator']) {
                $scope.setWizardStep($scope.wizardController.path['nodes-and-mediator']);
            }
        };

        $scope.finishStep = function finishStep() {
            if (!$scope.isValid()) {
                nocUI.handleFormValidity();
            } else {
                $scope.getPlacementInfo()
                    .then(function (placementInfo) {
                        if (placementInfo) {
                            _.assign($scope.wizardController.volumeRequest.haParams, placementInfo.floatingIps);
                            //These fields are mandatory but will be ignored by server:
                            _.assign($scope.wizardController.volumeRequest.haParams, {
                                mediatorKeyPairName: 'ignoreMe',
                                routeTableIds: []
                            });
                            $scope.createVolume(placementInfo)
                                .then($scope.backToVolumesPage);
                        } else {
                            _.assign($scope.wizardController.volumeRequest.haParams, {
                                clusterFloatingIP: null,
                                dataFloatingIP: null,
                                dataFloatingIP2: null,
                                mediatorKeyPairName: null
                            });
                            $scope.next('.floating-ip');
                        }
                    });
            }
        };

        $scope.init = function init() {
            $scope.editMode = false;
            $scope.wizardController.volumeRequest.haParams = {};
            $scope.filteredSubnets = {};
            $scope.selectDefaultAvailabilityZones();
        };

        $scope.init();


    });
}());
