(() => {
    'use strict';

    angular.module('noc').controller('ActionRequiredController', ($scope, nocHttp, nocDialog, actionRequiredService, workingEnvironmentResource) => {
        $scope.actionsRequiredStatic = _.cloneDeep($scope.actionsRequired);
        $scope.selectedWEActionsRequired = _.sortBy($scope.actionsRequiredStatic[$scope.selectedWorkingEnvironment.publicId], 'severity'); // 'error' then 'warning'

        $scope.isFixable = actionRequiredService.isFixable;

        let openedDialog;

        $scope.completeActionRequiredHandling = (action) => {
            $scope.actionsRequired[$scope.selectedWorkingEnvironment.publicId] = _.without($scope.actionsRequired[$scope.selectedWorkingEnvironment.publicId],
                _.find($scope.actionsRequired[$scope.selectedWorkingEnvironment.publicId], {parameters: {aggregateName: action.parameters.aggregateName}}));

            if ($scope.actionsRequired[$scope.selectedWorkingEnvironment.publicId].length === 0) {
                delete $scope.actionsRequired[$scope.selectedWorkingEnvironment.publicId];
            }
            $scope.actionsRequiredStatic = _.cloneDeep($scope.actionsRequired);
            //NOC-5155 Action required - After review and approve action item and there are more items to review need to stay in approval page and not to return to resource page - go back only on last action
            //NOC-9762 UI should refresh after approving manual action required fix -- in order to fix we will go back after each action required, which will cancel the above fix to 5155
            $scope.showMonitoring();
        };

        $scope.addDisks = (action) => {
            nocHttp.post({
                    url: $scope.selectedWorkingEnvironment._pathPrefix + 'aggregates/' + $scope.selectedWorkingEnvironment.publicId + '/' + action.parameters.aggregateName + '/disks',
                    data: {
                        numberOfDisks: action.parameters.numOfDisks,
                        name: action.parameters.aggregateName
                    }
                },
                'adding disks',
                nocHttp.BLOCK)
                .then(() => {
                    $scope.completeActionRequiredHandling(action);
                })
                .finally(() => {
                    openedDialog.resolve();
                });
        };

        $scope.deleteAggregate = (action) => {
            nocHttp.delete($scope.selectedWorkingEnvironment._pathPrefix + 'aggregates/' + $scope.selectedWorkingEnvironment.publicId + '/' + action.parameters.aggregateName,
                'removing disks',
                nocHttp.BLOCK)
                .then(() => {
                    $scope.completeActionRequiredHandling(action);
                })
                .finally(() => {
                    openedDialog.resolve();
                });
        };

        $scope.changeLicense = (action) => {
            workingEnvironmentResource.changeLicenseAndInstanceType(nocHttp, $scope.selectedWorkingEnvironment, action.parameters.licenseParameters.newLicenseType.type, action.parameters.licenseParameters.newInstanceType)
                .then(() => {
                    $scope.completeActionRequiredHandling(action);
                })
                .finally(openedDialog.resolve);
        };

        $scope.moveVolume = (action) => {
            const volumeMoveParameters = action.parameters.volumeMoveParameters;
            const url = $scope.selectedWorkingEnvironment._pathPrefix + 'volumes/' + $scope.selectedWorkingEnvironment.publicId + '/' + volumeMoveParameters.svmName + '/' + volumeMoveParameters.volumeName + '/move';
            const moveVolumeRequest = {
                createTargetAggregate: volumeMoveParameters.isNewTargetAggregate,
                numOfDisksToAdd: action.parameters.numOfDisks,
                targetAggregateName: volumeMoveParameters.targetAggregateName
            };
            nocHttp.post({url, data: moveVolumeRequest},
                'performing volume move action',
                nocHttp.BLOCK)
                .then(() => {
                    $scope.completeActionRequiredHandling(action);
                })
                .finally(() => {
                    openedDialog.resolve();
                });
        };

        $scope.fix = (action) => {
            /* istanbul ignore else: not interesting */
            if ($scope.isFixable(action)) {
                switch (action.actionType) {
                    case 'NeedsMoreDisks':
                        $scope.addDisks(action);
                        break;
                    case 'ShouldBeRemoved':
                        $scope.deleteAggregate(action);
                        break;
                    case 'MoreDisksNeededButDiskLimitExceeded':
                        $scope.moveVolume(action);
                        break;
                    case 'MoreDisksNeededButVsaLimitExceeded':
                        $scope.changeLicense(action);
                        break;
                    case 'LicenseUsedCapacityWarning':
                        $scope.changeLicense(action);
                        break;
                    case 'LicenseAllocatedCapacityError':
                        $scope.changeLicense(action);
                        break;
                    case 'NonCompliantConfig':
                        $scope.changeLicense(action);
                        break;
                    case 'deprecatedInstance':
                        $scope.changeLicense(action);
                        break;
                    /* istanbul ignore next: trivial */
                    default:
                }
            }
        };

        $scope.openReviewDialog = (action, workingEnvironment) => {
            openedDialog = nocDialog.confirm('', 'assets/templates/dialogs/review-action-required.html', {
                action,
                we: workingEnvironment,
                hasAdvancedInformation: action.actionType === 'MoreDisksNeededButDiskLimitExceeded' || action.actionType === 'ShouldBeRemoved',
                isFixable: $scope.isFixable,
                secondMessage: $scope.selectedWorkingEnvironment.ontapEncryption && action.actionType === 'ShouldBeRemoved' ?
                    'The encryption key used for this aggregate is listed in the timeline. You will need to manually delete the key from key managers.' :
                    ''
            })
                .then(() => {
                    $scope.fix(action);
                });
        };

        $scope.reviewable = (action) => {
            return action.actionType === 'NeedsMoreDisks' ||
                action.actionType === 'MoreDisksNeededButDiskLimitExceeded' ||
                action.actionType === 'MoreDisksNeededButVsaLimitExceeded' ||
                action.actionType === 'LicenseUsedCapacityWarning' ||
                action.actionType === 'LicenseAllocatedCapacityError' ||
                action.actionType === 'ShouldBeRemoved' ||
                action.actionType === 'EmptyWorkingEnvironment' ||
                action.actionType === 'EvaluationByol' ||
                action.actionType === 'NonCompliantConfig' ||
                action.actionType === 'MaxDisksLimitAboutToExceed' ||
                action.actionType === 'MaxDisksLimitExceeded' ||
                action.actionType === 'deprecatedInstance' ||
                action.actionType === 'PaygoNotRegisteredToSupport' ||
                _.startsWith(action.actionType, 'license');
        };

        if ($scope.selectedWEActionsRequired.length === 0) {
            $scope.showMonitoring();
        }
    });
})();
