(function () {
    'use strict';

    angular.module('noc').controller('AddDisksAggregateController', function ($scope, nocHttp, nocUI, nocUtil, aggregatesResource, pidgeon, nocDialog,
                                                                              nocApprovalMessageBuilder, $state, providerDiskTypes,
                                                                              $filter) {
        $scope.numberOfDisksOptions = [];
        $scope.addDisksRequest = {name: ''};

        var initializeModel = function initializeModel() {
            $scope.addDisksRequest.name = $scope.selectedAggregate.name;
            var curNumOfDisks = $scope.selectedAggregate.disks.length;
            var maxNumOfDisksInAggregateLimit = $scope.selectedWorkingEnvironment.capacityFeatures.maxDisksPerAggregate - (($scope.selectedWorkingEnvironment.isHA && $scope.selectedWorkingEnvironment._isAWS) ? curNumOfDisks / 2 : curNumOfDisks);

            if (maxNumOfDisksInAggregateLimit <= 0) {
                pidgeon.error('Cannot add any disks to aggregate, maximum capacity of ' + $scope.selectedWorkingEnvironment.capacityFeatures.maxDisksPerAggregate + ' disks per aggregate reached', '', 'initialize');
                return;
            }

            var diskSize = $scope.selectedAggregate.providerVolumes[0].size;
            var diskType = $scope.selectedAggregate.providerVolumes[0].diskType;
            var supportedSizes = $scope.selectedWorkingEnvironment._providerVolumeTypes[diskType].supportedSizes;
            var supportedSize = _.find(supportedSizes, {_identifier: $filter('normalizedCapacity')(diskSize, 2, true)});
            var maxNumOfDisksAllowedByServer = supportedSize.maxDisksAllow.numOfDisks;

             if (maxNumOfDisksAllowedByServer <= 0) {
                pidgeon.error('Cannot add any disks to aggregate: ' + supportedSize.maxDisksAllow.reason, '', 'initialize');
            } else {
                var allowedNumOfDisks = Math.min(maxNumOfDisksInAggregateLimit, maxNumOfDisksAllowedByServer);

                $scope.numberOfDisksOptions = _.range(1, allowedNumOfDisks + 1);
                $scope.addDisksRequest.numberOfDisks = $scope.numberOfDisksOptions[0];
            }
        };

        $scope.initializeController = function initializeController() {
            $scope.diskTypes = providerDiskTypes[$scope.selectedWorkingEnvironment._cloudProviderName];
            var selectedAggregateName = $state.params.aggregateName;

            pidgeon.clear('initialize');
            aggregatesResource.getAggregates(nocHttp, $scope.selectedWorkingEnvironment)
                .then(function (aggregatesResponse) {
                    $scope.selectedAggregate = _.find(aggregatesResponse, {name: selectedAggregateName});
                    if (!$scope.selectedAggregate) {
                        pidgeon.error(selectedAggregateName + " does not exist", "", 'initialize');
                        return;
                    }
                    initializeModel();
                });
        };

        $scope.updateSelectedWorkingEnvironment($state.params.workingEnvironmentId, ['capacityFeatures'])
            .then($scope.initializeController, $scope.backToGallery);

        $scope.addDisksAggregate = function () {
            if (!$scope.addDisksAggregateForm.$valid) {
                nocUI.handleFormValidity($scope.addDisksAggregateForm, $scope);
            } else {
                nocDialog
                    .confirm("", "assets/templates/dialogs/purchase-confirm-content.html", {html: nocApprovalMessageBuilder.buildTitle({
                        provider: $scope.selectedWorkingEnvironment._cloudProviderName,
                        numOfDisks: $scope.addDisksRequest.numberOfDisks,
                        diskSize: nocUtil.normalizeCapacity($scope.selectedAggregate.providerVolumes[0].size),
                        diskType: $scope.diskTypes[$scope.selectedAggregate._providerVolumeType].display,
                        isHA: $scope.selectedWorkingEnvironment.isHA
                        })})
                    .then(function () {
                        aggregatesResource.addDisksToAggregate(nocHttp, $scope.selectedWorkingEnvironment,
                            $scope.selectedAggregate, $scope.addDisksRequest)
                            .then(function () {
                                $scope.goToGadget('advancedAllocation');
                            });
                    });
            }
        };

        $scope.cancel = function () {
            $scope.goToGadget('advancedAllocation');
        };
    });
})();
