(() => {
    'use strict';
    angular.module('noc').controller('addKeyManagerController', ($scope, nocHttp, nocUI, nocUtil) => {
        $scope.getKeyManagers = () => {
            return nocHttp.get('/occm/encryption/key-managers', 'retrieving Cloud Manager key managers', nocHttp.LOADING)
                .then((results) => {
                    $scope.occmKeyManagers = results;
                    //Remove all existing key managers from the list.
                    $scope.keyManagersToDisplay = _.filter($scope.occmKeyManagers, (keyManager) => {
                        return !_.find($scope.workingEnvironmentKeyManagers, {address: keyManager.address});
                    });
                });
        };

        //We use local simplified version of nocUtil.formatSelect2 to solve: NOC-6520 - Encryption - Add KM with long name to VSA doesn't look good.
        //This version limits the width of the div and adds trimmed-text class.
        //If this problem repeats in more places we should consider improving the util function instead of using more local versions.
        $scope.formatSelect2 = function formatSelect2(address, name) {
            const title = address + ' (' + name + ')';

            return '<div class="select2-format-options clearfix trimmed-text" title="' + title + '" style="width: 321px;">' +
                '<span>' + address + '</span><span>' + name + '</span>' +
                '</div>';
        };

        $scope.formatKeyManager = (item) => {
            if (!item.text) return '';
            const keyManager = _.find($scope.keyManagersToDisplay, {address: item.text});

            return $scope.formatSelect2(keyManager.address, keyManager.name);
        };

        $scope.keyManagerMatcher = (searchString, itemText) => {
            const keyManager = _.find($scope.keyManagersToDisplay, {address: itemText});

            return nocUtil.genericItemFilter(searchString, keyManager, ['name', 'address']);
        };

        $scope.addKeyManagerOptions = {
            matcher: $scope.keyManagerMatcher,
            formatResult: $scope.formatKeyManager,
            formatSelection: $scope.formatKeyManager
        };

        $scope.okClicked = () => {
            if ($scope.addKeyManagerForm.$valid) {
                $scope.addKeyManager();
            } else {
                nocUI.handleFormValidity($scope.addKeyManagerForm, $scope);
            }
        };

        $scope.addKeyManager = () => {
            //POST /vsa/encryption/{vsa_id}/key-managers/{key_manager_ip}
            return nocHttp.post($scope.selectedWorkingEnvironment._pathPrefix + 'encryption/' + $scope.vsaResponse.id + '/key-managers/' + $scope.selectedKeyManager.address,
                'adding key manager',
                nocHttp.BLOCK)
                .then($scope.closeDialog);
        };

        $scope.initialize = () => {
            $scope.workingEnvironmentKeyManagers = $scope.vsaResponse.vsaEncryptionResponse.keyManagers;
            $scope.getKeyManagers();
            $scope.selectedKeyManager = {};
        };

        $scope.initialize();
    });
})();
