(() => {
    'use strict';
    angular.module('noc').controller('CifsConfigurationsController', ($scope, $q, CifsResource, $stateParams, pidgeon, nocUI,
                                                                      nocNormalizerService, nocHttp) => {
        nocUI.clearFormValidity();
        $scope.advanced = 'Advanced';
        $scope.basic = 'Simple';

        $scope.modes = {
            create: 'create',
            edit: 'edit'
        };

        $scope.mode = $scope.modes.create;

        $scope.bindDnsAndAD = {status: false};

        $scope.setupNtp = {status: false};

        $scope.forms = {};
        $scope.state = $scope.basic;

        const cifsResource = new CifsResource($stateParams.workingEnvironmentId, $scope.selectedWorkingEnvironment, null, nocHttp);
        const defaultOUValue = 'CN=Computers';

        $scope.switchState = (toState) => {
            $scope.state = toState;
        };

        const getNormalizedRequest = nocNormalizerService.getNormalizedCifsRequest;

        $scope.request = {
            dnsDomain: '',
            activeDirectoryDomain: '',
            ipAddresses: [],
            netBIOS: $scope.selectedWorkingEnvironment.name.substring(0, 15),
            organizationalUnit: defaultOUValue,
            activeDirectoryUsername: 'administrator',
            activeDirectoryPassword: ''
        };

        const exit = function exit() {
            if ($scope.wizardController) {
                if ($scope.singleStepVolumeWizard) {
                    $scope.startCreateVolumeProcess();
                } else {
                    $scope.wizardController.next('.usageProfile');
                }
            } else {
                $scope.showMonitoring();
            }
        };

        $scope.finish = () => {
            pidgeon.clear('cifs');
            nocUI.handleFormValidity();

            if (!$scope.forms.cifsForm.$valid) {
                const invalidState = $scope.forms.cifsForm[$scope.basic].$valid ? $scope.advanced : $scope.basic;
                $scope.switchState(invalidState);

                return;
            }

            let setupNtpFunc = () => $q.resolve();
            if ($scope.setupNtp.status) {
                setupNtpFunc = () => {
                    return cifsResource.createNTPConfigurations({
                        ntpServer: $scope.request.dnsDomain
                    });
                };
            }
            setupNtpFunc()
                .then(() => cifsResource.createCifsConfigurations(getNormalizedRequest($scope.request)))
                .then(exit);
        };

        let unwatch = _.noop;
        const bind = () => {
            $scope.request.dnsDomain = $scope.request.activeDirectoryDomain;
            unwatch = $scope.$watch('request.activeDirectoryDomain', (newValue) => {
                $scope.request.dnsDomain = newValue;
            });
        };

        const unbind = () => {
            unwatch();
        };

        $scope.bindingChanged = () => {
            if ($scope.bindDnsAndAD.status) {
                bind();
            } else {
                unbind();
            }
        };

        const unblock = $scope.isInDialog ? _.noop : nocUI.block();
        cifsResource.getCifsConfigurations()
            .then((response) => {
                $scope.request = response[0] || $scope.request;
                $scope.mode = response[0] ? $scope.modes.edit : $scope.modes.create;
                $scope.bindDnsAndAD.status = $scope.request.dnsDomain === $scope.request.activeDirectoryDomain;
                $scope.bindingChanged();
            })
            .finally(unblock);

        $scope.cancel = () => {
            return $scope.isInDialog ? $scope.modalCancel() : exit();
        };
    });
})();
