(() => {
    'use strict';

    angular.module('noc').controller('ChangeLicenseOrInstanceController', ($scope, $q, workingEnvironmentResource, nocHttp,
                                                                           MetadataResource, ManifestMapper, volumesResource, nocDialog) => {

        $scope.changeLicenseAndInstanceTypeConfirm = () => {
            workingEnvironmentResource.changeLicenseAndInstanceType(nocHttp, $scope.selectedWorkingEnvironment,
                $scope.changeLicenseSelectObjects.selectedLicense.licenseType, $scope.changeLicenseSelectObjects.selectedInstanceType.instanceType)
                .then(() => {
                    $scope.selectedWorkingEnvironment.capacityFeatures = undefined;
                    $scope.showMonitoring();
                });
        };

        $scope.onConfirm = () => {
            if($scope.selectedWorkingEnvironment.isHA){
                workingEnvironmentResource.checkIfCifsLockExists(nocHttp, $scope.selectedWorkingEnvironment)
                    .then((response) => {
                        if (response) {
                            return nocDialog.confirm('', 'assets/templates/dialogs/cifs-lock-warning-dialog.html', {
                                width: 560
                            }).then(() => {
                                $scope.changeLicenseAndInstanceTypeConfirm();
                            });
                        }
                        else {
                            $scope.changeLicenseAndInstanceTypeConfirm();
                        }
                    });
            } else {
                $scope.changeLicenseAndInstanceTypeConfirm();
            }
        };

        const checkNetworkUtilization = function checkNetworkUtilization(instanceType) {
            $scope.showNetworkUtilizationWarning = _.some(instanceType.supportedFeatures, (feature) => {
                return feature === 'network-utilization';
            });
        };

        $scope.checkOptionToAddLifs = (instanceType) => {
            $scope.instanceWithAdditionalLifs = !_.some(instanceType.supportedFeatures, (feature) => {
                return feature === 'four-nics';
            });
        };

        $scope.disableInstancesAndLicensesWithLessThan8Nics = () => {
            _.forEach($scope.licenses, (license) => {
                _.forEach(license.instanceTypes, (instanceType) => {
                    if (!instanceType.disabled) {
                        instanceType.disabled = _.some(instanceType.supportedFeatures, (feature) => {
                            return feature === 'four-nics';
                        });
                        if (instanceType.disabled) {
                            instanceType.disabledText = 'This VM size does not support the number of network interface cards (NICs) attached to this Cloud Volumes ONTAP system.';
                        }
                    }
                });
                if (!license._disabled) {
                    license._disabled = _.every(license.instanceTypes, 'disabled');
                    if (license._disabled) {
                        license._disableReason = 'This license does not contain VM types that support the number of network interface cards (NICs) attached to this Cloud Volumes ONTAP system.';
                    }
                }
            });
        };

        $scope.disableLegacy = (legacyTypes) => {
            _.forEach($scope.licenses, (license) => {
                _.forEach(license.instanceTypes, (instanceType) => {
                    if (!instanceType.disabled) {
                        instanceType.disabled = _.includes(legacyTypes, instanceType.instanceType);
                        if (instanceType.disabled) {
                            instanceType.disabledText = 'This instance type is not supported with Cloud Volumes ONTAP systems that use a tiered storage configuration.';
                        }
                    }
                });

                if (!license._disabled) {
                    license._disabled = _.every(license.instanceTypes, 'disabled');
                    if (license._disabled) {
                        license._disableReason = 'This ONTAP system uses a tiered storage configuration which is not supported by this license.';
                    }
                }
            });
        };

        $scope.disableDueToOntapEncryption = function disableDueToOntapEncryption() {
            if (!_.get($scope.localVsaResponse.vsaEncryptionResponse, 'ontapEncryption')) return;

            _.forEach($scope.licenses, (license) => {
                _.forEach(license.instanceTypes, (instanceType) => {
                    if (!instanceType.disabled) {
                        instanceType.disabled = !instanceType.supportsEncryption;
                        if (instanceType.disabled) {
                            instanceType.disabledText = 'This ' + $scope.selectedWorkingEnvironment._nameForInstance + ' type does not support ONTAP Encryption';
                        }
                    }
                });
                if (!license._disabled) {
                    license._disabled = _.every(license.instanceTypes, 'disabled');
                    if (license._disabled) {
                        license._disableReason = 'This license does not support ONTAP Encryption';
                    }
                }
            });
        };

        $scope.isInstanceLegacy = (selectedInstanceType, selectedLicenseType) => {
            return _.includes($scope.legacyInstanceTypes, selectedInstanceType) && selectedLicenseType !== 'Cloud Volumes ONTAP BYOL';
        };

        $scope.getLegacyTypes = function getLegacyTypes() {
            return MetadataResource.legacyInstanceTypes(nocHttp, $scope.selectedWorkingEnvironment._typeAndProvider !== 'HA', $scope.selectedWorkingEnvironment._isAzure)
                .then((legacyInstanceTypes) => {
                    return _.map(legacyInstanceTypes, 'instanceType');
                });
        };

        $scope.disableDueToComposite = (legacyTypes) => {
            if ($scope.selectedWorkingEnvironment.status.status !== 'on' || $scope.localVsaResponse.cluster.licenseType.name === 'Cloud Volumes ONTAP BYOL') {
                return $q.resolve();
            } else {
                volumesResource.getVolumes(nocHttp, $scope.selectedWorkingEnvironment.publicId, $scope.selectedWorkingEnvironment._pathPrefix)
                    .then((volumes) => {
                        const volumesWithTierPolicy = _.filter(volumes, (volume) => {
                            return volume.tieringPolicy && volume.tieringPolicy !== 'none';
                        }).length > 0;

                        _.forEach($scope.licenses, (license) => {
                            _.forEach(license.instanceTypes, (instanceType) => {
                                if (!instanceType.disabled) {
                                    instanceType.disabled = (volumesWithTierPolicy && _.includes(license.licenseType, 'explore'));

                                    if (instanceType.disabled) {
                                        instanceType.disabledText = 'This ONTAP system uses a tiered storage configuration which is not supported by this license.';
                                    }
                                }
                            });
                            if (!license._disabled) {
                                license._disabled = _.every(license.instanceTypes, 'disabled');
                                if (license._disabled) {
                                    license._disableReason = 'This ONTAP system uses a tiered storage configuration which is not supported by this license.';
                                }
                            }
                        });

                        if (volumesWithTierPolicy) {
                            $scope.disableLegacy(legacyTypes);
                        }
                    });
            }
        };

        $scope.$watch('changeLicenseSelectObjects.selectedInstanceType', (selectedInstance) => {
            if (selectedInstance) {
                checkNetworkUtilization(selectedInstance);
                if ($scope.selectedWorkingEnvironment._isAzure && ($scope.numberOfLifs < 6) && ($scope.numberOfAvailableIps < 2 || $scope.selectedWorkingEnvironment.status.status === 'off')) {
                    $scope.checkOptionToAddLifs(selectedInstance);
                }
            }
        });

        $scope.selectDefaultInstanceType = function selectDefaultInstanceType() {
            //1. Select the server default, if it's not disabled.
            const defaultInstanceFromManifestNotDisabled = _.find($scope.changeLicenseSelectObjects.selectedLicense.instanceTypes, (instance) => {
                const serverDefaultInstance = ManifestMapper.getDefaultInstance($scope.selectedWorkingEnvironment.regionName,
                    'ONTAP-' + $scope.selectedWorkingEnvironment._ontapVersion,
                    $scope.changeLicenseSelectObjects.selectedLicense.licenseType);

                return instance.instanceType === serverDefaultInstance && !instance.disabled;
            });

            //2. If  not found, select the first instance which is not legacy and not disabled.
            const firstNotLegacyNotDisabled = _.find($scope.changeLicenseSelectObjects.selectedLicense.instanceTypes, (instance) => {
                return !instance.disabled && !$scope.isInstanceLegacy(instance.instanceType, $scope.changeLicenseSelectObjects.selectedLicense.name);
            });

            //3. If none found, select the first not disables instance (even if it's legacy);
            const firstNotDisabled = _.find($scope.changeLicenseSelectObjects.selectedLicense.instanceTypes, (instance) => {
                return !instance.disabled;
            });

            $scope.changeLicenseSelectObjects.selectedInstanceType = defaultInstanceFromManifestNotDisabled || firstNotLegacyNotDisabled || firstNotDisabled;
        };

        $scope.initialize = function initialize() {
            $scope.localVsaResponse = $scope.vsaResponse || $scope.offVsaResponse;
            if (!$scope.localVsaResponse) {
                $scope.showMonitoring();
            } else {
                $q.all([
                    workingEnvironmentResource.getLicenseAndInstanceTypes(nocHttp, $scope.selectedWorkingEnvironment),
                    $scope.getLegacyTypes()
                ])
                    .then(_.spread((licensesResponse, legacyTypes) => {
                        $scope.changeLicenseSelectObjects = {};
                        $scope.legacyInstanceTypes = legacyTypes;
                        $scope.licenses = _.sortBy(licensesResponse, 'capacityLimit.size');
                        $scope.numberOfLifs = _.get($scope, 'selectedWorkingEnvironment.providerProperties.numOfNics') ? $scope.selectedWorkingEnvironment.providerProperties.numOfNics : 0;
                        $scope.changeLicenseSelectObjects.selectedLicense = _.find($scope.licenses, {name: $scope.localVsaResponse.cluster.licenseType.name});
                        const currentInstanceObject = _.find($scope.changeLicenseSelectObjects.selectedLicense.instanceTypes, {instanceType: $scope.localVsaResponse.instanceType});
                        $scope.changeLicenseSelectObjects.selectedLicense.instanceTypes = _.without($scope.changeLicenseSelectObjects.selectedLicense.instanceTypes, currentInstanceObject);
                        //If ($scope.legacyInstanceTypes.length > 0) {

                        //}
                        $scope.disableDueToComposite(legacyTypes);
                        $scope.disableDueToOntapEncryption();

                        if ($scope.selectedWorkingEnvironment._isAzure && !$scope.selectedWorkingEnvironment.providerProperties.singleNetworkInterface) {
                            if ($scope.numberOfLifs > 4) {
                                $scope.disableInstancesAndLicensesWithLessThan8Nics();
                            }
                        }

                        $scope.selectDefaultInstanceType();

                        if ($scope.selectedWorkingEnvironment._isAzure && !$scope.selectedWorkingEnvironment.providerProperties.singleNetworkInterface) {
                            if ($scope.numberOfLifs === 4) {
                                if ($scope.selectedWorkingEnvironment.status.status === 'off') {
                                    $scope.checkOptionToAddLifs($scope.changeLicenseSelectObjects.selectedInstanceType);
                                    $scope.noNewLifsCanBeAdded = true;
                                }
                                workingEnvironmentResource.getNumberOfAvailableIpsInSubnet(nocHttp, $scope.selectedWorkingEnvironment)
                                    .then((ipsResponse) => {
                                        $scope.numberOfAvailableIps = ipsResponse.availableIps;
                                        if ($scope.numberOfAvailableIps === 0) {
                                            $scope.checkOptionToAddLifs($scope.changeLicenseSelectObjects.selectedInstanceType);
                                        }
                                    });
                            }
                        }
                    }));
            }
        };

        $scope.initialize();
    });
})();
