(() => {
    'use strict';

    angular.module('noc').controller('ChangeOntapImageController', ($scope, workingEnvironmentResource, pidgeon, nocUI, nocDialog, nocUtil, nocHttp) => {
        $scope.extensionMatch = /^.+\w+\/[^<>:"/\\|?*]+\.(tgz)$/;
        $scope.selectedImage = null;
        $scope.externalUrl = null;
        $scope.externalUrlHa = null;
        $scope.values = {};
        $scope.values.mode = $scope.selectedWorkingEnvironment.isHA ?  "external" : "local";

        $scope.isValid = function(){
            return $scope.changeOntapImageForm.$valid;
        };

        $scope.changeOntapImageConfirm = function (workingEnvironment, selectedImage) {
            pidgeon.clear("url");
            if (!$scope.isValid()) {
                nocUI.handleFormValidity($scope.changeOntapImageForm, $scope);
            } else if ($scope.values.mode === "external" && !$scope.extensionMatch.test($scope.values.externalUrl)) {
                pidgeon.error("Please enter valid URL of file with .tgz extension", null, "url");
            } else {
                var payload = $scope.constructPayload(selectedImage, $scope.values.externalUrl , $scope.values.mode);
                nocDialog.confirm("Confirm the Cloud Volumes ONTAP image change", "assets/templates/dialogs/change-ontap-version-confirm-content.html", _.assignIn(_.clone(payload), {width: 600}))
                    .then(function () {
                        $scope.changeOntapImage(workingEnvironment, payload);
                    });
            }
        };

        $scope.constructPayload = function(selectedImage, externalUrl, mode){
            return mode === "external" ? {updateType: "EXTERNAL_FILE", updateParameter: externalUrl} : {updateType: "LOCAL_IMAGE", updateParameter: selectedImage.name};
        };

        $scope.changeOntapImage = function(workingEnvironment, payload){
            workingEnvironmentResource.changeOntapImage(nocHttp, workingEnvironment, payload)
                .then(function () {
                    workingEnvironmentResource.updateWorkingEnvironmentStatus(workingEnvironment, "updating");
                    $scope.showMonitoring();
                });
        };

        $scope.changeMode = function (mode) {
            $scope.values.mode = mode;

            if (mode === "external") $scope.selectedImageName = null;
        };

        $scope.handleLocalImageChange = (name) => {
            if (name) {
                $scope.changeMode('local');
            }
            $scope.selectedImage = _.find($scope.localImages, {name});
        };

        $scope.localImagesOptions = {
            formatResult: function (imageName) {
                if (!$scope.localImages) return '';
                var image = _.find($scope.localImages, {name: imageName.id});
                if (!image) return '';
                return nocUtil.formatSelect2(image.version, "Version " + image.version, null, image.name);
            },
            formatSelection: function(imageName){
                if (!$scope.localImages) return '';
                var image = _.find($scope.localImages, {name: imageName.id});
                if (!image) return '';
                return "Version " + image.version;
            }
        };

        workingEnvironmentResource.getLocalImages(nocHttp, $scope.selectedWorkingEnvironment)
            .then(function(localImages){
                $scope.localImages = localImages;
            });
    });
})();
