(() => {
    'use strict';
    angular.module('noc').controller('ConnectToKubernetesController', ($scope, nocHttp, KubernetesResource, pidgeon) => {
        $scope.initialize = () => {
            $scope.request = {
                setDefaultStorageClass: true,
                selectedCluster: {isOurStorageClassDefault: true}
            };
            $scope.exportInfo = {ips: $scope.getDefaultExportPolicyIps()};
            $scope.numOfConnectedClusters = _.get($scope.selectedWorkingEnvironment, 'k8sProperties.connectedClusterIds.length');

            KubernetesResource.getK8sClusters(nocHttp)
                .then((clusters) => {
                    const availableConnectedClusters = _.filter(clusters, 'connectivity');
                    if (availableConnectedClusters.length > 0) {
                        $scope.k8sEnabled = true;
                    }
                    $scope.availableClusters = _.reject(availableConnectedClusters, (cluster) => {
                        return _.includes(cluster.connectedWeIds, $scope.selectedWorkingEnvironment.publicId);
                    });
                    if ($scope.availableClusters.length === 1) {
                        $scope.request.selectedCluster = $scope.availableClusters[0];
                    }

                    const clustersConnectedToWE = _.filter(clusters, (cluster) => {
                        return _.includes(cluster.connectedWeIds, $scope.selectedWorkingEnvironment.publicId);
                    });
                    $scope.connectedClusterNames = _.map(clustersConnectedToWE, 'clusterName');
                    $scope.loaded = true;
                });
        };

        $scope.goToDiscoverCluster = () => {
            $scope.checkAndSetState('main.k8s-clusters.discover-k8s-cluster');
        };

        $scope.getDefaultExportPolicyIps = () => {
            // todo: gcp?
            switch ($scope.selectedWorkingEnvironment._typeAndProvider) {
                case 'VSA_AWS': return $scope.selectedWorkingEnvironment.awsProperties.vpc.cidrBlock;
                case 'VSA_Azure': return $scope.selectedWorkingEnvironment.providerProperties.vnetCidr;
                default: return '';
            }
        };

        $scope.connectToK8s = () => {
            KubernetesResource.connectWorkingEnvironment(nocHttp, $scope.selectedWorkingEnvironment.publicId, $scope.request.selectedCluster.publicId, {
                ips: !$scope.exportInfo.ips ? [] : _.split($scope.exportInfo.ips, (',')),
                setDefaultStorageClass: $scope.request.setDefaultStorageClass
            })
                .then(() => {
                    $scope.boxGoBack();
                    pidgeon.inProgress('Connecting Cloud Volumes ONTAP to Kubernetes cluster.', true);
                });
        };

        $scope.initialize();
    });
})();
