(() => {
    'use strict';

    angular.module('noc').controller('CotEncryptionSettingsController', ($scope, nocDialog, nocUI, nocHttp, pidgeon, nocEncryptionResourcesService) => {
        $scope.initialize = () => {
            $scope.getOccmCertificate();
        };

        $scope.getOccmCertificate = () => {
            nocEncryptionResourcesService.getOccmCertificate(nocHttp)
                .then((certificate) => {
                    $scope.occmCertificate = certificate;
                });
        };

        $scope.viewDetails = (certificate) => {
            nocDialog.dialog('Certificate details', 'assets/templates/dialogs/certificate-details.html', {certificate});
        };

        $scope.openUpdateKeyManagersCaCertificateDialog = () => {
            nocEncryptionResourcesService.getKeyManagersCaCertificatesWithOptions(nocHttp)
                .then((results) => {
                    const selectedKeyManagerCaCertificate = {};
                    if ($scope.serverCertificate) {
                        _.remove(results.data, {certificate: $scope.serverCertificate.certificate}); //Remove the current certificate
                    }
                    nocDialog.confirm('Update Key Manager CA Certificate', 'assets/templates/dialogs/update-key-managers-ca-certificate.html', {
                        keyManagersCaCertificates: results.data,
                        keyManagersCaCertificateOptions: results.select2options,
                        selectedKeyManagerCaCertificate,
                        okClicked: $scope.okClicked
                    })
                        .then(() => {
                            $scope.updateKeyManagerCaCertificate(selectedKeyManagerCaCertificate.id);
                        });
                });
        };

        $scope.okClicked = function okClicked() {
            if (this.updateKeyManagersCaCertificateForm.$valid) {
                this.modalOk();
            } else {
                nocUI.handleFormValidity(this.updateKeyManagersCaCertificateForm, $scope);
            }
        };

        $scope.updateKeyManagerCaCertificate = (newKeyManagerCaCertificateId) => {
            const params = {
                url: $scope.selectedWorkingEnvironment._pathPrefix + 'encryption/' + $scope.vsaResponse.id + '/update-key-manager-ca-certificate',
                data: {'keyManagerCaCertificateId': newKeyManagerCaCertificateId}
            };

            nocHttp.post(params, 'updating Key Managers CA Certificate', nocHttp.LOADING)
                .then((response) => {
                    nocEncryptionResourcesService.formatCertificate(response);
                    $scope.serverCertificate = response;
                    pidgeon.inProgress('Key Managers CA Certificate update started.', true);
                });
        };

        $scope.deleteKeyManager = (keyManager) => {
            //Do not allow the deletion of the last Key Manager
            if ($scope.vsaResponse.vsaEncryptionResponse.keyManagers.length > 1) {
                nocDialog
                    .confirm('Are you sure you want to delete Key Manager ' + keyManager.address + (keyManager.name ? ' (' + keyManager.name + ')' : '') + '?')
                    .then(() => {
                        //DELETE /vsa/encryption/{vsa_id}/key-managers/{key_manager_ip}
                        nocHttp.delete($scope.selectedWorkingEnvironment._pathPrefix + 'encryption/' + $scope.vsaResponse.id + '/key-managers/' + keyManager.address,
                            'deleting key manager',
                            nocHttp.BLOCK);
                        //We ignore the success case because the poller is responsible for requesting new VsaResponse.
                    });
            }
        };

        $scope.updateClientCertificate = () => {
            if (!$scope.occmCertificate) {
                nocDialog.alert('Cloud Manager could not renew the Cloud Volumes ONTAP certificate because Cloud Manager is not configured to be an intermediate CA. Cloud Manager Admin can configure this in Encryption Setup.');
            } else if ($scope.occmCertificate._expirationState.state === 'certificate-expired') {
                nocDialog.alert('Cloud Manager could not renew the Cloud Volumes ONTAP certificate because the Cloud Manager intermediate CA certificate has expired. The Cloud Manager Admin must renew the intermediate CA certificate from the Encryption Setup page.');
            } else {
                nocDialog
                    .confirm('Are you sure that you want to generate a new client certificate for this Cloud Volumes ONTAP system?')
                    .then(() => {
                        nocHttp.post($scope.selectedWorkingEnvironment._pathPrefix + 'encryption/' + $scope.vsaResponse.id + '/update-client-certificate',
                            'generating new ONTAP client certificate',
                            nocHttp.LOADING)
                            .then((result) => {
                                nocEncryptionResourcesService.formatCertificate(result);
                                $scope.clientCertificate = result;
                                pidgeon.inProgress('ONTAP certificate renew started.', true);
                            });
                    });
            }
        };

        $scope.showAddKeyManagerDialog = function showAddKeyManagerDialog() {
            if ($scope.vsaResponse.vsaEncryptionResponse.keyManagers.length >= 4) {
                nocDialog.alert('You cannot configure Cloud Volumes ONTAP to use more than 4 key managers.');
            } else {
                nocDialog.dialog('Add a Key Manager', 'assets/templates/dialogs/add-key-manager.html', {
                    vsaResponse: $scope.vsaResponse,
                    selectedWorkingEnvironment: $scope.selectedWorkingEnvironment,
                    width: 415
                });
            }
        };

        $scope.initialize();
    });
})();
