(() => {
    'use strict';
    angular.module('noc').controller('EditVolumeController', ($scope, nocUI, nocUtil, pidgeon, BaseCreateVolume, $state, usageProfiles,
                                                              nocHttp, nocNormalizerService, BaseCreateVolumeOnExistingWorkingEnvironment,
                                                              $filter, $log) => {
        BaseCreateVolume.call($scope);

        $scope.dataReady = false;

        $scope.buildCifsProtocolInfo = function(shareInfo){
            var users = shareInfo.accessControlList.length > 0 ? shareInfo.accessControlList[0].users : [];
            return {
                accessControl: {
                    users: users,
                    _users: users.join(";"),
                    permission: shareInfo.accessControlList.length > 0 ? shareInfo.accessControlList[0].permission : ""
                },
                shareName: shareInfo.shareName,
                //access control list can be empty in case the user entered a wrong group/user and the access control list was not created
                shouldShowPermissions: shareInfo.accessControlList.length <= 1
            };
        };

        $scope.selectShareInfo = function() {
            const shareInfo = $scope.findShareInfoByName($scope.shareInfo.shareName);
            $scope.volumeRequest.shareInfo = $scope.buildCifsProtocolInfo(shareInfo);
        };

        $scope.generateEditVolumeRequest = function generateEditVolumeRequest(volume) {
            let ips;
            if (volume.exportPolicyInfo) {
                ips = _.isEmpty(volume.exportPolicyInfo.ips) ? $scope.getDefaultExportPolicyIps() : volume.exportPolicyInfo.ips;
            }

            return {
                name: volume.name,
                sizeGb: parseInt($filter('normalizedCapacity')(volume.size, 2, true, 'GB', true)),
                workingEnvironmentId: $scope.selectedWorkingEnvironment.publicId,
                svmName: volume.svmName,
                aggregateName: volume.aggregateName,
                exportPolicyInfo: volume.exportPolicyInfo ? {
                    policyType: volume.exportPolicyInfo.policyType,
                    ips,
                    _ips: ips.join(', ')
                } : undefined,
                shareInfo: volume.shareInfo.length > 0 ? $scope.buildCifsProtocolInfo(volume.shareInfo[0]) : undefined,
                usageProfile: usageProfiles.fromFlags(volume.thinProvisioning, volume.deduplication, volume.compression),
                snapshotPolicyName: volume.snapshotPolicy,
                providerVolumeType: volume.providerVolumeType
            };
        };

        $scope.shareInfoOptions = {
            formatResult: function(shareInfoItem){
                var shareInfo = $scope.findShareInfoByName(shareInfoItem.id);
                return nocUtil.formatSelect2(shareInfo.shareName, shareInfo.shareName,"", shareInfo.accessControlList.length + (shareInfo.accessControlList.length === 1 ? " permission":" permissions"));
            }
        };

        $scope.fillPermissions = () => {
            const shareInfo = $scope.findShareInfoByName($scope.shareInfo.shareName);
            return `${shareInfo.accessControlList.length}${shareInfo.accessControlList.length === 1 ? ' permission' : ' permissions'} `;
        };

        $scope.findShareInfoByName = function(shareName){
            return _.find($scope.selectedVolume.shareInfo, {shareName: shareName});
        };

        $scope.initializeController = function initializeController() {
            nocUI.clearFormValidity();
            BaseCreateVolumeOnExistingWorkingEnvironment.call($scope, pidgeon, nocHttp);

            $scope.$watch('volumes', function (volumes) {
                /* istanbul ignore else */
                if (volumes) {
                    $scope.resolveVolumeBySvmNameAndVolumeName($state.params.selectedVolumeSVM, $state.params.selectedVolumeName);

                    if ($scope.selectedVolume) {
                        $scope.selectDefaultExportPolicy($scope.selectedVolume.exportPolicyInfo.policyType);
                        $scope.volumeRequest = $scope.generateEditVolumeRequest($scope.selectedVolume);
                        $scope.shareInfo = {
                            shareName : _.get($scope.volumeRequest, 'shareInfo.shareName')
                        };

                        $scope.originalEditVolumeRequest = _.cloneDeep($scope.volumeRequest);
                        $scope.volumeProtocol = {type: $scope.volumeRequest.shareInfo ? 'CIFS' : 'NFS'};

                        $scope.fetchSnapshotPolicy($scope.selectedWorkingEnvironment, $scope.selectedWorkingEnvironment._pathPrefix, nocHttp);
                    }

                    $scope.dataReady = true;
                }
            });
        };

        $scope.resolveVolumeBySvmNameAndVolumeName = function(svmName, volumeName){
            if($scope.volumes){
                var selectedVolume = _.find($scope.volumes, {svmName: svmName, name: volumeName});
                if(selectedVolume){
                    $scope.selectedVolume = selectedVolume;
                }
                else {
                    $scope.cancelAddVolume($scope.selectedWorkingEnvironment);
                }
            }
            else $scope.cancelAddVolume($scope.selectedWorkingEnvironment);
        };

        $scope.isDirty = function () {
            var clone = _.cloneDeep($scope.volumeRequest);
            return !_.isEqual($scope.originalEditVolumeRequest, clone);
        };

        $scope.cancelAddVolume = function (workingEnvironment) {
            $scope.showMonitoring(workingEnvironment);
        };

        $scope.buildUpdatedShareInfo = function(shareName, permission, users) {
            var originalShareInfo = $scope.findShareInfoByName(shareName);
            return {
                shareName: shareName,
                //access control list can be empty in case the user entered a wrong group/user and the access control list was not created
                accessControlList: originalShareInfo.accessControlList.length <= 1 ? [{permission: permission, users: users}] : originalShareInfo.accessControlList
            };
        };

        $scope.saveAddVolume = function () {
            if (!$scope.editVolumeForm.$valid) {
                nocUI.handleFormValidity($scope.addVolumeForm, $scope);
            } else {
                if ($scope.isDirty()) {
                    var url = $scope.selectedWorkingEnvironment._pathPrefix + "volumes/" + $scope.selectedWorkingEnvironment.publicId + "/" + $scope.selectedVolume.svmName + "/" + $scope.selectedVolume.name;
                    var updatedVolumeRequest = $scope.generateRequestBody(null, $scope.volumeProtocol.type);
                    var updateVolumeData = {
                        originalVolumeInfo : {
                            snapshotPolicyName: $scope.selectedVolume.snapshotPolicy,
                            shareInfo: $scope.selectedVolume.shareInfo || undefined,
                            exportPolicyInfo: $scope.selectedVolume.exportPolicyInfo || undefined
                        },
                        snapshotPolicyName: updatedVolumeRequest.snapshotPolicyName,
                        shareInfo: updatedVolumeRequest.shareInfo ? $scope.buildUpdatedShareInfo(updatedVolumeRequest.shareInfo.shareName, updatedVolumeRequest.shareInfo.accessControl.permission, updatedVolumeRequest.shareInfo.accessControl.users): undefined,
                        exportPolicyInfo: updatedVolumeRequest.exportPolicyInfo ? {
                            policyType: updatedVolumeRequest.exportPolicyInfo.policyType,
                            ips: updatedVolumeRequest.exportPolicyInfo.policyType === "custom" ? nocNormalizerService.normalizeCustomAccessControl(updatedVolumeRequest.exportPolicyInfo._ips) : []
                        } : undefined
                    };
                    nocHttp.put({url: url, data: updateVolumeData}, "updating volume", nocHttp.BLOCK)
                        .then(function () {
                            delete $scope.originalEditVolumeRequest;
                            delete $scope.volumeRequest;
                            $scope.showMonitoring($scope.selectedWorkingEnvironment);
                        });
                }
                else {
                    $scope.cancelAddVolume($scope.selectedWorkingEnvironment);
                }
            }
        };

        $scope.updateSelectedWorkingEnvironment($state.params.workingEnvironmentId)
            .then($scope.initializeController)
            .catch($log.error);
    });
})();
