(() => {
    'use strict';

    angular.module('noc').controller('EnableCapacityTieringController', ($scope, workingEnvironmentResource, nocHttp, nocDialog,
                                                                         pidgeon, MetadataResource, CloudProviderAccountResource) => {
        $scope.initialize = () => {
            $scope.getObjectStorageName = () => {
                const providerToObjectStorage = {
                    AWS: 'S3',
                    Azure: 'Azure Blob storage',
                    GCP: 'Google Cloud Storage'
                };

                return providerToObjectStorage[$scope.selectedWorkingEnvironment._cloudProviderName];
            };
            $scope.capacityTier = $scope.getObjectStorageName();
            $scope.instanceProfileDefaultRole = {
                name: 'Let Cloud Manager create the role (recommended)',
                instanceProfileId: null
            };
            if ($scope.selectedWorkingEnvironment._isAWS) {
                MetadataResource.getInstanceProfiles(nocHttp, $scope.selectedWorkingEnvironment._pathPrefix,
                    _.get($scope, 'selectedWorkingEnvironment.awsProperties.cloudProviderAccountId'))
                    .then((instanceProfiles) => {
                        $scope.selected = {};
                        $scope.instanceProfiles = instanceProfiles;
                        $scope.instanceProfiles.unshift($scope.instanceProfileDefaultRole);
                        $scope.selected.selectedInstanceProfile = $scope.selected.selectedInstanceProfile || $scope.instanceProfiles[0];
                    });
            }

            if ($scope.selectedWorkingEnvironment._isGCP) {
                $scope.gcpAccountRequest = {};
                CloudProviderAccountResource.getAccounts(nocHttp)
                    .then((accounts) => {
                        $scope.gcpStorageAccounts = accounts.gcpStorageAccounts;
                        if (!$scope.selectedGcpAccount) {
                            $scope.selectedGcpAccount = accounts.gcpStorageAccounts[0];
                        }
                    });
            }
        };

        $scope.goToAccountSettings = () => {
            $scope.checkAndSetState('cloud-provider-accounts.account-settings');
        };

        $scope.openLearnMoreDialog = () => {
            nocDialog.confirm('', 'assets/templates/cloud-provider-accounts/gcp-learn-more.html', {
                width: 920,
                fromTop: '20%'
            });
        };

        const configureFabricPool = (selectedInstanceProfileName, gcpStorageAccountId) => {
            workingEnvironmentResource.configureFabricPool(nocHttp, $scope.selectedWorkingEnvironment, selectedInstanceProfileName, gcpStorageAccountId)
                .then(() => {
                    $scope.boxGoBack();
                    pidgeon.inProgress('Configuring fabric pool resources.', true);
                });
        };

        $scope.enableCapacityTiering = () => {
            if ($scope.selectedWorkingEnvironment._isGCP) {
                if ($scope.selectedGcpAccount) {
                    configureFabricPool(null, $scope.selectedGcpAccount.publicId);
                } else {
                    CloudProviderAccountResource.createAccount(nocHttp, 'gcp', $scope.gcpAccountRequest.accountName, $scope.gcpAccountRequest.providerKeys)
                        .then((account) => {
                            $scope.selectedGcpAccount = account;
                            configureFabricPool(null, $scope.selectedGcpAccount.publicId);
                        });
                }
            } else {
                const selectedInstanceProfileName = $scope.selectedWorkingEnvironment._isAWS && ($scope.selected.selectedInstanceProfile !== $scope.instanceProfileDefaultRole) ? $scope.selected.selectedInstanceProfile.name : null;
                configureFabricPool(selectedInstanceProfileName);
            }
        };

        $scope.initialize();
    });
})();
