(() => {
    'use strict';

    angular.module('noc').controller('GalleryGridController', ($scope, galleryTableColumnsService, $timeout, columnPickerService,
                                                               uiGridFilterService, $filter, workingEnvironmentStatuses,
                                                               nocDialog, nocUtil, uiGridConstants, nocCurrentUserService, nocSelectedWorkspaceOrTenantService) => {
        $scope.search = {term: ''};

        const rowTemplate = `<div ng-mouseenter="grid.appScope.over(row.entity)"
                                  ng-mouseleave="grid.appScope.leave(row.entity)"
                                  ng-repeat="(colRenderIndex, col) in colContainer.renderedColumns track by col.colDef.name"
                                  class="ui-grid-cell"
                                  ng-class="{'ui-grid-row-header-cell': col.isRowHeader, 'hovered': grid.appScope.hoveredRow === row.entity}"
                                  ui-grid-cell></div>`;
        $scope.hoveredRow = null;

        $scope.isAllowed = () => {
            return nocCurrentUserService.isAllowed('we:create', nocSelectedWorkspaceOrTenantService.getSelectedWorkspace());
        };

        $scope.over = function(entity) {
            $scope.hoveredRow = entity;
        };

        $scope.leave = function(entity) {
            if ($scope.hoveredRow === entity){
                $scope.hoveredRow = null;
            }
        };

        function setGridHeight() {
            const uiGrid = $('[ui-grid]'),
                top = uiGrid.offset().top,
                mainFooterTop = $('.main-footer').offset().top;
            uiGrid.removeClass('visible');
            uiGrid.height(mainFooterTop - top - 40);
            $timeout(() => {
                uiGrid.addClass('visible');
            });
        }

        var onScroll = function(scrollEvent) {
            if (scrollEvent.x) $scope.gridIsScrolled = scrollEvent.newScrollLeft > 0;
        };

        $scope.gridOptions = {
            enableHorizontalScrollbar: uiGridConstants.scrollbars.NEVER,
            enableVerticalScrollbar: uiGridConstants.scrollbars.NEVER,
            useExternalFiltering: true,
            columnVirtualizationThreshold: 50,
            virtualizationThreshold: 30,
            rowTemplate: rowTemplate,
            columnDefs: galleryTableColumnsService.getColumnsDef(),
            onRegisterApi: (gridAPI) => {
                $scope.gridAPI = gridAPI;
                uiGridFilterService.installFilter(gridAPI, $scope);
                $scope.gridAPI.core.addRowHeaderColumn({
                    name: 'rowHeaderCol',
                    displayName: '',
                    width: 60,
                    enableColumnResizing: false,
                    cellTemplate: 'assets/templates/tabs/working-environments/gallery-grid-popup-btn.html'
                });
                $scope.gridAPI.colMovable.on.columnPositionChanged($scope, () => {
                    galleryTableColumnsService.saveGridState($scope.gridAPI, $scope);
                });
                $timeout(() => {
                    galleryTableColumnsService.restoreGridState($scope.gridAPI, $scope);
                    $timeout(() => {
                        $(window).trigger('resize');
                    });
                });
                $scope.gridAPI.core.on.scrollBegin($scope, onScroll);
                $scope.gridAPI.core.on.scrollEnd($scope, onScroll);
                $scope.$watch('gridAPI.grid.isScrollingVertically', (isScrolling) => {
                    if (isScrolling && $scope.popoverHelper.popoverApi) $scope.popoverHelper.popoverApi.remove();
                });
            }
        };

        $(window).on("resize", setGridHeight);

        $scope.startColumnPicker = function() {
            columnPickerService.startColumnPicker($scope.gridOptions, $scope.gridAPI,galleryTableColumnsService, "Working Environments Table Columns");
        };

        $scope.targetWorkingEnvironmentsMatcher = function (searchString, itemText) {
            var workingEnvironment = _.find($scope.workingEnvironments, {name: itemText});
            workingEnvironment._formattedRegion = (workingEnvironment.workingEnvironmentType === "ON_PREM") ? "" : $filter("regionFormat")(workingEnvironment.regionName);
            return nocUtil.genericItemFilter(searchString, workingEnvironment, ["name", "_typeLongName", "_formattedRegion"]);
        };

        $scope.formatTargetWorkingEnvironments = function (workingEnvironmentItem) {
            var workingEnvironment = _.find($scope.workingEnvironments, {name: workingEnvironmentItem.id});
            if (workingEnvironment) {
                return nocUtil.formatSelect2(
                    workingEnvironment.name,
                    workingEnvironment.name,
                    workingEnvironment._typeLongName,
                    workingEnvironment.workingEnvironmentType === "ON_PREM" ? "" : $filter("regionFormat")(workingEnvironment.regionName));
            } else {
                return workingEnvironmentItem.text;
            }
        };

        $scope.targetWorkingEnvironmentsOptions = {
            matcher: $scope.targetWorkingEnvironmentsMatcher,
            formatNoMatches: _.constant('No working environments to which you can replicate data found.'),
            formatResult: $scope.formatTargetWorkingEnvironments
        };

        $scope.replicateFromTable = (source) => {
            const targetWorkingEnvironments = [];
            _.forEach($scope.workingEnvironments, (workingEnvironment) => {
                if (workingEnvironmentStatuses.isReplicationAllowed(source, workingEnvironment) && source.publicId !== workingEnvironment.publicId) {
                    targetWorkingEnvironments.push(workingEnvironment);
                }
            });

            $scope.target = targetWorkingEnvironments.length > 0 ? targetWorkingEnvironments[0] : null;
            $scope.targetName = $scope.target ? $scope.target.name : null;
            $scope.setTarget = function(targetName) {
                $scope.target = _.find($scope.workingEnvironments, {name: targetName});
            };

            nocDialog.confirm("Select the working environment to which you want to replicate data", "assets/templates/dialogs/select-target-working-environment-dialog.html", {
                targetWorkingEnvironments: targetWorkingEnvironments,
                targetWorkingEnvironmentsOptions: $scope.targetWorkingEnvironmentsOptions,
                target: $scope.target,
                targetName: $scope.targetName,
                setTarget: $scope.setTarget,
                width: 520,
                fromTop: "10%"
            })
                .then(function() {
                    if (workingEnvironmentStatuses.isReplicationAllowed(source, $scope.target)) {
                        $scope.checkAndSetState('snap-mirror-wizard', {from: "main.working-environments.index", slTenantId: nocSelectedWorkspaceOrTenantService.getPublicId(),
                            sourceId: source.publicId, targetId: $scope.target.publicId, sourceType:source._workingEnvironmentTypeByLocation, sourceProvider: source._cloudProviderName, targetType:$scope.target._workingEnvironmentTypeByLocation, targetProvider: $scope.target._cloudProviderName});
                    }
                });
        };

        $scope.initialize = () => {
            unwatchWorkingEnvironments();
            $scope.$watchCollection('workingEnvironments', () => {
                $scope.gridOptions.data = _.cloneDeep($scope.workingEnvironments);
            });
        };

        var unwatchWorkingEnvironments = $scope.$watch('workingEnvironments', $scope.initialize);

        $scope.$on('$destroy', function() {
            galleryTableColumnsService.saveGridState($scope.gridAPI);
            $(window).off('resize', setGridHeight);
        });
    });
})();
