(() => {
    'use strict';
    angular.module('noc').controller('MonitoringSVMController', ($scope, workingEnvironmentResource, actionRequiredResource,
                                                                 pidgeon, nocUtil, volumesResource, $window, $timeout, nocUI, nocCurrentUserService) => {
        const unloading = nocUI.loading();
        let vsaPollingStopper = _.noop;
        let vsaActiveActionsPollingStopper = _.noop;

        let actionRequiredPollstopper = _.noop;
        const startPollingActionRequired = () => {
            actionRequiredPollstopper = actionRequiredResource.startActionRequiredPolling($scope.selectedWorkingEnvironment, () => {
                $scope.setComputingCapacity(true);
            });
        };

        let volumesPollingStopper = _.noop;
        const startVolumePolling = () => {
            const status = $scope.selectedWorkingEnvironment.status ? $scope.selectedWorkingEnvironment.status.status : 'undefined';
            if (volumesPollingStopper === _.noop && (status === 'on' || status === 'degraded')) {
                volumesPollingStopper = volumesResource.startVolumesPolling($scope.selectedWorkingEnvironment, unloading);
            }
        };

        $scope.initialize = function initialize() {
            startVolumePolling();

            $scope.volumesMenuAvailable = true;
            $scope.capacityRadius = 50;
            $scope.capacityCircumference = 2 * Math.PI * $scope.capacityRadius;

            if ($scope.selectedWorkingEnvironment.workingEnvironmentType === 'VSA') {
                // First initialization
                if ($scope.selectedWorkingEnvironment && !$scope.vsaResponse) {
                    $scope.vsaResponse = workingEnvironmentResource.workingEnvironmentResponseToVsaResponse($scope.selectedWorkingEnvironment);
                }

                $scope.$on(workingEnvironmentResource.VSA_GET_EVENT2, (event, payload) => {
                    pidgeon.clear('vsaInformation');

                    if (actionRequiredPollstopper === _.noop) {
                        startPollingActionRequired();
                    }
                    $scope.handleActiveActionsUpdate(payload.workingEnvironment.activeActions, payload.currentActiveActions);
                });

                vsaPollingStopper = workingEnvironmentResource.startVsaPolling($scope.selectedWorkingEnvironment);
                vsaActiveActionsPollingStopper = workingEnvironmentResource.startVsaActiveActionsPolling($scope.selectedWorkingEnvironment);

                $scope.$on($scope.selectedWorkingEnvironment.publicId + workingEnvironmentResource.VSA_ACTIVE_ACTIONS_GET_EVENT, (event, newActiveActions) => {
                    pidgeon.clear('vsa-state');

                    if (!$scope.vsaResponse) return;

                    $scope.handleActiveActionsUpdate(newActiveActions, $scope.vsaResponse.activeActions);

                    $scope.vsaResponse.activeActions = newActiveActions;
                });

                $scope.$on($scope.selectedWorkingEnvironment.publicId + workingEnvironmentResource.VSA_ACTIVE_ACTIONS_ERROR_EVENT, (event, error) => {
                    pidgeon.error(pidgeon.serverErrorPrefix + 'retrieving Cloud Volumes ONTAP state for working environment: ' + $scope.selectedWorkingEnvironment.name, error, 'vsa-state');
                });
            } else if ($scope.selectedWorkingEnvironment.workingEnvironmentType === 'ON_PREM') {
                if (actionRequiredPollstopper === _.noop) {
                    startPollingActionRequired();
                }
            }

            $scope.$on(actionRequiredResource.ACTION_REQUIRED_GET_EVENT, (event, actionRequiredResponse) => {
                const mappedResponse = {}; // The server has changed the API response NOC-4518 no action required in resources
                mappedResponse[$scope.selectedWorkingEnvironment.publicId] = actionRequiredResponse;
                pidgeon.clear('action-required');
                $scope.setActionRequiredFor($scope.selectedWorkingEnvironment.publicId, !_.isEmpty(_.get($scope, 'vsaResponse.activeActions')), mappedResponse);
                $scope.setComputingCapacity(false);
            });

            $scope.$on(actionRequiredResource.ACTION_REQUIRED_ERROR_EVENT, (event, error) => {
                pidgeon.error(pidgeon.serverErrorPrefix + 'retrieving required capacity actions', error, 'action-required');
                $scope.setComputingCapacity(false);
            });

            $scope.$on(workingEnvironmentResource.WORKING_ENVIRONMENT_STATUS_UPDATE, (event, payload) => {
                const updatedWE = payload.workingEnvironment;
                if (updatedWE.publicId === $scope.selectedWorkingEnvironment.publicId) {
                    _.assignInWith($scope.selectedWorkingEnvironment, updatedWE, nocUtil.dontOverrideObjectWithNull);
                    if (!$scope.selectedWorkingEnvironment.status || ($scope.selectedWorkingEnvironment.status.status !== 'on' && $scope.selectedWorkingEnvironment.status.status !== 'degraded')) {
                        volumesPollingStopper();
                    } else {
                        startVolumePolling();
                    }
                }
            });
        };

        $scope.handleActiveActionsUpdate = function handleActiveActionsUpdate(newActiveActions, currentActiveActions) {
            if (!_.isEmpty(newActiveActions)) {
                $scope.setComputingCapacity(true);
            }

            if (!_.isEmpty(currentActiveActions) && _.isEmpty(newActiveActions)) {
                delete $scope.actionsRequired[$scope.selectedWorkingEnvironment.publicId];
                if (actionRequiredPollstopper) actionRequiredPollstopper();
                startPollingActionRequired();
            }
        };

        $scope.calculateVolumesState = function calculateVolumesState() {
            if (!$scope.selectedWorkingEnvironment) return;

            if (($scope.selectedWorkingEnvironment.status.status === 'on' && $scope.volumes) ||
                $scope.selectedWorkingEnvironment.status.status === 'degraded' ||
                ($scope.selectedWorkingEnvironment.status.status === 'on' && $scope.selectedWorkingEnvironment._cloudProviderName === 'Azure')) {
                return 'volumes';
            }
        };

        /* istanbul ignore next: no logic */
        $scope.changeTierDisabledMessage = function changeTierDisabledMessage(volume) {
            if (volume.parentVolumeName) return 'Cloned or restored volume tier cannot be changed';
            if (volume.cloneNames && volume.cloneNames.length) return 'Parent volume tier cannot be changed';
            if (volume.state === 'offline') return 'Offline volume tier cannot be changed';
            if (volume.state === 'undefined') return 'Undefined volume tier cannot be changed';
            if (volume.moving) return 'Volume is already in move process';
            if (!_.get($scope.selectedWorkingEnvironment, 'supportedFeatures.supportsMixedAggregates')) {
                return 'This Cloud Volumes ONTAP version does not support changing volume\'s tier';
            }
            if ($scope.selectedWorkingEnvironment.status.status === 'degraded') {
                return 'Degraded HA volume\'s tier cannot be changed';
            }
            if (volume.volumeType === 'dp') return 'Data protection volume tier cannot be changed';
            if (volume.snaplockType === 'compliance' || volume.snaplockType === 'enterprise') return 'WORM storage volume tier cannot be changed';

            return '';
        };

        $scope.disabledTooltipForSnapshotsVolume = function disabledTooltipForSnapshotsVolume(volume) {
            if (volume.volumeType === 'dp') return 'You cannot create a Snapshot copy of a data protection volume';
            if (volume.volumeType !== 'rw') return 'You can create a Snapshot copy only for read-write volumes';
            if (volume.state === 'offline') return 'You cannot create a Snapshot copy of an offline volume';
            if (volume.state === 'undefined') return 'You cannot create a Snapshot copy of an undefined volume';
            if (volume.moving) return 'Volume operations cannot be performed while changing tier';

            return '';
        };

        /* istanbul ignore next: no logic */
        $scope.disabledTooltipForVolumeAction = function disabledTooltipForVolumeAction(volume, action, disabledStates) {
            if (disabledStates.dp && volume.volumeType === 'dp') return 'Data protection volume cannot be ' + action;
            if (disabledStates.cifs && volume.securityStyle === 'ntfs') return 'CIFS volume cannot be ' + action;
            if (disabledStates.offline && volume.state === 'offline') return 'Offline volume cannot be ' + action;
            if (volume.state === 'undefined' && action === 'info') return 'There is no information available for an undefined volume';
            if (volume.state === 'undefined' && action !== 'deleted') return 'Undefined volume cannot be ' + action;
            if (disabledStates.moving && volume.moving) return 'Volume operations cannot be performed while changing tier';
            if (disabledStates.parent && volume.cloneNames.length > 0) return 'Parent volume cannot be ' + action;
            if (disabledStates.noJunctionPath && !volume.junctionPath) return 'Volume without junction path cannot be ' + action;
            if (disabledStates.noMountPoint && !volume.mountPoint && !volume.cifsShareAccessPoint) {
                return 'Could not calculate ' +
                    (volume.securityStyle === 'ntfs' ? 'share access point' : 'mount point') +
                    ', volume cannot be ' + action;
            }

            return '';
        };

        $scope.disableVolumeActionsForProblematicSync = function disableVolumeActionsForProblematicSync() {
            if (_.get($scope.selectedWorkingEnvironment.cloudSyncProperties, 'status.status') !== 'COMPLETE') {
                return 'Volume cannot be synced to S3, Sync is not available';
            }
            const expired = nocCurrentUserService.getCurrentUserCloudSyncLicenseExpired();
            if (expired) {
                return 'Cloud Sync license expired';
            }
        };

        $scope.disabledTooltipForSyncVolume = function disabledTooltipForSyncVolume(volume) {
            return $scope.disableVolumeActionsForProblematicSync() || $scope.disabledTooltipForVolumeAction(volume, 'synced to S3', {
                dp: true,
                cifs: true,
                offline: true,
                undefined: true,
                moving: true,
                noJunctionPath: true,
                noMountPoint: true
            });
        };

        $window.onclick = function onclick() {
            $timeout(() => {
                if (_.isNumber($scope.selectedCardIndex)) {
                    $scope.selectedCardIndex = null;
                }
            }); //Timeout actives digest cycle so that volume menu instantly closes
        };

        $scope.toggleCardMenu = function toggleCardMenu(index, event) {
            $scope.selectedCardIndex = index;
            if (event) event.stopPropagation();
        };

        const doFiltering = () => {
            $scope.filteredVolumes = nocUtil.filterVolumes($scope.volumes, $scope.search);
        };

        $scope.$watch('search', doFiltering);
        $scope.$watchCollection('volumes', doFiltering);

        $scope.$on('$destroy', () => {
            volumesPollingStopper();
            vsaPollingStopper();
            actionRequiredPollstopper();
            vsaActiveActionsPollingStopper();
        });

        if ($scope.monitoringInitialized) {
            $scope.initialize();
        } else {
            $scope.$watch('monitoringInitialized', () => {
                if ($scope.monitoringInitialized) {
                    $scope.initialize();
                }
            });
        }
    });
})();
