(function() {
    "use strict";
    angular.module("noc").controller("ShutdownVSAController", function($scope, workingEnvironmentResource, nocUI, $timeout, nocHttp){
        $scope.daysMap = {
            0: "Sun",
            1: "Mon",
            2: "Tue",
            3: "Wed",
            4: "Thu",
            5: "Fri",
            6: "Sat"
        };

        $scope.buildMomentTime = function(range){
            return moment(_.pick(range, "hour","minute")).day(range.day);
        };

        $scope.initialize = function() {
            $scope.selectedWorkingEnvironment._schedulesListInitialized = $scope.initializeSchedules($scope.selectedWorkingEnvironment._originalSchedules);
            $scope.selectedWorkingEnvironment._originalSchedulesListInitialized = _.cloneDeep($scope.selectedWorkingEnvironment._schedulesListInitialized);
        };

        $scope.fixDaysValues = function (schedule) {
            _.forEach(schedule.schedule, function (range) {
                //server expects sunday to be 7 instead of 0
                range.start.day = range.start.day === 7 ? 0 : range.start.day;
                range.stop.day = range.stop.day === 7 ? 0 : range.stop.day;

                //make sure the start is after the stop
                range.start.day = range.start.day < range.stop.day ? (range.start.day + 7) : range.start.day;
            });
        };

        $scope.buildDaysList = function (schedule) {
            return _.map(schedule.schedule, function (schd) {
                return $scope.daysMap[schd.stop.day];
            });
        };

        $scope.fixInvalidShutdownPeriod = function(schedule, period){
            if (period > schedule._maxPeriod) return schedule._maxPeriod;
            if (period < 1) return 1;
            return Math.round(period);
        };

        $scope.initializeSchedules = (schedules) => {
            const schedulesToDisplay = [];
            _.forEach(schedules, (schedule) => {
                const tmpSched = _.cloneDeep(schedule);
                $scope.fixDaysValues(schedule);
                tmpSched._formattedName = _.toLower(schedule.name);
                const stopMoment = $scope.buildMomentTime(schedule.schedule[0].stop);
                tmpSched._stopTime = stopMoment.toDate();
                const startMoment = $scope.buildMomentTime(schedule.schedule[0].start);
                tmpSched._startTime = startMoment.toDate();

                tmpSched._stopDays = $scope.buildDaysList(schedule);
                tmpSched._maxPeriod = schedule.name === 'Weekend' ? 48 : 24;

                tmpSched._shutdownPeriod = $scope.fixInvalidShutdownPeriod(tmpSched, moment.duration(startMoment.diff(stopMoment)).asHours());
                tmpSched.enabled = schedule.enabled;
                schedulesToDisplay.unshift(tmpSched);
            });
            return schedulesToDisplay;
        };

        $scope.isScheduleAllowed = (name) => {
            const allDaysEnabled = _.get(_.find($scope.selectedWorkingEnvironment._schedulesListInitialized, ['name', 'day']), 'enabled');
            const nonAllDaysEnabled = _.filter($scope.selectedWorkingEnvironment._schedulesListInitialized, (s) => {
                return s.name !== 'day' && s.enabled;
            }).length > 0;

            if (name !== 'day' && allDaysEnabled) {
                return true;
            }

            if (name === 'day' && nonAllDaysEnabled) {
                return true;
            }

            return false;
        };

        $scope.buildSchedulesRequest = function(schedules) {
            const schedulesCopy = _.cloneDeep(schedules);
            const request = [];
            _.forEach(schedulesCopy, (schedule) => {
                _.forEach(schedule.schedule, (range) => {
                    range.stop.hour = schedule._stopTime.getHours();
                    range.stop.minute = schedule._stopTime.getMinutes();
                    range.start.minute = schedule._stopTime.getMinutes();

                    //for each inner schedule we must build again the stop time since the days differ in each schedule...
                    schedule._startTime = $scope.buildMomentTime(range.stop).add(schedule._shutdownPeriod, "hours").toDate();
                    range.start.hour = schedule._startTime.getHours();

                    //server expects sunday to be 7 instead of 0
                    range.start.day = moment(schedule._startTime).day() === 0 ? 7 : moment(schedule._startTime).day();
                    range.stop.day = range.stop.day === 0 ? 7 : range.stop.day;

                });
                request.push(_.pick(schedule,"name","enabled","schedule"));
            });

            return request;
        };

        $scope.isValid = function(){
            return $scope.shutdownVSAForm.$valid;
        };

        $scope.returnUncheckedScheduleToOriginalValue = function (schedule) {
            if (!schedule.enabled) {
                var index = _.findIndex($scope.selectedWorkingEnvironment._schedulesListInitialized, {name: schedule.name});
                var scheduleObjectWithoutCheckedStatus = _.omit($scope.selectedWorkingEnvironment._originalSchedulesListInitialized[index], 'enabled');
                _.assign($scope.selectedWorkingEnvironment._schedulesListInitialized[index], scheduleObjectWithoutCheckedStatus);
            }
        };

        $scope.saveSchedules = function() {
            if(!$scope.isValid()) {
                nocUI.handleFormValidity($scope.shutdownVSAForm, $scope);
            }else{
                var schedulesRequest = $scope.buildSchedulesRequest($scope.selectedWorkingEnvironment._schedulesListInitialized);
                workingEnvironmentResource.updateSchedules(nocHttp, $scope.selectedWorkingEnvironment, schedulesRequest)
                    .then(function () {
                        //update the schedules only after saved.
                        $scope.selectedWorkingEnvironment._originalSchedules = schedulesRequest;
                        $scope.showMonitoring();
                    });
            }
        };

        var clearWatch = $scope.$watchCollection("selectedWorkingEnvironment._originalSchedules",function(){
            if ($scope.selectedWorkingEnvironment._originalSchedules.length > 0) {
                $scope.initialize();
                clearWatch();
            }
        });

        $scope.updateSelectedSchedule = function(schedule) {
            if (!schedule.enabled && (!schedule._shutdownPeriod || schedule._shutdownPeriod < 1 || schedule._shutdownPeriod > schedule._maxPeriod)){
                $timeout(function(){
                    schedule._shutdownPeriod = schedule._originalShutdownPeriod;
                });
            }
        };
    });
})();