(function () {
    "use strict";
    angular.module("noc").controller("SupportRegistrationFormController", function ($scope, nocHttp, $q, pidgeon, nocDialog, $rootScope, nocTenantsService, configService, CloudProviderAccountResource) {

        $scope.initialize = function () {
            $scope.loading = true;
            $q.all([
                nocHttp.get($scope.selectedWorkingEnvironment._pathPrefix + 'working-environments/' + $scope.selectedWorkingEnvironment.publicId + '?fields=supportRegistrationInformation',
                    'retrieving support registration information'),
                configService.getConfig(nocHttp, ['ipaManualUrl'], true),
                CloudProviderAccountResource.getAccounts(nocHttp),
                nocHttp.get($scope.selectedWorkingEnvironment._pathPrefix + 'working-environments/' + $scope.selectedWorkingEnvironment.publicId + '/support-registration',
                    'retrieving support registration information')
            ])
                .then(_.spread((vsaResponse, config, accounts, supportRegistrationResponse) => {
                    $scope.registrationStatusMessages = _.map(vsaResponse.supportRegistrationInformation, (supportInfo) => {
                        // Extract supportRegistrationStatus while replacing 'deregistered' with 'not-registered' (same treatment)
                        return supportInfo.supportRegistrationStatus === 'deregistered' ? 'not-registered' : supportInfo.supportRegistrationStatus;
                    });
                    $scope.registrationStatusMessage = $scope.registrationStatusMessages[0];
                    $scope.supportRegistrationInformation = vsaResponse.supportRegistrationInformation;
                    $scope.isNotRegistered = _.includes($scope.registrationStatusMessages, 'not-registered');

                    $scope.manualRegistrationUrl = config.ipaManualUrl;

                    $scope.nssAccounts = accounts.nssAccounts;
                    if (!$scope.selectedNssAccount.nssAccount) {
                        $scope.selectedNssAccount.nssAccount = accounts.nssAccounts[0];
                    }

                    $scope.moreInfoParameters = {
                        nssUserName: _.get(_.find($scope.nssAccounts, {publicId: vsaResponse.supportRegistrationInformation[0].nssAccountId}), 'accountName'),
                        expiryDate: vsaResponse.supportRegistrationInformation[0].licenseExpiryDate ? moment(vsaResponse.supportRegistrationInformation[0].licenseExpiryDate).format('LL') : null,
                        startDate: supportRegistrationResponse.licenseStartDate ? moment(supportRegistrationResponse.licenseStartDate).format('LL') : null,
                        goToAccountSettings: $scope.goToAccountSettings
                    };

                }))
                .finally(() => {
                    $scope.loading = false;
                });
        };

        const queryBuilder = (instanceId, serialNumber, accountId, instanceId2, serialNumber2) => {
            if (!instanceId || !serialNumber || !accountId) return '';
            let templateString = '?instanceId={{instanceId}}&serialNumber={{serialNumber}}&accountId={{accountId}}&hc={{hashCode}}';
            const templateProperties = {
                instanceId,
                serialNumber,
                accountId,
                hashCode: $scope.hash(instanceId.toString() + serialNumber.toString() + accountId.toString())
            };

            // If has second node (HA) - add second node parameters to query url
            if (instanceId2 && serialNumber2) {
                templateString += '&instanceId={{instanceId2}}&serialNumber={{serialNumber2}}'; // Using same parameter name in a query sends the values as an array
                _.assign(templateProperties, {
                    instanceId2,
                    serialNumber2,
                    hashCode: $scope.hash(instanceId.toString() + instanceId2.toString() + serialNumber.toString() + serialNumber2.toString() + accountId.toString())
                });
            }

            const template = _.template(templateString);

            return template(templateProperties);
        };

        $scope.manualRegistrationPopup = () => {
            let query = null;

            if ($scope.selectedWorkingEnvironment._isAWS && !$scope.selectedWorkingEnvironment.isHA) {
                query = queryBuilder($scope.vsaResponse.instanceId, $scope.vsaResponse.cluster.serialNumbers, $scope.vsaResponse.accountId);
            } else if ($scope.selectedWorkingEnvironment._isAzure) {
                query = queryBuilder($scope.vsaResponse.deploymentId, $scope.vsaResponse.cluster.serialNumbers, $scope.vsaResponse.subscriptionId);
            } else if ($scope.selectedWorkingEnvironment.isHA) {
                const node1 = $scope.vsaResponse.cluster.nodes[0];
                const node2 = $scope.vsaResponse.cluster.nodes[1];
                query = queryBuilder(node1.cloudProviderId, node1.serialNumber, $scope.vsaResponse.accountId, node2.cloudProviderId, node2.serialNumber);
            }

            nocDialog.dialog('NetApp Support Registration', 'assets/templates/dialogs/support-registration-manual-register-popup.html', {
                query,
                manualRegistrationUrl: $scope.manualRegistrationUrl,
                manual: true
            });
        };

        $scope.openLearnMoreDialog = () => {
            nocDialog.confirm('', 'assets/templates/cloud-provider-accounts/nss-learn-more.html', {
                width: 890,
                fromTop: '20%'
            });
        };

        $scope.toggleNodeInformation = (nodeIndex) => {
            if ($scope.currentOpenSection === nodeIndex) {
                $scope.currentOpenSection = '';
            } else {
                $scope.currentOpenSection = nodeIndex;
            }
        };

        const performAutomaticRegistration = () => {
            $scope.registrationStatusMessage = 'in-progress';
            const nssRegisterUrl = $scope.selectedWorkingEnvironment._pathPrefix + 'working-environments/' + $scope.selectedWorkingEnvironment.publicId +
                '/support-registration?nssAccountId=' + $scope.selectedNssAccount.nssAccount.publicId;
            nocHttp.post(nssRegisterUrl, '', nocHttp.BLOCK)
                .then((response) => {
                    $scope.registrationStatusMessage = 'registered';
                    $scope.moreInfoParameters = {
                        nssUserName: _.find($scope.nssAccounts, {publicId: response.nssAccountId}).accountName,
                        expiryDate: moment(response.licenseExpiryDate).format('LL'),
                        startDate: moment(response.licenseStartDate).format('LL'),
                        goToAccountSettings: $scope.goToAccountSettings
                    };
                })
                .catch((error) => {
                    $scope.registrationStatusMessage = 'not-registered';
                    $scope.errorMessage = error.causeMessage.substring(_.indexOf(error.causeMessage, ':') + 2, error.causeMessage.length);
                });
        };

        $scope.registerForSupport = () => {
            if (_.get($scope.nssAccountRequest, 'accountName')) {
                $scope.errorMessage = '';
                $scope.nameErrorMessage = '';
                CloudProviderAccountResource.createAccount(nocHttp, 'nss', $scope.nssAccountRequest.accountName, $scope.nssAccountRequest.providerKeys, [], true)
                    .then((account) => {
                        $scope.nssAccounts = [account];
                        $scope.selectedNssAccount.nssAccount = account;
                        performAutomaticRegistration();
                    })
                    .catch((error) => {
                        if (_.endsWith(error.message, 'Account name is already in use.')) {
                            $scope.nameErrorMessage = 'Account name already exists';
                        } else {
                            $scope.errorMessage = error.causeMessage.substring(_.indexOf(error.causeMessage, ':') + 2, error.causeMessage.length);
                        }
                    });
            } else {
                performAutomaticRegistration();
            }
        };

        $scope.goToAccountSettings = () => {
            $scope.checkAndSetState('cloud-provider-accounts.account-settings');
        };

        /* eslint-disable no-bitwise */
        $scope.hash = (text) => {
            let hash = 0;
            if (text.length === 0) return hash;
            for (let i = 0; i < text.length; i++) {
                const char = text.charCodeAt(i);
                hash = ((hash << 5) - hash) + char;
                hash = hash & hash;
            }

            return hash;
        };
        /* eslint-enable no-bitwise */

        $scope.initialize();
    });
})();
