(function () {
    'use strict';

    angular.module('noc').controller('SyncS3StatusMonitoringController', function ($rootScope, $scope, cloudSyncGoThereService, $state, nocHttp,
                                                                                   workingEnvironmentResource, nocDialog, nocPoller,
                                                                                   nocCloudSyncService, configService, nocCurrentUserService) {
        $scope.poller = _.noop;

        $scope.currentUserCloudSyncLicenseExpirationDate = nocCurrentUserService.getCurrentUserCloudSyncLicenseExpirationDate();

        function getSyncStatus() {
            if (_.get(nocCurrentUserService.getCurrentUserCloudSyncLicenseInformation(), '._cloudSyncLicenseExpired')) {
                if (_.isEqual(nocCurrentUserService.getCurrentUserCloudSyncLicenseTypes(), ['trial'])) {
                    return 'TRIAL_EXPIRED';
                }
                return 'LICENSE_EXPIRED';
            }
            if ($scope.syncData.isSynced) {
                switch ($scope.syncData.status.status) {
                    case 'IN_PROGRESS':
                        return 'IN_PROGRESS';
                    case 'DELETING':
                        return 'DELETING';
                    case 'USER_NOT_LOGGED_IN':
                        return 'USER_NOT_LOGGED_IN';
                    default:
                        return 'SYNCED';
                }
            }
            return 'NOT_SYNCED';
        }

        function processWorkingEnvironment(workingEnvironment) {
            workingEnvironmentResource.initWorkingEnvironment(workingEnvironment);
            $scope.selectedWorkingEnvironment.cloudSyncProperties = workingEnvironment.cloudSyncProperties;

            $scope.syncData = workingEnvironment.cloudSyncProperties;

            $scope.syncStatus = getSyncStatus();

            $scope.relationshipsStatusesCount = _($scope.syncData.relationships).countBy('relationshipStatus')
                .map(function (value, key) {
                    return value + ' ' + _.capitalize(key);
                })
                .join(', ');

            if ($scope.syncData.isSynced && !$scope.isLoading && $scope.syncData.relationships.length > 0) {
                $scope.hasOtherLocations = !_.every($scope.syncData.relationships, function (relation) {
                    return _.includes(relation.s3Location, $scope.syncData.s3Location);
                });
            }
        }

        function pollForStatus(workingEnvironment) {
            var opts = {
                method: 'GET',
                url: workingEnvironmentResource.getUrlWithFields(workingEnvironment.publicId, workingEnvironment._pathPrefix, 'cloudSyncProperties')
            };
            $scope.poller = nocPoller.start(opts, 60 * 1000, processWorkingEnvironment, _.noop, null, true);
        }

        $scope.initializeController = function initializeController() {
            $scope.cloudSyncConstants = {};
            configService.getConfig(nocHttp, ['cloudSyncConstants'])
                .then(function (config) {
                    $scope.cloudSyncConstants = config.cloudSyncConstants;
                });

            $scope.serviceUrl = $rootScope.cloudSyncUrl;

            processWorkingEnvironment($scope.selectedWorkingEnvironment);
            if ($scope.syncData.isSynced) {
                pollForStatus($scope.selectedWorkingEnvironment);
            }
        };



        $scope.launchCloudSyncWizard = function launchCloudSyncWizard() {
            cloudSyncGoThereService.launchCloudSyncWizard(nocHttp, $scope.selectedWorkingEnvironment);
        };

        $scope.deleteSync = function () {
            nocDialog.confirm('Delete Sync', 'assets/templates/dialogs/cloud-sync-delete-integration.html', {
                width: 660
            })
                .then(function () {
                    nocCloudSyncService.deleteSync(nocHttp, $scope.selectedWorkingEnvironment.publicId, $state.params.type)
                        .then(function () {
                            $scope.poller();
                            $scope.initializeController($scope.selectedWorkingEnvironment);
                        });
                });
        };


        $scope.getFreeTrialNotificationLevel = function getFreeTrialNotificationLevel() {
            if (!nocCurrentUserService.getCurrentUserCloudSyncLicenseInformation()) return;
            if (!_.isEqual(nocCurrentUserService.getCurrentUserCloudSyncLicenseTypes(), ['trial'])) return;

            //Substracting 1 minute to overcome minor time differences between server and UI:
            var licenseExpirationDate = nocCurrentUserService.getCurrentUserCloudSyncLicenseExpirationDate() - (60 * 1000);
            var daysToExpiry = moment.duration(moment(licenseExpirationDate).diff(moment())).asDays();

            if (daysToExpiry <= $scope.cloudSyncConstants.warningPeriod) return 'error';
            if (daysToExpiry <= $scope.cloudSyncConstants.reminderPeriod) return 'warning';

        };

        $scope.$on('$destroy', function () {
            $scope.poller();
        });

        $scope.updateSelectedWorkingEnvironment($state.params.workingEnvironmentId, ['cloudSyncProperties'])
            .then($scope.initializeController, $scope.backToGallery);
    });
}());
