(function () {
    'use strict';

    angular.module('noc').controller('WorkingEnvironmentInformationController', function ($scope, $state, nocDialog, nocHttp) {
        // remove cluster LIF IPs from the information tab (NOC-10864)

        $scope.initializeController = function initializeController() {
            $scope.access = [];
            if ($scope.selectedWorkingEnvironment.workingEnvironmentType === 'ON_PREM') {
                $scope.clusterResponse = $scope.selectedWorkingEnvironment.ontapClusterProperties;
            } else { //VSA
                $scope.access[0] = privateIpLifs();
                var floatingIps = floatingIpLifs();
                if (floatingIps.length > 0) {
                    $scope.access[1] = floatingIps;
                    seperateNfsCifsFromPrivateIpLifs();
                }
            }
        };

        $scope.hasFloatingIps = function () {
            return $scope.access[1];
        };

        $scope.removeClusterLif = function removeClusterLif(lifs) {
            return _.reject(lifs, {lifType: 'Cluster'});
        };

        var privateIpLifs = function () {
            return _.filter($scope.removeClusterLif($scope.vsaResponse.cluster.lifs), {privateIp: true});
        };

        var seperateNfsCifsFromPrivateIpLifs = function () {
            var nfsLifs = function(lif) {
              return _.includes(lif.key, 'nfs');
            };
            var privateIpNfsCifs = _.filter($scope.access[0], function(lif){
                return nfsLifs(lif);
            });
            if (privateIpNfsCifs.length === 1) {
                $scope.access[0] = _.reject($scope.access[0], function(lif){
                    return nfsLifs(lif);
                });
                $scope.access[2] = privateIpNfsCifs;
            }
        };

        var floatingIpLifs = function () {
            return _.filter($scope.removeClusterLif($scope.vsaResponse.cluster.lifs), {privateIp: false});
        };

        $scope.updateSelectedWorkingEnvironment($state.params.workingEnvironmentId, ['ontapClusterProperties'])
            .then($scope.initializeController, $scope.backToGallery);

        $scope.showEditRouteTablesDialog = function showEditRouteTablesDialog() {
            nocDialog.dialog('Route Tables', 'assets/templates/tabs/working-environments/ha-edit-route-tables-dialog.html', {
                width: 1030,
                selectedWorkingEnvironment: $scope.selectedWorkingEnvironment,
                mainScreenNocHttp: nocHttp
            });
        };
    });
})();
