(() => {
    'use strict';

    angular.module('noc').controller('WorkingEnvironmentsController', ($scope, $window, nocHttp, nocUI, $q, volumesResource, $location,
                                                                       workingEnvironmentResource, Messages, nocDialog,
                                                                       $state, replicationResource, nocUtil, ransomwareProtectionResource, nocCurrentUserService, nocSelectedWorkspaceOrTenantService) => {
        $scope.initialize = () => {
            $scope.workingEnvironments = null;
            $scope.actionsRequired = {};
            $scope.actionsRequiredHighestLevel = {};
            $scope.computingCapacity = false;
            $scope.canDoDataRefresh = true;
            $scope.ransomwareProtection = {
                isSnapshotProtected: undefined,
                isFpolicyProtected: undefined,
                isProtected: undefined
            };
            $window.AOS.init({duration: 1000}); // Initialize the library for volume cards animation (moved here from monitoring controller due to NOC-18083)
        };

        $scope.handleWorkingEnvironmentDrop = function($event, ui, workingEnvironment){
            var sourceId = ui.draggable.scope().workingEnvironment.publicId;
            var targetId = workingEnvironment.publicId;
            if (sourceId === targetId) return;
            $scope.checkAndSetState('snap-mirror-wizard', {from: "main.working-environments.index", slTenantId: nocSelectedWorkspaceOrTenantService.getPublicId(), sourceId: sourceId, targetId: targetId});
        };

        $scope.handleWorkingEnvironmentDragStart = function($event){
            $event.stopImmediatePropagation();
        };

        $scope.setComputingCapacity = function(state) {
            $scope.computingCapacity = state;
        };

        $scope.clearActionsRequired = function() {
            $scope.actionsRequired = {};
        };

        $scope.updateActionsLevel = function (publicId, actionRequiredResponse) {
            var action = _.find(actionRequiredResponse[publicId], {severity: "error"}) || _.find(actionRequiredResponse[publicId], {severity: "warning"});
            $scope.actionsRequiredHighestLevel[publicId] = action ? action.severity : undefined;
        };

        $scope.getActionsRequiredHighestLevel = function () {
            var values = _.values($scope.actionsRequiredHighestLevel) || [];
            return values ? (_.includes(values, "error") ? "error" : (_.includes(values, "warning") ? "warning" : undefined)) : undefined;
        };

        $scope.setActionRequiredFor = function (publicId, isActive, actionRequiredResponse) {
            $scope.updateActionsLevel(publicId, actionRequiredResponse);
            if ($scope.actionsRequiredHighestLevel[publicId] === "error") {
                $scope.actionsRequired[publicId] = actionRequiredResponse[publicId];
            } else {
                $scope.actionsRequired[publicId] = actionRequiredResponse[publicId] && actionRequiredResponse[publicId].length && !isActive ? actionRequiredResponse[publicId] : undefined;
            }

            var workingEnvironment = _.find($scope.workingEnvironments, {publicId: publicId});
            if(workingEnvironment) $scope.setNotificationsOnVSA(workingEnvironment);
        };

        $scope.setNotificationsOnVSA = function(workingEnvironment){
            var actionsRequired = $scope.actionsRequiredHighestLevel[workingEnvironment.publicId];
            //This number is used for sorting the tabular view
            workingEnvironment._notifications = (function(){
                if (actionsRequired === "error") return 3;
                if (actionsRequired === "warning") return 2;
                if (!_.isEmpty(_.get(workingEnvironment, 'ontapClusterProperties.upgradeVersions'))) return 1;
                return 0;
            })();
        };

        $scope.createReplicationMapByType = function (workingEnvironments) {
            //Creates a map of how many working environments are available for replication by type, for example {VSA: 3, ON_PREM: 1}
            $scope.replicationMapByType = _(workingEnvironments).filter("_allowedOperations.replicate").groupBy("workingEnvironmentType").mapValues("length").value();
        };

        $scope.setWorkingEnvironments = function (workingEnvironments) {
            $scope.workingEnvironments = workingEnvironments;
            $scope.createReplicationMapByType(workingEnvironments);
            _.forEach($scope.workingEnvironments, function (workingEnvironment) {
                if (workingEnvironment.workingEnvironmentType === "VSA") $scope.setNotificationsOnVSA(workingEnvironment);
            });
        };

        $scope.refreshWorkingEnvironmentData = function refreshWorkingEnvironmentData(workingEnvironment) {
            $scope.canDoDataRefresh = false;

            $q.all([
                workingEnvironmentResource.refreshVsaData(nocHttp, workingEnvironment),
                volumesResource.refreshVolumesData(nocHttp, workingEnvironment)
            ])
                .finally(function () {
                    $scope.canDoDataRefresh = true;
                });
        };

        $scope.moveToTimeline = function(workingEnvironment) {
            $state.go("main.timeline", {we: workingEnvironment.name, weId: workingEnvironment.publicId, fromWe: $location.url()});
        };

        function deleteFunction(workingEnvironment, gotoMonitoring, forceDelete) {
            return workingEnvironmentResource.deleteWorkingEnvironment(nocHttp, workingEnvironment, forceDelete)
                .then(function () {
                    if (gotoMonitoring) {
                        $scope.showMonitoring(workingEnvironment);
                    }
                });
        }

        $scope.deleteWorkingEnvironment = function deleteWorkingEnvironment(workingEnvironment, gotoMonitoring) {
            const unblock = nocUI.block(); //NOC-11589 AWS - Delete VSA Icon should not be active after one click
            //It is not allowed to delete WE with replication relationships, hence getting replication status.

            return replicationResource.getReplicationStatus(nocHttp, null, workingEnvironment)
                .then((replicationStatusResponse) => {
                    unblock();

                    return nocDialog.confirm('',
                        'assets/templates/tabs/working-environments/delete-working-environment-dialog-content.html', {
                            workingEnvironment,
                            hasReplications: (replicationStatusResponse && replicationStatusResponse.length),
                            isMain: _.get(workingEnvironment, 'k8sProperties.mainClusterIds') && workingEnvironment.k8sProperties.mainClusterIds.length > 0,
                            width: 650
                        })
                        .then((dialogScope) => {
                            return deleteFunction(workingEnvironment, gotoMonitoring, dialogScope.forceDeleteCheckbox);
                        });
                })
                .finally(unblock);
        };

        $scope.removeWorkingEnvironment = function (workingEnvironment, gotoMonitoring) {
            return nocDialog.confirm(Messages.removeConfirm({name: workingEnvironment.name}),
                "assets/templates/dialogs/remove-confirm-content.html",
                {workingEnvironmentType: workingEnvironment.workingEnvironmentType})
                .then(function () {
                    return deleteFunction(workingEnvironment, gotoMonitoring);
                });
        };


        $scope.backToGallery = function () {
            $scope.checkAndSetState("main.working-environments.index");
        };

        $scope.startCreateNewVolumeOnAggregateWizard = function startCreateNewVolumeOnAggregateWizard(workingEnvironment, aggregate) {
            if (!workingEnvironment._allowedOperations['aggregate-add-capacity']) return;

            $scope.checkAndSetState('create-new-volume-wizard', {
                workingEnvironmentId: workingEnvironment.publicId,
                from: 'main.working-environments.monitoring.index',
                gadget: 'advancedAllocation',
                type: workingEnvironment._workingEnvironmentTypeByLocation,
                provider: workingEnvironment._cloudProviderName,
                aggregateName: aggregate.name,
                providerVolumeType: aggregate._providerVolumeType,
                capacityTier: aggregate.capacityTier,
                iops: aggregate.providerVolumes[0].iops
            });
        };

        $scope.startCreateNewVolumeWizard = function startCreateNewVolumeWizard(workingEnvironment, from) {
            if (!workingEnvironment._allowedOperations["add-capacity"]) return;

            $scope.checkAndSetState('create-new-volume-wizard', {
                workingEnvironmentId: workingEnvironment.publicId,
                from: from,
                type: workingEnvironment._workingEnvironmentTypeByLocation,
                provider: workingEnvironment._cloudProviderName
            });
        };

        $scope.gotoCreateWorkingEnvironment = function () {
            $scope.checkAndSetState('wizard', {from: "main.working-environments.index"});
        };

        $scope.showMonitoring = function (workingEnvironment) {
            $scope.goToGadget('', '', workingEnvironment);
        };

        $scope.goToActionRequired = function(workingEnvironment) {
            $scope.goToGadget('actionRequired', null, workingEnvironment);
        };

        $scope.goToGadget = function(gadget, volume, workingEnvironment) {
            volume = volume || {};
            workingEnvironment = workingEnvironment || {
                    publicId: $state.params.workingEnvironmentId,
                    _workingEnvironmentTypeByLocation: $state.params.type,
                    _cloudProviderName: $state.params.provider
                };
            var preventReload = false;

            //Route only when needed
            if ($state.current.name === 'main.working-environments.monitoring.index' &&
                $state.params.workingEnvironmentId === workingEnvironment.publicId &&
                $state.params.type === workingEnvironment._workingEnvironmentTypeByLocation &&
                ($state.params.provider || '') === (workingEnvironment._cloudProviderName || '')) {
                preventReload = true;
            }
            $scope.checkAndSetState("main.working-environments.monitoring.index", {
                workingEnvironmentId: workingEnvironment.publicId,
                gadget: gadget,
                type: workingEnvironment._workingEnvironmentTypeByLocation,
                provider: workingEnvironment._cloudProviderName,
                selectedVolumeName: volume.name,
                selectedVolumeSVM: volume.svmName
            }, preventReload);
        };

        $scope.goToMonitoringTab = function goToMonitoringTab(workingEnvironment, tab) {
            $scope.checkAndSetState('main.working-environments.monitoring.' + tab, {
                workingEnvironmentId: workingEnvironment.publicId,
                type: workingEnvironment._workingEnvironmentTypeByLocation,
                provider: workingEnvironment._cloudProviderName,
                gadget: ''
            });
        };

        $scope.$on(volumesResource.VOLUMES_GET_EVENT, (event, payload) => {
            $scope.updateIsRansomwareProtected(payload.volumes, null);
        });

        $scope.$on(workingEnvironmentResource.VSA_GET_EVENT, (event, payload) => {
            $scope.updateIsRansomwareProtected(null, payload.workingEnvironment);
        });

        $scope.updateIsRansomwareProtected = (volumes, we) => {
            if (volumes) {
                const countVolumesNoSnapshot = ransomwareProtectionResource.countVolumesNoSnapshot(ransomwareProtectionResource.getEligibleVolumes(volumes));
                $scope.ransomwareProtection.isSnapshotProtected = countVolumesNoSnapshot === 0;
            }
            if (we) {
                $scope.ransomwareProtection.isFpolicyProtected = _.get(we.fpolicyProperties, 'fPolicyProtocolStatus.cifs') !== 0 && _.get(we.fpolicyProperties, 'status') !== 0; // Cifs activated or N/A & nfs activated
            }
            $scope.ransomwareProtection.isProtected = $scope.ransomwareProtection.isSnapshotProtected && $scope.ransomwareProtection.isFpolicyProtected;
        };

        $scope.goToRansomwareProtection = (workingEnvironment) => {
            $scope.goToGadget('ransomwareProtection', null, workingEnvironment);
        };

        $scope.isAllowed = (action) => {
            return nocCurrentUserService.isAllowed(action, nocSelectedWorkspaceOrTenantService.getSelectedWorkspace());
        };

        $scope.gotoUpgradeMediator = (workingEnvironment) => {
            workingEnvironmentResource.checkIfCifsLockExists(nocHttp, workingEnvironment)
                .then((cifsLockExists) => {
                    const first = cifsLockExists ? 'cifs' : 'eula';
                    $scope.checkAndSetState('mediator-upgrade', {
                        from: $state.current.name,
                        workingEnvironmentId: workingEnvironment.publicId,
                        type: workingEnvironment._workingEnvironmentTypeByLocation,
                        provider: workingEnvironment._cloudProviderName,
                        first
                    });
                });
        };

        $scope.gotoUpgradeWorkingEnvironment = (workingEnvironment, gotoSupportRegistrationFunction) => {
            const unblock = nocUI.block();
            $q.all([
                workingEnvironmentResource.eligibilityTests(nocHttp, workingEnvironment),
                replicationResource.getReplicationStatus(nocHttp, null, workingEnvironment),
                workingEnvironment.isHA ? workingEnvironmentResource.checkIfCifsLockExists(nocHttp, workingEnvironment) : ''
            ]).then(_.spread(function (eligibilityResponse, replicationStatuses, cifsLockExists) {
                unblock(); // Unblock appears in .then() & .catch() instead of .finally() so that dialogs in the .then() won't be blocked
                let first = cifsLockExists ? 'cifs' : 'version';

                const gotoWizard = () => {
                    $scope.checkAndSetState('cloud-ontap-upgrade', {
                        from: $state.current.name,
                        workingEnvironmentId: workingEnvironment.publicId,
                        type: workingEnvironment._workingEnvironmentTypeByLocation,
                        provider: workingEnvironment._cloudProviderName,
                        first
                    });
                };

                switch (eligibilityResponse.eligibilityResponse) {
                    case 'VALID':
                        if (replicationStatuses.length > 0) {
                            nocDialog.confirm('Cloud Volumes ONTAP has existing SnapMirror relationships', 'assets/templates/dialogs/eligibility/replication-warning.html')
                                .then(gotoWizard);
                        } else {
                            gotoWizard();
                        }
                        break;
                    case 'LICENSE_EXPIRED':
                        nocDialog.alert('The Cloud Volumes ONTAP\'s license has expired. It is not possible to upgrade.');
                        break;
                    case 'NOT_REGISTERED':
                        gotoSupportRegistrationFunction = gotoSupportRegistrationFunction || function () {
                            $scope.goToGadget('supportRegistration', null, workingEnvironment);
                        };
                        nocDialog.dialog('Cloud Volumes ONTAP not registered', 'assets/templates/dialogs/eligibility/not-registered.html', {gotoSupportRegistrationFunction: gotoSupportRegistrationFunction});
                        break;
                    case 'NSS_NOT_EXISTS':
                    case 'NSS_NOT_VALID':
                        first = 'nss';
                        gotoWizard();
                        break;
                    case 'IPA_PROBLEM':
                        nocDialog.alert('Unable to contact IPA. Cannot proceed with upgrade process.');
                        break;
                    default:
                        throw new Error('Unknown registration status');
                }
            }))
                .catch(unblock);
        };

        $scope.isOntapUpgradeAvailable = function(workingEnvironment) {
            return workingEnvironment.ontapClusterProperties && workingEnvironment.ontapClusterProperties.upgradeVersions &&
                workingEnvironment.ontapClusterProperties.upgradeVersions.length > 0 &&
                workingEnvironment._allowedOperations['change-ontap-image'];
        };

        $scope.isMediatorUpgradeAvailable = function(workingEnvironment) {
            return _.get(workingEnvironment.haProperties, 'mediatorVersionsToUpdate') &&
                workingEnvironment.haProperties.mediatorVersionsToUpdate.length > 0 &&
                workingEnvironment._allowedOperations['upgrade-mediator'] &&
                !$scope.isOntapUpgradeAvailable(workingEnvironment);
        };

        $scope.$on(workingEnvironmentResource.WORKING_ENVIRONMENT_STATUS_UPDATE, function(event, payload) {
            var updatedWE = payload.workingEnvironment;
            var we = _.find($scope.workingEnvironments, {publicId: updatedWE.publicId});
            if (we) _.assignInWith(we, updatedWE, nocUtil.dontOverrideObjectWithNull);
        });

        $scope.startVsa = function startVsa(workingEnvironment) {
            workingEnvironmentResource.startVsa(nocHttp, workingEnvironment)
                .then(function () {
                    workingEnvironmentResource.updateWorkingEnvironmentStatus(workingEnvironment, "starting");
                });
        };

        $scope.shouldShowTurnOnAction = function (workingEnvironment) {
            return workingEnvironmentResource.shouldShowTurnOnAction($scope, workingEnvironment);
        };

        $scope.initialize();
    });
})();
