(function tabsController() {
    'use strict';

    angular.module('noc').controller('TabsController', ($scope, $location, layoutOptions,
                                                        $state, KubernetesResource, nocHttp, nocAuth0Service) => {
        $scope.narrowTabs = ['Settings', 'Support Dashboard'];

        $scope.tabs = layoutOptions.tabs;
        $scope.weTabUnselected = _.startsWith($state.current.name, 'main.working-environments.monitoring');
        if ($scope.tabs.length === 0) {
            throw new Error('Unable to initialize tabs controller with no tabs!');
        }

        $scope.basePath = layoutOptions.basePath;

        $scope.handleStateChange = (event, toState) => {
            $scope.weTabUnselected = _.startsWith(toState.name, 'main.working-environments.monitoring');
            const splitStateName = _.split(toState.name, '.');
            if (splitStateName[0] === $scope.basePath) {
                const exists = $scope.setCurrentTab(splitStateName[1]);
                if (!exists) {
                    if (event) {
                        event.preventDefault();
                    }
                    $scope.onTabClick($scope.tabs[0]);
                }
            }
        };

        $scope.doesCurrentStateContain = (state) => {
            const currentState = $state.current.name;

            return _.includes(currentState, state);
        };

        $scope.tabName = function tabName() {
            if (_.toLower($state.current.name) === 'main.timeline' && $state.params.fromWe) {
                return 'Timeline for Working Environment ' + $state.params.we;
            } else {
                return $scope.currentTab ? $scope.currentTab.title : '';
            }
        };

        $scope.setTabWidthClassName = (tabName) => {
            if (_.includes($scope.narrowTabs, tabName) || $state.current.name === 'main.users.settings') {
                return 'header-1280px';
            } else if ($state.current.name === 'main.users.add') {
                return 'header-900px';
            }
        };

        $scope.showBackIcon = function showBackIcon() {
            return _.toLower($state.current.name) === 'main.timeline' && $state.params.fromWe;
        };

        $scope.boxGoBack = () => {
            $location.url($state.params.fromWe);
        };

        $scope.clearWeTimelineFilter = () => {
            $state.go('main.timeline', {we: undefined, fromWe: undefined});
        };

        $scope.$on('$stateChangeStart', $scope.handleStateChange);

        $scope.setCurrentTab = (tabName) => {
            const newCurrentTab = _.find($scope.tabs, {tabName});

            if (newCurrentTab) {
                $scope.currentTab = newCurrentTab;

                return true;
            }

            return false;
        };

        $scope.onTabClick = (tab) => {
            const state = $state.get(layoutOptions.basePath + '.' + tab.tabName);

             if (tab.tabName === 'k8s-clusters') {
                KubernetesResource.getK8sClusters(nocHttp)
                    .then((clusters) => {
                        if (clusters.length > 0) {
                            $scope.checkAndSetState('main.k8s-clusters.k8s-cluster-settings');
                        } else {
                            const audience = 'https://api.cloud.netapp.com';
                            nocAuth0Service.generateTokenForCustomAudience(audience)
                                .then((token) => {
                                    KubernetesResource.getExternalClusters(nocHttp, token)
                                        .then((externalClusters) => {
                                            if (externalClusters.length > 0) {
                                                $scope.checkAndSetState('main.k8s-clusters.k8s-cluster-settings');
                                            } else {
                                                $scope.checkAndSetState('main.k8s-clusters.index');
                                            }
                                        })
                                        .catch(() => {
                                            $scope.checkAndSetState('main.k8s-clusters.index');
                                        });
                                })
                                .catch(() => {
                                    $scope.checkAndSetState('main.k8s-clusters.index');
                                });
                        }
                    });
            } else {
                $scope.checkAndSetState(layoutOptions.basePath + '.' + tab.tabName + (state.abstract ? '.index' : ''));
            }
        };

        $scope.handleStateChange(null, $state.current);
    });
}());
