(() => {
    'use strict';

    angular.module('noc').controller('TopMenuController', ($rootScope, $scope, $state, nocHttp, CloudProviderAccountResource, nocCurrentUserService, nocSelectedWorkspaceOrTenantService, configService, nocTenantsService, nocWorkspacesService) => {
        $scope.isMainMenuEnabled = nocCurrentUserService.isAllowed('configuration:set');
        $scope.isAccountWidgetEnabled = nocCurrentUserService.isAllowed('account:widget');
        $scope.currentUser = nocCurrentUserService.getCurrentUser();
        $scope.workspaceMenuClosed = true;
        $scope.isAgent = nocSelectedWorkspaceOrTenantService.getIsAgentMode();

        $scope.selected = {};
        $scope.userNotifications = {};

        $scope.initialize = () => {
            $scope.userNotifications.value = $scope.currentUser.notificationsOptIn;
            $scope.selected.selectedTenantWS = $scope.selected.selectedTenantWS || nocSelectedWorkspaceOrTenantService.getSelectedTenant() || nocSelectedWorkspaceOrTenantService.getSelectedWorkspace();

            if (!$scope.isAgent) {
                nocTenantsService.reloadTenants();
                $scope.list = nocTenantsService.getTenants();
            } else {
                nocWorkspacesService.reloadWorkspaces();
                $scope.list = nocWorkspacesService.getWorkspaces();
            }

            configService.getConfig(nocHttp, ['asupCompany'])
                .then((configuration) => {
                    $scope.company = configuration.asupCompany;
                });
        };

        $scope.isAllowed = (action) => {
            return nocCurrentUserService.isAllowed(action, nocSelectedWorkspaceOrTenantService.getSelectedWorkspace());
        };

        $scope.saveUserNotifications = (value) => {
            if ($scope.isAgent) {
                nocHttp.put({
                    url: '/occm/tenancy-service/users/update-current-user',
                    data: {cloudManagerNotificationsOptIn: value}
                }, 'sending user notifcations', nocHttp.BLOCK)
                    .then(() => {
                        $scope.userNotifications.value = value;
                    });

            } else {
                $scope.currentUser.notificationsOptIn = value;
                nocHttp.put({
                    url: '/users/' + $scope.currentUser.publicId,
                    data: $scope.currentUser
                }, 'updating user notifcations', nocHttp.BLOCK)
                    .then(() => {
                        $scope.userNotifications.value = value;
                    });
            }
        };

        if ($scope.isAgent) {
            window.accountManagementWidget.sendMessage({
                type: 'SET-TENANCY-BASE-URL',
                payload: {tenancyBaseUrl: _.get($rootScope.supportServices, 'tenancyServiceInformation.url')}
            });
            window.accountManagementWidget.sendMessage({
                type: 'SET-AGENTS-BASE-URL',
                payload: {agentsBaseUrl: _.get($rootScope.supportServices, 'tenancyServiceInformation.agentsMgmtUrl')}
            });
            window.accountManagementWidget.sendMessage({
                type: 'SET-ACCOUNT-ID',
                payload: {accountId: _.get($rootScope.supportServices, 'tenancyServiceInformation.accountId')}
            });

            $scope.openAccountsWidget = () => {
                window.accountManagementWidget.sendMessage({type: 'OPEN-WIDGET'});
            };

            window.accountManagementWidget.accountDataChanged = (payload) => {
                if (_.includes(payload.action, 'workspace')) {

                    nocWorkspacesService.reloadWorkspaces()
                        .then((workspaces) => {
                            $scope.list = workspaces;
                            $scope.selected.selectedTenantWS = nocSelectedWorkspaceOrTenantService.resolveTenantOrWorkspaceFromCurrentUserOrUrl(nocSelectedWorkspaceOrTenantService.getPublicId(), true);
                        });
                }
            };
        }

        $scope.goToCloudProviderAccounts = () => {
            CloudProviderAccountResource.getFormattedAccounts(nocHttp)
                .then((accounts) => {
                    if (accounts.length === 0) {
                        $state.go('cloud-provider-accounts.add');
                    } else {
                        $state.go('cloud-provider-accounts.account-settings');
                    }
                });
        };

        $scope.$watch('selected.selectedTenantWS', (newVal) => {
            $scope.selected.selectedTenantWS = newVal;
        });

        $scope.initialize();
    });
})();
