(() => {
    'use strict';

    angular.module('noc').controller('VirginWizardController', ($scope, $state, layoutOptions, exitOptions, initOptions, nocDialog,
                                                                Messages, nocNavigateAway, nocUI, pidgeon, $rootScope, $timeout) => {
        _.assignIn($scope, layoutOptions);
        _.assignIn($scope, exitOptions);
        _.assignIn($scope, initOptions);
        $scope.wizardController = $scope;

        $scope.wizardController.backupData = {};
        $scope.wizardController.path = {};
        $scope.backupStepData = function(stepName, data){
            $scope.wizardController.backupData[stepName] = data;
        };

        $scope.clearAllStepData = function(){
            $scope.wizardController.backupData = {};
        };

        $scope.getStepData = function(stepName) {
            return $scope.wizardController.backupData[stepName];
        };

        $scope.clearStepData = function (stepName) {
            //Has to be in timeout to make sure this happens AFTER the step destroys itself.
            //Otherwise the step overrides this clearing by saving it's data.
            $timeout(function(){
                $scope.wizardController.backupData[stepName] = {};
            });
        };

        $scope.feedbacks = {};

        $scope.staticFeedback = function(text) {
            $scope.feedbacks["_static"] = text;
        };

        $scope.feedback = function(text) {
            $scope.feedbacks[this.virginStuff.stateName] = text;
        };

        $scope.clearFeedback = function() {
            delete $scope.feedbacks[this.virginStuff.stateName];
        };

        $scope.clearStaticFeedback = function() {
            delete $scope.feedbacks["_static"];
        };

        $scope.$on('$stateChangeSuccess', (event, toState) => {
            //Create a map of ("step-name":"path.to.that.step").
            //This is useful for noc-set-wizard-step and for other cases when we need to know how did we get to a step.
            $scope.wizardController.path[$state.current.data.name] = $state.current.name;
            _.forEach($scope.feedbacks, (value, key) => {
                if (key === '_static') return true;
                if (!_.includes(toState.name, key) || toState.name === key) delete $scope.feedbacks[key];
            });
        });

        $scope.setPathToPreviousOrRoot = () => {
            $scope.wizardController.wizardCompletedSucssesfully = true;
            if ($state.params.from) {
                $scope.checkAndSetState($state.params.from);
            } else {
                $scope.checkAndSetState($rootScope.appView === 'volume-view' ? 'volume-view.volumes' : 'main');
            }
        };

        $scope.isValid = function(){
            return $scope.VirginWizardForm.$valid;
        };

        $scope.$on("$stateChangeSuccess", function(){
            $scope.currentStateName = $state.current.data.name;
            $scope.currentStepPath = $state.current.name;
        });

        $scope.isInitialStep = function(){
            return _.split($state.current.name, '.').length <= ($scope.hasInitStep ? 3 : 2);
        };

        $scope.back = () => {
            if (!$scope.isInitialStep()) {
                $state.transitionTo('^', {}, {
                    location: 'replace',
                    inherit: true,
                    relative: $state.$current,
                    notify: true
                });
            }
        };

        $scope.backFunctionForDetailsCredentialsStepOrRegion = () => {
            $state.go('wizard.init.choose');
        };

        $scope.canceled = false;

        $scope.canLeave = function (proceedFunction) {
            var markAsCanceledAndLeave = function() {
                $scope.canceled = true;
                proceedFunction();
            };

            //Do not warn for losing changes upon: a. Successfully finishing wizard; b. leaving on the first step.
            if ($scope.wizardController.wizardCompletedSucssesfully || $scope.isInitialStep()) {
                markAsCanceledAndLeave();
            } else if (!$scope.canceled) {

                /*NOC-5794 WE Wizards - When exit wizards after the first step, warning is displayed for the first field
                * it is not enought to ignore the form validity when opening the dialog. it can be too late. so ignoring here as well just in case. will be cleared when the dialog is actually opened.*/
                nocUI.ignoreFormValidity();
                nocDialog
                    .confirm(Messages.defaultConfirm)
                    .then(function(){
                        $timeout(markAsCanceledAndLeave, 250);  //250ms is the time it take the animation of dialog:confirm leave wizard to fade. This solves NOC-5361.
                    });
            }
        };

        $scope.setWizardStep = function(step){
            $state.go(step);
        };

        $scope.next = function (nextStep) {
            if ($scope.isValid()) {
                pidgeon.clear();
                nocUI.clearFormValidity(); //This to prevent fields in future steps to appear invalid although never touched.
                $state.go(nextStep);
            } else {
                nocUI.handleErrorsInWizard(true); //Parameter 'true' makes the screen scroll to the first error.
            }
        };

        if ($rootScope.appView !== 'volume-view') {
            $scope.$on(nocNavigateAway.eventName, function (event, payload) {
                nocNavigateAway.handleNavigateAway(payload, $scope.canLeave);
            });
        }

        $scope.scrollToElementByClass = function(className) {
            var elementToScrollTo = $("." + className).first(),
                topFillHeight = elementToScrollTo.parents(".layout-content-holder").children("[noc-match-header-height]").height(),
                theView = elementToScrollTo.closest("[ui-view]");
            $timeout(function() {
                $("html, body").animate({ scrollTop: elementToScrollTo.offset().top - topFillHeight - parseInt(theView.css('margin-top')) - 200}, 300, "linear");
            }, 600); //must be a timeout bigger than the one in super duper scrolling up to the upper view on destroy!!!
        };

        if (initOptions.firstStep) {
            $state.go(initOptions.firstStep);
        } else {
            $scope.exit();
        }
    });
})();
