(() => {
    'use strict';

    angular.module('noc').controller('VirginAddFirstAccountController', ($rootScope, $scope, nocHttp, $state, nocUI,
                                                                         CloudProviderAccountResource, nocDialog) => {

        $scope.initialize = () => {
            $scope.isVirginAddFirstAccountController = true;
            $scope.isAWS = $scope.wizardController.workingEnvironmentTypeAndProvider.providerName === 'AWS';

            if ($scope.isAWS) {
                $scope.awsAccountRequest = {};
                $scope.accountType = {};
                const instancePlacement = $rootScope.instancePlacement;
                $scope.hasInstanceProfile = !!(instancePlacement && instancePlacement.role && instancePlacement.providerRoleType === 'AWS');
                $scope.accountType.name = 'keys';
            } else {
                $scope.azureAccountRequest = {};
            }
        };

        $scope.openLearnMoreDialog = () => {
            if ($scope.isAWS) {
                nocDialog.confirm('', 'assets/templates/cloud-provider-accounts/aws-learn-more.html', {
                    width: 890
                });
            } else {
                nocDialog.confirm('', 'assets/templates/cloud-provider-accounts/azure-learn-more.html', {
                    width: 890
                });
            }
        };

        function nextStepToGoTo() {
            if ($scope.wizardController.choice === 'Recover' || $scope.wizardController.choice === 'RecoverAzure') {
                $state.go('wizard.init.choose.no-accounts.region');
            } else {
                $state.go('wizard.init.choose.no-accounts.create-detailsCredentials');
            }
        }

        $scope.next = () => {
            if ($scope.isValid()) {
                if ($scope.isAWS) {
                    if ($scope.accountType.name === 'assume-role') {
                        $scope.awsAccountRequest.providerKeys.awsAccessKeys = null;
                    } else {
                        $scope.awsAccountRequest.providerKeys.assumeRoleArn = null;
                    }
                    CloudProviderAccountResource.createAccount(nocHttp, 'aws', $scope.awsAccountRequest.accountName, $scope.awsAccountRequest.providerKeys)
                        .then((account) => {
                            $scope.wizardController.providerAccounts.awsAccounts = [account];
                            nextStepToGoTo();
                        });
                } else {
                    CloudProviderAccountResource.createAccount(nocHttp, 'azure', $scope.azureAccountRequest.accountName, $scope.azureAccountRequest.providerKeys)
                        .then((account) => {
                            $scope.wizardController.providerAccounts.azureAccounts = [account];
                            nextStepToGoTo();
                        });
                }
            } else {
                nocUI.handleFormValidity($scope.VirginWizardForm);
            }
        };

        $scope.initialize();
    });
})();
