(() => {
    'use strict';

    angular.module('noc').controller('AnteriorSetupWizardController', ($scope, $state, $log, nocUI, nocHttp, nocDialog, nocLocalStorage,
                                                                       $timeout, $rootScope, $q, AuthResource, ServerUpgradeService) => {
        const instancePlacement = $rootScope.instancePlacement;

        $scope.hasInstanceProfile = !!(instancePlacement && instancePlacement.role);
        $scope.instanceProfileName = (instancePlacement && instancePlacement.role);

        $scope.wizardStep = "welcome";
        $scope.proxyShowAuthenticationData = false;
        $scope.myFormHolder = {};
        $scope.setupRequest = {
            proxyUrl: {},
            autoUpgrade: false,
            userRequest: {
                roleId: "Role-1"
            }
        };


        $scope.expandProxyAuthenticationData = function() {
            $scope.proxyShowAuthenticationData = !$scope.proxyShowAuthenticationData;
        };

        $scope.startOccmSetup = function(){
            $state.go("setup-wizard.welcome");
        };

        $scope.nextFromWelcome = function () {
            $scope.showCheckingConnectivityLayout = true;
            $q.all([
                nocHttp.post('/occm/setup/validate/connectivity'),
                //Timeout to ensure the showCheckingConnectivityLayout is displayed for minimum 1.5 seconds.
                $timeout(_.noop, 1.5 * 1000)
            ])
                .then(function () {
                    $scope.wizardStep = "sign-up";
                })
                .catch(function () {
                    $scope.wizardStep = "proxy";
                })
                .finally(function () {
                    $scope.showCheckingConnectivityLayout = false;
                });
        };

        $scope.validateProxyAndProceed = function () {
            if ($scope.myFormHolder.anteriorSetupWizardProxyForm.$valid) {
                nocHttp.post({
                    url: '/occm/setup/validate/proxy',
                    data: $scope.setupRequest.proxyUrl
                }, "validating Proxy", nocHttp.LOADING)
                    .then(function () {
                        nocUI.clearFormValidity(); //To prevent sign-up step from being invalid
                        $scope.wizardStep = "sign-up";
                    });
            } else {
                nocUI.handleFormValidity();
            }
        };

        $scope.cancel = function cancel() {
            nocUI.clearFormValidity();
            $scope.wizardStep = "welcome";
        };

        $scope.openEulaDialog = function () {
            nocDialog.dialog("End User License Agreement", "assets/templates/wizards/anterior-setup/eula-dialog.html", {width: 650});
        };

        $scope.isValid = function () {
            return $scope.anteriorSetupWizardSignUpStepForm.$valid;
        };

        function doLogin() {
            AuthResource.login($scope.setupRequest.userRequest.email, $scope.setupRequest.userRequest.password)
                .then(() => {
                    nocUI.reloadSilently(true);
                });
        }

        $scope.signUp = function signUp() {
            if ($scope.myFormHolder.anteriorSetupWizardSignUpStepForm.$valid) {
                nocHttp.post({
                    url: '/occm/setup/volume-view-init',
                    data: $scope.setupRequest
                }, 'creating user', nocHttp.LOADING)
                    .then((setupResponse) => {
                        nocLocalStorage.VOLUME_VIEW.save(true);
                        if (setupResponse.upgradeToVersion) {
                            $log.info('upgrade during setup performed');
                            const unblockDuringSetup = nocUI.block();
                            ServerUpgradeService.pollForServerUpgrade(doLogin)
                                .finally(unblockDuringSetup);
                        } else {
                            $log.info('going to login');
                            doLogin();
                        }
                    });
            } else {
                nocUI.handleFormValidity();
            }
        };
    });
})();
