(() => {
    'use strict';

    angular.module('noc').controller('CloudSyncActivationS3BucketStepController', ($scope, BaseVirginStep, nocCloudSyncService,
                                                                                   nocHttp, nocDialog, $q, $timeout) => {
        const dummy = [{dirName: 'dummy sub dir'}];

        $scope.initialize = function initialize() {
            BaseVirginStep.call($scope);
            $scope.currentPath = [];
            $scope.search = {};
            $scope.subdirectoriesStack = [dummy];

            let getBucketsPromise;
            if ($scope.wizardController.type === 'VSA') {
                getBucketsPromise = nocCloudSyncService.getBuckets(nocHttp, $scope.wizardController.workingEnvironmentId);
            } else {
                getBucketsPromise = nocCloudSyncService.getBucketsByUserId(nocHttp, $scope.wizardController.userId);
            }

            getBucketsPromise
                .then((buckets) => {
                    $scope.buckets = buckets;
                });

        };

        $scope.locationClicked = function locationClicked(location) {
            if ($scope.wizardController.s3Location === location) {
                $scope.wizardController.s3Location = $scope.currentPath.join('/');
            } else {
                $scope.wizardController.s3Location = location;
            }
        };

        /*istanbul ignore next*/
        $scope.showTagsDialog = function showTagsDialog(bucket) {
            const tags = bucket.tags;
            const list = _.sortBy(_.map(tags, (value, key) => {
                return {text: value, key: key, trimmedText: true, trimmedKey: true};
            }), 'key');

            const lists = {leftList: list};
            lists.keysHeader = 'Tag Key';
            lists.valuesHeader = 'Tag Value';
            nocDialog.dialog(bucket.bucketName + ' tags:', 'assets/templates/dialogs/general-info-dialog.html',
                {width: 800, lists: lists, limitedHeight: true});
        };

        $scope.getLocationSubdirectories = function getLocationSubdirectories(location) {
            const fetchSubDirPromise = $scope.wizardController.type === 'VSA' ?
                nocCloudSyncService.getBucketSubdirectories(nocHttp, location.bucketName, $scope.wizardController.workingEnvironmentId, location.fullPath) :
                nocCloudSyncService.getBucketSubdirectoriesByUserId(nocHttp, location.bucketName, $scope.wizardController.userId, location.fullPath);
            const getSubdirectoriesPromise = location.subdirectories ? $q.when(location.subdirectories) :
                fetchSubDirPromise
                    .then((response) => {
                        _.forEach(response, (dir) => {
                            const splittedPath = _.split(dir.fullPath, '/');

                            dir.dirName = splittedPath[splittedPath.length - 1];
                        });
                        location.subdirectories = response;

                        return location.subdirectories;
                    });

            getSubdirectoriesPromise
                .then((subdirectories) => {
                    $timeout(() => {
                        $scope.wizardController.s3Location = location.bucketName + (location.fullPath ? ('/' + location.fullPath) : '');
                        $scope.currentPath.push(location.dirName || location.bucketName);
                        $scope.subdirectoriesStack.pop(); //popdummy
                        $scope.subdirectoriesStack.push(subdirectories);
                        $timeout(() => {
                            $scope.subdirectoriesStack.push(dummy);
                        });
                        $scope.bucketsUpTheStream = true;
                    });
                });
        };

        $scope.goBackOneLevel = () => {
            $scope.currentPath.pop();
            $scope.subdirectoriesStack.pop(); //pop dummy

            return $timeout(() => {
                $scope.subdirectoriesStack.pop();
                $scope.subdirectoriesStack.push(dummy);
                if ($scope.subdirectoriesStack.length === 1) {
                    $scope.bucketsUpTheStream = false;
                }
            });
        };

        //Recursion !
        //   \(•_•)    (•_•)/
        //    (  (>   <(  )
        //    /  \     /  \
        $scope.goBackToDir = (dirName) => {
            if (!$scope.currentPath || $scope.currentPath[$scope.currentPath.length - 1] === dirName) return;

            return $scope.goBackOneLevel()
                .then(() => {
                    return $scope.goBackToDir(dirName);
                })
                .then(() => {
                    $scope.wizardController.s3Location = $scope.currentPath.join('/');
                });
        };

        $scope.next = function next() {
            if (!$scope.wizardController.s3Location) {
                $scope.userClickedNext = true;
            } else {
                $scope.wizardController.next('.dataBroker');
            }
        };

        $scope.initialize();

        $scope.$on('$destroy', () => {
            $scope.wizardController.backupStepData('s3Bucket', {
                currentPath: $scope.currentPath,
                subdirectoriesStack: $scope.subdirectoriesStack,
                bucketsUpTheStream: $scope.bucketsUpTheStream
            });
        });

        _.assignIn($scope, $scope.wizardController.getStepData('s3Bucket'));
    });
})();
