(() => {
    'use strict';

    angular.module('noc').controller('CreateNewAggregateDiskSizeAndIopsStepController', ($scope, aggregatesResource,
                                                                                         nocApprovalMessageBuilder, nocDialog, Constants,
                                                                                         nocHttp, nocUI, nocUtil, pidgeon, configService) => {
        $scope.initializeController = function initializeController() {
            $scope.allowDiskSizeSelection = true;
            $scope.allowNumOfDisksSelection = true;
            $scope.moreInfoParameters = {
                cloudProviderName: $scope.wizardController.selectedWorkingEnvironment._cloudProviderName,
                selectedTierDiskName: $scope.wizardController.selectedTier.diskName
            };

            const isKvm = _.includes($scope.selectedWorkingEnvironment._instanceType, '5');
            const maxDisksProp = isKvm ? 'externalConfig.disksConfiguration.aws.maxDisksSingleNodeKvm' : 'externalConfig.disksConfiguration.aws.maxDisksSingleNode';
            const maxDisksHaProp = isKvm ? 'externalConfig.disksConfiguration.aws.maxDisksHaKvm' : 'externalConfig.disksConfiguration.aws.maxDisksHa';


            const maxDisksSingleNode = _.get($scope, maxDisksProp);
            const maxDisksHa = _.get($scope, maxDisksHaProp);
            const numReservedDisksHa = _.get($scope, 'externalConfig.disksConfiguration.aws.numReservedDisksHa');
            const numReservedDisksSingleNode = _.get($scope, 'externalConfig.disksConfiguration.aws.numReservedDisksSingleNode');
            const maxDisksAllowed = ($scope.selectedWorkingEnvironment.isHA ? maxDisksHa : maxDisksSingleNode) - ($scope.selectedWorkingEnvironment.isHA ? numReservedDisksHa : numReservedDisksSingleNode);
            $scope.maxDisksMessage = 'In AWS, ' + ($scope.selectedWorkingEnvironment.isHA ? 'each Cloud Volumes ONTAP node in an HA pair ' : 'a single-node Cloud Volumes ONTAP system ') +
                `has a limit of ${maxDisksAllowed} data disks.`;
            const licenseTypeName = _.get($scope.selectedWorkingEnvironment, '_licenseType');
            $scope.showMaxDisksMessage = $scope.moreInfoParameters.cloudProviderName === 'AWS' && (licenseTypeName === 'Cloud Volumes ONTAP BYOL' || licenseTypeName === 'Cloud Volumes ONTAP Premium' || licenseTypeName === 'Cloud Volumes ONTAP Standard');

            configService.getConfig(nocHttp, ['ebsSizeToPiopsRatio'])
                .then((configResponse) => {
                    $scope.ebsSizeToPiopsRatio = configResponse.ebsSizeToPiopsRatio;
                    $scope.onDiskSizeChange();
                });
        };

        $scope.onDiskSizeChange = function onDiskSizeChange() {
            $scope.wizardController.providerVolumeSizeGB = $scope.wizardController.providerVolumeSize.unit === 'GB' ? $scope.wizardController.providerVolumeSize.size : $scope.wizardController.providerVolumeSize.size * 1024;
            $scope.maxIopsValue = _.min([Constants.awsIopsLimitPerVolume, $scope.wizardController.providerVolumeSizeGB * $scope.ebsSizeToPiopsRatio]);
            $scope.numberOfDisksOptions = _.range(1, $scope.wizardController.providerVolumeSize.maxDisksAllow.numOfDisks + 1);
            $scope.wizardController.addAggregateRequest.numberOfDisks = $scope.numberOfDisksOptions[0];
        };


        $scope.createAggregate = function createAggregate() {
            if (!$scope.isValid()) {
                nocUI.handleErrorsInWizard(true);
            } else {
                $scope.wizardController.addAggregateRequest.providerVolumeType = $scope.wizardController.selectedTier.diskName;
                const providerToObjectStorage = {
                    AWS: 'S3',
                    Azure: 'Blob',
                    GCP: 'cloudStorage'
                };
                $scope.wizardController.addAggregateRequest.capacityTier = $scope.wizardController.tieringEnabled ? providerToObjectStorage[$scope.wizardController.provider] : undefined;
                $scope.wizardController.addAggregateRequest.diskSize = $scope.wizardController.providerVolumeSize;
                $scope.wizardController.addAggregateRequest.iops = $scope.wizardController.selectedTier.diskName === 'io1' ? $scope.wizardController.iops : null;
                nocDialog.confirm('', 'assets/templates/dialogs/purchase-confirm-content.html', {
                    html: nocApprovalMessageBuilder.buildTitle({
                        provider: $scope.wizardController.selectedWorkingEnvironment._cloudProviderName,
                        numOfDisks: $scope.wizardController.addAggregateRequest.numberOfDisks,
                        diskSize: nocUtil.normalizeCapacity($scope.wizardController.addAggregateRequest.diskSize),
                        diskType: _.find($scope.possibleTiers, {name: $scope.wizardController.selectedTier.diskName}).display,
                        isHA: $scope.wizardController.selectedWorkingEnvironment.isHA
                    })
                })
                    .then(() => {
                        aggregatesResource.createAggregate(nocHttp, $scope.wizardController.selectedWorkingEnvironment, $scope.wizardController.addAggregateRequest)
                            .then(() => {
                                $scope.wizardController.wizardCompletedSucssesfully = true;
                                $scope.wizardController.exit();
                                pidgeon.inProgress('Aggregate creation in progress.', true);
                            });
                    });
            }
        };

        $scope.initializeController();
    });
})();
