(() => {
    'use strict';

    angular.module('noc').controller('CreateNewAggregateTieringStepController', ($filter, $timeout, $scope,
                                                                                 nocAdvancedAllocationService) => {
        $scope.initializeController = function initializeController() {
            $scope.createNewAggregateWizard = true;
            //Initialize possibleTiers: remove types with no supported sizes:
            $scope.possibleTiers = nocAdvancedAllocationService.getPossibleTiers($scope.wizardController.selectedWorkingEnvironment);

            const defaultDiskType = $scope.wizardController.selectedWorkingEnvironment.capacityFeatures.defaultProviderVolumeType.diskType;

            if ($scope.wizardController.selectedTier) {
                _.assignIn($scope.wizardController, $scope.wizardController.getStepData('tiering'));
            } else {
                $scope.tierSelected($scope.possibleTiers[defaultDiskType]);
                $scope.wizardController.forceCompositeVersion = $scope.selectedWorkingEnvironment.capacityFeatures.supportedCapacityTiers.forceCompositeVersion;
                $scope.wizardController.compositeSupported = $scope.selectedWorkingEnvironment.capacityFeatures.supportedCapacityTiers.compositeSupported;

                if ($scope.selectedWorkingEnvironment.capacityFeatures.supportedCapacityTiers.capacityTiersDisableReasons) {
                    $scope.wizardController.capacityTiersDisableReasons = $scope.selectedWorkingEnvironment.capacityFeatures.supportedCapacityTiers.capacityTiersDisableReasons;
                }

                $scope.wizardController.tieringEnabled = $scope.selectCapacityTierRadioDefault();
            }

            $scope.moreInfoParameters = {
                provider: $scope.wizardController.provider
            };

            $timeout(() => {
                //Focus on the selected box:
                const element = $('.' + $scope.wizardController.selectedTier.name);
                if (element) element.focus();
            });
        };

        $scope.selectCapacityTierRadioDefault = () => {
            if (!$scope.wizardController.compositeSupported || $scope.wizardController.encryptionType === 'ONTAP' ||
                !_.some(['S3', 'Blob', 'cloudStorage'], (objectStorageType) => _.includes(_.get($scope.wizardController.selectedTier, 'supportedCapacityTiers'), objectStorageType)) ||
                ($scope.wizardController.capacityTiersDisableReasons && $scope.wizardController.capacityTiersDisableReasons.length > 0)) return false;

            return $scope.wizardController.forceCompositeVersion;
        };

        $scope.disabledTooltipForTier = function disabledTooltipForTier(tier) {
            if ($scope.wizardController.capacityTiersDisableReasons && $scope.wizardController.capacityTiersDisableReasons.length > 0) {
                return $scope.wizardController.capacityTiersDisableReasons[0];
            }
            if (tier.disabledMessage) {
                return tier.disabledMessage;
            }
            if (tier && !_.some(['S3', 'Blob', 'cloudStorage'], (objectStorageType) => _.includes(_.get(tier, 'supportedCapacityTiers'), objectStorageType))) {
                return $scope.selectedWorkingEnvironment._objectStorageDisplayName + ' tiering is not available for this disk type.';
            }
        };

        $scope.tierSelected = function tierSelected(tier) {
            if ($scope.wizardController.selectedTier === tier) return;

            $scope.wizardController.setWizardStep($scope.wizardController.path.tiering);
            $scope.wizardController.selectedTier = tier;
            if (!_.some(['S3', 'Blob', 'cloudStorage'], (objectStorageType) => _.includes(_.get($scope.wizardController.selectedTier.supportedCapacityTiers, 'supportedCapacityTiers'), objectStorageType))) {
                $scope.wizardController.tieringEnabled = false;
            }
            const supportedSizes = $scope.wizardController.selectedTier.supportedSizes;
            const defaultProviderVolumeSize = $scope.wizardController.selectedWorkingEnvironment.capacityFeatures.defaultProviderVolumeType.size;
            const defaultDiskSize = _.find(supportedSizes, {_identifier: $filter('normalizedCapacity')(defaultProviderVolumeSize, 2, true)});
            $scope.wizardController.providerVolumeSize = defaultDiskSize || supportedSizes[0];
        };

        $scope.initializeController();
    });
})();
