(() => {
    'use strict';

    angular.module('noc').controller('CreateNewVolumeDetailsStepController', ($scope, $rootScope, nocUI, nocUtil, nocHttp,
                                                                              pidgeon, volumesResource, BaseCreateVolume, $state,
                                                                              BaseCreateVolumeOnExistingWorkingEnvironment, $q,
                                                                              nocAdvancedAllocationService, workingEnvironmentResource,
                                                                              $log) => {
        $scope.initializeController = function initializeController() {
            nocUI.clearFormValidity();
            BaseCreateVolume.call($scope);
            BaseCreateVolumeOnExistingWorkingEnvironment.call($scope, pidgeon, nocHttp);
            $scope.selectDefaultExportPolicy($scope.selectedWorkingEnvironment.workingEnvironmentType === 'VSA' ? 'custom' : 'none');
            $scope.wizardController.volumeRequest = $scope.createVolumeRequest();

            if ($scope.selectedWorkingEnvironment.workingEnvironmentType === 'VSA') {
                const defaultDiskType = $scope.selectedWorkingEnvironment.capacityFeatures.defaultProviderVolumeType.diskType;

                $scope.wizardController.selectedTier = $scope.selectedWorkingEnvironment._providerVolumeTypes[defaultDiskType];
                $scope.wizardController.tieringEnabled = $scope.selectedWorkingEnvironment.capacityFeatures.supportedCapacityTiers.compositeSupported && $scope.selectedWorkingEnvironment.capacityFeatures.supportedCapacityTiers.capacityTiersDisableReasons.length === 0;
                $scope.wizardController.selectedDiskTypeDisplay = $scope.selectedWorkingEnvironment._providerVolumeTypes[$scope.wizardController.selectedTier.diskName].display;
            }

            //Advanced mode: create volume on specific aggregate
            if ($state.params.aggregateName) {
                $scope.wizardController.selectedTier = $scope.selectedWorkingEnvironment._providerVolumeTypes[$state.params.providerVolumeType];
                $scope.wizardController.tieringEnabled = $state.params.capacityTier;
                $scope.wizardController.iops = _.toNumber($state.params.iops);
                $scope.disabledUsageProfile = $scope.wizardController.tieringEnabled ? 'No Storage Efficiency' : '';

                $scope.wizardController.title += ' on ' + $state.params.aggregateName;
                _.assignIn($scope.volumeRequest, {aggregateName: $state.params.aggregateName});
            } else {//Simple mode
                $scope.wizardController.title += ' in ' + $scope.wizardController.selectedWorkingEnvironment.name;
            }

            $scope.stateParamSyncToS3Status = $scope.syncToS3StatusForAddNewVolume($scope.wizardController.selectedWorkingEnvironment);
            $scope.singleStepVolumeWizard = $scope.wizardController.selectedWorkingEnvironment.workingEnvironmentType === 'ON_PREM';

            $scope.initSyncToS3Status();
            $scope.policyTypeChanged();
            $q.all([
                $scope.fetchSnapshotPolicy($scope.wizardController.selectedWorkingEnvironment, $scope.wizardController.selectedWorkingEnvironment._pathPrefix, nocHttp),
                $scope.wizardController.selectedWorkingEnvironment.workingEnvironmentType === 'ON_PREM' ? $scope.initializeAdvancedAllocation(nocHttp, pidgeon) : $q.when()
            ])
                .then(() => {
                    $scope.moreInfoParameters = {
                        schedules: _.find($scope.snapshotPolicies, {name: $scope.wizardController.volumeRequest.snapshotPolicyName}).schedules
                    };
                    $scope.initialized = true;
                });
        };

        $scope.initializeAdvancedAllocation = function initializeAdvancedAllocation(nocHttp, pidgeon) {
            return $q.all([
                nocAdvancedAllocationService.getAggregates(nocHttp, pidgeon, $scope.wizardController.selectedWorkingEnvironment)
                    .then(function (aggregates) {
                        $scope.aggregates = aggregates;
                        $scope.wizardController.volumeRequest.aggregateName = $scope.aggregates[0].name;
                    }),

                nocAdvancedAllocationService.getSVMs(nocHttp, $scope.wizardController.selectedWorkingEnvironment, $scope)
                    .then(function () {
                        if ($scope.destinationSvms.length === 1) {
                            $scope.wizardController.volumeRequest.svmName = $scope.destinationSvms[0].name;
                        }
                    })
            ]);
        };

        $scope.syncToS3StatusForAddNewVolume = function syncToS3StatusForAddNewVolume(workingEnvironment) {
            return _.get(workingEnvironment, 'cloudSyncProperties.isSynced') ?
                (_.get(workingEnvironment, 'cloudSyncProperties.status.status') === 'COMPLETE' ? 'ENABLED' : 'DISABLED') :
                'NOT_AVAILABLE';
        };

        $scope.getSelectedWorkingEnvironment = function getSelectedWorkingEnvironment(workingEnvironmentId) {
            var pathPrefix = nocUtil.workingEnvironmentTypeToRequestPathPrefix($state.params.type, $state.params.provider);
            var workingEnvironmentsUnloading = nocUI.loading();
            var providerProperties = $state.params.provider === 'AWS' ? 'awsProperties' : 'providerProperties';
            var fields = 'svmName,capacityFeatures,cloudSyncProperties,ontapClusterProperties,' + providerProperties;

            return workingEnvironmentResource.getWorkingEnvironmentByPublicId(nocHttp, workingEnvironmentId, pathPrefix,
                fields)
                .then(function (selectedWe) {
                    $scope.wizardController.selectedWorkingEnvironment = selectedWe;
                })
                .finally(workingEnvironmentsUnloading);
        };

        $scope.getSelectedWorkingEnvironment($state.params.workingEnvironmentId)
            .then($scope.initializeController)
            .catch($log.error);

        var actionName = 'creating volume';

        $scope.createNewVolume = function createNewVolume(quoteResponse) {
            const aggregateSuffix = ($scope.selectedWorkingEnvironment.workingEnvironmentType === 'VSA' && !$state.params.aggregateName) ? '?createAggregateIfNotFound=true' : '';

            return nocHttp.post({
                url: $scope.selectedWorkingEnvironment._pathPrefix + 'volumes' + aggregateSuffix,
                data: $scope.generateRequestBody(quoteResponse, $scope.volumeProtocol.type)
            }, actionName, nocHttp.BLOCK)
                .then(() => {
                    $scope.wizardController.wizardCompletedSucssesfully = true;
                    $scope.wizardController.exit();
                });
        };

        $scope.getObjectStorage = () => {
            const providerToObjectStorage = {
                AWS: 'S3',
                Azure: 'Blob',
                GCP: 'cloudStorage'
            };

            return providerToObjectStorage[$scope.selectedWorkingEnvironment._cloudProviderName];
        };

        $scope.getCapacityTier = () => {
            //In case of thick volume or unsupported disk dont send capacityTier.
            if (!$scope.wizardController.volumeRequest.usageProfile.flags.enableThinProvisioning ||
                (!_.some(['S3', 'Blob', 'cloudStorage'], (objectStorageType) => _.includes($scope.wizardController.selectedTier.supportedCapacityTiers, objectStorageType)))) return undefined;

            //In case of disabled tiered aggregate, dont send capacityTier. If aggregate name exists we are in advanced mode
            if ($scope.wizardController.volumeRequest.aggregateName && !$scope.wizardController.tieringEnabled) return undefined;

            //In advanced mode if license is explore but aggregate is composite, send capacityTier but don't send tiering policy
            if ($scope.wizardController.volumeRequest.aggregateName && $scope.wizardController.tieringEnabled && $scope.selectedWorkingEnvironment._licenseType === 'Cloud Volumes ONTAP Explore') {
                $scope.wizardController.tieringPolicy = {};

                return $scope.getObjectStorage();
            }

            //Send capacityTier and dont send tieringPolicy case
            if ($scope.wizardController.tieringEnabled &&
                $scope.wizardController.forceCompositeVersion &&
                ($scope.wizardController.capacityTiersDisableReasons && $scope.wizardController.capacityTiersDisableReasons.length > 0 && $scope.wizardController.encryptionType !== 'ONTAP')) {
                //Send capacityTier but dont send tieringPolicy
                $scope.wizardController.tieringPolicy = {};

                return $scope.getObjectStorage();
            }

            //Check whether to send both or not at all
            return $scope.wizardController.tieringEnabled && $scope.wizardController.compositeSupported ? $scope.getObjectStorage() : undefined;
        };

        $scope.createVolumeOnVsa = function createVolumeOnVsa() {
            $scope.wizardController.volumeRequest.iops = $scope.wizardController.selectedTier.diskName === 'io1' ? $scope.wizardController.iops : null;
            $scope.wizardController.volumeRequest.providerVolumeType = $scope.wizardController.selectedTier.diskName;
            $scope.wizardController.volumeRequest.capacityTier = $scope.getCapacityTier();
            $scope.wizardController.volumeRequest.tieringPolicy = $scope.wizardController.volumeRequest.capacityTier ? _.get($scope.wizardController, 'tieringPolicy.type') : undefined;
            volumesResource.askForQuote(nocHttp,
                $scope.selectedWorkingEnvironment._pathPrefix,
                $scope.generateRequestBody(null, $scope.volumeProtocol.type),
                actionName)
                .then((quoteResponse) => {
                    return volumesResource.getUserApprovalForNewVolumeIfNeeded(quoteResponse,
                        $scope.selectedWorkingEnvironment._cloudProviderName,
                        $scope.wizardController.selectedDiskTypeDisplay,
                        $scope.selectedWorkingEnvironment.isHA,
                        $scope.wizardController.volumeRequest.name);
                })
                .then($scope.createNewVolume);
        };

        $scope.startCreateVolumeProcess = function startCreateVolumeProcess() {
            if ($scope.selectedWorkingEnvironment.workingEnvironmentType === 'VSA') {
                $scope.createVolumeOnVsa();
            } else { // OnPrem
                $scope.createNewVolume();
            }
        };

        $scope.next = function next() {
            if (!$scope.isValid()) {
                nocUI.handleErrorsInWizard(true);
            } else {
                if ($scope.singleStepVolumeWizard && !$scope.needCifsConfiguration) {
                    $scope.startCreateVolumeProcess();
                } else if ($scope.needCifsConfiguration) {
                    $scope.wizardController.next('.cifs');
                } else {
                    $scope.wizardController.next('.usageProfile');
                }
            }
        };

        $scope.$watch('wizardController.volumeRequest.snapshotPolicyName', () => {
            if (_.get($scope.wizardController, 'volumeRequest.snapshotPolicyName')) {
                $scope.moreInfoParameters = {
                    schedules: _.find($scope.snapshotPolicies, {name: $scope.wizardController.volumeRequest.snapshotPolicyName}).schedules
                };
            }
        });
    });
})();
