(() => {
    'use strict';

    angular.module('noc').controller('CreateNewVolumeUsageProfileStepController', ($scope, nocUI, $state) => {
        const providerTypes = _.get($scope.selectedWorkingEnvironment, '_providerVolumeTypes');
        $scope.possibleTiers =
            _(providerTypes).filter((volumeType) => {
                return _.find(volumeType.supportedSizes, (size) => {
                    return size.maxDisksAllow.numOfDisks > 0;
                });
            }).values().value();

        if ($scope.selectedWorkingEnvironment) {
            $scope.availableTieringPoliciesForTier = _.find($scope.selectedWorkingEnvironment.capacityFeatures.supportedCapacityTiers.supportedCapacityTiersPerVolumeType,
                {volumeType: $scope.wizardController.selectedTier.name})
                .availableTieringPolicies;
            $scope.wizardController.forceCompositeVersion = $scope.selectedWorkingEnvironment.capacityFeatures.supportedCapacityTiers.forceCompositeVersion;
            $scope.wizardController.compositeSupported = $scope.selectedWorkingEnvironment.capacityFeatures.supportedCapacityTiers.compositeSupported;
            if ($scope.selectedWorkingEnvironment.capacityFeatures.supportedCapacityTiers.capacityTiersDisableReasons) {
                $scope.wizardController.capacityTiersDisableReasons = $scope.selectedWorkingEnvironment.capacityFeatures.supportedCapacityTiers.capacityTiersDisableReasons;
            }
        }

        $scope.getObjectStorageName = () => {
            const providerToObjectStorage = {
                AWS: 'S3',
                Azure: 'Blob',
                GCP: 'Cloud Storage'
            };

            return providerToObjectStorage[$scope.selectedWorkingEnvironment._cloudProviderName];
        };

        $scope.usageProfileChanged = function usageProfileChanged(profile) {
            if (profile.display === 'No Storage Efficiency') {
                $scope.wizardController.tieringEnabled = false;
                $scope.tierSelected($scope.selectedWorkingEnvironment._providerVolumeTypes[$scope.wizardController.selectedTier.diskName]);
            }
        };

        $scope.disabledTooltipForTier = function disabledTooltipForTier(tier) {
            if ($scope.wizardController.capacityTiersDisableReasons && $scope.wizardController.capacityTiersDisableReasons.length > 0) {
                return $scope.wizardController.capacityTiersDisableReasons[0];
            }
            if (tier.disabledMessage) {
                return tier.disabledMessage;
            }
            if (!$scope.wizardController.volumeRequest.usageProfile.flags.enableThinProvisioning) {
                return 'If you want to use storage tiers, you must enable the Storage Efficiency usage profile.';
            }
            if (!_.some(['S3', 'Blob', 'cloudStorage'], (objectStorageType) => _.includes(_.get(tier, 'supportedCapacityTiers'), objectStorageType))) {
                tier.disabledMessage = $scope.getObjectStorageName() + ' tiering is not available for this disk type.';

                return tier.disabledMessage;
            }
        };

        $scope.tierSelected = function tierSelected(tier) {
            $scope.wizardController.selectedTier = tier;
            $scope.availableTieringPoliciesForTier = _.find($scope.selectedWorkingEnvironment.capacityFeatures.supportedCapacityTiers.supportedCapacityTiersPerVolumeType, {volumeType: tier.name}).availableTieringPolicies;

            if ($scope.wizardController.tieringPolicy && $scope.availableTieringPoliciesForTier.length === 0) {
                $scope.wizardController.tieringPolicy = {};
            } else if (!_.includes($scope.availableTieringPoliciesForTier, _.get($scope.wizardController, 'tieringPolicy.type'))) {
                $scope.wizardController.tieringPolicy = {
                    type: _.includes($scope.availableTieringPoliciesForTier, 'auto') ? 'auto' : (_.includes($scope.availableTieringPoliciesForTier, 'snapshot_only') ? 'snapshot_only' : 'none')
                };
            }
            $scope.wizardController.setWizardStep($scope.wizardController.path.usageProfile);
            $scope.wizardController.selectedDiskTypeDisplay = $scope.selectedWorkingEnvironment._providerVolumeTypes[$scope.wizardController.selectedTier.diskName].display;
        };

        $scope.isAdvancedMode = () => {
            return $state.params.aggregateName;
        };

        $scope.saveAddVolume = function saveAddVolume() {
            if (!$scope.isValid()) {
                nocUI.handleErrorsInWizard(true);
            } else {
                $scope.createVolumeOnVsa();
            }
        };
    });
})();
