(function () {
    "use strict";

    angular.module("noc").controller("RemoveWeSelectWEController", function ($scope, BaseVirginStep, uiGridFilterService,
                                                                             nocTenantsService, workingEnvironmentResource,
                                                                             $timeout, nocHttp, $q, uiGridConstants, nocSelectedWorkspaceOrTenantService) {
        BaseVirginStep.call($scope);
        $scope.search = {term: ''};

        $q.all([
            workingEnvironmentResource.getWorkingEnvironments(nocHttp, null, "status"),
            nocTenantsService.getTenantsFromServer(nocHttp, {workingEnvironmentsInformation: true})
        ])
            .then(_.spread(function (workingEnvironments, tenants) {
                _.forEach(workingEnvironments, function (we) {
                    workingEnvironmentResource.initWorkingEnvironment(we);
                    we._tenantName = _.get(_.find(tenants, {publicId: we.tenantId}), 'name');
                });

                $scope.workingEnvironments = workingEnvironments;
                $scope.gridOptions.data = workingEnvironments;
            }));

        $scope.gridOptions = {
            useExternalFiltering: true,
            noUnselect:true,
            enableRowSelection: true,
            enableVerticalScrollbar: uiGridConstants.scrollbars.NEVER,
            rowTemplate: " <div title=\"{{row.enableSelection ? '' : 'Only Cloud Volumes ONTAP working environments can be removed.'}}\" ng-repeat=\"(colRenderIndex, col) in colContainer.renderedColumns track by col.uid\" class=\"ui-grid-cell\" ng-class=\"{ 'ui-grid-row-header-cell': col.isRowHeader, 'disabled-row': !row.enableSelection, 'selectable-row': row.enableSelection}\" ng-click=\"grid.appScope.onClick(row)\" ui-grid-cell></div>",
            onRegisterApi: function(api) {
                uiGridFilterService.installFilter(api,$scope);
            },
            isRowSelectable: function(row){
                return row.entity.workingEnvironmentType === "VSA";
            },
            appScopeProvider: {
                onClick: function (row) {
                    if (row.isSelected) {
                        if ($scope.wizardController.currentStateName !== "select-we") {
                            $scope.wizardController.setWizardStep("remove-we-wizard.select-we");
                        }
                        $timeout(function () {
                            $scope.selectedWorkingEnvironment = row.entity;
                            $scope.wizardController.next('.review');
                        });
                    }
                }
            },
            columnDefs: [
                {field: 'name', visible: true},
                {field: '_typeLongName', displayName: 'Working Environment Type', visible: true},
                {field: '_tenantName', displayName: 'Tenant', visible: !nocSelectedWorkspaceOrTenantService.getIsAgentMode()},
                {field: '_displayStatus', displayName: 'Status', cellClass: 'capitalize', visible: true}
            ]
        };
    });

    angular.module('noc').controller('RemoveWeReviewController', ($scope, BaseVirginStep, nocHttp, pidgeon) => {
        BaseVirginStep.call($scope);

        $scope.removeWELocally = (we) => {
            const url = we._pathPrefix + 'working-environments/' + we.publicId + '?localDelete=true';
            nocHttp.delete(url,
                'removing working environment \'' + we.name + '\' from Cloud Manager',
                nocHttp.LOADING)
                .then(() => {
                    pidgeon.success('Working environment \'' + we.name + '\' has been removed from Cloud Manager.');
                    $scope.setPathToPreviousOrRoot();
                });
        };
    });
}());
