(() => {
    'use strict';
    angular.module('noc').controller('SetupWizardWelcomeStepController', ($scope, $state, nocHttp, nocUI) => {
        $scope.eulaAgree = false;
        $scope.setupRequest = {
            userRequest: {
                roleId: 'Role-1',
                notificationsOptIn: true
            },
            tenantRequest: {name: '', description: '', costCenter: ''},
            site: '',
            company: '',
            proxyUrl: {
                uri: '',
                proxyUserName: '',
                proxyPassword: '',
                proxyDomain: ''
            }
        };

        $scope.restorePrevious = () => {
            $state.go('.restore-previous');
        };

        $scope.installNew = () => {
            nocHttp.post('/occm/setup/validate/connectivity', null, nocHttp.BLOCK)
                .then(() => {
                    $state.go('.company');
                })
                .catch(() => {
                    $state.go('.proxy');
                });
        };

        $scope.cancel = () => {
            nocUI.clearFormValidity();
            $state.go('anterior-setup-wizard');
        };
    });

    angular.module('noc').controller('SetupWizardRestorePreviousController', ($scope, BaseVirginStep, nocUI, nocHttp, nocDialog,
                                                                              AuthResource, $log, ServerUpgradeService) => {
        BaseVirginStep.call($scope);

        $scope.wizardController.restoreRequest = $scope.wizardController.restoreRequest || {};

        $scope.showEula = function showEula() {
            nocDialog.dialog('End User License Agreement', 'assets/templates/wizards/anterior-setup/eula-dialog.html',
                {width: 650}
            );
        };

        function doLogin() {
            AuthResource.login($scope.restoreRequest.email, $scope.restoreRequest.password)
                .then(() => {
                    nocUI.reloadSilently(true);
                });
        }

        $scope.restore = function restore() {
            if ($scope.isValid($scope)) {
                nocHttp.post({
                    url: '/occm/setup/restore',
                    data: $scope.wizardController.restoreRequest
                }, 'restoring working environment', nocHttp.BLOCK)
                    .then((setupResponse) => {
                        if (setupResponse.upgradeToVersion) {
                            $log.info('upgrade during setup performed');
                            const unblockDuringSetup = nocUI.block();
                            ServerUpgradeService.pollForServerUpgrade(doLogin)
                                .finally(unblockDuringSetup);
                        } else {
                            $log.info('going to login');
                            doLogin();
                        }
                    });
            } else {
                nocUI.handleErrorsInWizard();
            }
        };
    });

    angular.module('noc').controller('SetupWizardProxyController', ($scope, nocHttp, nocUI, $state, BaseVirginStep, OCCMResource) => {
        BaseVirginStep.call($scope);
        $scope.proxyShowAuthenticationData = false;

        $scope.expandProxyAuthenticationData = () => {
            $scope.proxyShowAuthenticationData = !$scope.proxyShowAuthenticationData;
        };

        $scope.next = () => {
            if ($scope.isValid($scope)) {
                if ($scope.setupRequest.proxyUrl.uri) {
                    nocHttp.post({
                        url: '/occm/setup/validate/proxy',
                        data: $scope.setupRequest.proxyUrl
                    }, 'validating proxy', nocHttp.BLOCK)
                        .then(() => {
                            nocUI.clearFormValidity();
                            $state.go('.company');
                        });
                } else {
                    OCCMResource.validateConnectivity(nocHttp)
                        .then(() => {
                            nocUI.clearFormValidity();
                            $state.go('.company');
                        });
                }
            } else {
                nocUI.handleErrorsInWizard();
            }
        };
    });

    angular.module('noc').controller('SetupWizardSimpleController', ($scope, $rootScope, BaseVirginStep) => {
        BaseVirginStep.call($scope);

        $scope.nextFromTenant = function nextFromTenant() {
            $scope.wizardController.next($rootScope.runningInIC ? '.review' : '.nss');
        };
    });

    angular.module('noc').controller('SetupWizardNssController', ($scope, BaseVirginStep, nocHttp, $state, nocUI) => {
        BaseVirginStep.call($scope);

        const httpHeaders = {
            'Setup-Proxy': $scope.setupRequest.proxyUrl.uri,
            'Setup-Proxy-User': $scope.setupRequest.proxyUrl.proxyUserName,
            'Setup-Proxy-Password': $scope.setupRequest.proxyUrl.proxyPassword,
            'Setup-Company': $scope.setupRequest.company,
            'Setup-Site': $scope.setupRequest.site
        };

        $scope.next = () => {
            if ($scope.isValid($scope)) {
                $scope.setupRequest.tenantRequest.nssKeys = _.pick($scope.wizardController, 'nssUserName', 'nssPassword');
                nocHttp.post({
                    url: '/occm/setup/validate/tenant',
                    data: $scope.setupRequest.tenantRequest,
                    headers: httpHeaders
                }, 'validating tenant', nocHttp.BLOCK)
                    .then(() => {
                        $state.go('.review');
                    });
            } else {
                nocUI.handleErrorsInWizard();
            }
        };

    });

    angular.module('noc').controller('SetupWizardReviewController', ($scope, $rootScope, $log, nocUI, pidgeon, nocHttp,
                                                                     $q, BaseVirginStep, AuthResource, nocLocalStorage,
                                                                     nocDialog, ServerUpgradeService) => {
        BaseVirginStep.call($scope);
        const instancePlacementRole = _.get($rootScope.instancePlacement, 'role');

        $scope.providers = () => {
            const providers = [];

            if (instancePlacementRole) return instancePlacementRole + ' (AWS)';

            return providers.join(', ') || 'N/A';
        };

        $scope.values = {autoVsaCapacityManagementViewValue: 'auto', autoUpgrade: true};

        $scope.showEula = () => {
            nocDialog.dialog('End User License Agreement', 'assets/templates/wizards/anterior-setup/eula-dialog.html', {width: 650});
        };

        function doLogin() {
            const unblock = nocUI.block();

            return AuthResource.login($scope.setupRequest.userRequest.email, $scope.setupRequest.userRequest.password)
                .then(() => {
                    $scope.wizardController.wizardCompletedSucssesfully = true;
                    nocUI.reloadSilently(true);
                })
                .catch((errorResponse) => {
                    pidgeon.error(pidgeon.serverErrorPrefix + 'logging in', errorResponse.data, 'finish');
                })
                .finally(unblock);
        }

        $scope.finish = function finish() {
            if ($scope.isValid($scope)) {
                $scope.setupRequest.tenantRequest.nssKeys = _.pick($scope.wizardController, 'nssUserName', 'nssPassword');
                $scope.setupRequest.autoVsaCapacityManagement = $scope.values.autoVsaCapacityManagementViewValue === 'auto';
                $scope.setupRequest.autoUpgrade = true;

                pidgeon.clear('finish');
                const httpHeaders = {
                    'Setup-Proxy': $scope.setupRequest.proxyUrl.uri,
                    'Setup-Proxy-User': $scope.setupRequest.proxyUrl.proxyUserName,
                    'Setup-Proxy-Password': $scope.setupRequest.proxyUrl.proxyPassword
                };
                $q.all([
                    nocHttp.post({
                        url: '/occm/setup/validate/tenant',
                        data: $scope.setupRequest.tenantRequest,
                        headers: httpHeaders
                    }, 'validating tenant', nocHttp.BLOCK),
                    nocHttp.post({
                        url: '/occm/setup/validate/user',
                        data: $scope.setupRequest.userRequest,
                        headers: httpHeaders
                    }, 'validating user', nocHttp.BLOCK),
                    nocHttp.post({
                        url: '/occm/setup/validate/proxy',
                        data: $scope.setupRequest.proxyUrl
                    }, 'validating proxy', nocHttp.BLOCK)
                ])
                    .then(() => {
                        nocHttp.post({
                            url: '/occm/setup/init',
                            data: $scope.setupRequest
                        }, 'setting up Cloud Manager', nocHttp.BLOCK)
                            .then((setupResponse) => {
                                $scope.setupResponse = setupResponse;
                                $log.info('setup response ' + JSON.stringify(setupResponse));
                                nocLocalStorage.VOLUME_VIEW.save(false);
                                if (setupResponse.upgradeToVersion) {
                                    $log.info('upgrade during setup performed');
                                    ServerUpgradeService.pollForServerUpgrade(doLogin);
                                } else {
                                    $log.info('going to login');
                                    doLogin();
                                }
                            });
                    });
            } else {
                nocUI.handleErrorsInWizard(true);
            }
        };
    });
})();
