(() => {
    'use strict';

    angular.module('noc').factory('BaseCreateVpc', ($http, ManifestMapper, $state,
                                                    MetadataResource, $q, $rootScope) => {
        return function BaseCreateVpc(pidgeon, nocUI, nocHttp) {
            const scope = this;

            scope.initialize = () => {
                const backedupStepData = scope.wizardController.getStepData('vpc-base');
                scope.wizardController.networkRequirements = _.get(backedupStepData, 'networkRequirements');
                _.assignIn(scope.wizardController.createVpcSelectObjects, _.get(backedupStepData, 'createVpcSelectObjects'));
                scope.wizardController.selectedRegion = _.get(backedupStepData, 'selectedRegion');
                scope.regions = _.get(backedupStepData, 'regions');

                if (scope.wizardController.vpcBeenHereBefore === scope.wizardController.workingEnvironmentType) {
                    return $q.reject();
                }
                scope.wizardController.vpcBeenHereBefore = scope.wizardController.workingEnvironmentType;
                scope.wizardController.useExistingSecurityGroup = false;
                scope.clearAll();
                if (scope.wizardController.workingEnvironmentType === 'HA') {
                    scope.wizardController.haParameters = {};
                }
                scope.wizardController.instancePlacementDefaultForWizard = _.cloneDeep($rootScope.instancePlacement);

                if (scope.wizardController.instancePlacementDefaultForWizard && !scope.wizardController.instancePlacementDefaultForWizard.occmSubnetId) {
                    scope.wizardController.instancePlacementDefaultForWizard.occmSubnetId = scope.wizardController.instancePlacementDefaultForWizard.subnetId;
                }
                if (scope.wizardController.failedRequest) {
                    _.set(scope.wizardController.instancePlacementDefaultForWizard, 'region', scope.wizardController.failedRequest.region);
                    _.set(scope.wizardController.instancePlacementDefaultForWizard, 'vpcId', scope.wizardController.failedRequest.vpcId);
                    _.set(scope.wizardController.instancePlacementDefaultForWizard, 'subnetId', scope.wizardController.failedRequest.subnetId || scope.wizardController.failedRequest.haParams.node1SubnetId);
                }

                return MetadataResource.getNetworkRequirements(nocHttp, scope.wizardController.pathPrefix,
                    scope.wizardController.workingEnvironmentTypeAndProvider.type, scope.wizardController.failoverMode)
                    .then((networkRequirements) => {
                        scope.wizardController.networkRequirements = networkRequirements;

                        return scope.getRegions();
                    });
            };

            scope.securityGroupTitle = (securityGroup) => {
                return securityGroup.name + (securityGroup.description ? ' (' + securityGroup.description + ')' : '');
            };

            scope.subnetTitle = (subnet, ipsNeeded, showAz) => {
                if (ipsNeeded > subnet.availableIps) {
                    return ('The instance requires ' + ipsNeeded + ' IP address' +
                        (ipsNeeded !== 1 ? 'es' : '') + ', but the subnet ' +
                        (subnet.availableIps === 0 ? 'has none.' : 'only has ' + subnet.availableIps + ' available.'));
                } else {
                    const sname = subnet.subnetName ? subnet.subnetName : '';

                    return sname + (showAz ? ' (' + subnet.availabilityZone + ')' : '');
                }
            };

            scope.clearAllButRegionsAndVpcs = () => {
                const region = _.get(scope.wizardController, 'createVpcSelectObjects.region');
                const access = _.get(scope.wizardController, 'createVpcSelectObjects.access');
                const key = _.get(scope.wizardController, 'createVpcSelectObjects.key');
                scope.wizardController.createVpcSelectObjects = {region, access, key};
                $state.go(scope.wizardController.path[scope.wizardController.workingEnvironmentType === 'HA' && scope.wizardController.failoverMode === 'FloatingIP' ? 'ha-vpc' : 'vpc']);
            };

            scope.clearAll = () => {
                const access = _.get(scope.wizardController, 'createVpcSelectObjects.access');
                scope.wizardController.createVpcSelectObjects = {access};
                scope.wizardController.vpcsWithSubnets = null;
            };

            scope.getKeyPairsForAllRegions = () => {
                return MetadataResource.getKeyPairsForAllRegions(nocHttp, pidgeon, scope.wizardController.workingEnvironmentType !== 'HA',
                    scope.wizardController.cloudProviderAccountId).then((keyData) => {
                    return keyData.regionToKeyPairs;
                });
            };

            scope.getRegions = () => {
                pidgeon.clear('regions');
                //Filters out the regions that no ontap version supports
                const regions = ManifestMapper.getRegions();
                if (regions && regions.length > 0) {
                    scope.regions = regions;

                    return scope.getKeyPairsForAllRegions()
                        .then((keyData) => {
                            //Select default region:
                            scope.keyData = keyData;
                            if (scope.wizardController.instancePlacementDefaultForWizard && _.find(scope.regions, {code: scope.wizardController.instancePlacementDefaultForWizard.region})) {
                                scope.wizardController.createVpcSelectObjects.region = _.find(scope.regions, {code: scope.wizardController.instancePlacementDefaultForWizard.region});
                            } else {
                                //Find the first region which has keyPair
                                const firstRegion = _.find(scope.regions, (region) => {
                                    return scope.keyData[region.code] && scope.keyData[region.code].length > 1;
                                });

                                if (!firstRegion) {
                                    pidgeon.error('Unable to find region with key pair', null, 'regions');
                                } else {
                                    scope.wizardController.createVpcSelectObjects.region = firstRegion;
                                }
                            }

                            return scope.handleRegionChange(scope.wizardController.createVpcSelectObjects.region);
                        });
                } else {
                    pidgeon.error('Unable to retrieve list of regions', null, 'regions');

                    return $q.reject();
                }
            };

            scope.getVPCs = () => {
                const unloadVpcs = nocUI.loading();

                //In case the user changes the region often, and this function is called time after time,
                //we don't want that newer VPCs response would be overridden by old ones. So we cancel the old request
                //and create new one.
                if (scope.lastVPCConfig) scope.lastVPCConfig._.cancelled = true;
                scope.lastVPCConfig = {
                    method: 'GET',
                    url: scope.wizardController.pathPrefix + 'metadata/vpcs' + (scope.wizardController.cloudProviderAccountId ? '?cloudProviderAccountId=' + scope.wizardController.cloudProviderAccountId : ''),
                    params: {region: scope.wizardController.createVpcSelectObjects.region.code},
                    _: {}
                };
                pidgeon.clear('vpcsWithSubnets');

                /*NocHttp does not fit here, because we need to access the config*/
                return $http(scope.lastVPCConfig)
                    .then((response) => {
                        if (response.config._.cancelled) return;

                        const vpcsWithSubnets = response.data;
                        scope.wizardController.vpcsWithSubnets = vpcsWithSubnets;
                        _.forEach(vpcsWithSubnets, (vpc) => {
                            vpc._hasTags = !_.isEmpty(vpc.tags);
                            const nameTag = _.find(vpc.tags, {key: 'Name'});
                            vpc._name = nameTag ? nameTag.value : null;

                            // //If WE is HA push the default security group in the beginning
                            if (scope.wizardController.workingEnvironmentType === 'HA') {
                                vpc.securityGroupsIncludingGenerated = _.cloneDeep(vpc.securityGroups);

                                scope.wizardController.defaultSecurityGroup = {
                                    securityGroupId: 'Use a generated security group',
                                    description: 'Default Cloud Volumes ONTAP Firewall Rules',
                                    name: ''
                                };
                                vpc.securityGroupsIncludingGenerated.unshift(scope.wizardController.defaultSecurityGroup);
                            }
                        });

                        if (scope.wizardController.instancePlacementDefaultForWizard && scope.wizardController.instancePlacementDefaultForWizard.vpcId) {
                            if (_.some(scope.wizardController.vpcsWithSubnets, {vpcId: scope.wizardController.instancePlacementDefaultForWizard.vpcId})) {
                                //Define for OCCM VPC or current VPC
                                scope.wizardController.createVpcSelectObjects.vpc = _.find(scope.wizardController.vpcsWithSubnets, {vpcId: scope.wizardController.instancePlacementDefaultForWizard.vpcId});
                                // || scope.wizardController.createVpcSelectObjects.vpc
                                scope.selectVPC();
                                if (scope.wizardController.workingEnvironmentType === 'HA') {
                                    const subnet = _.find(scope.wizardController.createVpcSelectObjects.vpc.subnets, {subnetId: scope.wizardController.instancePlacementDefaultForWizard.subnetId});
                                    if (!subnet) return;
                                    scope.wizardController.createVpcSelectObjects.node1AvailabilityZone = subnet.availabilityZone;
                                    scope.wizardController.createVpcSelectObjects.subnet = subnet;
                                }
                            }
                            if (!scope.wizardController.createVpcSelectObjects.vpc && scope.wizardController.vpcsWithSubnets.length === 1) {
                                scope.wizardController.createVpcSelectObjects.vpc = scope.wizardController.vpcsWithSubnets[0];
                                scope.selectVPC();
                            }
                        }

                        if (!_.get(scope.wizardController.failedRequest, 'securityGroupId')) {
                            scope.selectSecurityGroups();
                        }
                    })
                    .catch((errorResponse) => {
                        if (errorResponse.config._.cancelled) return;

                        pidgeon.error(pidgeon.serverErrorPrefix + 'retrieving list of VPCs with subnets', errorResponse.data, 'vpcsWithSubnets');

                        return $q.reject();
                    })
                    .finally(unloadVpcs);
            };

            scope.selectSecurityGroups = () => {
                const vpc = scope.wizardController.createVpcSelectObjects.vpc;
                scope.wizardController.createVpcSelectObjects.securityGroup = scope.wizardController.useExistingSecurityGroup ? _.get(vpc, 'securityGroups[0]') :
                    (scope.wizardController.failoverMode === 'FloatingIP' ? vpc.securityGroupsIncludingGenerated[0] : null);
            };

            scope.selectVPC = () => {
                const vpc = scope.wizardController.createVpcSelectObjects.vpc;
                scope.clearAllButRegionsAndVpcs();
                scope.wizardController.createVpcSelectObjects.vpc = vpc;
                //Select the vpc only if has subnets. on selection clear last selected subnet
                if (vpc && vpc.subnets.length > 0) {
                    nocUI.clearFormValidity();
                    delete scope.wizardController.routeTables;
                    //Select by default only if there is 1 subnet, otherwise force the user to select. See NOC-2630
                    scope.wizardController.createVpcSelectObjects.subnet = (vpc.subnets.length === 1 && vpc.subnets[0].availableIps >= scope.wizardController.networkRequirements.node) ? vpc.subnets[0] : undefined;

                    if (scope.wizardController.instancePlacementDefaultForWizard && scope.wizardController.instancePlacementDefaultForWizard.subnetId && !scope.wizardController.selectedSubnet) {
                        scope.wizardController.createVpcSelectObjects.subnet = _.find(scope.wizardController.createVpcSelectObjects.vpc.subnets, {subnetId: scope.wizardController.instancePlacementDefaultForWizard.subnetId});
                        if (scope.wizardController.createVpcSelectObjects.subnet && scope.wizardController.createVpcSelectObjects.subnet.availableIps < scope.wizardController.networkRequirements.node) scope.wizardController.createVpcSelectObjects.subnet = undefined;
                    }

                    if (scope.wizardController.createVpcSelectObjects.vpc) {
                        if (_.get(scope.wizardController.failedRequest, 'securityGroupId')) {
                            if (!_.find(vpc.securityGroups, {securityGroupId: scope.wizardController.failedRequest.securityGroupId})) {
                                scope.wizardController.useExistingSecurityGroup = false;
                            } else {
                                scope.wizardController.useExistingSecurityGroup = true;
                                scope.wizardController.createVpcSelectObjects.securityGroup = _.find(vpc.securityGroups, {securityGroupId: scope.wizardController.failedRequest.securityGroupId});
                            }
                        } else {
                            scope.selectSecurityGroups();
                        }
                        if (scope.wizardController.workingEnvironmentType === 'HA') {
                            if (_.get(scope.wizardController.failedRequest, 'haParams.mediatorSecurityGroupId')) {
                                scope.wizardController.haParameters.haMediatorSecurityGroup = _.find(vpc.securityGroups, {securityGroupId: scope.wizardController.failedRequest.haParams.mediatorSecurityGroupId});
                            } else {
                                scope.wizardController.createVpcSelectObjects.securityGroup = vpc.securityGroupsIncludingGenerated[0];
                                scope.wizardController.haParameters.haMediatorSecurityGroup = vpc.securityGroupsIncludingGenerated[0];
                            }
                        }
                    }
                    if (scope.wizardController.workingEnvironmentType === 'HA' && scope.wizardController.failoverMode === 'FloatingIP') {
                        scope.getAvailabilityZones();
                        scope.clearAvailabilityZones();
                    }
                    if (!scope.wizardController.createVpcSelectObjects.subnet && scope.wizardController.createVpcSelectObjects.vpc.subnets.length === 1) {
                        scope.wizardController.createVpcSelectObjects.subnet = scope.wizardController.createVpcSelectObjects.vpc.subnets[0];
                    }
                } else {
                    scope.wizardController.createVpcSelectObjects.vpc = undefined;
                }
            };

            scope.handleRegionChange = (newRegion) => {
                if (!newRegion) return $q.reject();
                scope.wizardController.createVpcSelectObjects.region = newRegion;
                scope.wizardController.selectedRegion = scope.wizardController.createVpcSelectObjects.region.code;

                scope.clearAllButRegionsAndVpcs();

                //Clearing all licence and version data to force the user to reselect.
                scope.wizardController.clearStepData('license');

                scope.getKeyPairs();

                return scope.getVPCs();
            };

            scope.$on('$destroy', () => {
                scope.wizardController.backupStepData('vpc-base', {
                    createVpcSelectObjects: {
                        region: _.get(scope.wizardController, 'createVpcSelectObjects.region'),
                        vpc: _.get(scope.wizardController, 'createVpcSelectObjects.vpc'),
                        subnet: _.get(scope.wizardController, 'createVpcSelectObjects.subnet'),
                        securityGroup: _.get(scope.wizardController, 'createVpcSelectObjects.securityGroup')
                    },
                    networkRequirements: scope.wizardController.networkRequirements,
                    selectedRegion: scope.wizardController.selectedRegion,
                    regions: scope.regions
                });
                scope.wizardController.networkRequirements = undefined;
                scope.wizardController.selectedRegion = undefined;
            });
        };
    });
})();
