(() => {
    'use strict';

    angular.module('noc').controller('CreateWorkingEnvironmentByolStepController', ($scope, $timeout, ManifestMapper,
                                                                                    nocHttp, $rootScope, nocUI,
                                                                                    CloudProviderAccountResource, nocDialog, pidgeon) => {
        $scope.initialize = function initialize() {
            $scope.licenses = ManifestMapper.getAvailableLicenses($scope.wizardController.selectedRegion, $scope.getLatestOntapVersions());
            $scope.uploadProblem = {};
            $scope.licenseFileName = {};
            $scope.expirationDate = {};
            $scope.licenseNocHttp = nocHttp;
            $scope.azureGermany = $scope.wizardController.workingEnvironmentTypeAndProvider.providerName === 'Azure' && _.includes($scope.wizardController.selectedRegion, 'germany');
            $scope.nssAccountRequest = {};
            $scope.wizardController.useByol = false;

            //Find at least one paygo
            $scope.paygoAvailable = _.find($scope.licenses, (license) => {
                return !_.includes(license.type, 'byol');
            }) && !$scope.azureGermany;
            if (!$scope.paygoAvailable && !$scope.wizardController.failedRequestType) {
                $scope.hasByolLicense();
            }

            CloudProviderAccountResource.getAccounts(nocHttp)
                .then((accounts) => {
                    $scope.wizardController.nssAccounts = accounts.nssAccounts;
                    if (!$scope.wizardController.selectedNSSAccount) {
                        $scope.wizardController.selectedNSSAccount = accounts.nssAccounts[0];
                    }
                });

            if ($scope.wizardController.failedRequestType) {
                if ($scope.wizardController.failedRequest.vsaMetadata.platformSerialNumber || $scope.wizardController.failedRequest.serialNumber) {
                    $scope.wizardController.platformSerialNumber = $scope.wizardController.failedRequest.vsaMetadata.platformSerialNumber || $scope.wizardController.failedRequest.serialNumber;
                    $scope.hasByolLicense();
                } else if ($scope.wizardController.failedRequest.haParams && $scope.wizardController.failedRequest.haParams.platformSerialNumberNode1) {
                    $scope.wizardController.platformSerialNumberNode1 = $scope.wizardController.failedRequest.haParams.platformSerialNumberNode1;
                    $scope.wizardController.platformSerialNumberNode2 = $scope.wizardController.failedRequest.haParams.platformSerialNumberNode2;
                    $scope.hasByolLicense();
                } else {
                    $scope.goingPayGo();
                }
            }
        };

        $scope.getLatestOntapVersions = function getLatestOntapVersions() {
            return ManifestMapper.getAvailableVersions($scope.wizardController.selectedRegion)[0];
        };

        $scope.clearFutureStepsData = function clearFutureStepsData() {
            if ($scope.wizardController.currentStateName !== 'byol') {
                $scope.wizardController.setWizardStep($scope.wizardController.path.byol);
            }
            $scope.wizardController.clearStepData('license');
            $scope.wizardController.clearStepData('packages');
        };

        $scope.hasByolLicense = function hasByolLicense() {
            //When changing selection:
            if (!$scope.wizardController.selectedLicense || !_.includes($scope.wizardController.selectedLicense.type, 'byol')) {
                $scope.clearFutureStepsData();
            }

            $timeout(() => {
                //Has to be in timeout to make sure this happens AFTER future steps destroy themselves.
                //Otherwise this is overridden by steps saving their data.
                $scope.wizardController.selectedLicense = _.find($scope.licenses, (license) => {
                    return _.includes(license.type, 'byol');
                });
                $scope.wizardController.useByol = true;
            });
        };

        $scope.goingPayGo = function goingPayGo() {
            //When changing selection:
            if (_.includes(_.get($scope.wizardController, 'selectedLicense.type'), 'byol')) {
                $scope.clearFutureStepsData();
                //In case user previously clicked 'yes', need to clear byol selection:
                $scope.wizardController.selectedLicense = null;
            }
        };

        $scope.openLearnMoreDialog = () => {
            nocDialog.confirm('', 'assets/templates/cloud-provider-accounts/nss-learn-more.html', {
                width: 890,
                fromTop: '20%'
            });
        };

        $scope.goToAccountSettings = () => {
            $scope.checkAndSetState('cloud-provider-accounts.account-settings');
        };

        const handleIcMode = () => {
            if (_.get($scope.wizardController, 'selectedLicense.name') === 'Cloud Volumes ONTAP BYOL') {
                if ($scope.wizardController.workingEnvironmentType === 'VSA') {
                    if (!$scope.wizardController.providedLicense) {
                        $scope.uploadProblem[1] = true;
                    } else {
                        $scope.wizardController.platformSerialNumber = JSON.parse($scope.wizardController.providedLicense).statusResp.serialNumber;
                        $scope.wizardController.next('.packages');
                    }
                } else if ($scope.wizardController.workingEnvironmentType === 'HA') {
                    if (!$scope.wizardController.providedLicenseNode1) $scope.uploadProblem[1] = true;
                    if (!$scope.wizardController.providedLicenseNode2) $scope.uploadProblem[2] = true;
                    if ($scope.wizardController.providedLicenseNode1 &&
                        $scope.wizardController.providedLicenseNode2 &&
                        ($scope.wizardController.providedLicenseNode1 !== $scope.wizardController.providedLicenseNode2)) {
                        $scope.wizardController.platformSerialNumberNode1 = JSON.parse($scope.wizardController.providedLicenseNode1).statusResp.serialNumber;
                        $scope.wizardController.platformSerialNumberNode2 = JSON.parse($scope.wizardController.providedLicenseNode2).statusResp.serialNumber;
                        $scope.wizardController.next('.packages');
                    }
                }
            } else {
                $scope.wizardController.next('.packages');
            }
        };

        $scope.usingByolAndNotEval = () => {
            const isSingleEval = $scope.wizardController.platformSerialNumber && _.startsWith($scope.wizardController.platformSerialNumber, 'Eval-');
            const isHaEval = $scope.wizardController.platformSerialNumberNode1 && _.startsWith($scope.wizardController.platformSerialNumberNode1, 'Eval-');

            return $scope.wizardController.useByol && !isSingleEval && !isHaEval;
        };

        $scope.next = () => {
            if ($scope.isValid()) {
                if ($rootScope.runningInIC) {
                    handleIcMode();
                } else {
                    if (_.get($scope.nssAccountRequest, 'accountName')) {
                        CloudProviderAccountResource.createAccount(nocHttp, 'nss', $scope.nssAccountRequest.accountName, $scope.nssAccountRequest.providerKeys, [], true)
                            .then((account) => {
                                pidgeon.clear('createNssAccount');
                                $scope.wizardController.nssAccounts = [account];
                                $scope.wizardController.selectedNSSAccount = account;
                                $scope.nssAccountRequest = {};
                                $scope.wizardController.next('.packages');
                            })
                            .catch((error) => {
                                pidgeon.error(error.causeMessage.substring(_.indexOf(error.causeMessage, ':') + 2, error.causeMessage.length), null, 'createNssAccount');
                            });
                    } else {
                        $scope.wizardController.next('.packages');
                    }
                }
            } else {
                nocUI.handleFormValidity($scope.VirginWizardForm);
            }
        };

        $scope.$on('$destroy', () => {
            $scope.wizardController.backupStepData('byol', {
                selectedLicense: $scope.wizardController.selectedLicense,
                platformSerialNumberNode1: $scope.wizardController.platformSerialNumberNode1,
                platformSerialNumberNode2: $scope.wizardController.platformSerialNumberNode2,
                providedLicense: $scope.wizardController.providedLicense,
                providedLicenseNode1: $scope.wizardController.providedLicenseNode1,
                providedLicenseNode2: $scope.wizardController.providedLicenseNode2,
                platformSerialNumber: $scope.wizardController.platformSerialNumber
            });

            $scope.wizardController.selectedLicense = null;
            $scope.wizardController.platformSerialNumberNode1 = null;
            $scope.wizardController.platformSerialNumberNode2 = null;
            $scope.wizardController.providedLicense = null;
            $scope.wizardController.providedLicenseNode1 = null;
            $scope.wizardController.providedLicenseNode2 = null;
            $scope.wizardController.platformSerialNumber = null;
        });

        _.assignIn($scope.wizardController, $scope.wizardController.getStepData('byol'));
        $scope.initialize();
    });
})();
