(() => {
    'use strict';

    angular.module('noc').controller('VirginCreateWorkingEnvironmentEncryptionStepController', ($scope, $q, nocHttp, nocDialog,
                                                                                                nocEncryptionResourcesService,
                                                                                                $timeout) => {
        $scope.initialize = () => {
            $scope.wizardController.encryptionType = $scope.wizardController.encryptionType || ($scope.wizardController.workingEnvironmentTypeAndProvider.providerName === 'Azure' ? 'AZURE' : 'AWS');
            $timeout(() => {
                //Focus on the selected box:
                const element = $('.' + _.toLower($scope.wizardController.encryptionType) + '-encryption');
                if (element) element.focus();
            });

            $scope.disableEditawsEncryptionKeys = true;
            $q.all([
                nocEncryptionResourcesService.getOccmCertificate(nocHttp),
                nocEncryptionResourcesService.getKeyManagers(nocHttp),
                nocEncryptionResourcesService.getKeyManagersCaCertificatesWithOptions(nocHttp)
            ]).then(_.spread(function (certificate, keyManagers, keyManagersCaCertificateWithOptions) {
                $scope.wizardController.certificate = certificate;
                $scope.wizardController.keyManagers = keyManagers;
                $scope.wizardController.keyManagersCaCertificates = keyManagersCaCertificateWithOptions.data;
                $scope.wizardController.keyManagersCaCertificateOptions = keyManagersCaCertificateWithOptions.select2options;
                $scope.cotEncryptionDisabled = $scope.shouldCotEncryptionBeDisabled();
            }));
            if ($scope.wizardController.workingEnvironmentTypeAndProvider.providerName !== 'Azure') {
                nocEncryptionResourcesService.getAwsEncryptionKeys(nocHttp, $scope.wizardController.createVpcSelectObjects.region.code, $scope.wizardController.cloudProviderAccountId)
                    .then((awsEncryptionKeysObject) => {
                        $scope.wizardController.awsEncryptionKeys = awsEncryptionKeysObject.awsEncryptionKeys;
                        _.forEach($scope.wizardController.awsEncryptionKeys, (key) => {
                            nocEncryptionResourcesService.formatAwsEncryptionKeyExpiration(key);
                            key.status = _.startCase(key.status);
                        });
                        $scope.keyWithStatusNotEnabled = _.some($scope.wizardController.awsEncryptionKeys, (key) => {
                            return key.status !== 'Enabled' && !key.generated;
                        });
                        _.remove($scope.wizardController.awsEncryptionKeys, (key) => {
                            return key.status !== '' && key.status !== 'Enabled';
                        });
                        $scope.wizardController.useKeysFromAnotherAccount = $scope.wizardController.useKeysFromAnotherAccount || false;
                        $scope.wizardController.awsEncryptionKey = $scope.wizardController.awsEncryptionKey || awsEncryptionKeysObject.awsEncryptionKey;
                        $scope.wizardController.awsEncryptionKey.chosen = true;
                        $scope.disableEditawsEncryptionKeys = awsEncryptionKeysObject.awsEncryptionKeys.length === 1;
                        $scope.disableEditawsEncryptionKeysTooltip = $scope.disableEditawsEncryptionKeys ? 'You cannot choose a different master key because no other enabled keys exist in the region.' : '';
                        if ($scope.wizardController.failedRequestType) {
                            if ($scope.wizardController.failedRequest.dataEncryptionType === 'AWS') {
                                $scope.wizardController.useKeysFromAnotherAccount = !!$scope.wizardController.failedRequest.awsEncryptionParameters.kmsKeyArn;
                                $scope.wizardController.awsEncryptionKey = $scope.wizardController.useKeysFromAnotherAccount ?
                                    {kmsKeyArn: $scope.wizardController.failedRequest.awsEncryptionParameters.kmsKeyArn} :
                                    _.find($scope.wizardController.awsEncryptionKeys, {keyId: $scope.wizardController.failedRequest.awsEncryptionParameters.kmsKeyId});
                            }
                        }
                    });
            }
            if ($scope.wizardController.failedRequestType) {
                $scope.wizardController.encryptionType = $scope.wizardController.failedRequest.dataEncryptionType;
            }
        };

        $scope.shouldCotEncryptionBeDisabled = () => {
            return !$scope.wizardController.certificate ||
                $scope.wizardController.certificate._expirationState.state === 'certificate-expired' ||
                _.isEmpty($scope.wizardController.keyManagers) ||
                _.isEmpty($scope.wizardController.keyManagersCaCertificates) ||
                _.every($scope.wizardController.keyManagersCaCertificates, {_expirationState: {state: 'certificate-expired'}});
        };

        $scope.selectEncryptionType = function selectEncryptionType(encryptionType) {
            // If previously selected no encryption or AWS encryption and now selecting ONTAP encryption (or vise versa),
            // clearing all licence and version and packages data to force the user to reselect and collapse wizard to
            // this step.
            if ((encryptionType === 'ONTAP') !== ($scope.wizardController.encryptionType === 'ONTAP')) {
                $scope.wizardController.setWizardStep($scope.wizardController.path['encryption']);
                $scope.wizardController.clearStepData('packages');
                $scope.wizardController.clearStepData('license');
            }
            $scope.wizardController.encryptionType = encryptionType;
        };

        $scope.editAwsEncryptionKey = () => {
            if ($scope.disableEditawsEncryptionKeys) return;
            nocDialog.dialog('', 'assets/templates/wizards/virgin-create-working-environment/edit-aws-encryption-keys-dialog.html', {
                wizardController: $scope.wizardController,
                keyWithStatusNotEnabled: $scope.keyWithStatusNotEnabled,
                width: 890,
                fromTop: '12%'
            });
        };

        $scope.$on('$destroy', () => {
            $scope.wizardController.backupStepData('create-encryption', {
                encryptionType: $scope.wizardController.encryptionType,
                awsEncryptionKey: $scope.wizardController.awsEncryptionKey,
                useKeysFromAnotherAccount: $scope.wizardController.useKeysFromAnotherAccount
            });
            $scope.wizardController.encryptionType = undefined;
        });

        _.assignIn($scope.wizardController, $scope.wizardController.getStepData('create-encryption'));
        $scope.initialize();
    });
})();
