(() => {
    'use strict';

    angular.module('noc').controller('VirginCreateWorkingEnvironmentHAPublicKeyController', ($scope, $rootScope, nocHttp, pidgeon, MetadataResource,
                                                                                             configService, nocUI) => {
        const keyAccess = {name: 'Key Pair', requestValue: 'key'};
        const defaultAccess = {name: 'Password', requestValue: 'password'};
        $scope.initialize = () => {
            $scope.accesses = $rootScope.runningInIC ? [] : [defaultAccess, keyAccess];
            $scope.internetAccesses = [
                {name: 'Public IP address', requestValue: 'public'},
                {name: 'Proxy server', requestValue: 'proxy'},
                {name: 'Manual option', requestValue: 'manual'}
            ];

            $scope.wizardController.haKeySelectObjects = $scope.wizardController.haKeySelectObjects || ($rootScope.runningInIC ? {access: keyAccess} : {access: defaultAccess});
            if ($scope.wizardController.keyPairs) {
                $scope.wizardController.haKeySelectObjects.nodesKey = $scope.wizardController.haKeySelectObjects.nodesKey || _.get($scope.wizardController, 'keyPairs[0]');
                $scope.wizardController.haKeySelectObjects.mediatorKey = $scope.wizardController.haKeySelectObjects.mediatorKey || _.get($scope.wizardController, 'keyPairs[0]');
                if ($scope.wizardController.failedRequestType) recoverFailedRequestKey();
            } else {
                getKeyPairs().then((keyPairs) => {
                    $scope.wizardController.keyPairs = keyPairs;
                    $scope.wizardController.haKeySelectObjects.nodesKey = _.get($scope.wizardController, 'keyPairs[0]');
                    $scope.wizardController.haKeySelectObjects.mediatorKey = _.get($scope.wizardController, 'keyPairs[0]');
                    if ($scope.wizardController.failedRequestType) recoverFailedRequestKey();
                });
            }

            $scope.wizardController.mediatorProxy = {};

            if ($scope.wizardController.failedRequestType) {
                $scope.proxyValues = {
                    proxyUri: _.get($scope.wizardController.failedRequest.haParams, 'mediatorProxy.url'),
                    proxyUserName: _.get($scope.wizardController.failedRequest.haParams, 'mediatorProxy.userName'),
                    proxyPassword: ''
                };

                $scope.wizardController.mediatorProxy.url = $scope.proxyValues.proxyUri;
                $scope.wizardController.mediatorProxy.userName = $scope.proxyValues.proxyUserName;
                $scope.wizardController.mediatorProxy.password = $scope.proxyValues.proxyPassword;

                if ($scope.wizardController.mediatorProxy.url) {
                    $scope.wizardController.mediatorAccess = {access: _.find($scope.internetAccesses, {requestValue: 'proxy'})};
                } else {
                    if ($scope.wizardController.failedRequest.haParams.mediatorAssignPublicIP) {
                        $scope.wizardController.mediatorAccess = {access: _.find($scope.internetAccesses, {requestValue: 'public'})};
                    } else {
                        $scope.wizardController.mediatorAccess = {access: _.find($scope.internetAccesses, {requestValue: 'manual'})};
                    }
                }
            } else {
                configService.getConfig(nocHttp, ['proxyUri', 'proxyUserName', 'proxyPassword'])
                    .then((configResponse) => {
                        $scope.proxyValues = configResponse;

                        $scope.wizardController.mediatorProxy.url = $scope.proxyValues.proxyUri;
                        $scope.wizardController.mediatorProxy.userName = $scope.proxyValues.proxyUserName;
                        $scope.wizardController.mediatorProxy.password = $scope.proxyValues.proxyPassword;

                        if ($scope.wizardController.mediatorProxy.url) {
                            $scope.wizardController.mediatorAccess = {access: _.find($scope.internetAccesses, {requestValue: 'proxy'})};
                        } else {
                            $scope.wizardController.mediatorAccess = $scope.wizardController.mediatorAccess || {access: _.find($scope.internetAccesses, {requestValue: 'public'})};
                        }
                    });
            }
        };

        const recoverFailedRequestKey = () => {
            if ($scope.wizardController.failedRequest.clusterKeyPairName) {
                $scope.wizardController.haKeySelectObjects.nodesKey = _.find($scope.wizardController.keyPairs, {name: $scope.wizardController.failedRequest.clusterKeyPairName});
                $scope.wizardController.haKeySelectObjects.access = keyAccess;
            }
            $scope.wizardController.haKeySelectObjects.mediatorKey = _.find($scope.wizardController.keyPairs, {name: $scope.wizardController.failedRequest.haParams.mediatorKeyPairName});
        };

        const getKeyPairs = () => {
            return MetadataResource.getKeyPairs(nocHttp, pidgeon, _.get($scope.wizardController, 'createVpcSelectObjects.region.code'), false, $scope.wizardController.cloudProviderAccountId)
                .then((keyPairs) => {
                    return _.map(keyPairs, (keyPair) => {
                        return {name: keyPair};
                    });
                });
        };

        $scope.next = () => {
            if ($scope.wizardController.haKeySelectObjects.access.requestValue === 'proxy') {
                if (!$scope.setupMediatorProxyForm.$valid) {
                    nocUI.handleFormValidity();

                    return;
                }
            }
            $scope.wizardController.next(getNextStep(_.get($scope.wizardController, 'failoverMode')));
        };

        function getNextStep(haFailoverMode) {
            return haFailoverMode === 'FloatingIP' ? '.ha-floating-ip' : '.encryption';
        }

        $scope.$on('$destroy', () => {
            $scope.wizardController.backupStepData('ha-public-key', {
                keyPairs: $scope.wizardController.keyPairs,
                haKeySelectObjects: $scope.wizardController.haKeySelectObjects,
                mediatorAccess: $scope.wizardController.mediatorAccess
            });

            $scope.wizardController.backupStepData('ha-public-key-region', {
                regionCode: _.get($scope.wizardController, 'createVpcSelectObjects.region.code')
            });
            $scope.wizardController.haKeySelectObjects = null;
            $scope.wizardController.keyPairs = null;
            $scope.wizardController.mediatorAccess = null;
        });

        // If the region didn't change, it is OK to load from backup
        if (_.get($scope.wizardController.getStepData('ha-public-key-region'), 'regionCode') === $scope.wizardController.createVpcSelectObjects.region.code) {
            _.assignIn($scope.wizardController, $scope.wizardController.getStepData('ha-public-key'));
        }

        $scope.initialize();
    });
})();
