(() => {
    "use strict";

    angular.module("noc").controller("VirginCreateWorkingEnvironmentKeyManagersStepController", function ($scope, nocDialog, nocUtil, nocHttp, pidgeon) {
        $scope.maxKeyManagersAllowedToSelect = 4;

        $scope.initialize = function () {
            $scope.wizardController.ontapEncryption = $scope.wizardController.ontapEncryption || {keyManagers: []};
            $scope.wizardController.keyManagersSelectedMap = $scope.wizardController.keyManagersSelectedMap || {};

            //if there is only one key manager, auto select it:
            if ($scope.wizardController.keyManagers && $scope.wizardController.keyManagers.length === 1) {
                var onlyKeyManager = $scope.wizardController.keyManagers[0];
                $scope.wizardController.ontapEncryption.keyManagers = [onlyKeyManager];
                $scope.wizardController.keyManagersSelectedMap[onlyKeyManager.publicId] = true;
            }

            //if there is only one key managers' CA certificate, auto select it:
            if ($scope.wizardController.keyManagersCaCertificates && $scope.wizardController.keyManagersCaCertificates.length === 1) {
                $scope.wizardController.ontapEncryption.keyManagerCaCertificateId = $scope.wizardController.keyManagersCaCertificates[0].publicId;
            }

            if ($scope.wizardController.failedRequestType) {
                $scope.wizardController.keyManagersSelectedMap = {};
                $scope.wizardController.ontapEncryption.keyManagers = [];
                _.forEach($scope.wizardController.failedRequest.ontapEncryptionParameters.keyManagerIds, (keyManagerId) => {
                    $scope.keyManagerClicked(_.find($scope.wizardController.keyManagers, {publicId: keyManagerId}));
                });
                $scope.wizardController.ontapEncryption.keyManagerCaCertificateId = $scope.wizardController.failedRequest.ontapEncryptionParameters.keyManagerCaCertificateId;
            }

            pidgeon.clear("create-key-managers");
            nocHttp.get('/occm/config', "initializing", nocHttp.LOADING)
                .then(function (config) {
                    $scope.certificateValidityPeriodDays = config.certificateValidityPeriod;
                    $scope.certificateValidityPeriod = moment.duration($scope.certificateValidityPeriodDays, 'days').humanize();
                });
        };

        $scope.keyManagerClicked = function (keyManager) {
            //If scrolled up to here from advanced step, bring the wizard back to the current step to enforce validation.
            if ($scope.wizardController.currentStateName !== 'key-managers'){
                $scope.wizardController.setWizardStep($scope.wizardController.path["key-managers"]);
            }
            if ($scope.wizardController.keyManagersSelectedMap[keyManager.publicId]) { //un-selecting
                _.pull($scope.wizardController.ontapEncryption.keyManagers, keyManager);
            } else { //selecting
                $scope.wizardController.ontapEncryption.keyManagers.push(keyManager);
            }
            $scope.wizardController.keyManagersSelectedMap[keyManager.publicId] = !$scope.wizardController.keyManagersSelectedMap[keyManager.publicId];
        };

        $scope.next = function () {
            var allObjectsHasCommonValues = function (collection, field) {
                return _.uniq(_.map(collection,field)).length === 1;
            };

            if (_.isEmpty($scope.wizardController.ontapEncryption.keyManagers) || $scope.wizardController.ontapEncryption.keyManagers.length > $scope.maxKeyManagersAllowedToSelect) {
                nocDialog.alert("You must select 1 to " + $scope.maxKeyManagersAllowedToSelect + " Key Managers before you can proceed.");
            } else
            //NOC-5994 - if user chooses more than one key manager with FIPS – all should have the same FIPS settings – validate in UI
            if (!allObjectsHasCommonValues($scope.wizardController.ontapEncryption.keyManagers, 'usernameField') ||
                !allObjectsHasCommonValues($scope.wizardController.ontapEncryption.keyManagers, 'usernameValue')) {
                nocDialog.alert("Key manager client authentication fields should be the same for all selected key managers.");
            } else {
                $scope.wizardController.next('.byol');
            }
        };

        $scope.occmCertificateExpiresInCertificateValidityPeriod = function () {
            return nocUtil.certificateExpirationState($scope.wizardController.certificate.endDate, $scope.certificateValidityPeriodDays).state === 'certificate-warning';
        };

        $scope.$on("$destroy", function(){
            $scope.wizardController.backupStepData("create-key-managers", {
                ontapEncryption: $scope.wizardController.ontapEncryption,
                keyManagersSelectedMap: $scope.wizardController.keyManagersSelectedMap
            });
            $scope.wizardController.ontapEncryption = null;
            $scope.wizardController.keyManagersSelectedMap = null;
        });

        _.assignIn($scope.wizardController, $scope.wizardController.getStepData("create-key-managers"));
        $scope.initialize();

    });
})();
