(() => {
    'use strict';

    angular.module('noc').controller('VirginCreateCheckMarketPlaceRegistration', ($scope, $timeout, nocHttp, MetadataResource,
                                                                                  CreateWorkingEnvironmentWizardService) => {
        CreateWorkingEnvironmentWizardService.call($scope, nocHttp);
        $scope.providerName = $scope.wizardController.choice === 'VSA_AZURE' || $scope.wizardController.choice === 'HA_AZURE' ? 'Azure' : 'AWS';
        $scope.isAzure = $scope.wizardController.choice === 'VSA_AZURE' || $scope.wizardController.choice === 'HA_AZURE';
        $scope.state = 'not-registered';
        $scope.dummy = null;
        if ($scope.isAzure) {
            $scope.dummy = 'dummy';
        }

        let checkStartTime = new Date().getTime();

        function processResponse(registered) {
            if (registered.subscribed) {
                $scope.state = 'registered';
                $scope.dummy = 'dummy';
            } else {
                $scope.ontapRegistrationLink = registered.url;
                if ((new Date().getTime() - checkStartTime) < (20 * 60 * 1000)) {
                    $timeout(pollForStatus, 60 * 500);
                } else {
                    $scope.state = 'timeout';
                }
            }
        }

        function pollForStatus() {
            const provider = $scope.wizardController.workingEnvironmentTypeAndProvider.providerName;
            const request = {
                region: $scope.wizardController.selectedRegion,
                subnetId: provider === 'Azure' ? $scope.wizardController.selectedSubnet.id : $scope.wizardController.createVpcSelectObjects.subnet.subnetId,
                vsaMetadata: {
                    ontapVersion: $scope.wizardController.selectedOntapVersion,
                    licenseType: $scope.wizardController.selectedLicense.type,
                    instanceType: $scope.wizardController.selectedInstanceType,
                    providedLicense: $scope.wizardController.providedLicense
                }
            };

            if ($scope.wizardController.cloudProviderAccountId) {
                request.cloudProviderAccount = $scope.wizardController.cloudProviderAccountId;
            }

            MetadataResource.isRegisteredToMarketPlace(nocHttp, request, $scope.wizardController.workingEnvironmentType, provider)
                .then((response) => {
                    processResponse(response);
                });
        }

        $scope.repoll = () => {
            $scope.state = 'not-registered';
            checkStartTime = new Date().getTime();
            pollForStatus();
        };

        $scope.next = () => {
            const nextStep = _.includes($scope.wizardController.selectedPackage.name, 'custom') ? '.tiering' : '.volume';

            if (!$scope.dummy) {
                $scope.clicked = true;
            } else {
                $scope.wizardController.next(nextStep);
            }
        };

        if (!$scope.isAzure) {
            pollForStatus();
        }
    });
})();
