(() => {
    'use strict';

    angular.module('noc').controller('CreateWorkingEnvironmentPackagesStepController', ($scope, MetadataResource, nocHttp,
                                                                                        ManifestMapper, nocUI, CreateWorkingEnvironmentWizardService,
                                                                                        providerDiskTypes, $timeout, $rootScope,
                                                                                        $log) => {
        $scope.initialize = function initialize() {
            CreateWorkingEnvironmentWizardService.call($scope, nocHttp, nocUI);

            $scope.licenses = ManifestMapper.getAvailableLicenses($scope.wizardController.selectedRegion, $scope.getLatestOntapVersions());
            MetadataResource.getPackages(nocHttp, $scope.wizardController.workingEnvironmentType, $scope.wizardController.workingEnvironmentTypeAndProvider.providerName)
                .then((packagesResponse) => {
                    _.forEach(packagesResponse, (preConfPackage) => {
                        preConfPackage._diskType = _.find(providerDiskTypes[$scope.wizardController.workingEnvironmentTypeAndProvider.providerName], {name: preConfPackage.diskType});
                        preConfPackage._tierName = preConfPackage.diskType + (preConfPackage.capacityTier ? '+' + preConfPackage.capacityTier : '');
                        preConfPackage._diskTypeCap = preConfPackage._diskType.display;

                        const regionInstanceType = _.find(preConfPackage.instanceTypeMapping, {'region': $scope.wizardController.selectedRegion});
                        if (!regionInstanceType) {
                            const defaultRegionInstanceType = _.find(preConfPackage.instanceTypeMapping, {'region': 'default'});
                            preConfPackage.instanceType = defaultRegionInstanceType.instanceType;
                        } else if (regionInstanceType.instanceType.length) {
                            preConfPackage.instanceType = regionInstanceType.instanceType;
                        } else {
                            preConfPackage.disabled = true;
                            preConfPackage.disabledText = 'There is no supported instance types in this region.';
                        }

                        if ($scope.wizardController.encryptionType === 'ONTAP' && preConfPackage.capacityTier) {
                            preConfPackage.disabled = true;
                            preConfPackage.disabledText = 'This package is not supported with ONTAP encryption.';
                        }
                        if (_.includes(_.get($scope.wizardController, 'selectedLicense.type'), 'byol')) {
                            preConfPackage._license = $scope.wizardController.selectedLicense;
                        } else {
                            preConfPackage._license = _.find($scope.licenses, {type: preConfPackage.licenseType});
                        }
                    });
                    $scope.packages = packagesResponse;
                    if ($scope.wizardController.failedRequestType) {
                        $scope.wizardController.selectedPackage = _.find($scope.packages, {'name': $scope.wizardController.failedRequest.packageName});
                    }
                })
                .catch($log.error);
        };

        $scope.getLatestOntapVersions = function getLatestOntapVersions() {
            return ManifestMapper.getAvailableVersions($scope.wizardController.selectedRegion)[0];
        };

        $scope.clearFutureStepsDataIfNeeded = function clearFutureStepsDataIfNeeded(clickedPackage) {
            //0. Should do nothing if selecting package for the first time:
            if (!$scope.wizardController.selectedPackage) return;

            //1. Switching from 'Custom' to pre-configured packaged or vice versa:
            const cond1 = clickedPackage.isCustom !== $scope.wizardController.selectedPackage.isCustom;

            //2. Switching from pre-configured packaged with tiering to S3 to another pre-configured packaged without it:
            const cond2 = !clickedPackage.isCustom && !$scope.wizardController.selectedPackage.isCustom &&
                $scope.wizardController.selectedPackage.capacityTier !== clickedPackage.capacityTier;

            if (cond1 || cond2) {
                $scope.wizardController.setWizardStep($scope.wizardController.path.packages);
                $scope.wizardController.clearStepData('volume');
            }
        };

        $scope.selectPackage = function selectPackage(preConfPackage) {
            if (preConfPackage.disabled) return;

            $scope.clearFutureStepsDataIfNeeded(preConfPackage);

            $timeout(() => {
                //Has to be in timeout to make sure this happens AFTER future steps destroy themselves.
                //Otherwise this is overridden by steps saving their data.
                $scope.wizardController.selectedPackage = preConfPackage;

                $scope.wizardController.selectedTier = preConfPackage._diskType;
                $scope.wizardController.tieringEnabled = Boolean(preConfPackage.capacityTier);
                $scope.wizardController.providerVolumeSize = $scope.isAzure ? {size: preConfPackage.diskSize} : preConfPackage.diskSize;
                if ($scope.isAzure) {
                    $scope.wizardController.providerVolumeSize._identifier = preConfPackage.diskSize._identifier;
                }
                $scope.wizardController.selectedInstanceType = preConfPackage.instanceType;
                $scope.wizardController.selectedLicense = preConfPackage._license;
                const tenancies = [
                    {name: 'Shared', requestName: 'default'},
                    {name: 'Dedicated', requestName: 'dedicated'}
                ];
                $scope.wizardController.tenancy = _.find(tenancies, {requestName: preConfPackage.instanceTenancy});
                $scope.wizardController.writingSpeedState = preConfPackage.writingSpeedState;
                $scope.wizardController.selectedOntapVersion = $scope.getLatestOntapVersions();

                $scope.next();
            });
        };

        function needToShowRolePageInIC() {
            if ($rootScope.runningInIC && $scope.wizardController.workingEnvironmentTypeAndProvider.combined === 'HA_AWS') {
                return true;
            }

            if (!$rootScope.runningInIC) {
                return true;
            }
        }
        $scope.selectCreateMyOwn = function selectCreateMyOwn() {
            const typeAndProviderToCustomPackageNameMap = {
                'VSA_AWS': 'aws_custom',
                'HA_AWS': 'aws_ha_custom',
                'VSA_Azure': 'azure_custom',
                'HA_Azure': 'azure_ha_custom',
                'VSA_GCP': 'gcp_custom'
            };

            $scope.wizardController.selectedTier = null;
            $scope.clearFutureStepsDataIfNeeded({isCustom: true});

            $timeout(() => {
                $scope.wizardController.selectedPackage = {
                    name: typeAndProviderToCustomPackageNameMap[$scope.wizardController.workingEnvironmentTypeAndProvider.combined],
                    isCustom: true
                };

                if ($scope.isAWS && $scope.wizardController.currentStateName === 'packages' && needToShowRolePageInIC()) {
                    $scope.wizardController.next('.roles');
                } else {
                    $scope.wizardController.next('.license');
                }
            });
        };

        $scope.next = function next() {
            if ($rootScope.runningInIC && $scope.wizardController.workingEnvironmentTypeAndProvider.combined === 'HA_AWS') {
                $scope.wizardController.next('.roles');
            } else {
                $scope.continueAfterLicenseOrPackagesStep('.volume');
            }
        };

        $scope.$on('$destroy', () => {
            $scope.wizardController.backupStepData('packages', {
                selectedPackage: $scope.wizardController.selectedPackage,
                selectedTier: $scope.wizardController.selectedTier,
                providerVolumeSize: $scope.wizardController.providerVolumeSize,
                selectedInstanceType: $scope.wizardController.selectedInstanceType,
                selectedLicense: $scope.wizardController.selectedLicense,
                tenancy: $scope.wizardController.tenancy,
                writingSpeedState: $scope.wizardController.writingSpeedState,
                selectedOntapVersion: $scope.wizardController.selectedOntapVersion
            });

            $scope.wizardController.selectedPackage = null;
            $scope.wizardController.selectedTier = null;
            $scope.wizardController.providerVolumeSize = null;
            $scope.wizardController.selectedInstanceType = null;
            if (!_.includes(_.get($scope.wizardController, 'selectedLicense.type'), 'byol')) {
                //Only if it is paygo delete the license, as it was selected in prev step
                $scope.wizardController.selectedLicense = null;
            }

            $scope.wizardController.tenancy = null;
            $scope.wizardController.writingSpeedState = null;
            $scope.wizardController.selectedOntapVersion = null;
        });

        _.assignIn($scope.wizardController, $scope.wizardController.getStepData('packages'));
        $scope.initialize();
    });
})();
