(() => {
    'use strict';

    angular.module('noc').controller('VirginCreateWorkingEnvironmentTieringStepController', ($scope, nocHttp, TieringMetadataResource,
                                                                                             pidgeon, ManifestMapper, $timeout,
                                                                                             configService, Constants, MetadataResource,
                                                                                             nocDialog) => {
        $scope.initialize = function initialize() {
            $scope.allowDiskSizeSelection = true;
            $scope.maxIopsValue = Constants.awsIopsLimitPerVolume;

            configService.getConfig(nocHttp, ['ebsSizeToPiopsRatio'])
                .then(function (configResponse) {
                    $scope.ebsSizeToPiopsRatio = configResponse.ebsSizeToPiopsRatio;
                    $scope.onDiskSizeChange();
                });

            $scope.getPossibleTiers()
                .then(() => {
                    $timeout(() => {
                        //Focus on the selected box:
                        const elementClass = '.' + $scope.wizardController.selectedTier.name;
                        const element = $(elementClass);
                        if (element) element.focus();
                    });

                    TieringMetadataResource.getSupportedCapacityTiers(
                        nocHttp,
                        $scope.wizardController.pathPrefix,
                        $scope.wizardController.selectedRegion,
                        $scope.wizardController.selectedOntapVersion,
                        $scope.wizardController.encryptionType || 'NONE',
                        $scope.wizardController.selectedLicense.type,
                        $scope.wizardController.selectedInstanceType,
                        $scope.wizardController.selectedAzureSubscription ? $scope.wizardController.selectedAzureSubscription.subscriptionId : null,
                        $scope.wizardController.cloudProviderAccountId
                    )
                        .then((response) => {
                            $scope.wizardController.compositeSupported = response.compositeSupported;
                            $scope.wizardController.forceCompositeVersion = response.forceCompositeVersion;
                            if (response.capacityTiersDisableReasons) {
                                $scope.wizardController.capacityTiersDisableReasons = response.capacityTiersDisableReasons;
                            }
                            $scope.tieringDisabledMessage = $scope.disabledMessageForTieringCheckbox();
                            $scope.wizardController.tieringDisabled = Boolean($scope.tieringDisabledMessage);

                        });

                });

            //This is here because it needs to be after license step:
            if (_.includes(ManifestMapper.getNetworkUtilizationEnabledInstanceTypes($scope.wizardController.selectedOntapVersion),
                    $scope.wizardController.selectedInstanceType)) {
                $scope.wizardController.optimizedNetworkUtilization = 'optimized';
            }

            MetadataResource.getSupportedFeatures(
                nocHttp,
                $scope.wizardController.pathPrefix,
                $scope.wizardController.selectedRegion,
                $scope.wizardController.selectedOntapVersion,
                $scope.wizardController.encryptionType || 'NONE',
                $scope.wizardController.selectedLicense.type,
                $scope.wizardController.selectedInstanceType
            )
                .then((response) => {
                    $scope.wizardController.wormSupportedVersion = response.wormSupportedVersion;
                });

        };

        $scope.onDiskSizeChange = function onDiskSizeChange() {
            $scope.wizardController.providerVolumeSizeGB = $scope.wizardController.providerVolumeSize.unit === 'GB' ? $scope.wizardController.providerVolumeSize.size : $scope.wizardController.providerVolumeSize.size * 1024;
            $scope.maxIopsValue = _.min([Constants.awsIopsLimitPerVolume, $scope.wizardController.providerVolumeSizeGB * $scope.ebsSizeToPiopsRatio]);
        };

        function recoverFailedRequestSettings() {
            if ($scope.wizardController.workingEnvironmentTypeAndProvider.providerName === 'AWS') {
                var failedRequestDiskType = $scope.wizardController.failedRequest.ebsVolumeType;
                $scope.wizardController.selectedTier = _.find($scope.possibleTiers, {name: failedRequestDiskType});
                $scope.wizardController.tieringEnabled = Boolean($scope.wizardController.failedRequest.capacityTier);
                $scope.wizardController.providerVolumeSize = $scope.wizardController.failedRequest.ebsVolumeSize;
                $scope.wizardController.providerVolumeSize._identifier = $scope.wizardController.providerVolumeSize.size + ' ' + $scope.wizardController.providerVolumeSize.unit;
            }
            if ($scope.wizardController.workingEnvironmentTypeAndProvider.providerName === 'Azure') {
                $scope.wizardController.selectedTier = _.find($scope.possibleTiers, {name: $scope.wizardController.failedRequest.storageType});
                $scope.wizardController.providerVolumeSize = _.find($scope.wizardController.selectedTier.supportedSizes,
                    {size: {size: $scope.wizardController.failedRequest.diskSize.size}});
            }
        }

        $scope.getPossibleTiers = function getPossibleTiers() {
            pidgeon.clear('getPossibleTiers');

            return TieringMetadataResource.getPossibleTiers(nocHttp,
                $scope.wizardController.workingEnvironmentTypeAndProvider.type,
                $scope.wizardController.workingEnvironmentTypeAndProvider.providerName,
                $scope.wizardController.selectedLicense,
                $scope.wizardController.selectedRegion,
                $scope.wizardController.selectedOntapVersion,
                $scope.wizardController.encryptionType,
                $scope.wizardController.selectedInstanceType,
                $scope.wizardController.cloudProviderAccountId)
                .then((possibleTiers) => {
                    $scope.possibleTiers = possibleTiers;
                    //We allocate S3 bucket to any working environment that supports composite tiers, even if initial
                    // tier is not composite:
                    $scope.wizardController.isAllocatingS3Bucket = _.find($scope.possibleTiers, (diskType) => {
                        return !diskType.disabledMessage && _.includes(diskType.supportedCapacityTiers, 'S3');
                    });
                    if ($scope.wizardController.workingEnvironmentTypeAndProvider.providerName === 'AWS') {
                        delete $scope.possibleTiers['standard'];
                    }
                    if ($scope.wizardController.failedRequestType) recoverFailedRequestSettings();
                    //In case selectedTier was restored from previous visit to this step, keep it.
                    if (!$scope.wizardController.selectedTier) {
                        $scope.tierSelected(_.find($scope.possibleTiers, 'default'));
                    }
                })
                .catch((message) => {
                    pidgeon.error(message, null, 'getPossibleTiers');
                });
        };

        $scope.disabledTooltipForTier = function disabledTooltipForTier(tier) {
            if (tier && tier.disabledMessage) {
                $scope.wizardController.tieringEnabled = false;
                return tier.disabledMessage;
            }
            if (!_.includes(_.get(tier, 'supportedCapacityTiers'), 'S3') && !_.includes(_.get(tier, 'supportedCapacityTiers'), 'Blob')) {
                return $scope.getObjectStorageName() + ' tiering is not available for this disk type.';
            }
        };

        $scope.disabledMessageForTieringCheckbox = function disabledMessageForTieringCheckbox(tier) {
            if ($scope.wizardController.capacityTiersDisableReasons && $scope.wizardController.capacityTiersDisableReasons.length > 0) {
                return $scope.wizardController.capacityTiersDisableReasons[0];
            }
            if (tier && tier.disabledMessage) {
                return tier.disabledMessage;
            }
            if (tier && !_.some(['S3', 'Blob', 'cloudStorage'], (objectStorageType) => _.includes(_.get(tier, 'supportedCapacityTiers'), objectStorageType))) {
                tier.disabledMessage = $scope.getObjectStorageName() + ' tiering is not available for this disk type.';

                return tier.disabledMessage;
            }
        };

        $scope.tierSelected = function tierSelected(tier) {
            if ($scope.wizardController.selectedTier && ($scope.wizardController.selectedTier.diskName !== 'io1') && (tier.diskName === 'io1')) {
                $scope.wizardController.setWizardStep($scope.wizardController.path['tiering']);
            }

            $scope.wizardController.selectedTier = tier;
            if (!_.includes($scope.wizardController.selectedTier.supportedCapacityTiers, 'S3')) {
                $scope.wizardController.tieringEnabled = false;
            }
            $scope.wizardController.providerVolumeSize = _.find(tier.supportedSizes, {_identifier: (tier.name === 'sc1') ? '2 TB' : '1 TB'});
            $scope.tieringDisabledMessage = $scope.disabledMessageForTieringCheckbox(tier);
            $scope.wizardController.tieringDisabled = Boolean($scope.tieringDisabledMessage);
        };

        $scope.tieringCheckboxSelected = function tieringCheckboxSelected() {
            //Handle case when user selects 'No Storage Efficiency' volume then scrolls up and changes tier selection.
            $scope.wizardController.setWizardStep($scope.wizardController.path['tiering']);
            $scope.wizardController.clearStepData('volume');
        };

        $scope.getObjectStorageName = () => {
            const providerToObjectStorage = {
                AWS: 'S3',
                Azure: 'Blob',
                GCP: 'Cloud Storage'
            };

            return providerToObjectStorage[$scope.wizardController.workingEnvironmentTypeAndProvider.providerName];
        };

        $scope.next = () => {
            if ($scope.wizardController.workingEnvironmentType === 'HA') {
                if ($scope.wizardController.wormSupportedVersion) {
                    $scope.wizardController.next('.worm');
                } else {
                    $scope.wizardController.next('.volume');
                }
            } else {
                if ($scope.wizardController.workingEnvironmentTypeAndProvider.providerName === 'GCP') {
                    $scope.wizardController.next('.worm');
                } else {
                    $scope.wizardController.next('.writing-speed');
                }
            }
        };

        $scope.openTieringCheckboxDialog = () => {
            return nocDialog.confirm('',
                'assets/templates/tabs/working-environments/tiering-checkbox-dialog.html', {
                    tieringEnabled: {value: !$scope.wizardController.tieringDisabled},
                    wizardController: $scope.wizardController,
                    objectStore: $scope.getObjectStorageName(),
                    width: 560
                }).then((dialogScope) => {
                $scope.wizardController.tieringDisabled = !dialogScope.tieringEnabled.value;
            });
        };

        $scope.$on('$destroy', () => {
            $scope.wizardController.backupStepData('tiering', {
                selectedTier: $scope.wizardController.selectedTier,
                providerVolumeSize: $scope.wizardController.providerVolumeSize
            });

            $scope.wizardController.selectedTier = undefined;
            $scope.wizardController.providerVolumeSize = undefined;
        });

        $scope.initialize();
        _.assignIn($scope.wizardController, $scope.wizardController.getStepData('tiering'));
    });
})();
