(function () {
    "use strict";

    angular.module("noc").controller("SnapMirrorInitController", function ($scope, BaseVirginStep, $state, nocUI,
                                                                           nocHttp, pidgeon, workingEnvironmentResource,
                                                                           $q, nocUtil, replicationResource) {
        BaseVirginStep.call($scope);

        $scope.handleLifs = function (response) {
            var buildSelectedLifs = function (isOnPrem, lifs) {
                if (!isOnPrem || (isOnPrem && lifs.length === 1)) {
                    return _.map(lifs, 'address');
                }
            };

            var sourceIsOnPrem = $scope.wizardController.sourceWorkingEnvironment.workingEnvironmentType === 'ON_PREM';
            var targetIsOnPrem = $scope.wizardController.targetWorkingEnvironment.workingEnvironmentType === 'ON_PREM';

            $scope.wizardController.lifs = {};
            $scope.wizardController.lifs["sourceLifs"] = response.interClusterLifs;
            $scope.wizardController.lifs["destinationLifs"] = response.peerInterClusterLifs;

            var peeredSourceLifs = _.map(_.filter(response.interClusterLifs, 'isPeered'), 'address');
            var peeredDestinationLifs = _.map(_.filter(response.peerInterClusterLifs, 'isPeered'), 'address');
            $scope.wizardController.selectedLifsAddresses = {};
            $scope.wizardController.selectedLifsAddresses["sourceLifs"] = peeredSourceLifs.length > 0 ? peeredSourceLifs : buildSelectedLifs(sourceIsOnPrem, response.interClusterLifs);
            $scope.wizardController.selectedLifsAddresses["destinationLifs"] = peeredDestinationLifs.length > 0 ? peeredDestinationLifs : buildSelectedLifs(targetIsOnPrem, response.peerInterClusterLifs);

            // if no side is OnPrem or peering exists or one LIF in both sides - go directly to volumes step
            if ((!sourceIsOnPrem && !targetIsOnPrem) ||
                (_.find(response.interClusterLifs, 'isPeered') && _.find(response.peerInterClusterLifs, 'isPeered')) || (response.interClusterLifs.length === 1 && response.peerInterClusterLifs.length === 1)) {
                $scope.wizardController.nextStep = {init: ".volumes"};
            }
            // else at least one side is OnPrem which is not peered and has more than one leaf - build steps
            else {
                $scope.wizardController.nextStep = {};
                var currentStep = "init";
                if (sourceIsOnPrem && response.interClusterLifs.length > 1) {
                    $scope.wizardController.nextStep[currentStep] = ".sourceLifs";
                    currentStep = "sourceLifs";
                }
                if (targetIsOnPrem && response.peerInterClusterLifs.length > 1) {
                    $scope.wizardController.nextStep[currentStep] = ".destinationLifs";
                    currentStep = "destinationLifs";
                }
                $scope.wizardController.nextStep[currentStep] = ".volumes";
            }
        };

        $scope.getInterclusterLifs = function () {
            replicationResource.getInterclusterLifs(nocHttp,
                $scope.wizardController.sourceWorkingEnvironment.publicId,
                $scope.wizardController.targetWorkingEnvironment.publicId)
                .then(function (response) {
                    $scope.handleLifs(response);
                    $scope.next();
                })
                .catch(function (errorResponse) {
                    var prefix = pidgeon.serverErrorPrefix + "retrieving LIFs for Replication";
                    //the error is pushed to the DataFabricController's pidgeon so it will be shown in the gallery and not in this hidden step
                    $scope.pushError(prefix + (errorResponse ? ": " + errorResponse.messageForDisplay() : ""), errorResponse ? errorResponse : {}, "DataFabricController", "replicationWizard");
                    $scope.exit();
                });
        };

        $scope.getObjectStorageName = () => {
            const providerToObjectStorage = {
                AWS: 'S3',
                Azure: 'Blob',
                GCP: 'Cloud Storage'
            };

            return providerToObjectStorage[$scope.wizardController.sourceWorkingEnvironment._cloudProviderName];
        };

        $scope.initialize = function () {
            pidgeon.clear("init");
            var sourcePathPrefix = nocUtil.workingEnvironmentTypeToRequestPathPrefix($state.params.sourceType, $state.params.sourceProvider);
            var targetPathPrefix = nocUtil.workingEnvironmentTypeToRequestPathPrefix($state.params.targetType, $state.params.targetProvider);

            if (sourcePathPrefix && targetPathPrefix) {
                var unloading = nocUI.loading();

                $q.all([
                    workingEnvironmentResource.getWorkingEnvironmentByPublicId(nocHttp, $state.params.sourceId, sourcePathPrefix, 'status,svmName,capacityFeatures,cloudSyncProperties'),
                    workingEnvironmentResource.getWorkingEnvironmentByPublicId(nocHttp, $state.params.targetId, targetPathPrefix, 'status,svmName,capacityFeatures')
                ])
                    .then(_.spread(function (sourceWorkingEnvironment, targetWorkingEnvironment) {
                        $scope.wizardController.sourceWorkingEnvironment = sourceWorkingEnvironment;
                        $scope.wizardController.targetWorkingEnvironment = targetWorkingEnvironment;
                        $scope.getInterclusterLifs();
                    }))
                    .catch(function (errorResponse) {
                        var prefix = pidgeon.serverErrorPrefix + "retrieving working environments details for replication";
                        //the error is pushed to the DataFabricController's pidgeon so it will be shown in the gallery and not in this hidden step
                        $scope.pushError(prefix + (errorResponse ? ": " + errorResponse.messageForDisplay() : ""), errorResponse ? errorResponse : {}, "DataFabricController", "replicationWizard");
                        $scope.exit();
                    })
                    .finally(unloading);

            } else {
                $scope.exit();
            }

        };

        $scope.next = function () {
            $state.go("snap-mirror-wizard.init" + $scope.wizardController.nextStep["init"]);
        };

        $scope.initialize();
    });
}());
