(() => {
    'use strict';

    /* Gets a Capacity object like {size: 210230130, unit: "Byte"} and formats it nicely to "2.12 TB".
     * Number of decimal digits depends on decimalDigits, default is 0.
     * If the capacity size is small compared to the passed unit, and rounded to zero with the restrictions of
     * decimalDigits parameter, displays "< 0.01 " or "< 0.001 " etc according to decimalDigits instead
     * of "0".
     */
    angular.module('noc').filter('normalizedCapacity', function(numberFilter, nocUtil) {
        return  function (input, decimalDigits, hideZeroDecimal, conversionUnit, sizeOnly) {
            var createNumberPart = function () {
                // If size was rounded to zero display "< 0.0*1"
                if ((input.size !== 0) && (nocUtil.floorToDecimalPoint(normalized.size, decimalDigits) === 0)) {
                    return "< " + Math.pow(10, -decimalDigits);
                } else {
                    var truncateDecimal = hideZeroDecimal ? isZeroDecimal(normalized.size, decimalDigits) : false;
                    return numberFilter(normalized.size, truncateDecimal ? 0 : decimalDigits);
                }
            };
            //initializing un-initialized parameters
            if (!input || !_.isNumber(input.size)) input = {size: 0, unit: conversionUnit ? conversionUnit : "KB"};
            decimalDigits = decimalDigits || 0;

            var normalized = conversionUnit ? nocUtil.convertTo(input, conversionUnit) : nocUtil.normalizeCapacity(input);
            return createNumberPart() + (sizeOnly ? "" : ' ' + normalized.unit);
        };
    });

    var isZeroDecimal= function(number, decimalDigits) {
        var power = Math.pow(10, decimalDigits);
        return Math.floor(number * power) % power === 0;
    };

    angular.module('noc').filter('removeDecimalZeros', function() {
        return function(input) {
            return parseFloat(input).toString();
        };
    });

    /*
     * gets a number like 5.15 and formats it according to way we want to show in the gallery
     */
    angular.module('noc').filter('galleryCapacity', function(numberFilter) {
        return function(input) {
            return input < 0.01 ? '< 0.01' : numberFilter(input, (input * 100 % 100) >= 1  ? 2 : 0);
        };
    });

    angular.module('noc').filter('allocatedAggregatesCapacity', function(nocUtil) {
        return function(aggregates) {
            var sum = {size: 0, unit: "Byte"};
            _.forEach(aggregates, function(aggregate){
                _.forEach(aggregate.providerVolumes, function(vol) {
                    var normalizedSize = nocUtil.convertTo(vol.size, "Byte");
                    sum.size += normalizedSize.size;
                });
            });
            return sum;
        };
    });

    angular.module('noc').filter('volumePermissionToString', () => {
        const map = {
            'full_control': 'Full Control',
            'change': 'Read/Write',
            'read': 'Read',
            'no_access': 'No Access'
        };

        return (permission) => map[permission] || permission;
    });
})();
