(() => {
    'use strict';

    $.ui.dialog.prototype._allowInteraction = function(e) {
        return !!$(e.target).closest('.ui-dialog, .ui-datepicker, .select2-drop').length;
    };

    //Create NOC namespace object
    var noc = {};

    //Create noc module
    noc.ng = angular.module("noc", ["noc.components", "ngAnimate", "ui.utils", "ui.grid", "ui.grid.pinning", "ui.grid.autoResize", "ui.grid.selection", "ui.grid.resizeColumns", "ui.grid.moveColumns", "ui.grid.saveState", "ui.router", "ui.select2", "angularMoment", "ngCookies", "ngSanitize", "ui.bootstrap", 'ui.select']);

    //Create components module
    angular.module("noc.components", []);

    noc.ng.config(function (uiSelectConfig) {
        uiSelectConfig.theme = 'bootstrap';
    });

    //Preventing console errors triggered by unhandled $q rejections. See here: https://docs.angularjs.org/guide/migration#migrate1.5to1.6-ng-services-$q
    noc.ng.config(['$qProvider', function ($qProvider) {
        $qProvider.errorOnUnhandledRejections(false);
    }]);

    noc.ng.config(($httpProvider, $provide, $stateProvider) => {
        $provide.decorator('$controller', function($delegate, nocUIFactory,pidgeonFactory,nocHttpFactory,nocUtil){
            return function (className, locals) {
                if(_.isString(className)) {
                    locals.nocUI = nocUIFactory.spawn(className, locals.$scope);

                    if (locals.$scope && nocUtil.isMessageable(locals.$scope)){
                        locals.pidgeon = pidgeonFactory.spawn(className, locals.$scope);
                        locals.nocHttp = nocHttpFactory.spawn(locals.pidgeon, locals.nocUI);
                    }
                }

                return $delegate.apply(this, arguments);
            };
        });

        // ***********************************************************************************************************************************
        // For Angular Component:
        // In order to allow nocHttp injection to components' controller, the above code has to be replaced with the below code.
        // This is currently not implemented because ideally components shouldn't deal with http requests.
        // Also take into consideration that in order for Pigeon to appear, the component's template has to ng-include 'messages-container.html'
        // ***********************************************************************************************************************************
        // $provide.decorator('$controller', function($delegate, nocUIFactory, pidgeonFactory, nocHttpFactory, nocUtil, Messageable){
        //     return (className, locals) => {
        //         if(_.isString(className) || (_.isFunction(className) && className.name === 'controller')){
        //             locals.nocUI = nocUIFactory.spawn(className, locals.$scope);
        //             //In case it is a component's controller, define the scope as Messageable
        //             if (_.isFunction(className) && className.name === 'controller'){
        //                 Messageable.call(locals.$scope);
        //             }
        //             if (locals.$scope && nocUtil.isMessageable(locals.$scope)) {
        //                 locals.pidgeon = pidgeonFactory.spawn(className, locals.$scope);
        //                 locals.nocHttp = nocHttpFactory.spawn(locals.pidgeon, locals.nocUI);
        //             }
        //         }
        //
        //         return $delegate.apply(this, arguments);
        //     };
        // });

        $provide.decorator('GridOptions', function($delegate) {
            var gridOptions;
            gridOptions = angular.copy($delegate);
            gridOptions.initialize = function(options) {
                var defaultOptions = {
                    rowHeight: 58,
                    enableColumnResizing: true,
                    enableSorting: true,

                    showGridFooter: false,

                    multiSelect: false,
                    enableSelectAll: false,
                    enableRowSelection: false,
                    enableRowHeaderSelection: false,
                    modifierKeysToMultiSelect: false,

                    saveScroll: true,
                    saveOrder: true,
                    saveVisible: true,
                    saveSort: false,
                    saveWidths: false,
                    saveFocus: false,

                    enableColumnMenus: false
                };

                _.defaults(options, defaultOptions);
                return $delegate.initialize(options);
            };
            return gridOptions;
        });

        $provide.decorator('timepickerDirective', function($delegate) {
            $delegate[0].templateUrl = "assets/templates/timepicker/timepicker.html";
            return $delegate;
        });

        const doNotMessWith = ['ui-grid']; //Don't mess with the response if the url contains one of these
        const isSaint = (url) => {
            return _.some(doNotMessWith, (saint) => url.match(saint));
        };

        //Reject non json responses if they are expected
        $provide.factory('nonJsonResponseRejector', ($q, NocErrorResponse, $log) => {
            return {
                response: (response) => {
                    const expectingJson = response.data && //Response contains data
                        response.config.headers['Accept'] === 'application/json' && //Expected json
                        !(/html$/).test(response.config.url) && //Not requesting an html
                        !(/svg$/).test(response.config.url) && //Not requesting an svg
                        !isSaint(response.config.url);

                    if (expectingJson) {
                        const responseContentType = response.headers('content-type');
                        if (responseContentType !== 'application/json') {
                            const message = "Could not handle response. Expected 'application/json' but got '" + responseContentType + "'";
                            $log.error(message + '. Request was to ' + response.config.url + '. Got response with the following headers: ' + JSON.stringify(response.headers()));

                            return $q.reject({data: new NocErrorResponse({message})});
                        }
                    }

                    return response;
                }
            };
        });

        $provide.factory('v8Interceptor', function($q, $templateCache, nocBaseUrl, nocUIMain, $rootScope, NocErrorResponse, $injector, Constants) {
            return {
                'request': function(config) {
                    if($templateCache.get(config.url) && $templateCache.get(config.url).length > 0) {
                        return config;
                    }

                    if ((config.url.match(/((http|https):\/\/|\/?assets\/images)/) === null &&
                         config.url.match(/((http|https):\/\/|\/?assets\/templates)/) === null) && !isSaint(config.url) && !config.__retryAttempt) {
                        if (config.url.length && config.url[0] !== '/') config.url = '/' + config.url;
                        config.url = nocBaseUrl + config.url;
                    }
                    if (config.url.match(/.*\/api\/.*/) !== null){
                        //disable IE ajax request caching
                        config.headers['If-Modified-Since'] = '0';
                        config.headers['Cache-Control'] = 'no-cache';
                        config.headers['Pragma'] = 'no-cache';
                    }

                    return config;
                },
                // optional method
                'responseError': function(rejection) {
                    if(rejection.status === 401 && $rootScope.appState === "loaded"){
                        nocUIMain.reloadSilently(true);
                    }

                    if(Constants.system.retriesEnabled && rejection.status === 400 && !rejection.data && !rejection.config.__retryAttempt){
                        rejection.config.__retryAttempt = true;
                        return $injector.get("$http")(rejection.config);
                    }else{
                        if(rejection.status === 0 && !$rootScope.expectNetworkFailures){
                            rejection.data = new NocErrorResponse({message: "Unable to send request. Possibly due to network failure. Please check connection."});
                            rejection.data.status = "NETWORK_FAILURE";
                        }else{
                            rejection.data = new NocErrorResponse(rejection.data);
                        }

                        return $q.reject(rejection);
                    }
                }
            };
        });

        $provide.factory('unauthorizedInterceptor', ($q, $rootScope) => {
            return {
                'responseError': (rejection) => {
                    if (rejection.status === 403 && _.get($rootScope, 'supportServices.portalService.usePortalAuthentication')) {
                        $rootScope.appState = 'unauthorized';
                        $rootScope.checkAndSetState('unauthorized');
                    }

                    return $q.reject(rejection);
                }
            };
        });

        $provide.factory('asyncInterceptor', function(nocLongPollingService, $rootScope, nocLocalStorage) {
            var asyncDisabled = nocLocalStorage.DISABLE_ASYNC_RESPONDER.restore(false);
            if (!asyncDisabled) {
                nocLocalStorage.DISABLE_ASYNC_RESPONDER.save(false);
            }

            return {
                'request': function(config) {
                    var isAsync = $rootScope.appState === 'loaded' && !config.url.match(/\/(auth|occm|audit|async)\//);

                    if (!asyncDisabled && isAsync) {
                        config.headers[nocLongPollingService.REQUEST_ID_REQUEST_HEADER] = nocLongPollingService.getClientId();
                    }

                    return config;
                },

                'response': function(response) {
                    if (response.headers(nocLongPollingService.REQUEST_ID_RESPONSE_HEADER)) {
                        return nocLongPollingService.getAsyncResponse(response);
                    } else {
                        return response;
                    }
                }
            };
        });

        $provide.factory('validationInterceptor', function(NocErrorResponse, $q) {
            return {
                "response" : function(response) {
                    // in case of validation error
                    if (response.data && !_.isUndefined(response.data.result) && !_.isUndefined(response.data.reasons)) {
                        if (!response.data.result) {
                            response.data = new NocErrorResponse({message: response.data.reasons.join(', ')});
                            return $q.reject(response);
                        }
                    }
                    return response;
                }
            };
        });

        //for each request, tell the server we want to receive json
        $httpProvider.defaults.headers.common["Accept"] = "application/json";

        //for each request, tell the server we are sending json
        $httpProvider.defaults.headers.common["Content-Type"] = "application/json";

        $httpProvider.interceptors.push('validationInterceptor');

        $httpProvider.interceptors.push('asyncInterceptor');

        $httpProvider.interceptors.push('v8Interceptor');

        $httpProvider.interceptors.push('unauthorizedInterceptor');

        //Extracts oncloud-request-id header from POST|PUT|DELETE requests and starts polling the audit to check when they are completed
        $httpProvider.interceptors.push('statusRequestInterceptor');

        //Set up route change authorization
        var originalStateF = $stateProvider.state;
        $stateProvider.state = function(name, state){
            if(!_.includes(name,".")){
                state.resolve = state.resolve || {};
            }
            originalStateF.apply(this, arguments);
        };
    });

    _.templateSettings = {
        'interpolate': /{{([\s\S]+?)}}/g
    };

    //default select2 configuration.
    noc.ng.run(function(uiSelect2Config,$rootScope,nocSelect2Options) {
        uiSelect2Config.minimumResultsForSearch = 6;
        uiSelect2Config.dropdownAutoWidth = true;
        uiSelect2Config.width = 'copy';

        //shared select2 options
        $rootScope.nocSelect2Options = nocSelect2Options;
    });

    return noc;
})();
