(() => {
    'use strict';

    angular.module('noc').factory('CloudProviderAccountResource', (workingEnvironmentResource) => {
        const resource = {};

        resource.createAccount = (nocHttp, provider, accountName, providerKeys, vsaList = null, dontShowPidgeon) => {
            const pidgeonString = dontShowPidgeon ? '' : 'creating the ' + provider + ' account';

            return nocHttp.post({
                url: '/accounts/' + provider,
                data: {
                    accountName,
                    providerKeys,
                    vsaList
                }
            }, pidgeonString, nocHttp.BLOCK);
        };

        resource.getAccounts = (nocHttp) => {
            return nocHttp.get('/accounts', 'retrieving Accounts', nocHttp.LOADING);
        };

        resource.getFormattedAccounts = (nocHttp) => {
            return resource.getAccounts(nocHttp)
                .then((response) => {
                    const awsAccounts = response.awsAccounts;
                    const azureAccounts = response.azureAccounts;
                    const gcpAccounts = response.gcpStorageAccounts;
                    const nssAccounts = response.nssAccounts;
                    _.forEach(awsAccounts, (account) => {
                        account._provider = 'aws';
                        account._prettyAccountType = 'AWS Keys';
                        account._roleName = _.get(account, 'assumeRole.roleName');
                    });
                    _.forEach(azureAccounts, (account) => {
                        account._provider = 'azure';
                        account._prettyAccountType = 'Azure Keys';
                    });
                    _.forEach(gcpAccounts, (account) => {
                        account._provider = 'gcp';
                        account._prettyAccountType = 'GCP Keys';
                    });
                    _.forEach(nssAccounts, (account) => {
                        account._provider = 'nss';
                        account._prettyAccountType = 'NSS';
                    });

                    return ((awsAccounts.concat(azureAccounts)).concat(gcpAccounts)).concat(nssAccounts);
                });
        };

        resource.editAccount = (nocHttp, provider, accountId, accountName, providerKeys, vsaList = null) => {
            return nocHttp.put({
                url: '/accounts/' + provider + '/' + accountId,
                data: {
                    accountName,
                    providerKeys,
                    vsaList
                }
            }, '', nocHttp.BLOCK);
        };

        resource.deleteAccount = (nocHttp, accountId) => {
            return nocHttp.delete('/accounts/' + accountId, 'deleting Account', nocHttp.BLOCK);
        };

        resource.getUnregisteredVsas = (nocHttp) => {
            return workingEnvironmentResource.getWorkingEnvironments(nocHttp, null, 'status,supportRegistrationInformation')
                .then((vsas) => {
                    const unregisteredVsas = _.filter(vsas, (vsa) => {
                        return vsa.status.status === 'on' && vsa.workingEnvironmentType !== 'ON_PREM';
                    });

                    return _.reject(unregisteredVsas, (vsa) => {
                        return _.get(vsa, 'supportRegistrationInformation[0].supportRegistrationStatus') === 'registered' ||
                               _.get(vsa, 'supportRegistrationInformation[1].supportRegistrationStatus') === 'registered';
                    });
                });
        };

        resource.registerVsaToNssAccount = (nocHttp, accountId, vsaPublicId) => {
            return nocHttp.put({
                url: `/accounts/nss/set-vsa-list/${accountId}`,
                data: {
                    vsaList: [vsaPublicId]
                }
            }, '', nocHttp.BLOCK);
        };

        return resource;
    });
})();
