(function () {
    "use strict";

    angular.module("noc").factory("replicationResource", function (workingEnvironmentStatuses, nocUtil, $filter, Constants, nocPoliciesService, nocPoller, $rootScope) {
        var replicationResource = {};

        replicationResource.moment = moment;

        replicationResource.initRelationships = function (replicationStatusResponse) {
            var unavailable = "";
            _.forEach(replicationStatusResponse, function (relationship) {
                _.forEach(_.keys(workingEnvironmentStatuses.relationshipActionsSourceAndTargetToWorkingEnvironmentStates), function (operation) {
                    relationship._allowedRelationshipOperations = relationship._allowedRelationshipOperations || {};
                    relationship._allowedRelationshipOperations[operation] = workingEnvironmentStatuses.isRelationshipOperationAllowed(operation, relationship.source.workingEnvironmentStatus, relationship.destination.workingEnvironmentStatus, relationship.policyType);
                });

                relationship._lagDuration = {
                    sortValue: relationship.lagTime ? relationship.lagTime.length : "",
                    viewValue: relationship.lagTime ? _.capitalize(replicationResource.moment.duration((-1) * relationship.lagTime.length, relationship.lagTime.unit).humanize()) : unavailable
                };

                relationship._health = relationship.unhealthyReason || "Healthy";

                relationship.lastTransferInfo = relationship.lastTransferInfo || {};
                relationship.lastTransferInfo._transferEnded = relationship.lastTransferInfo.transferEnded ? $filter('dateAndTime')(replicationResource.moment.unix(relationship.lastTransferInfo.transferEnded)) : unavailable;
                relationship.lastTransferInfo._transferSize = relationship.lastTransferInfo.transferSize ? $filter('normalizedCapacity')(relationship.lastTransferInfo.transferSize, 2, true) : unavailable; //Add normalized size field to every volume to optimize filtering and display.
                relationship.lastTransferInfo._transferDuration = relationship.lastTransferInfo.transferDuration ? _.capitalize(replicationResource.moment.duration(relationship.lastTransferInfo.transferDuration, relationship.lastTransferInfo.transferDuration.unit).humanize()) : unavailable;

                relationship._maxTransferRate = (relationship.maxTransferRate && relationship.maxTransferRate.size > 0) ? $filter('normalizedCapacity')(relationship.maxTransferRate, 2, true) + "/s" : "Unlimited";

                relationship._totalTransferTime = {
                    sortValue: relationship.totalTransferTime && relationship.totalTransferTime.length,
                    viewValue: relationship.totalTransferTime ? _.capitalize(replicationResource.moment.duration(relationship.totalTransferTime.length, relationship.totalTransferTime.unit).humanize()) : unavailable
                };

                relationship._totalTransferSize = {
                    sortValue: nocUtil.convertTo(relationship.totalTransferSize, 'Byte').size,
                    viewValue: relationship.totalTransferSize ? $filter('normalizedCapacity')(relationship.totalTransferSize, 2, true) : unavailable
                };

                relationship._newestSnapshotCreated = {
                    sortValue: relationship.newestSnapshotCreated,
                    viewValue: relationship.newestSnapshotCreated ? $filter('dateAndTime')(replicationResource.moment.unix(relationship.newestSnapshotCreated)) : unavailable
                };

                relationship._relationshipProgress = relationship.relationshipProgress ? $filter('normalizedCapacity')(relationship.relationshipProgress, 2, true) : unavailable;
                relationship._schedule = _.isEmpty(relationship.schedule) ? (_.toLower(relationship.destination.workingEnvironmentStatus) === 'on' ? Constants.emptyReplicationScheduleDescription : unavailable) : relationship.schedule;
                relationship._volumeUsedSize = $filter('normalizedCapacity')(relationship.volumeUsedSize, 2, true);
                relationship._volumeCapacityTier = $filter('normalizedCapacity')(relationship.volumeCapacityTier, 2, true);

                // Add readable status for tooltip in the grid
                relationship.source._normalizedState = nocUtil.normalizeStatus(relationship.source.workingEnvironmentStatus);
                relationship.source._stateInfo = _.cloneDeep(workingEnvironmentStatuses.stateInfoMap[relationship.source._normalizedState]) || /* istanbul ignore next: trivial */ {};

                relationship.destination._normalizedState = nocUtil.normalizeStatus(relationship.destination.workingEnvironmentStatus);
                relationship.destination._stateInfo = _.cloneDeep(workingEnvironmentStatuses.stateInfoMap[relationship.destination._normalizedState]) || /* istanbul ignore next: trivial */ {};
                relationship._policyName = {
                    displayName: nocPoliciesService.getPolicyInfo(relationship.policy).displayName,
                    tooltip: relationship.policy
                }; //convert the name in any case
            });

            return replicationStatusResponse;
        };

        // This function should get workingEnvironment OR tenantId.
        // If workingEnvironment is passed, tenantId is ignored.
        replicationResource.getReplicationStatus = function (nocHttp, tenantOrWsId, workingEnvironment) {
            var weString = workingEnvironment ? "/" + workingEnvironment.publicId : "";
            var opts = {url: "/replication/status" + weString,
                        params: {},
                        headers: {
                                    "X-Occm-Bypass-Cache": true
                                }};
            if (tenantOrWsId) opts.params.tenantId = tenantOrWsId;

            return nocHttp.get(opts,
                "retrieving replication status" + (workingEnvironment ? " for working environment " + workingEnvironment.name : ""),
                nocHttp.LOADING)
                .then(replicationResource.initRelationships);
        };

        //===========================================================================================================================
        /*
         Replication Status polling

         */
        replicationResource.REPLICATION_STATUS_GET_EVENT = "replicationStatusGetSuccess";
        replicationResource.REPLICATION_STATUS_GET_ERROR_EVENT = "replicationStatusGetError";

        replicationResource.startReplicationStatusPolling = function (workingEnvironment) {
            var opts = {method: "GET", url: "/replication/status/" + workingEnvironment.publicId, params: {}};
            return nocPoller.start(opts, 5 * 60 * 1000,
                function (replicationStatusResponse) {
                    replicationResource.initRelationships(replicationStatusResponse);
                    $rootScope.$broadcast(replicationResource.REPLICATION_STATUS_GET_EVENT, {
                        workingEnvironment: workingEnvironment,
                        replicationStatusResponse: replicationStatusResponse
                    });
                },
                function (error) {
                    $rootScope.$broadcast(replicationResource.REPLICATION_STATUS_GET_ERROR_EVENT, {
                        workingEnvironment: workingEnvironment,
                        error: error
                    });
                }, null, true);
        };

        replicationResource.REPLICATION_STATUS_GET_ALL_EVENT = "replicationStatusGetAllSuccess";
        replicationResource.REPLICATION_STATUS_GET_ALL_ERROR_EVENT = "replicationStatusGetAllError";

        replicationResource.startReplicationStatusAllPolling = function (tenantOrWsId) {
            var opts = {method: "GET", url: "/replication/status", params: {tenantId: tenantOrWsId}};

            return nocPoller.start(opts, 5 * 60 * 1000,
                function (replicationStatusResponse, status, headers, config) {
                    replicationResource.initRelationships(replicationStatusResponse);
                    $rootScope.$broadcast(replicationResource.REPLICATION_STATUS_GET_ALL_EVENT, {
                        replicationStatusResponse: replicationStatusResponse,
                        tenantId: config.params.tenantId
                    });
                },
                function (error, status, headers, config) {
                    $rootScope.$broadcast(replicationResource.REPLICATION_STATUS_GET_ALL_ERROR_EVENT, {
                        error: error,
                        tenantId: config.params.tenantId
                    });
                }, null, true);
        };

        replicationResource.getInterclusterLifs = function (nocHttp, workingEnvironmentId, peerWorkingEnvironmentId) {
            return nocHttp.get({
                url: "/replication/intercluster-lifs",
                params: {
                    "workingEnvironmentId": workingEnvironmentId,
                    "peerWorkingEnvironmentId": peerWorkingEnvironmentId
                }
            });
        };

        replicationResource.getSchedules = function (nocHttp, targetPublicId) {
            return nocHttp.get("/replication/schedules/" + targetPublicId,
                "retrieving schedules for target volume", nocHttp.LOADING);
        };

        replicationResource.updateRelationship = function(nocHttp, destination, updatedData, description){
            return nocHttp.put({
                url: "/replication/" + destination.workingEnvironmentId + "/" + destination.svmName + "/" + destination.volumeName,
                data: updatedData
            }, description, nocHttp.BLOCK);

        };

        return replicationResource;
    });
}());