(() => {
    'use strict';

    angular.module('noc').service('usersResource', () => {
        const service = {};

        service.getUser = (nocHttp, publicId) => {
            return nocHttp.get('/users/' + publicId, 'retrieving user data', nocHttp.BLOCK)
                .then((user) => {
                    if (_.get(user, 'cloudSyncLicenseInformation.licenseExpirationDate') && user.cloudSyncLicenseInformation.licenseExpirationDate < moment()) {
                        user.cloudSyncLicenseInformation._cloudSyncLicenseExpired = true;
                    }

                    return user;
                });
        };

        service.getUsers = (nocHttp) => {
            return nocHttp.get('/users', 'retrieving users information', nocHttp.LOADING);
        };

        service.addUser = (nocHttp, userToAdd) => {
            const userData = {
                email: userToAdd.email,
                roleId: userToAdd.role.publicId
            };
            if (userToAdd.tenant) {
                userData.tenantId = userToAdd.tenant.publicId;
            }

            return nocHttp.post({
                url: '/users/add-user',
                data: userData
            }, 'adding user', nocHttp.BLOCK);
        };

        service.grantRevokeWePermissions = (nocHttp, userId, grantPermission) => {
            return nocHttp.post({
                url: `/users/${userId}/working-environments-grant-revoke`,
                data: grantPermission
            }, "changing user's working environment permissions", nocHttp.BLOCK);
        };

        service.removeUser = (nocHttp, userId) => {
            return nocHttp.delete({
                url: `/users/${userId}/remove-user`
            }, 'removing user', nocHttp.BLOCK);
        };

        return service;
    });
})();
