(() => {
    'use strict';
    angular.module('noc').config(($urlRouterProvider) => {
        $urlRouterProvider.when('/volume-view', '/volume-view/volumes');

        $urlRouterProvider.otherwise('/');
    });

    angular.module('noc').config(($stateProvider) => {
        function topState(name, url, tabs) {
            $stateProvider.state(name, {
                url,
                templateUrl: 'assets/templates/tabs-main.html',
                controller: 'TabsController',
                onEnter(autoCapacityService, occmUpdateService) {
                    occmUpdateService.showUpdatesDialog('updatesAvailable', false);
                },
                resolve: {
                    layoutOptions(nocCurrentUserService) {
                        return {
                            basePath: name,
                            tabs: _.filter(tabs, (tab) => {
                                return nocCurrentUserService.isAllowed(tab.permission);
                            })
                        };
                    }
                }
            });
        }

        //////////////////////////////////////////////////////////////////////////////////////////////
        //
        //  Main
        //
        //////////////////////////////////////////////////////////////////////////////////////////////

        const mainTabs = [
            {
                tabName: 'working-environments',
                title: 'Working Environments',
                permission: 'we:get'
            },
            {
                tabName: 'replication-status',
                title: 'Replication Status',
                permission: 'we:get'
            },
            {
                tabName: 'k8s-clusters',
                title: 'Kubernetes Clusters',
                permission: 'k8s:*'
            },
            {
                tabName: 'users',
                title: 'Users',
                permission: 'user:get',
                noShow: true
            },
            {
                tabName: 'tools',
                title: 'Tools',
                permission: 'backup-restore:set',
                noShow: true
            },
            {
                tabName: 'asup',
                title: 'Support Dashboard',
                permission: 'asup:set',
                noShow: true
            },
            {
                tabName: 'settings',
                title: 'Settings',
                permission: 'configuration:set',
                noShow: true
            },
            {
                tabName: 'view-selection',
                title: 'View Selection',
                permission: 'managed_volume',
                noShow: true
            },
            {
                tabName: 'update',
                title: 'Cloud Manager Software Update',
                permission: 'patch:*',
                noShow: true
            },
            {
                tabName: 'encryption',
                title: 'Encryption',
                permission: 'encryption:*',
                noShow: true
            },
            {
                tabName: 'tls-setup',
                title: 'HTTPS Setup',
                permission: 'key_store:generate',
                noShow: true
            },
            {
                tabName: 'ad-setup',
                title: 'Active Directory Setup',
                permission: 'activedirectory:*',
                noShow: true
            },
            {
                tabName: 'tenants',
                title: 'Tenants',
                permission: 'tenant:update',
                noShow: true
            },
            {
                tabName: 'timeline',
                title: 'Timeline'
            }
        ];

        $stateProvider.state('unauthorized', {
            url: '/unauthorized',
            templateUrl: 'assets/templates/unauthorized.html'
        });

        // An intermediate state, when the entry point - volume view or good'ol OCCM - is unknown
        $stateProvider.state('i', {
            url: '/',
            template: '<div></div>',
            controller: 'RouteResolverController'
        });

        topState('main', '/main?slTenantId&nopoll', mainTabs);

        $stateProvider.state('main.tools', {
            tenantsTopMenu: 'hide',
            url: '/tools',
            templateUrl: 'assets/templates/standalone/tools.html'
        });

        $stateProvider.state('main.update', {
            tenantsTopMenu: 'hide',
            url: '/update',
            templateUrl: 'assets/templates/standalone/update.html'
        });

        $stateProvider.state('main.encryption', {
            tenantsTopMenu: 'hide',
            url: '/encryption/',
            templateUrl: 'assets/templates/tabs/encryption/encryption.html'
        });

        $stateProvider.state('main.tls-setup', {
            tenantsTopMenu: 'hide',
            url: '/https-setup',
            templateUrl: 'assets/templates/tabs/https-setup/tls-certificate.html'
        });

        $stateProvider.state('main.ad-setup', {
            tenantsTopMenu: 'hide',
            url: '/ad-setup',
            templateUrl: 'assets/templates/standalone/ad-setup.html'
        });

        $stateProvider.state('main.asup', {
            tenantsTopMenu: 'hide',
            url: '/asup',
            controller: 'AsupController',
            templateUrl: 'assets/templates/standalone/asup.html'
        });

        $stateProvider.state('main.settings', {
            tenantsTopMenu: 'hide',
            url: '/settings',
            templateUrl: 'assets/templates/standalone/settings.html'
        });

        $stateProvider.state('main.view-selection', {
            tenantsTopMenu: 'hide',
            url: '/view-selection',
            templateUrl: 'assets/templates/standalone/view-selection.html'
        });

        $stateProvider.state('main.users', {
            abstract: true,
            url: '/users',
            controller: 'UsersController',
            templateUrl: 'assets/templates/tabs/abstractTab.html'
        });

        $stateProvider.state('main.users.index', {
            tenantsTopMenu: 'hide',
            url: '',
            templateUrl: 'assets/templates/tabs/users/users-table.html',
            controller: 'UsersTableController'
        });

        $stateProvider.state('main.users.add', {
            tenantsTopMenu: 'hide',
            url: '/add',
            templateUrl: 'assets/templates/tabs/users/users-add.html',
            controller: 'UsersAddController',
            controllerAs: 'ctrlUserAdd'
        });

        $stateProvider.state('main.users.new', {
            tenantsTopMenu: 'hide',
            url: '/new',
            templateUrl: 'assets/templates/tabs/users/users-new-edit.html',
            controller: 'UsersNewEditController'
        });

        $stateProvider.state('main.users.settings', {
            tenantsTopMenu: 'hide',
            url: '/:userId/settings',
            templateUrl: 'assets/templates/tabs/users/user-settings.html',
            controller: 'UserSettingsController'
        });

        $stateProvider.state('main.users.edit', {
            tenantsTopMenu: 'hide',
            url: '/:userId/edit',
            templateUrl: 'assets/templates/tabs/users/users-new-edit.html',
            controller: 'UsersNewEditController'
        });

        $stateProvider.state('main.timeline', {
            tenantsTopMenu: 'hide',
            url: '/timeline',
            controller: 'AuditController',
            templateUrl: 'assets/templates/tabs/audit.html',
            params: {
                we: null,
                fromWe: null,
                weId: null
            }
        });

        $stateProvider.state('main.tenants', {
            abstract: true,
            url: '/tenants',
            controller: 'TenantsController',
            templateUrl: 'assets/templates/tabs/abstractTab.html'
        });

        $stateProvider.state('main.tenants.index', {
            tenantsTopMenu: 'hide',
            url: '',
            templateUrl: 'assets/templates/tabs/tenants/tenants-gallery.html',
            controller: 'TenantsGalleryController'
        });

        $stateProvider.state('main.tenants.new', {
            tenantsTopMenu: 'hide',
            url: '/new',
            templateUrl: 'assets/templates/tabs/tenants/tenants-new-edit.html',
            controller: 'TenantsNewEditController',
            resolve: {tenantsMode: _.constant('new')}
        });

        $stateProvider.state('main.tenants.edit', {
            tenantsTopMenu: 'hide',
            url: '/:tenantId/edit',
            templateUrl: 'assets/templates/tabs/tenants/tenants-new-edit.html',
            controller: 'TenantsNewEditController',
            resolve: {tenantsMode: _.constant('edit')}
        });

        $stateProvider.state('main.replication-status', {
            abstract: true,
            url: '/replication-status',
            controller: 'ReplicationStatusTabController',
            templateUrl: 'assets/templates/tabs/abstractTab.html'
        });

        $stateProvider.state('main.replication-status.index', {
            tenantsTopMenu: 'selectable',
            url: '',
            templateUrl: 'assets/templates/tabs/replication-status/replication-status-tab.html'
        });

        $stateProvider.state('main.working-environments', {
            abstract: true,
            url: '/working-environments',
            controller: 'WorkingEnvironmentsController',
            templateUrl: 'assets/templates/tabs/abstractTab.html'
        });

        $stateProvider.state('main.working-environments.index', {
            tenantsTopMenu: 'selectable',
            url: '',
            templateUrl: 'assets/templates/tabs/working-environments/data-fabric.html',
            controller: 'DataFabricController'
        });

        $stateProvider.state('main.working-environments.monitoring', {
            abstract: true,
            url: '/:workingEnvironmentId/monitoring?gadget&type&provider&selectedVolumeName&selectedVolumeSVM',
            controller: 'MonitoringController',
            templateUrl: 'assets/templates/tabs/abstractTab.html'
        });

        $stateProvider.state('main.working-environments.monitoring.index', {
            url: '',
            templateUrl: 'assets/templates/tabs/working-environments/working-environments-monitoring.html'
        });

        $stateProvider.state('main.working-environments.monitoring.add-volume', {
            url: '/add-volume?from&syncToS3',
            templateUrl: 'assets/templates/tabs/working-environments/add-volume.html',
            controller: 'AddVolumeController'
        });

        $stateProvider.state('main.working-environments.monitoring.cost', {
            url: '/cost',
            templateUrl: 'assets/templates/tabs/working-environments/cost.html',
            controller: 'CostController'
        });

        $stateProvider.state('main.working-environments.monitoring.instances', {
            url: '/instances',
            templateUrl: 'assets/templates/tabs/working-environments/instances.html'
        });

        $stateProvider.state('main.working-environments.monitoring.add-disks-aggregate', {
            url: '/:aggregateName/add-disks-aggregate',
            templateUrl: 'assets/templates/tabs/working-environments/add-disks-aggregate.html',
            controller: 'AddDisksAggregateController'
        });

        $stateProvider.state('main.working-environments.monitoring.create-volume-aggregate', {
            url: '/:aggregateName/:aggregateProviderVolumeType/create-volume-aggregate?syncToS3',
            templateUrl: 'assets/templates/tabs/working-environments/create-volume-aggregate.html',
            controller: 'CreateVolumeAggregateController'
        });

        $stateProvider.state('main.working-environments.monitoring.replication-status', {
            url: '/replication-status',
            templateUrl: 'assets/templates/tabs/working-environments/replication-status-monitoring.html',
            controller: 'ReplicationStatusMonitoringController'
        });

        $stateProvider.state('main.working-environments.monitoring.sync-s3-status', {
            url: '/sync-s3-status',
            templateUrl: 'assets/templates/tabs/working-environments/sync-s3-status-monitoring.html',
            controller: 'SyncS3StatusMonitoringController'
        });

        //////////////////////////////////////////////////////////////////////////////////////////////
        //
        //  Kubernetes Clusters
        //
        //////////////////////////////////////////////////////////////////////////////////////////////

        $stateProvider.state('main.k8s-clusters', {
            abstract: true,
            url: '/kubernetes-clusters',
            controller: 'K8sClustersController',
            template: '<div ui-view></div>'
        });

        $stateProvider.state('main.k8s-clusters.index', {
            onEnter: () => {
                 $('body').addClass('k8s-clusters-background');
            },
            onExit: () => {
                $('body').removeClass('k8s-clusters-background');
            },
            url: '',
            templateUrl: 'assets/templates/k8s-clusters/first-k8s-cluster.html'
        });

        $stateProvider.state('main.k8s-clusters.discover-k8s-cluster', {
            url: '/discover-kubernetes-cluster',
            templateUrl: 'assets/templates/k8s-clusters/discover-k8s-cluster.html',
            controller: 'DiscoverK8sClusterController'
        });

        $stateProvider.state('main.k8s-clusters.k8s-cluster-settings', {
            url: '/cluster-settings',
            templateUrl: 'assets/templates/k8s-clusters/cluster-settings.html',
            controller: 'KubernetesClusterSettingsController'
        });

        //////////////////////////////////////////////////////////////////////////////////////////////
        //
        //  Cloud Provider Accounts
        //
        //////////////////////////////////////////////////////////////////////////////////////////////

        $stateProvider.state('cloud-provider-accounts', {
            abstract: true,
            url: '/cloud-provider-accounts',
            controller: 'AccountsController',
            template: '<div ui-view></div>'
        });

        $stateProvider.state('cloud-provider-accounts.add', {
            url: '',
            controller: 'CloudProviderAccountsAddAccountController',
            templateUrl: 'assets/templates/cloud-provider-accounts/add-account.html'
        });

        $stateProvider.state('cloud-provider-accounts.add-azure-account', {
            url: '/add-azure-account?accountPublicId',
            templateUrl: 'assets/templates/cloud-provider-accounts/add-azure-account.html',
            controller: 'AddAzureAccountController'
        });

        $stateProvider.state('cloud-provider-accounts.add-gcp-account', {
            url: '/add-gcp-account?accountPublicId',
            templateUrl: 'assets/templates/cloud-provider-accounts/add-gcp-account.html',
            controller: 'AddGcpAccountController'
        });

        $stateProvider.state('cloud-provider-accounts.add-aws-account', {
            url: '/add-aws-account?accountPublicId',
            templateUrl: 'assets/templates/cloud-provider-accounts/add-aws-account.html',
            controller: 'AddAwsAccountController'
        });

        $stateProvider.state('cloud-provider-accounts.add-nss-account', {
            url: '/add-nss-account?accountPublicId',
            templateUrl: 'assets/templates/cloud-provider-accounts/add-nss-account.html',
            controller: 'AddNssAccountController'
        });

        $stateProvider.state('cloud-provider-accounts.account-settings', {
            url: '/account-settings',
            templateUrl: 'assets/templates/cloud-provider-accounts/account-settings.html',
            controller: 'AccountSettingsController'
        });

        //////////////////////////////////////////////////////////////////////////////////////////////
        //
        //  Volumes
        //
        //////////////////////////////////////////////////////////////////////////////////////////////

        $stateProvider.state('volume-view', {
            onEnter: () => {
                $('body').addClass('vol-view-grey-background');
            },
            onExit: () => {
                $('body').removeClass('vol-view-grey-background');
            },
            resolve: {
                isVolViewAllowed: ($rootScope, $state, nocRoutingResolverService, nocUIMain) => {
                    const unloading = nocUIMain.loading();

                    return nocRoutingResolverService.isVolumeViewAllowed($rootScope)
                        .catch(() => {
                            $state.go('main');
                        })
                        .finally(unloading);
                }
            },
            url: '/volume-view',
            abstract: true,
            controller: 'VolumesLayoutController',
            templateUrl: 'assets/templates/vol-view-layout.html'
        });

        $stateProvider.state('volume-view.timeline', {
            tenantsTopMenu: 'hide',
            url: '/timeline',
            controller: 'VolumeViewAuditController',
            templateUrl: 'assets/templates/tabs/volume_view/volume-view-timeline.html'
        });

        $stateProvider.state('volume-view.volumes', {
            url: '/volumes',
            controller: 'VolumeViewController',
            templateUrl: 'assets/templates/tabs/volume_view/volume-view.html'
        });

        $stateProvider.state('volume-view.resources', {
            url: '/resources',
            controller: 'VolumeViewResourcesController',
            templateUrl: 'assets/templates/tabs/volume_view/volume-view-resources.html'
        });

        $stateProvider.state('volume-view.volume-view-user-settings', {
            onEnter() {
                $('body').removeClass('vol-view-grey-background');
            },
            onExit() {
                $('body').addClass('vol-view-grey-background');
            },
            url: '/user-settings/:userId',
            templateUrl: 'assets/templates/tabs/volume_view/volume-view-user-settings.html',
            controller: 'UsersController'
        });

        $stateProvider.state('volume-view.action-required', {
            onEnter() {
                $('body').removeClass('vol-view-grey-background');
            },
            onExit() {
                $('body').addClass('vol-view-grey-background');
            },
            url: '/action-required',
            templateUrl: 'assets/templates/tabs/volume_view/volume-view-action-required.html',
            controller: 'VolumeViewActionRequiredController'
        });

        //////////////////////////////////////////////////////////////////////////////////////////////
        //
        //  Anterior Setup Wizard
        //
        //////////////////////////////////////////////////////////////////////////////////////////////

        $stateProvider.state('anterior-setup-wizard', {
            url: '/anterior-setup-wizard/welcome',
            templateUrl: 'assets/templates/wizards/anterior-setup/anterior-setup-wizard.html',
            controller: 'AnteriorSetupWizardController',
            resolve: {
                // Prevents the user from navigating to a forbidden location
                isAnteriorSetupWizardAllowed: ($rootScope, $state) => {
                    if (_.get($rootScope, 'supportServices.volumeViewEnabled')) {
                        $state.go('setup-wizard.welcome');
                    }
                }
            }
        });

        $stateProvider.state('auth0-setup-wizard', {
            url: '/auth0-setup-wizard',
            templateUrl: 'assets/templates/wizards/portal-setup/portal-setup-wizard.html',
            controller: 'PortalSetupWizardController'
        });

        $stateProvider.state('auth0-setup-failed-register', {
            url: '/auth0-setup-failed-register',
            templateUrl: 'assets/templates/standalone/setup-server-registration-error.html'
        });

        $stateProvider.state('auth0-failed-reach', {
            url: '/auth0-failed-reach',
            templateUrl: 'assets/templates/standalone/failed-reach-auth0.html'
        });

        $stateProvider.state('after-migration', {
            url: '/after-migration',
            templateUrl: 'assets/templates/standalone/portal-migration.html'
        });
    });
})();
