(function(){
    "use strict";

    angular.module("noc").service("volumeViewColumnsService", function (uiGridConstants, nocLocalStorage, uiGridSortUtilities, uiGridUtilities) {
        var service = {};

        var titledCellTemplate = uiGridUtilities.titledCellTemplate,
            statusCellTemplate = "assets/templates/tabs/volume_view/grid-cell-templates/status-cell-template.html",
            headerCellTemplate = "assets/templates/tabs/volume_view/grid-cell-templates/header-cell-template.html";
        var volumeNameTemplate = "assets/templates/tabs/volume_view/grid-cell-templates/volume-name-cell-template.html";
        var diskTypeTemplate =  "assets/templates/tabs/volume_view/grid-cell-templates/volume-disk-type-cell-template.html";
        var attributesCellTemplate = "assets/templates/tabs/volume_view/grid-cell-templates/volume-attributes-cell-template.html";

        var defaultVolumesColumnsDef = [
            { displayName: "Volume Name", headerCellClass: "volume", visible: true, width: 180, sort: { direction: uiGridConstants.DESC}, field: "name", headerCellTemplate: headerCellTemplate, cellTemplate: volumeNameTemplate},
            { displayName: "Attributes", headerCellClass: "volume", visible: true, width: 170, field: "_attributes", headerCellTemplate: headerCellTemplate, cellTemplate: attributesCellTemplate},
            { displayName: "Capacity", headerCellClass: "capacity", visible: true, width: 140, field: "size", cellFilter: "normalizedCapacity:2:true", _filterBy: ["_size"], headerCellTemplate: headerCellTemplate,
                sortingAlgorithm: uiGridSortUtilities.sortCapacityObjects},
            { displayName: "Used Capacity", headerCellClass: "used-capacity", visible: true, maxWidth: 170, field: "usedSize", cellFilter: "normalizedCapacity:2:true", _filterBy: ["_usedSize"], headerCellTemplate: headerCellTemplate,
                sortingAlgorithm: uiGridSortUtilities.sortCapacityObjects},
            { displayName: "Disk Type", headerCellClass: "disk-type", visible: true, field: "_diskType", headerCellTemplate: headerCellTemplate, cellTemplate: diskTypeTemplate},
            { displayName: "Exported as", headerCellClass: "exported-as", visible: true, field: "_mountPoint", headerCellTemplate: headerCellTemplate, cellTemplate: titledCellTemplate},
            { displayName: "Location", width: 250, headerCellClass: "location", visible: true, field: "_location", headerCellTemplate: headerCellTemplate, cellTemplate: titledCellTemplate},
            { displayName: "Status", headerCellClass: "status", visible: true, field: "_displayStatus", headerCellTemplate: headerCellTemplate, cellTemplate: statusCellTemplate},
            { displayName: "", visible: true, minWidth:50, maxWidth: 50, width: 50, field: "state", cellTemplate: "<vol-view-grid-menu grid-name='volume-view-volumes'></vol-view-grid-menu>"}
        ];

        var ebsVolumesCellTemplate = "<div class='ui-grid-cell-contents ebs-volumes'>{{row.entity._ebsVolumeTotalCount}} <div class='vol-view-more-info-widget' ng-click='popoverHelper.ebsVolumes = row.entity._ebsVolumes;' noc-popover=\"'assets/templates/tabs/volume_view/dialogs/ebs-volumes-per-instance-popover.html'\" ng-if=\"row.entity._ebsVolumeTotalCount\"></div></div>";
        var tagsCellTemplate = "<div class='ui-grid-cell-contents aws-tags'>{{row.entity._awsTagsCount}} <div class='vol-view-more-info-widget' ng-click='popoverHelper.awsTags = row.entity._awsTags;' noc-popover=\"'assets/templates/tabs/volume_view/dialogs/aws-tags-per-instance-popover.html'\" ng-if=\"row.entity._awsTagsCount\"></div></div>";
        var instanceNameCellTemplate = "<div class='ui-grid-cell-contents trimmed-text' title='{{row.entity._displayName}}'>{{row.entity._displayName}}</div>";

        var defaultResourcesColumnsDef = [
            { displayName: "Instance Name", width: 280, visible: true, sort: { direction: uiGridConstants.DESC}, field: '_displayName', cellTemplate: instanceNameCellTemplate, headerCellTemplate: headerCellTemplate},
            { displayName: "Instance Type", visible: true, field: "instanceType", headerCellTemplate: headerCellTemplate},
            { displayName: "Location: Availability Zone | Subnet", width: 300, headerCellClass: "location", visible: true, field: "_location", headerCellTemplate: headerCellTemplate, cellTemplate: titledCellTemplate},
            { displayName: "Volumes", headerCellClass: "volume", visible: true, field: "volumes", headerCellTemplate: headerCellTemplate, cellTemplate: "assets/templates/tabs/volume_view/grid-cell-templates/ontap-volumes-cell-template.html"},
            { displayName: "EBS Volumes", headerCellClass: "disk-type", visible: true, field: "_ebsVolumeTotalCount", headerCellTemplate: headerCellTemplate, cellTemplate: ebsVolumesCellTemplate},
            { displayName: "AWS Tags", visible: true, field: "_awsTagsCount", headerCellTemplate: headerCellTemplate, cellTemplate: tagsCellTemplate},
            { displayName: "Status", headerCellClass: "status", visible: true, field: "_displayStatus", headerCellTemplate: headerCellTemplate, cellTemplate: statusCellTemplate},
            { displayName: "", visible: true, minWidth:50, maxWidth: 50, width: 50, field: "status", cellTemplate: "<vol-view-grid-menu grid-name='volume-view-resources'></vol-view-grid-menu>"}
        ];

        _.forEach(defaultVolumesColumnsDef, function (col) {
            if (col.displayName && !col.width) col.width = "*";
            col.cellClass = function (grid, row) {
                return (row.entity._displayStatus === 'online' || row.entity._displayStatus === 'degraded') ? '' : 'disabled';
            };
        });

        _.forEach(defaultResourcesColumnsDef, function (col) {
            if (col.displayName && !col.width) col.width = "*";
        });

        service.getVolumesColumnsDef = function () {
            return defaultVolumesColumnsDef;
        };

        service.getResourcesColumnsDef = function () {
            return defaultResourcesColumnsDef;
        };

        service.restoreGridState = function (gridApi, scope, gridName) {
            var restored = nocLocalStorage[gridName].restore(gridApi);
            if (restored) {
                gridApi.saveState.restore(scope, restored);
            }
        };

        service.saveGridState = function (gridApi, gridName) {
            if (gridApi) {
                var state = gridApi.saveState.save();
                nocLocalStorage[gridName].save(state);
            }
        };

        return service;
    });
}());