(function () {
    "use strict";

    angular.module("noc").factory("nocHttpsResourcesService", function (nocUtil, $timeout, $log, $http, nocUIMain) {
        var httpsService = {},
            deployStartTime;

        httpsService.restartOCCM = function restartOCCM(nocHttp, pidgeon) {
            return nocHttp.post('/occm/system/restart-server', '', nocHttp.BLOCK)
                .catch(function (error) {
                    if (error && _.startsWith(error.causeMessage, "OnGoingAsyncOperationException")) return;
                    pidgeon.inProgress("Restarting OnCommand Cloud Manager...", false, pidgeon.DURATION_LONG);
                    nocUIMain.block();
                    deployStartTime = new Date().getTime();
                    $timeout(pollUIAndLoginAfterServerRedeployed, 5000);
                });
        };

        function pollUIAndLoginAfterServerRedeployed() {
            var pollInterval = 1000; //ms
            var scheduleAnotherCheck = function () {
                if ((new Date().getTime() - deployStartTime) < ( 5 * 60 * 1000)) {
                    $timeout(pollUIAndLoginAfterServerRedeployed, pollInterval);
                } else {
                    $log.info('An error occurred while restarting the server. Server did not redeploy.');
                }
            };
            var httpConfig = {
                method: 'GET',
                url: '/occm/system/about',
                timeout: 5000
            };
            $http(httpConfig)
                .then(nocUIMain.logout)
                .catch(scheduleAnotherCheck);
        }

        httpsService.getTrustedCaCertificates = function (nocHttp) {
            return nocHttp.get('/occm/key-store/user-trusted-certificates',
                "retrieving trusted CA certificates", nocHttp.LOADING)
                .then(function (trustedCaCertificates) {
                    var trustedCertificateInfos = _.map(trustedCaCertificates, function (trustedCaCertificate) {
                        trustedCaCertificate.certificateInfo.status = trustedCaCertificate.status;
                        trustedCaCertificate.certificateInfo.id = trustedCaCertificate.id;
                        httpsService.formatCertificate(trustedCaCertificate.certificateInfo);
                        return trustedCaCertificate.certificateInfo;
                    });
                    return trustedCertificateInfos;
                });
        };

        httpsService.formatCertificate = function(certificate){
            if (!certificate) return;
            //1. Calculate expiration state.
            certificate._expirationState = nocUtil.certificateExpirationState(certificate.endDate, certificate.startDate);

            //2. Add space after each comma to allow logical line-breaks.
            certificate.directIssuerName = _.replace(certificate.directIssuerName, /,/g, ', ');
            certificate.subjectName = _.replace(certificate.subjectName, /,/g, ', ');
            if(certificate.status === 'PENDING_DELETED') {
                certificate._statusText = 'Delete pending Cloud Manager restart';
            }
            if(certificate.status === 'PENDING_INSTALLED') {
                certificate._statusText = 'Install pending Cloud Manager restart';
            }
            if(certificate.status === 'ACTIVE') {
                certificate._statusText = 'Active';
            }
            if(certificate._expirationState.state === 'certificate-expired') {
                certificate._statusText = 'Expired';
            }
            if(certificate._expirationState.state === 'certificate-warning') {
                certificate._statusText = 'Active';
            }
            if(certificate._expirationState.state === 'future-activation') {
                certificate._statusText = 'Future activation';
            }
        };

        return httpsService;
    });
}());