/* eslint-disable new-cap, camelcase, id-length */
(() => {
    'use strict';


    function basicData(about, user, screen, roles, appId, instancePlacement) {
        const data = {
            app_id: appId,
            beta: about.beta,
            build: about.build,
            build_time: about.buildTimestamp,
            environment: about.environment,
            version: about.version,
            system_id: about.systemId,
            screen_resolution: screen.width + ' x ' + screen.height,
            install_location: instancePlacement.installLocation
        };

        if (user) {
            _.assignIn(data, {
                user_id: user.auth0Id || about.systemId + '/' + user.publicId
            });
        }

        return data;
    }
    function siteData(about, privateInfoApproved) {
        return about.siteIdentifier && privateInfoApproved ?
            {
                site: about.siteIdentifier.site,
                company: about.siteIdentifier.company,
                host_name: about.siteIdentifier.hostName
            } :
            {};
    }

    function userData(user, privateInfoApproved) {
        return user && privateInfoApproved ?
            {
                name: user.firstName + ' ' + user.lastName,
                email: user.email
            } :
            {};
    }

    angular.module('noc').factory('intercomNotifier', (nocCurrentUserService, $rootScope, $window, $injector, nocLocalStorage) => {
        let booted = false;

        function intercomEnabled() {
            return  $rootScope.appState !== 'setup-wizard' && //Show intercom widget only in normal (logged in) mode
                $rootScope.about && //About is needed for many properties sent to intercom
                $rootScope.supportServices.intercomEnabled; //Disabled in gov cloud and when opted out by user
        }

        function calcData(includeBasic, appId) {
            const about = $rootScope.about;
            const privateInfoApproved = true;//localStorage.restore();)

            return _({})
                .assignIn(includeBasic ? basicData(about, nocCurrentUserService.getCurrentUser(), $window.screen, nocCurrentUserService.getRoles(), appId, $rootScope.instancePlacement) : {})
                .assignIn(siteData(about, privateInfoApproved))
                .assignIn(userData( nocCurrentUserService.getCurrentUser(), privateInfoApproved))
                .value();
        }

        function boot() {
            if (intercomEnabled() && !booted) {
                const userPublicId = nocCurrentUserService.getCurrentUserPublicId();
                const localStorage = nocLocalStorage.INTERCOM_PRIVATE_INFO(userPublicId);

                const appId = $rootScope.about.environment === 'production' ?
                    $rootScope.externalConfig.intercom_prod_app_id || 'i2d81hca' :
                    $rootScope.externalConfig.intercom_test_app_id || 'brf2h510';

                booted = true;

                //Read from localStorage if the user already approved sending name and email.
                //if the user declines, only anonymous information is sent


                $window.Intercom('boot', calcData(true, appId));

                $window.Intercom('onShow', () => {
                    if (!localStorage.restore()) {
                        //Acknowledging the user about the Intercom usage
                        $injector.get('nocDialog').confirm('Privacy note',
                            'assets/templates/dialogs/intercom-private-info.html', {
                                width: 582,
                                dialogOptions: {
                                    confirmBtnText: 'Agree',
                                    cancelBtnText: 'Decline'
                                }
                            })
                            .then(() => {
                                //Keep user confirmation in the localStorage to avoid the confirmation next time
                                localStorage.save(true);

                                //Update intercom with the user data
                                update(calcData(false));
                            })
                            .catch(() => {
                                //Close the Intercom messenger as the user declined using it
                                $window.Intercom('hide');

                                //Keep user decline in the localStorage.
                                localStorage.save(false);
                            });
                    }
                });

            }
        }
        function logout() {
            if (booted) {
                booted = false;
                $window.Intercom('shutdown');
            }
        }
        function update(newData) {
            if (booted) {
                $window.Intercom('update', newData);
            }
        }

        function event(name, meta) {
            if (booted) {
                $window.Intercom('trackEvent', name, meta);
            }
        }

        return {
            boot,
            logout,
            update,
            event
        };
    });
})();
