(() => {
    'use strict';

    angular.module('noc').factory('marketingPopupService', (nocLocalStorage, nocDialog, $window,
                                                               $timeout, $rootScope) => {
        const service = {};

        service.getPopupData = () => {
            if (!nocLocalStorage.MARKETING_POPUP_COUNTER.restore()) {
                nocLocalStorage.MARKETING_POPUP_COUNTER.save('0');
            }

            if (($rootScope.instancePlacement.installLocation === 'AWS') && (nocLocalStorage.MARKETING_POPUP_COUNTER.restore() < 2) && !nocLocalStorage.DONT_SHOW_MARKETING_POPUP.restore()) {
                $timeout(() => {
                    service.showMarketingDialog();
                }, 2000);
            }
        };

        service.dontShowAgainInThisSession = false;

        function isDismissedForever() {
            return nocLocalStorage.DONT_SHOW_MARKETING_POPUP.restore();
        }

        service.dontShowAgain = function dontShowAgain() {
            nocLocalStorage.DONT_SHOW_MARKETING_POPUP.save(true);
        };

        service.learnMore = () => {
            $window.open('https://docs.netapp.com/us-en/occm/task_setting_up_cbs.html', '_blank');
        };

        service.showMarketingDialog = function showMarketingDialog() {
            if (true || isDismissedForever() || service.dontShowAgainInThisSession) return;

            return nocDialog.confirm('', 'assets/templates/dialogs/marketing-popup.html', {
                dialogClass: 'marketing-popup no-title-dialog',
                dontShowAgain: service.dontShowAgain,
                learnMore: service.learnMore,
                width: 700,
                fromTop: '10%',
                clickOutsideToClose: true
            })
                .finally(() => {
                    let marketingPopupCounter = nocLocalStorage.MARKETING_POPUP_COUNTER.restore();
                    marketingPopupCounter++;
                    nocLocalStorage.MARKETING_POPUP_COUNTER.save(marketingPopupCounter);
                    service.dontShowAgainInThisSession = true;
                });
        };

        return service;
    });
})();
