(() => {
    'use strict';

    angular.module('noc').factory('nocUIFactory', (nocUIMain, $q) => {
        function NocUI(className){
            this._className = className;
        }

        var unloadingMethods = [];
        var unblockMethods = [];

        NocUI.prototype.loading = function() {
            if(this._dead) return;
            var unloading = nocUIMain.loading(this._className);
            unloadingMethods.push(unloading);
            return unloading;
        };

        NocUI.prototype.block = function() {
            if(this._dead) return;
            var unblock = nocUIMain.block(this._className);
            unblockMethods.push(unblock);
            return unblock;
        };

        NocUI.prototype.kill = function(){
            if(this._dead) return; //You cannot kill what does not die
            this._dead = true;
            //Call unloading and unblock methods
            _.forEach(unloadingMethods, function(unloading) {
                unloading();
            });
            _.forEach(unblockMethods, function(unblock) {
                unblock();
            });
        };

        NocUI.prototype.formValidate = function (validationFunction) {
            if (this._dead) return $q.when();
            if (validationFunction()) return $q.when();

            this.handleFormValidity();
            return $q.reject();
        };

        NocUI.prototype.handleFormValidity = function() {
            if(this._dead) return;
            nocUIMain.handleFormValidity();
        };

        NocUI.prototype.handleErrorsInWizard = function(withScrolling) {
            if(this._dead) return;
            nocUIMain.handleErrorsInWizard(withScrolling);
        };

        NocUI.prototype.clearFormValidity = function() {
            if(this._dead) return;
            nocUIMain.clearFormValidity();
        };

        NocUI.prototype.ignoreFormValidity = function() {
            if(this._dead) return;
            nocUIMain.ignoreFormValidity();
        };

        NocUI.prototype.reloadSilently = function(backToRoot) {
            if(this._dead) return;
            nocUIMain.reloadSilently(backToRoot);
        };

        NocUI.prototype.logout = function(delayMillis) {
            nocUIMain.logout(delayMillis);
        };

        return {spawn: function(className, scope){
            var nocUI = new NocUI(className);
            scope.$on("$destroy", function(){
                nocUI.kill();
            });
            return nocUI;
        }};
    });
})();
