(function(){
    "use strict";

    angular.module("noc").factory("nocSchedulesService", function (replicationResource) {
        var schedulesService = {};

        schedulesService.getSchedules = function (nocHttp, targetPublicId) {
            return replicationResource.getSchedules(nocHttp, targetPublicId)
                .then(function (schedules) {
                    //Add nice description field to every schedule:
                    _.forEach(schedules, function (sched) {
                        sched.descArr = schedulesService.stringifyCron(sched.cronJobSchedule);
                    });
                    return schedules;
                });
        };

        schedulesService.stringifyTimeUnit = function (timeUnit, arr) {
            //Returns the ordinal number for n: 1 => '1st', 22 => '22nd' etc;
            function ord(n) {
                var s = ["th", "st", "nd", "rd"],
                    v = n % 100;
                return n + (s[(v - 20) % 10] || s[v] || s[0]);
            }

            //['a', 'b', 'c'] => 'a, b and c'
            //['a'] => 'a'
            //Assuming arr is not null and not empty.
            function prettifyList(arr) {
                var lastItem = arr.pop();
                var str = arr.join(', ');
                return str ? (str + " and " + lastItem) : lastItem;
            }

            //[1, 3, 5, 22] => '1st, 3rd, 5th and 22nd'
            function ordList(arr) {
                return (prettifyList(_.map(arr, ord)));
            }

            var timesNames = {"months": "Months", "weekDays": "Days", "days": "Days", "hours": "Hours", "minutes": "Minutes"};
            var timesSingle = {"months": "month", "weeks": "week", "weekDays": "day", "days": "day", "hours": "hour", "minutes": "minute"};
            var monthsNames = ["Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"];
            var weekDays = ["Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"]; //0 represents Sunday.

            var str;
            if (!arr || arr.length === 0) {
                str = "Every " + timesSingle[timeUnit];
            } else {
                str = timesNames[timeUnit] + ": ";
                switch (timeUnit) {
                    case "months":
                        // [0, 3, 5] ==> "Months: Jan, Mar and May"
                        str += prettifyList(_.at(monthsNames, arr));
                        break;
                    case "weekDays":
                        // [0, 3, 5] ==> "Week days: Sun, Wed and Fri"
                        str += prettifyList(_.at(weekDays, arr));
                        break;
                    case "days":
                        // [0, 9, 19] ==> "Days: 1st, 10th and 20th"
                        str += ordList(_.map(arr, function (num) {
                            return num + 1;
                        }));
                        break;
                    case "hours":
                        // [0, 3, 13] ==> "Hours: 12 AM, 3 AM and 1 PM"
                        // 12-hours clock is stupid system. Americans, you are lame.
                        str += prettifyList(_.map(arr, function (hour) {
                            //it is pm if hours from 12 onwards
                            var suffix = (hour >= 12) ? ' PM' : ' AM';

                            //translate to 12hours clock
                            hour = ((hour + 11) % 12 + 1);

                            return hour + suffix;
                        }));
                        break;
                    case "minutes":
                        // [1, 3, 6] ==> "Minutes: 1st, 5th and 6th minute"
                        str += ordList(arr) + " minute";
                        break;
                    default:
                        str += "Something is wrong here!";
                        break;
                }
            }
            return str;
        };

        schedulesService.stringifyCron = function (cron) {
            var times = ["months", "days", "weekDays", "hours", "minutes"];
            //If the cron has days, weekDays or both they will be displayed. If none of them exist we will display days.
            if (!cron["weekDays"] || cron["weekDays"].length === 0) {
                times[2] = "";
                times = _.compact(times);
            } else if ((!cron["days"] || cron["days"].length === 0) && (cron["weekDays"] || cron["days"].length !== 0)) {
                times[1] = "weeks";
                cron["weeks"] = [];
            }

            var description = [];
            var i = 0;
            //We want to print one line of "Every timeUnit" for the unit that is one larger than the first non-null time unit:
            //We find it here...:
            while ((!cron[times[i]].length || cron[times[i]].length === 0) && (!cron[times[i + 1]].length || cron[times[i + 1]].length === 0)) {
                i++;
            }
            //...And print a line for every time-unit after it, even if it's null.
            for (i; i < times.length; i++) {
                description.push(schedulesService.stringifyTimeUnit(times[i], cron[times[i]]));
            }
            return description;
        };


        return schedulesService;
    });
}());