(() => {
    'use strict';
    angular.module('noc').factory('ServerUpgradeService', (nocPoller, $rootScope, $timeout, $http, $log) => {
        const service = {};

        let deployStartTime;

        service.pollForServerUpgrade = (callback) => {
            deployStartTime = new Date().getTime();
            if (!callback || !_.isFunction(callback)) {
                $log.debug('pollForServerUpgrade called with no callback, using location.reload as default.');
                callback = location.reload;
            }
            nocPoller.pause();
            $rootScope.expectNetworkFailures = true;

            return waitForServerRenaissanceAndDoCallback(callback);
        };

        const pollInterval = 1000; //Milliseconds
        const timeThreshold = 10 * 60 * 1000; //10 Minutes
        const waitForServerRenaissanceAndDoCallback = (callback) => {
            const scheduleAnotherCheckOrDoCallbackIfTimeIsUp = () => {
                if ((new Date().getTime() - deployStartTime) < timeThreshold) {
                    return $timeout(() => {
                        return waitForServerRenaissanceAndDoCallback(callback);
                    }, pollInterval);
                } else {
                    return callback();
                }
            };

            const httpConfig = {
                method: 'GET',
                url: '/occm/system/about',
                timeout: 7000
            };

            return $http(httpConfig)
                .then((about) => {
                    if ($rootScope.about.buildTimestamp === about.data.buildTimestamp) { //Still in the 'old' server
                        return scheduleAnotherCheckOrDoCallbackIfTimeIsUp();
                    } else { //'new' server is deployed
                        return callback();
                    }
                })
                //Most likely 'error' means that we are in the limbo between the deployments.
                .catch(scheduleAnotherCheckOrDoCallbackIfTimeIsUp);
        };

        return service;
    });
})();
