'use strict';

var _ = require('lodash');
var rule = require('../rules/catch-or-return');
var RuleTester = require('eslint').RuleTester;

function invalid(codeSnippets) {
    return _.map(codeSnippets, function (codeSnippetOrObject) {
        const obj = _.isString(codeSnippetOrObject) ? {code: codeSnippetOrObject} : codeSnippetOrObject;
        obj.errors = [{message: 'Expected catch() or return'}];

        return obj;
    });
}

new RuleTester().run('catch-or-return', rule, {
    valid: [

        // catch
        'frank().then(go).catch(doIt)',
        'frank().then(go).then().then().then().catch(doIt)',
        'frank().then(go).then().catch(function() { /* why bother */ })',
        'frank.then(go).then(to).catch(jail)',
        'frank.then(go).then(go).then(go).then(go).then(go).then(go).then(go).then(go).then(go).then(go).then(go).then(go).catch(jail)',
        'frank.then(go).catch(jail).finally(cleanup)',
        'frank.then(go).then().catch(jail).finally(cleanup)',

        // return
        'function a() { return frank().then(go) }',
        'function a() { return frank().then(go).then().then().then() }',
        'function a() { return frank().then(go).then()}',
        'function a() { return frank.then(go).then(to) }',

        //make sure passing the options do not break - these should pass anyways
        {code:'function a() { return nocHttp.then(go) }',               options:['nocHttp']},
        {code:'function a() { nocHttp.then(go).catch(something) }',     options:['nocHttp']},

        //ok only because the promise producer is in the exceptions list
        {code:'function a() { nocHttp.then(go) }',                      options:['nocHttp']},
        {code:'function a() { nocHttp.then(go).then(go) }',             options:['nocHttp']},
        {code:'function a() { nocHttp.then(go).finally(go) }' ,         options:['nocHttp']},
        {code:'function a() { something.then(go) }',                    options:['nocHttp','something']},

        //ok only because the promise producer ends with Resource
        'function a() { titleCaseResource.then(go) }',
        'function a() { lowerCaseresource.then(go).then(go) }',
        'function a() { anotherResource.then(go).finally(go) }'
    ],

    invalid: invalid([

        // catch failures
        'function callPromise(promise, cb) { promise.then(cb) }',
        'fetch("http://www.yahoo.com").then(console.log.bind(console))',
        'a.then(function() { return "x"; }).then(function(y) { throw y; })',
        'a.then(function() { return "x"; }).then(function(y) { throw y; })',

        // return failures
        'function a() { frank().then(go) }',
        'function a() { frank().then(go).then().then().then() }',
        'function a() { frank().then(go).then()}',
        'function a() { frank.then(go).then(to) }',
        'function a() { frank.then(go).then(to) }',

        {code:'function a() { nocHttp.then(go) }',                      options:['something']},
        {code:'function a() { nocHttp.then(go).then(go) }',             options:['something']},
        {code:'function a() { nocHttp.then(go).finally(go) }' ,         options:['something']}
    ])
});
