'use strict';

const _ = require('lodash');
const rule = require('../rules/no-fake-promise');
const RuleTester = require('eslint').RuleTester;

function invalidFakeConfirmNew(codeSnippets) {
    return _.map(codeSnippets, (codeSnippet) => {
        return {
            code: codeSnippet,
            errors: [{message: 'fakeConfirmNew is deprecated, use mockConfirm instead. Don\'t forget to trigger the digest cycle.'}]
        };
    });
}

function invalidMockDeferredPromise(codeSnippets) {
    return _.map(codeSnippets, (codeSnippet) => {
        return {
            code: codeSnippet,
            errors: [{message: 'mockDeferredPromise is deprecated, use spyOn(...).return(j.$q.when(...)\\.reject(...)) instead. Don\'t forget to trigger the digest cycle.'}]
        };
    });
}

new RuleTester().run('no-fake-promise', rule, {
    valid: [
        'magnifispec.otherFunction()',
        'magnifispec.fakeConfirmNew',
        'magnifispec.mockDeferredPromise'
    ],

    invalid: _.concat(
        invalidFakeConfirmNew([
            'magnifispec.fakeConfirmNew()',
            'magnifispec.fakeConfirmNew(true)',
            'magnifispec.fakeConfirmNew(false)'
        ]),
        invalidMockDeferredPromise([
            'magnifispec.mockDeferredPromise()',
            'magnifispec.mockDeferredPromise(true)',
            'magnifispec.mockDeferredPromise(false)'
        ]))
});
