'use strict';

var rule = require('../rules/noc-http-description');
var RuleTester = require('eslint').RuleTester;

new RuleTester().run('noc-http-description', rule, {
    valid: [

        'nocHttp.get("url", "retrieving data", nocHttp.LOADING)',
        'nocHttp.get("url", "retrieving data for volume " + volName, nocHttp.LOADING)',
        'nocHttp.post("url", "saving data")',
        'nocHttp.post({url: "url", params: "params"}, "saving data", nocHttp.LOADING)',
        'nocHttp.delete("url", "deleting data", nocHttp.BLOCKING)',
        'nocHttp.put("url", "changing data", nocHttp.LOADING)',

        'function a(){nocHttp.get("url", "retrieving data", nocHttp.LOADING);}',
        'function a(){return nocHttp.get("url", "retrieving data", nocHttp.LOADING);}',

        'nocHttp.get("url")',
        'nocHttp.get("url", "", nocHttp.LOADING)',
        'nocHttp.get("url", null, nocHttp.LOADING)',
        'nocHttp.get("url", undefined, nocHttp.LOADING)'
    ],

    invalid: [

        //nocHttp message must start with lowercase:
        {code: 'nocHttp.get("url", "Retrieving data", nocHttp.LOADING)', errors: [{message: 'nocHttp message must start with lowercase'}]},
        {code: 'function foo(){return nocHttp.get("url", "Retrieving data", nocHttp.LOADING);}', errors: [{message: 'nocHttp message must start with lowercase'}]},

        //nocHttp message must start with present progressive verb (-ing)
        {code: 'nocHttp.get("url", "retrieve data", nocHttp.LOADING)', errors: [{message: 'nocHttp message must start with present progressive verb (-ing)'}]},
        {code: 'nocHttp.post({url:\'/occm/encryption/certificate\', data:{"occmCertificate": $scope.occmCertificate}}, "certificate installation", nocHttp.LOADING)', errors: [{message: 'nocHttp message must start with present progressive verb (-ing)'}]},

        //nocHttp.get() message should start with 'retrieving', not 'fetching' or 'getting'
        {code: 'nocHttp.get("url", "getting data", nocHttp.LOADING)', errors: [{message: "nocHttp.get() message should start with 'retrieving', not 'fetching' or 'getting'"}]},

        //Multiple problems:
        {
            code: 'nocHttp.get("url", "Retrieve data", nocHttp.LOADING)',
            errors: [{message: 'nocHttp message must start with lowercase'}, {message: 'nocHttp message must start with present progressive verb (-ing)'}]
        }

        //Limitation: should be invalid but currently overlooked by the rules:
        //,{code: 'nocHttp.get("url", "Retrieving data for resource " + resource1, nocHttp.LOADING)', errors: [{message: 'nocHttp message must start with lowercase'}]}
    ]
});
