const gulp = require('gulp');
const connectInject = require('connect-injector');
const connect = require('gulp-connect');
const _ = require('lodash');
const url = require('url');
const proxy = require('proxy-middleware');

const program = require('../program');
const indexTask = require('./index');

module.exports = {
    registerTask: () => {
        //Connect server also manages the proxying to the server via middleware
        gulp.task('connect', (cb) => {
            let proxyOptions;

            connect.server({
                root: 'dist',
                port: 1337,
                livereload: true,
                middleware: () => {
                    const connectUrl = 'http://' + program.serverHost + '/occm';
                    program.info('connect: proxying requests to:' + connectUrl);

                    const middlewares = [
                        //This middleware intercepts responses https://github.com/daffl/connect-injector
                        (() => {
                            return connectInject(
                                (req) => {
                                    return (/api\/occm\/system\/about$/).test(req.url);
                                }, (data, req, res, callback) => {
                                    try {
                                        const aboutObj = JSON.parse(data);
                                        aboutObj.serverHost = program.serverHost;
                                        const converted = JSON.stringify(aboutObj);

                                        callback(null, converted); //Call callback with the new content
                                    } catch (ex) {
                                        program.error('Exception parsing expected json response from: ' + req.url);
                                        program.error(ex);
                                        callback(null, data); //Allow failed parse through
                                    }
                                });
                        })(),
                        (() => {
                            proxyOptions = url.parse(connectUrl);
                            proxyOptions.route = '/occm';

                            return proxy(proxyOptions);
                        })(),

                        (req, res, next) => {
                            const reqUrl = url.parse(req.url, true);
                            if (reqUrl.pathname === '/update-proxy') {
                                if (reqUrl.query.s) {
                                    program.server = reqUrl.query.s;
                                    const newUrl = 'http://' + program.serverHost + '/occm';

                                    _.assign(proxyOptions, url.parse(newUrl));
                                    program.info('connect: updated proxying requests to:' + newUrl);

                                    gulp.src(indexTask.glob).pipe(connect.reload());
                                }

                                res.end();
                            } else {
                                next();
                            }
                        }
                    ];

                    if (program.logConnect) middlewares.push(require('connect').logger());

                    return middlewares;
                }
            });
            cb();

            program.info('To change proxy backend address: curl localhost:1337/update-proxy?s=<backend ip address>');
        });
    }
};
