const lazypipe = require('lazypipe');
const gulp = require('gulp');
const eslint = require('gulp-eslint');
const ignore = require('gulp-ignore');
const teamcityReporter = require('eslint-teamcity');
const gulpif = require('gulp-if');
const replace = require('gulp-replace');
const sourceMaps = require('gulp-sourcemaps');
const babel = require('gulp-babel');

const program = require('../program');
const Task = require('../task');
const iconStore = require('../icon_store');

module.exports = new Task(
    'scripts',
    {
        prod: 'app/scripts/**/*.js',
        test: 'test/jasmine/**/*.js',
        build: 'gulp/**/*.js',
        exclude: '!**/node_modules/**'
    },
    true,
    lazypipe()
        .pipe(eslint, {
            quiet: true
        })
        .pipe(() => {
            return gulpif(program.teamcity,
                eslint.format(teamcityReporter),
                eslint.formatEach()
            );
        })
        .pipe(eslint.failAfterError)
        .pipe(sourceMaps.init)
        .pipe(babel)
        .pipe(sourceMaps.write, '.')

        //Pass only production script files - discard test and build
        .pipe(ignore.exclude, (file) => {
            return (/jasmine|gulp/).test(file.path);
        })

        //Build the list of svg icons in the icon directive
        .pipe(() => {
            return replace("'INJECT_ICON_MAP_HERE'", iconStore.toString());
        })

        .pipe(gulp.dest, './dist/assets/scripts')

        //Filter out added and deleted files so they will be reloaded only by index task
        .pipe(ignore.include, (file) => file.event === 'change'),

    ['icons']
);
