describe.skip('Action required notice directive', function () {
    'use strict';

    var j;
    beforeEach(() => {
        j = magnifispec.setup();
    });
    var scope, element;
    beforeEach(function () {
        scope = j.$rootScope.$new();
        element = j.$compile('<noc-action-required-notice></noc-action-required-notice>')(scope);
        scope.selectedWorkingEnvironment = {publicId: 'someId'};
        scope.actionsRequired = {};
        scope.actionsRequiredHighestLevel = {};

        //initial setup, will be modified in test cases:
        scope.actionsRequired[scope.selectedWorkingEnvironment.publicId] = true;
        scope.actionsRequiredHighestLevel[scope.selectedWorkingEnvironment.publicId] = 'warning';
        scope.vsaResponse = {activeActions: []};
        scope.vsaComputingCapacity = false;

        scope.$digest();
    });

    it('should show yellow \'action required\' when it is required', function () {
        expect(element.children().hasClass('action-required-notice')).toEqual(true);
        expect(element.children().hasClass('warning')).toEqual(true);
        expect(element.find('i').hasClass('fa-exclamation-triangle')).toEqual(true);
    });

    it('should show red \'action required\' when it is required', function () {
        scope.actionsRequiredHighestLevel[scope.selectedWorkingEnvironment.publicId] = 'error';
        scope.selectedWorkingEnvironment.status = 'off';
        scope.$digest();

        expect(element.children().hasClass('action-required-notice')).toEqual(true);
        expect(element.children().hasClass('error')).toEqual(true);
        expect(element.find('i').hasClass('fa-exclamation-triangle')).toEqual(true);
    });

    it('should show \'Analyzing\' when it is required and will keep showing it for 2 secs', function () {
        scope.actionsRequired[scope.selectedWorkingEnvironment.publicId] = false;
        scope.vsaResponse = {activeActions: ['a']};
        scope.$digest();
        expect(element.children().hasClass('action-required-notice')).toEqual(true);
        expect(element.find('i').hasClass('fa-refresh')).toEqual(true);

        scope.vsaResponse = {activeActions: []};
        scope.vsaComputingCapacity = true;
        scope.$digest();
        expect(element.children().hasClass('action-required-notice')).toEqual(true);
        expect(element.find('i').hasClass('fa-refresh')).toEqual(true);

        scope.vsaComputingCapacity = false;
        scope.$digest();
        expect(element.find('i').hasClass('fa-refresh')).toEqual(true);

        j.$timeout.flush();
        scope.$digest();
        expect(element.find('i').hasClass('fa-refresh')).toEqual(false);
    });

    it('should show empty placeholder when no action required items exist', function () {
        scope.actionsRequired[scope.selectedWorkingEnvironment.publicId] = false;
        scope.$digest();
        expect(element.children().length).toEqual(1);
    });

    it('should show \'action required\' on ON_PREM', () => {
        scope.selectedWorkingEnvironment.workingEnvironmentType = 'ON_PREM';
        expect(element.children().hasClass('action-required-notice')).toEqual(true);
        expect(element.children().hasClass('warning')).toEqual(true);
        expect(element.find('i').hasClass('fa-exclamation-triangle')).toEqual(true);
    });

    describe('canShowIfOff', function () {
        beforeEach(function () {
            scope.vsaResponse = null;
            scope.selectedWorkingEnvironment.status = {status: 'off', message: null};
        });
        it('should show empty placeholder if no action required items are related to IPA', function () {
            scope.actionsRequired[scope.selectedWorkingEnvironment.publicId] = [{actionType: 'blah'}, {actionType: 'blah2'}];
            scope.$digest();
            expect(element.children().length).toEqual(1);
        });

        it('should show bar if at least one action required item is related to IPA', function () {
            scope.actionsRequired[scope.selectedWorkingEnvironment.publicId] = [{actionType: 'blah'}, {actionType: 'licenseExpirationWarning'}];
            scope.$digest();
            expect(element.children().hasClass('action-required-notice')).toEqual(true);
        });
    });
});
