describe.skip("Validation Directive", function () {
    "use strict";

    var j;    beforeEach(() => {        j = magnifispec.setup();    });

    describe("validation on div", function () {
        it("should add the form-control-feedback span to the child of the input", function () {
            var scope = j.$rootScope.$new();
            var element = j.$compile("<div><div class='verify-div' noc-div-verify></div></div>")(scope);
            var span = element.find("span");
            expect(span.hasClass("form-control-feedback")).toEqual(true);
            expect(span.find("i").length).toEqual(1);
        });

        it("should not add the form-control-feedback span if it's already exists", function () {
            var scope = j.$rootScope.$new();
            var element = j.$compile("<div><div class='verify-div' noc-div-verify></div><span class='form-control-feedback'></span></div>")(scope);
            var spans = element.children("span");
            expect(spans.length).toEqual(1);
        });

        it('should add the class noc-error to the parent when the input is invalid', () => {
            const scope = j.$rootScope.$new();
            const parent = j.$compile("<div><div class='verify-div' ng-model='model.value1' ng-pattern='/1234/' noc-div-verify></div>")(scope);
            scope.model = {value1: 'abcd'};

            j.$rootScope.$apply();

            expect(parent.hasClass('noc-error') || parent.hasClass('noc-empty')).toEqual(true);
        });

        it('should remove the class noc-error to the parent when the input is no longer invalid', () => {
            const scope = j.$rootScope.$new();
            const parent = j.$compile("<div><div class='verify-div' ng-model='model.value1' ng-pattern='/1234/' noc-div-verify></div>")(scope);
            scope.model = {value1: '1234'};

            j.$rootScope.$apply();

            expect(parent.hasClass('noc-error')).toEqual(false);
        });

        it("should add the class visited and remove the class focus to the parent when blurred", function () {
            var scope = j.$rootScope.$new();
            var parent = j.$compile("<div><div class='verify-div' noc-div-verify></div>")(scope);
            var div = parent.find("div.verify-div");

            scope.$apply(function(){
                div.blur();
            });

            j.$timeout.flush();
            expect(parent.hasClass("visited")).toEqual(true);
            expect(parent.hasClass("focus")).toEqual(false);
        });

        it("should add the class visited and remove the class focus to the parent when focused", function () {
            var scope = j.$rootScope.$new();
            var parent = j.$compile("<div><div class='verify-div' noc-div-verify></div>")(scope);
            var div = parent.find("div.verify-div");

            scope.$apply(function(){
                div.triggerHandler( "focus" );
            });

            expect(parent.hasClass("focus")).toEqual(true);
        });
    });

    describe("validation on select", function () {
        it("should add the form-control-feedback div to the child of the input", function () {
            var scope = j.$rootScope.$new();
            var element = j.$compile("<div><select ng-model='model'></select></div>")(scope);
            var span = element.find("span");
            expect(span.hasClass("form-control-feedback")).toEqual(true);
            expect(span.find("i").length).toEqual(1);
        });

        it('should add the class noc-error to the parent when the input is invalid', () => {
            const scope = j.$rootScope.$new();
            const parent = j.$compile("<div><select ng-model='model.value1' ng-pattern='/1234/'></select></div>")(scope);
            scope.model = {value1: 'abcd'};

            j.$rootScope.$apply();

            expect(parent.hasClass('noc-error')).toEqual(true);
        });

        it('should remove the class noc-error to the parent when the input is no longer invalid', () => {
            const scope = j.$rootScope.$new();
            const parent = j.$compile("<div><select ng-model='model.value1' ng-pattern='/1234/'></select></div>")(scope);
            scope.model = {value1: '1234'};

            j.$rootScope.$apply();

            expect(parent.hasClass('noc-error')).toEqual(false);
        });

        it("should add the class visited and remove the class focus to the parent when blurred", function () {
            var scope = j.$rootScope.$new();
            var parent = j.$compile("<div><select class='focus'  ng-model='model'></select></div>")(scope);
            var select = parent.find("select");

            select.blur();

            j.$timeout.flush();
            expect(parent.hasClass("visited")).toEqual(true);
            expect(parent.hasClass("focus")).toEqual(false);
        });

        it("should add the class visited and remove the class focus to the parent when focused", function () {
            var scope = j.$rootScope.$new();
            var parent = j.$compile("<div><select ng-model='model'></select></div>")(scope);
            var select = parent.find("select");

            scope.$apply(function(){
                select.triggerHandler( "focus" );
            });

            expect(parent.hasClass("focus")).toEqual(true);
        });
    });

    describe("validation on input", function () {
        it("should not execute directive if there is no ng-model", function(){
            var scope = j.$rootScope.$new();
            var element = j.$compile("<div><input></div>")(scope);
            var span = element.find("span");
            expect(span.length).toEqual(0);
        });

        it("should add the form-control-feedback div to the child of the input", function () {
            var scope = j.$rootScope.$new();
            var element = j.$compile("<div><input ng-model='model'></div>")(scope);
            var span = element.find("span");
            expect(span.hasClass("form-control-feedback")).toEqual(true);
            expect(span.find("i").length).toEqual(1);
        });

        it("should add the class no-label to an input whose parent does not have a label", function(){
            var scope = j.$rootScope.$new();
            var element = j.$compile("<div><input ng-model='model'></div>")(scope);
            expect(element.hasClass("no-label")).toEqual(true);
        });

        it("should add the class on-select to the select's div", function () {
            var scope = j.$rootScope.$new();
            var element = j.$compile("<div><select ng-model='model'></div>")(scope);
            expect(element.hasClass("on-select")).toEqual(true);
        });

        it("should not add the class no-label to an input whose parent does not have a label", function(){
            var scope = j.$rootScope.$new();
            var element = j.$compile("<div><label></label><input ng-model='model'></div>")(scope);
            expect(element.hasClass("no-label")).toEqual(false);
        });

        it('should add the class noc-error to the parent when the input is invalid', () => {
            const scope = j.$rootScope.$new();
            const parent = j.$compile("<div><input ng-model='model.value1' ng-pattern='/1234/'></div>")(scope);
            scope.model = {value1: 'abcd'};

            j.$rootScope.$apply();

            expect(parent.hasClass('noc-error')).toEqual(true);
        });

        it('should remove the class noc-error to the parent when the input is no longer invalid', () => {
            const scope = j.$rootScope.$new();
            const parent = j.$compile("<div><input ng-model='model.value1' ng-pattern='/1234/'></div>")(scope);
            scope.model = {value1: '1234'};

            j.$rootScope.$apply();

            expect(parent.hasClass('noc-error')).toEqual(false);
        });

        it("should add the class visited and remove the class focus to the parent when blurred", function () {
            var scope = j.$rootScope.$new();
            var parent = j.$compile("<div><input class='focus' ng-model='model'></div>")(scope);
            var input = parent.find("input");

            input.blur();

            j.$timeout.flush();
            expect(parent.hasClass("visited")).toEqual(true);
            expect(parent.hasClass("focus")).toEqual(false);
        });

        it("should add the class visited and remove the class focus to the parent when focused", function () {
            var scope = j.$rootScope.$new();
            var parent = j.$compile("<div><input ng-model='model'></div>")(scope);
            var input = parent.find("input");

            scope.$apply(function(){
                input.triggerHandler( "focus" );
            });

            expect(parent.hasClass("focus")).toEqual(true);
        });
    });

    describe("Popover", function () {
        it("should not add the popover if the element does not have ng-invalid", function () {
            var scope = j.$rootScope.$new();
            var holder = j.$compile("<div class='layout-content-holder'><div class='visited'><input ng-model='model'></div></div>")(scope);
            var parent = holder.children();
            var input = parent.find("input");

            scope.$apply(function(){
                input.triggerHandler( "focus" );
            });

            expect(holder.children().length).toEqual(1);
        });

        it("should not add the popover if the element's parent was not visited", function () {
            var scope = j.$rootScope.$new();
            var holder = j.$compile("<div class='layout-content-holder'><div><input class='ng-invalid' ng-model='model'></div></div>")(scope);
            var parent = holder.children();
            var input = parent.find("input");

            scope.$apply(function(){
                input.triggerHandler( "focus" );
            });

            expect(holder.children().length).toEqual(1);
        });

        it('should add the popover if the elements parent was visited and input is invalid', () => {
            const scope = j.$rootScope.$new();
            const holder = j.$compile("<div class='layout-content-holder'><div class='visited'><input ng-model='model.value1' ng-pattern='/1234/'></div></div>")(scope);
            scope.model = {value1: 'abcd'};
            const parent = holder.children();
            const input = parent.find('input');
            $('body').append(holder);

            scope.$apply(() => {
                input.triggerHandler('focus');
            });

            const popover = parent.children('.popover.error');
            expect(popover.length).toEqual(1);
        });

        describe("Text", function () {
            var scope;
            var holder;
            var input;
            beforeEach(function () {
                scope = j.$rootScope.$new();
                holder = j.$compile("<form name='formTest' class='layout-content-holder'><div class='visited'><input name='inputTest' ng-model='model'></div></form>")(scope);
                scope.formTest.inputTest.$setValidity('valid', false);
                const parent = holder.children();
                input = parent.find('input');

                $('body').append(holder);
            });

            let popover;
            const trigger = () => {
                scope.$apply(() => {
                    input.triggerHandler( 'focus' );
                });

                popover = input.parent().children('.popover.error');
            };

            it("should add the default error message if no corresponding message can be found", function () {
                trigger();
                expect(popover.text()).toEqual("Fill in valid input. ");
            });

            it("should add the message found from the given key if a key is present and corresponds to a message", function () {
                input.attr("data-error-message-any", "number");

                trigger();

                expect(popover.text()).toEqual("Enter a valid number. ");
            });

            it("should add the default message if a key is present but does not correspond to a message", function () {
                input.attr("data-error-key", "kuki");

                trigger();

                expect(popover.text()).toEqual("Fill in valid input. ");
            });

            it("should add a message whose key is in the keys array when the key is found on the class of the input", function () {
                input.addClass("ng-invalid-pattern");

                trigger();

                expect(popover.text()).toEqual('Input does not match the required pattern. ');
            });

            it("should not show popover for required fields, because its obvious", function () {
                input.addClass('ng-invalid-required');

                trigger();

                expect(popover.text()).toEqual('');
            });

            it("should add the leftmost message whose key is in the keys array when multiple keys are found on the class of the input", function () {
                input.addClass('ng-invalid-pattern');
                input.addClass("ng-invalid-email");

                trigger();

                expect(popover.text()).toEqual("Enter a valid email address. ");
            });

            it("should add the message corresponding to the given key even if the input has a class that matches one of the keys from the array", function () {
                input.attr("data-error-message-any", "number");
                input.addClass('ng-invalid-pattern');
                input.addClass("ng-invalid-email");

                trigger();

                expect(popover.text()).toEqual("Enter a valid number. ");
            });


            describe("data-error-message-any",function() {
                beforeEach(function() {
                    j.Messages.errors.bla = "bla";
                    j.Messages.errors.template= _.template("guys name is {{name}}");

                    scope = j.$rootScope.$new();
                });

                it('should use the provided data-error-message-any if available', () => {
                    holder = j.$compile("<div class='layout-content-holder'><div class='visited'><input ng-pattern='/1234/' data-error-message-pattern='bla' ng-model='model'></div></div>")(scope);
                    scope.model = {value1: 'abcd'};
                    const parent = holder.children();
                    input = parent.find('input');
                    $('body').append(holder);

                    trigger();

                    expect(popover.text()).toEqual('bla ');
                });

                it("should not use the provided data-error-message-any if the error is not related to the message", function() {
                    holder = j.$compile("<div class='layout-content-holder'><div class='visited'><input class='ng-invalid ng-invalid-required' data-error-message-pattern='wholeRange' ng-model='model'></div></div>")(scope);
                    var parent = holder.children();
                    input = parent.find("input");
                    $("body").append(holder);

                    trigger();

                    expect(popover.text()).not.toEqual("bla ");
                });

                it("should have templating functionality", function(){
                    holder = j.$compile("<div class='layout-content-holder'><div class='visited'><input class='ng-invalid ng-invalid-pattern' data-error-message-pattern='template' data-error-params='{name: \"guy\"}' ng-model='model'></div></div>")(scope);
                    var parent = holder.children();
                    input = parent.find("input");
                    $("body").append(holder);

                    trigger();

                    expect(popover.text()).not.toEqual("guys name is guy");
                });


                it("templating should fail gracfuly", function(){
                    holder = j.$compile("<div class='layout-content-holder'><div class='visited'><input class='ng-invalid ng-invalid-pattern' data-error-message-pattern='template' data-error-params='{name2: \"guy\"}' ng-model='model'></div></div>")(scope);
                    var parent = holder.children();
                    input = parent.find("input");
                    $("body").append(holder);

                    trigger();

                    expect(popover.text()).not.toEqual("guys name is");
                });
            });

            it("should remove the popover on blur", function () {
                trigger();

                input.triggerHandler( "blur" );

                expect(popover.parent().length).toEqual(0);
            });

            it("should remove the popover on destroy", function() {
                trigger();
                j.$rootScope.$broadcast('$destroy');
                expect(popover.parent().length).toEqual(0);
            });
        });
    });

    describe("noc-validation directive", function () {
        var scope;
        beforeEach(function () {
            scope = j.$rootScope.$new();
        });
        it("should replace itself with ng-pattern and data-error-message-pattern", function () {
            var element = j.$compile("<input type='text' noc-validation='workingEnvironmentName'>")(scope);
            expect(element.attr('ng-pattern')).toEqual(new RegExp(j.Patterns['workingEnvironmentName'].regex).toString());
            expect(element.attr('data-error-message-pattern')).toEqual('workingEnvironmentName');
        });
        it("should throw exception for wrong key", function () {
            var compileFunction = function () {
                j.$compile("<input type='text' noc-validation='thisKeyDoesNotExist'>")(scope);
            };
            expect(compileFunction).toThrow(new Error("Wrong or missing key for data-error-message-pattern"));
        });
        it("should throw exception for missing key", function () {
            var compileFunction = function () {
                j.$compile("<input type='text' noc-validation>")(scope);
            };
            expect(compileFunction).toThrow(new Error("Wrong or missing key for data-error-message-pattern"));
        });

        describe('validation behavior', function () {
            var form;
            beforeEach(function () {
                var element = '<form name="form">' +
                    '<input ng-model="model.value1" name="input1" noc-validation="workingEnvironmentName" />' +
                    '</form>';
                scope.model = {value1: null};
                j.$compile(element)(scope);
                form = scope.form;
            });

            it('should pass with integer', function () {
                form.input1.$setViewValue('abc');
                scope.$digest();
                expect(scope.model.value1).toEqual('abc');
                expect(form.input1.$valid).toBe(true);
            });

            it('should not pass with string', function () {
                form.input1.$setViewValue('3abc');
                scope.$digest();
                expect(scope.model.value1).toBeUndefined();
                expect(form.input1.$valid).toBe(false);
            });
        });
    });
});
