describe('action required resource', () => {
    var j;
    beforeEach(() => {
        j = magnifispec.setup();
    });
    let actionRequiredResource;
    let onPollFunc;
    let stopFunction;

    beforeEach(inject((_actionRequiredResource_) => {
        actionRequiredResource = _actionRequiredResource_;
        onPollFunc = jest.fn();
        stopFunction = jest.fn();
        jest.spyOn(j.nocPoller, 'start').mockReturnValue(stopFunction);
    }));

    describe('vsa action required polling', () => {
        it('should start the poller with the correct parameters', () => {
            const workingEnvironment = {_pathPrefix: '/vsa/', publicId: 'we1'};
            actionRequiredResource.startActionRequiredPolling(workingEnvironment, onPollFunc);

            expect(j.nocPoller.start).toHaveBeenCalled();
            const args = j.nocPoller.start.mock.calls[j.nocPoller.start.mock.calls.length - 1];

            expect(args[0]).toEqual({method: 'GET', url: '/vsa/working-environments/we1?fields=actionsRequired'});
            expect(args[1]).toEqual(3 * 60 * 1000);

            jest.spyOn(j.$rootScope, '$broadcast').mockImplementation(() => {
            });

            args[2]({actionsRequired: ['blah']});
            expect(j.$rootScope.$broadcast).toHaveBeenCalledWith(actionRequiredResource.ACTION_REQUIRED_GET_EVENT, ['blah']);

            args[3]({message: 'error'});
            expect(j.$rootScope.$broadcast).toHaveBeenCalledWith(actionRequiredResource.ACTION_REQUIRED_ERROR_EVENT, {message: 'error'});
        });

        it('should return a function which stops the poller when called', () => {
            const workingEnvironment = {_pathPrefix: '/vsa/', publicId: 'we1'};
            const stopper = actionRequiredResource.startActionRequiredPolling(workingEnvironment, onPollFunc);
            stopper();

            expect(stopFunction).toHaveBeenCalled();
        });
    });

    describe('global action required polling', () => {
        const apiResponse = {
            vsaWorkingEnvironments: [{
                publicId: 'publicId1',
                name: 'we1',
                actionsRequired: ['action1']
            }],
            onPremWorkingEnvironments: [{
                publicId: 'publicId2',
                name: 'we2',
                actionsRequired: ['action2a', 'action2b']
            }],
            azureVsaWorkingEnvironments: [{
                publicId: 'publicId3',
                name: 'we3',
                actionsRequired: ['action3']
            }]
        };

        it('should start the poller with the correct parameters', () => {
            actionRequiredResource.startGlobalActionRequiredPolling('tenantId1', onPollFunc);

            expect(j.nocPoller.start).toHaveBeenCalled();
            const args = mostRecentCall(j.nocPoller.start);

            expect(args[0]).toEqual({
                method: 'GET',
                url: 'working-environments?tenantId=tenantId1&fields=status,actionsRequired'
            });
            expect(args[1]).toEqual(5 * 60 * 1000);

            jest.spyOn(j.$rootScope, '$broadcast').mockImplementation(() => {
            });

            args[2](apiResponse);
            expect(j.$rootScope.$broadcast).toHaveBeenCalledWith(actionRequiredResource.GLOBAL_ACTION_REQUIRED_GET_EVENT, {
                publicId1: ['action1'],
                publicId2: ['action2a', 'action2b'],
                publicId3: ['action3']
            });

            args[3]({message: 'error'});
            expect(j.$rootScope.$broadcast).toHaveBeenCalledWith(actionRequiredResource.GLOBAL_ACTION_REQUIRED_ERROR_EVENT, {message: 'error'});
        });

        it('should return a function which stops the poller when called', () => {
            const workingEnvironment = {_pathPrefix: '/vsa/', publicId: 'we1'};
            const stopper = actionRequiredResource.startGlobalActionRequiredPolling(workingEnvironment, onPollFunc);
            stopper();

            expect(stopFunction).toHaveBeenCalled();
        });
    });
});
