describe("SystemResource", function () {
    var systemResource, systemResourceInstance;
    var j;    beforeEach(() => {        j = magnifispec.setup();    });
    var data = {data: "datum"}, response;

    beforeEach(inject(function (_systemResource_) {
        systemResource = _systemResource_;
    }));

    describe("Creating it with nocHttp", function () {
        beforeEach(function () {
            systemResourceInstance = systemResource(j.nocHttp);
            jest.spyOn(j.nocHttp, 'get');
        });

        it('should use the nocHttpInstance for http calls', function () {
            j.$httpBackend.expectGET("/occm/api/occm/system/occm-instance-placement").respond(200, data);
            systemResourceInstance.getInstancePlacement(j.nocHttp.LOADING).then(function(serverResponse) {
                response = serverResponse;
            });

            j.$httpBackend.flush();

            expect(j.nocHttp.get).toHaveBeenCalledWith({url: "/occm/system/occm-instance-placement", cache: true, method: "GET"}, "retrieving instance placement", j.nocHttp.LOADING);
            expect(response).toEqual(data);
        });
    });

    describe('Creating it without a nocHttp instance', function () {
        beforeEach(function () {
            systemResourceInstance = systemResource();
        });

        it("should throw an exception if improperly used", function () {
            expect(_.partial(systemResourceInstance.getInstancePlacement,"LOADING OR ANYTHING")).toThrowError();
        });

        it("should use the regular $http service if not given a nocHttp instance", function () {
            jest.spyOn(j.$http, 'get');

            j.$httpBackend.expectGET("/occm/api/occm/system/occm-instance-placement").respond(200, data);

            systemResourceInstance.getInstancePlacement().then(function(serverResponse) {
                response = serverResponse;
            });

            j.$httpBackend.flush();

            expect(response).toEqual(data);
            expect(j.$http.get).toHaveBeenCalledWith("/occm/system/occm-instance-placement", {cache: true});
        });

    });
});