describe("Navigate away service", function () {
    "use strict";
    var j;    beforeEach(() => {        j = magnifispec.setup();    });
    var nocNavigateAway;
    beforeEach(inject(function (_nocNavigateAway_) {
        nocNavigateAway = _nocNavigateAway_;
    }));

    it("should have the right event name", function () {
        expect(nocNavigateAway.eventName).toEqual("navigateAwayRequest");
    });

    describe("send NavigateAway", function () {
        it("should call proceedFunction if not cancelled",function() {
            var proceedFunction = jest.fn();

            j.$rootScope.$on(nocNavigateAway.eventName, function (event,payload) {
                expect(payload).toBeDefined();
            });

            nocNavigateAway.sendNavigateAwayMessage(proceedFunction);

            j.$timeout.flush();
            expect(proceedFunction).toHaveBeenCalled();
        });
        it("should do nothing if cancelled",function() {
            var proceedFunction = jest.fn();

            j.$rootScope.$on(nocNavigateAway.eventName, function (event,payload) {
                payload.cancel();
            });

            nocNavigateAway.sendNavigateAwayMessage(proceedFunction);

            expect(proceedFunction).not.toHaveBeenCalled();
        });
    });

    describe("handle navigate away request", function () {
        var payload;
        beforeEach(function () {
            payload = {
                cancel:jest.fn(),
                proceedFunction:jest.fn()
            };
        });

        it("should call cancel on payload", function () {
            magnifispec.fakeConfirmNew(false);
            nocNavigateAway.handleNavigateAway(payload);
            expect(payload.cancel).toHaveBeenCalled();
        });

        it("should call the function if it is a function, passing in the payload's proceedFunction", function () {
            magnifispec.fakeConfirmNew(false);
            var theFunction = jest.fn();
            nocNavigateAway.handleNavigateAway(payload, theFunction);

            expect(theFunction).toHaveBeenCalledWith(payload.proceedFunction);
        });

        it("should call window confirm with the passed in message, and if confirmed call payload's proceedFunction", function () {
            magnifispec.fakeConfirmNew(true);
            nocNavigateAway.handleNavigateAway(payload, "my message");

            expect(j.nocDialog.confirm).toHaveBeenCalledWith("my message");
            expect(payload.proceedFunction).toHaveBeenCalled();
        });

        it("should call window confirm with the default message when no message is passed in, and if confirmed call payload's proceedFunction", function () {
            magnifispec.fakeConfirmNew(false);
            nocNavigateAway.handleNavigateAway(payload);

            expect(j.nocDialog.confirm).toHaveBeenCalledWith(j.Messages.defaultConfirm);
        });

        it("should call window confirm, and if not confirmed should not call payload's proceedFunction", function () {
            magnifispec.fakeConfirmNew(false);
            nocNavigateAway.handleNavigateAway(payload);

            expect(payload.proceedFunction).not.toHaveBeenCalled();
        });


    });

    describe("promptOnLeaveBeforeDiscardingChanges", function () {
        var scope;
        beforeEach(function () {
            scope = j.$rootScope.$new();
            jest.spyOn(nocNavigateAway, 'handleNavigateAway').mockImplementation(() => {});
            nocNavigateAway.promptOnLeaveBeforeDiscardingChanges(scope);
        });

        it("should call handleNavigateAway if dirty", function () {
            scope.isDirty = _.constant(true);

            nocNavigateAway.sendNavigateAwayMessage();

            expect(nocNavigateAway.handleNavigateAway).toHaveBeenCalled();
        });

        it("should not call handleNavigateAway if clean", function () {
            scope.isDirty = _.constant(false);

            nocNavigateAway.sendNavigateAwayMessage();

            expect(nocNavigateAway.handleNavigateAway).not.toHaveBeenCalled();
        });
    });


});