describe("nocHttp factory Service", function () {
    "use strict";
    var j;    beforeEach(() => {        j = magnifispec.setup();    });
    var nocHttp, successFunction, failureFunction, finallyFunction;


    beforeEach(inject(function (_nocHttpFactory_) {
        nocHttp = _nocHttpFactory_.spawn(j.pidgeon, j.nocUI);
        successFunction = jest.fn();
        failureFunction = jest.fn();
        finallyFunction = jest.fn();
    }));

    function successExpectations(){
        expect(j.pidgeon.clear).toHaveBeenCalledWith("doing action 1");
        expect(j.pidgeon.error).not.toHaveBeenCalled();
        expect(j.nocUI.loading).toHaveBeenCalled();
        expect(j.nocUI.unloading).toHaveBeenCalled();
        expect(successFunction).toHaveBeenCalledWith("response");
        expect(failureFunction).not.toHaveBeenCalled();
        expect(finallyFunction).toHaveBeenCalled();
    }

    function errorExpectations(){
        expect(j.pidgeon.clear).toHaveBeenCalledWith("doing action 1");
        expect(j.pidgeon.error).toHaveBeenCalledWith("An error occurred while doing action 1", j.errorResponse, "doing action 1");
        expect(j.nocUI.loading).toHaveBeenCalled();
        expect(j.nocUI.unloading).toHaveBeenCalled();
        expect(successFunction).not.toHaveBeenCalled();
        expect(failureFunction).toHaveBeenCalledWith(j.errorResponse);
        expect(finallyFunction).toHaveBeenCalled();
    }

    describe("get", function () {
        it("should work upon success", function () {
            j.$httpBackend.expectGET(j.nocBaseUrl + '/myUrl').respond(200, "response");
            nocHttp.get('/myUrl', "doing action 1",nocHttp.LOADING)
                .then(successFunction, failureFunction)
                .finally(finallyFunction);

            j.$httpBackend.flush();

            successExpectations();
        });

        it("should work upon error", function () {
            j.$httpBackend.expectGET(j.nocBaseUrl + '/myUrl').respond(400, j.errorResponse);
            nocHttp.get('/myUrl', "doing action 1",nocHttp.LOADING)
                .then(successFunction, failureFunction)
                .finally(finallyFunction);

            j.$httpBackend.flush();

            errorExpectations();
        });

        it("should not pidgeon if errorMessage is null", function () {
            j.$httpBackend.expectGET(j.nocBaseUrl + '/myUrl').respond(400, j.errorResponse);
            nocHttp.get('/myUrl', null,nocHttp.LOADING);

            j.$httpBackend.flush();

            expect(j.pidgeon.clear).not.toHaveBeenCalled();
            expect(j.pidgeon.error).not.toHaveBeenCalled();
        });

        it("should call block when needed", function () {
            j.$httpBackend.expectGET(j.nocBaseUrl + '/myUrl').respond(200, j.errorResponse);
            nocHttp.get('/myUrl', null,nocHttp.BLOCK);

            j.$httpBackend.flush();

            expect(j.nocUI.block).toHaveBeenCalled();
            expect(j.nocUI.unblock).toHaveBeenCalled();
            expect(j.nocUI.loading).not.toHaveBeenCalled();
            expect(j.nocUI.unloading).not.toHaveBeenCalled();
        });

        it("should work without changing the UI", function () {
            j.$httpBackend.expectGET(j.nocBaseUrl + '/myUrl').respond(200, "response");
            nocHttp.get('/myUrl', null, null)
                .then(successFunction);

            j.$httpBackend.flush();

            expect(successFunction).toHaveBeenCalledWith("response");

            expect(j.nocUI.block).not.toHaveBeenCalled();
            expect(j.nocUI.unblock).not.toHaveBeenCalled();
            expect(j.nocUI.loading).not.toHaveBeenCalled();
            expect(j.nocUI.unloading).not.toHaveBeenCalled();
        });
    });

    describe("post", function () {
        var postedObject = {"posted": "object"};
        it("should work upon success", function () {
            j.$httpBackend.expectPOST(j.nocBaseUrl + '/myUrl', postedObject).respond(200, "response");
            nocHttp.post({url:'/myUrl', data:postedObject}, "doing action 1",nocHttp.LOADING)
                .then(successFunction, failureFunction)
                .finally(finallyFunction);

            j.$httpBackend.flush();

            successExpectations();
        });

        it("should work upon error", function () {
            j.$httpBackend.expectPOST(j.nocBaseUrl + '/myUrl').respond(400, j.errorResponse);
            nocHttp.post({url:'/myUrl', data:postedObject}, "doing action 1",nocHttp.LOADING)
                .then(successFunction, failureFunction)
                .finally(finallyFunction);

            j.$httpBackend.flush();

            errorExpectations();
        });
    });

    describe("put", function () {
        var puttedObject = {"posted": "object"};
        it("should work upon success", function () {
            j.$httpBackend.expectPUT(j.nocBaseUrl + '/myUrl', puttedObject).respond(200, "response");
            nocHttp.put({url:'/myUrl', data:puttedObject}, "doing action 1",nocHttp.LOADING)
                .then(successFunction, failureFunction)
                .finally(finallyFunction);

            j.$httpBackend.flush();

            successExpectations();
        });

        it("should work upon error", function () {
            j.$httpBackend.expectPUT(j.nocBaseUrl + '/myUrl').respond(400, j.errorResponse);
            nocHttp.put({url:'/myUrl', data:puttedObject}, "doing action 1",nocHttp.LOADING)
                .then(successFunction, failureFunction)
                .finally(finallyFunction);

            j.$httpBackend.flush();

            errorExpectations();
        });
    });

    describe("delete", function () {
        it("should work upon success", function () {
            j.$httpBackend.expectDELETE(j.nocBaseUrl + '/myUrl').respond(200, "response");
            nocHttp.delete('/myUrl', "doing action 1",nocHttp.LOADING)
                .then(successFunction, failureFunction)
                .finally(finallyFunction);

            j.$httpBackend.flush();

            successExpectations();
        });

        it("should work upon error", function () {
            j.$httpBackend.expectDELETE(j.nocBaseUrl + '/myUrl').respond(400, j.errorResponse);
            nocHttp.delete('/myUrl', "doing action 1",nocHttp.LOADING)
                .then(successFunction, failureFunction)
                .finally(finallyFunction);

            j.$httpBackend.flush();

            errorExpectations();
        });
    });
});