describe("OCCM Update Service", function () {
    "use strict";

    var occmUpdateService, updates = [{'name': 'update'}];
    var j;    beforeEach(() => {        j = magnifispec.setup();    });

    beforeEach(inject(function (_occmUpdateService_) {
        j.$rootScope.checkAndSetState = jest.fn();
        j.$rootScope.updates = updates;
        occmUpdateService = _occmUpdateService_;
    }));

    describe("before there're available updates from server", function () {
        it("should wait until the property gets assigned", function () {
            magnifispec.fakeConfirmNew("OK");

            j.$rootScope.updates = undefined;
            occmUpdateService.showUpdatesDialog('updates');
            j.$rootScope.$digest();
            expect(j.nocDialog.confirm).not.toHaveBeenCalled();
            j.$rootScope.updates = updates;
            j.$rootScope.$digest();
            expect(j.nocDialog.confirm).toHaveBeenCalled();

        });
    });

    describe("when there are updates available", function() {
        beforeEach(function () {
            magnifispec.fakeConfirmNew("OK");
        });

        it("should show the update dialog", function(){
            occmUpdateService.showUpdatesDialog('updates');
            j.$rootScope.$digest();
            expect(j.nocDialog.confirm).toHaveBeenCalled();
        });

        it("should not show the update dialog the second time its called", function(){
            occmUpdateService.showUpdatesDialog('updates');
            occmUpdateService.showUpdatesDialog('updates');
            j.$rootScope.$digest();
            expect(j.nocDialog.confirm.mock.calls.length).toEqual(1);
            expect(j.$rootScope.$$watchers.length).toEqual(1);

        });

        it("should not show the update dialog if there's a flag in the local storage with it's name", function(){
            j.nocLocalStorage.LATEST_UPDATE_IGNORED.save(updates[0].name);

            occmUpdateService.showUpdatesDialog('updates');

            j.$rootScope.$digest();

            expect(j.nocDialog.confirm).not.toHaveBeenCalled();

            j.nocLocalStorage.LATEST_UPDATE_IGNORED.save();
        });
    });

    describe("when there are no updates", function () {
        it("should not show the dialog", function () {
            j.$rootScope.updates = [];
            magnifispec.fakeConfirmNew();

            occmUpdateService.showUpdatesDialog('updates');

            j.$rootScope.$digest();

            expect(j.nocDialog.confirm).not.toHaveBeenCalled();
        });
    });
});
