describe('', function () {
    'use strict';
    let j, _nocCurrentUserService;
    beforeEach(() => {
        j = magnifispec.setup();
    });


    beforeEach(inject(function (_nocCurrentUserService_) {
        _nocCurrentUserService = _nocCurrentUserService_;

    }));

    var setTheUserPermissions = (arra) => {
        _nocCurrentUserService.initializeCurrentUser({permissions: arra});
    };

    beforeEach(inject(function (nocCurrentUserService) {
        _nocCurrentUserService = nocCurrentUserService;

        setTheUserPermissions(['billing:get', 'working-environments:*', 'resources:get:{puki}:kuki:*', 'status:show', 'users:*:WE admin:{tenant}:*']);
    }));


    it('should return true on empty string', function () {
        expect(_nocCurrentUserService.isAllowed('')).toEqual(true);
    });

    it('should return true on empty array', function () {
        expect(_nocCurrentUserService.isAllowed([])).toEqual(true);
    });

    it('should return true on undefined', function () {
        expect(_nocCurrentUserService.isAllowed()).toEqual(true);
    });

    it('should return true if the required permissions are a subset of the available permissions', function () {
        expect(_nocCurrentUserService.isAllowed('billing:get')).toEqual(true); //one at start as string

        expect(_nocCurrentUserService.isAllowed(['billing:get'])).toEqual(true); //one at start in array

        expect(_nocCurrentUserService.isAllowed(['resources:get'])).toEqual(true); // one in middle

        expect(_nocCurrentUserService.isAllowed(['resources:get', 'users:show'])).toEqual(true); //two in order

        expect(_nocCurrentUserService.isAllowed(['users:show', 'resources:get'])).toEqual(true);  //two not in order

        expect(_nocCurrentUserService.isAllowed(['working-environments:get', 'billing:get', 'users:show'])).toEqual(true); // three not in order

        expect(_nocCurrentUserService.isAllowed(['working-environments:create', 'resources:get', 'status:show', 'billing:get', 'users:show'])).toEqual(true); //whole list out of order
    });

    it('should return false if the required permissions are not a subset of the available permissions', function () {
        expect(_nocCurrentUserService.isAllowed('fuckyou:get')).toEqual(false); //one not in the list as string

        expect(_nocCurrentUserService.isAllowed('billing:create')).toEqual(false); //one not in the list as string (first is, second isnt)

        expect(_nocCurrentUserService.isAllowed(['fuckyou:get'])).toEqual(false); //one not in the list as array

        expect(_nocCurrentUserService.isAllowed(['billing:get', 'fuckyou:get'])).toEqual(false); //one in, one out

        expect(_nocCurrentUserService.isAllowed(['fuckyou:get', 'resources:get'])).toEqual(false);  //one out, one in

        expect(_nocCurrentUserService.isAllowed(['working-environments:get', 'resources:get', 'status:show', 'billing:get', 'users:update', 'fuckyou:get'])).toEqual(false); //five in one out
    });

    describe('IsPermitted', function () {
        it('simple complex true', function () {
            setTheUserPermissions(['work_env:*']);
            expect(_nocCurrentUserService.isAllowed('work_env:create')).toEqual(true);

            setTheUserPermissions(['work_env:create']);
            expect(_nocCurrentUserService.isAllowed('work_env:create')).toEqual(true);

            setTheUserPermissions(['work_env:create:*']);
            expect(_nocCurrentUserService.isAllowed('work_env:create')).toEqual(true);

            setTheUserPermissions(['work_env:create:even_more']);
            expect(_nocCurrentUserService.isAllowed('work_env:create')).toEqual(true);
        });

        it('simple complex false', function () {
            setTheUserPermissions(['work_:*']);
            expect(_nocCurrentUserService.isAllowed('work_env:create')).toEqual(false);

            setTheUserPermissions(['work_:create']);
            expect(_nocCurrentUserService.isAllowed('work_env:create')).toEqual(false);

            setTheUserPermissions(['work_env:create']);
            expect(_nocCurrentUserService.isAllowed('work_env:show')).toEqual(false);
        });

        it('complex complex true', function () {
            setTheUserPermissions(['work_env:*']);
            expect(_nocCurrentUserService.isAllowed(['work_env:create', 'work_env:update'])).toEqual(true);

            setTheUserPermissions(['work_env:create', 'work_env:update']);
            expect(_nocCurrentUserService.isAllowed(['work_env:create', 'work_env:update'])).toEqual(true);

            setTheUserPermissions(['work_env:create:*', 'work_env:update:*']);
            expect(_nocCurrentUserService.isAllowed(['work_env:create', 'work_env:update'])).toEqual(true);
        });

        it('complex complex false', function () {
            setTheUserPermissions(['work_env:create']);
            expect(_nocCurrentUserService.isAllowed(['work_env:create', 'work_env:update'])).toEqual(false);

            setTheUserPermissions(['work_env:get', 'work_env:update']);
            expect(_nocCurrentUserService.isAllowed(['work_env:create', 'work_env:update'])).toEqual(false);

            setTheUserPermissions(['work_env:create:*', 'work_env:delete:*']);
            expect(_nocCurrentUserService.isAllowed(['work_env:create', 'work_env:update'])).toEqual(false);
        });

        it('user permission is greater than required permission', function () {
            setTheUserPermissions(['work_env:create:hili:*']);
            expect(_nocCurrentUserService.isAllowed(['work_env:create'])).toEqual(true);
        });
    });
});
