describe.skip("Pidgeon factory", function () {
    "use strict";
    var j;
    beforeEach(() => {
        j = magnifispec.setup();
    });
    var pidgeon, scope, $onDestroyFunction;
    beforeEach(inject(function (_pidgeonFactory_) {
        scope = {
            $on: function (name, callback) {
                if (name === "$destroy") $onDestroyFunction = callback;
            },
            $broadcast: function(name){
                if (name === "$destroy") $onDestroyFunction();
            },
            pushError: jest.fn(),
            pushSuccess: jest.fn(),
            pushInProgress: jest.fn(),
            clearMessages: jest.fn()
        };
        pidgeon = _pidgeonFactory_.spawn("class", scope);

    }));

    describe("error", function () {
        var errorResponse;
        beforeEach(function () {
            errorResponse = {
                messageForDisplay: _.constant("Server is lazy.")
            };
        });
        it("should add ':' if errorResponse is supplied", function () {
            pidgeon.error("Cannot do something", errorResponse);
            expect(scope.pushError).toHaveBeenCalledWith("Cannot do something: Server is lazy.", errorResponse, 'class', undefined);
        });
        it("should work with errorResponse without prefix", function () {
            pidgeon.error("", errorResponse, "callerType1");
            expect(scope.pushError).toHaveBeenCalledWith("Server is lazy.", errorResponse, 'class', "callerType1");
        });
    });

    describe("sucess", function () {
        it("should call pushSuccess", function () {
            pidgeon.success("Something worked.");
            expect(scope.pushSuccess).toHaveBeenCalledWith("Something worked.");
        });
    });

    describe("inProgress", function () {
        it("should call pushInProgress with seeTimeline param true when passed", function () {
            pidgeon.inProgress("Something is in progress...", true);
            expect(scope.pushInProgress).toHaveBeenCalledWith("Something is in progress...", true, undefined);
        });
        it("should call pushInProgress with seeTimeline param false when passed", function () {
            pidgeon.inProgress("Something is in progress...", false);
            expect(scope.pushInProgress).toHaveBeenCalledWith("Something is in progress...", false, undefined);
        });
        it("should call pushInProgress with duration param when passed", function () {
            pidgeon.inProgress("Something is in progress...", false, 3456);
            expect(scope.pushInProgress).toHaveBeenCalledWith("Something is in progress...", false, 3456);
        });
    });

    describe("clear", function () {
        it("should call clearMessages", function () {
            pidgeon.clear("callerType1");
            expect(scope.clearMessages).toHaveBeenCalledWith("class", "callerType1");
        });
    });

    describe("kill", function () {
        it("should kill the pidgeon", function () {
            pidgeon.kill();
            expect(scope.clearMessages).toHaveBeenCalledWith("class");

            pidgeon.error("Cannot do something.");
            expect(scope.pushError).not.toHaveBeenCalled();

            pidgeon.success("Something worked.");
            expect(scope.pushSuccess).not.toHaveBeenCalled();

            pidgeon.inProgress("Something is in progress...", true);
            expect(scope.pushInProgress).not.toHaveBeenCalled();

            pidgeon.clear("callerType1");
            expect(scope.clearMessages).not.toHaveBeenCalledWith("class", "callerType1");

            pidgeon.kill(); //kill again should do nothing
            expect(scope.clearMessages.mock.calls.length).toEqual(1);
        });
    });

    describe("destroy", function () {
        it("should kill the pidgeon upon destroy", function () {
            jest.spyOn(pidgeon, "kill").mockImplementation(() => {});
            scope.$broadcast("$destroy");
            expect(pidgeon.kill).toHaveBeenCalled();
        });
    });
});