describe('routing resolver service spec', () => {
    'use strict';

    var j;
    beforeEach(() => {
        j = magnifispec.setup();
    });
    let routingResolverService;
    let currentUserService;

    beforeEach(inject((nocRoutingResolverService, nocCurrentUserService) => {
        routingResolverService = nocRoutingResolverService;
        currentUserService = nocCurrentUserService;
    }));

    describe('is volume view allowed', function () {
        var result;
        beforeEach(function () {
            result = undefined;
            j.$rootScope.supportServices = {volumeViewEnabled: true};
            const currentUser = {role: {publicId: 'Role-1'}};
            currentUserService.initializeCurrentUser(currentUser);
            jest.spyOn(j.$log, 'log').mockImplementation(() => {});
        });

        afterEach(function () {
            expect(j.$log.log).not.toHaveBeenCalled();
        });
        var callIsVolViewAllowed = function () {
            routingResolverService.isVolumeViewAllowed(j.$rootScope).then(function () {
                result = true;
            }, function (rejection) {
                result = rejection;
            });
        };

        it('should not allow if not admin', function () {
            j.$rootScope.currentUser = {role: {publicId: 'Role-2'}};

            callIsVolViewAllowed();
            j.$timeout.flush();

            expect(result).toEqual(routingResolverService.VOL_VIEW_NOT_ALLOWED);
        });

        it('should not allow if volume view not enabled', function () {
            j.$rootScope.supportServices = {volumeViewEnabled: false};

            callIsVolViewAllowed();
            j.$timeout.flush();

            expect(result).toEqual(routingResolverService.VOL_VIEW_NOT_ALLOWED);
        });

        //todo: fix
        // it('should allow if the instance placement is empty', function () {
        //     j.$httpBackend.expectGET('/occm/api/occm/system/occm-instance-placement').respond(204, '');
        //
        //     callIsVolViewAllowed();
        //     //j.$httpBackend.flush();
        //
        //     expect(result).toEqual(true);
        // });
        //
        //  it('should allow if the instance placement has anything, means we\'re on AWS', function () {
        //     j.$httpBackend.expectGET('/occm/api/occm/system/occm-instance-placement').respond({region: 'one'});
        //
        //     callIsVolViewAllowed();
        //     j.$httpBackend.flush();
        //
        //     expect(result).toEqual(true);
        // });
    });

    describe('resolve entry point', function () {
        beforeEach(function () {
            jest.spyOn(j.$state, 'go').mockImplementation(() => {
            });
            j.$rootScope.supportServices = {volumeViewEnabled: true};
            j.$rootScope.currentUser = {role: {publicId: 'Role-1'}};
            j.$rootScope.appState = 'normal';
        });

        it('should go to volume view if its allowed and is set in local storage', function () {
            jest.spyOn(j.nocLocalStorage.VOLUME_VIEW, 'isDefined').mockReturnValue(true);
            jest.spyOn(j.nocLocalStorage.VOLUME_VIEW, 'restore').mockReturnValue(true);
            jest.spyOn(routingResolverService, 'isVolumeViewAllowed').mockReturnValue(j.$q.when());
            routingResolverService.resolveEntryPoint(j.nocHttp);
            j.$timeout.flush();
            expect(j.$state.go).toHaveBeenCalledWith('volume-view.volumes');
        });

        it('should go to main view if is set in local storage', function () {
            jest.spyOn(j.nocLocalStorage.VOLUME_VIEW, 'isDefined').mockReturnValue(true);
            jest.spyOn(j.nocLocalStorage.VOLUME_VIEW, 'restore').mockReturnValue(false);
            jest.spyOn(routingResolverService, 'isVolumeViewAllowed').mockReturnValue(j.$q.when());
            routingResolverService.resolveEntryPoint(j.nocHttp);
            j.$timeout.flush();
            expect(j.$state.go).toHaveBeenCalledWith('main');
        });

        it('should ask OCCM where to go if nothing is in the local storage and set the local storage to the value that came from the server', function () {
            jest.spyOn(j.nocLocalStorage.VOLUME_VIEW, 'isDefined').mockReturnValue(false);
            jest.spyOn(routingResolverService, 'isVolumeViewAllowed').mockReturnValue(j.$q.when());
            jest.spyOn(j.nocLocalStorage.VOLUME_VIEW, 'save').mockImplementation(() => {
            });
            j.$httpBackend.expectGET('/occm/api/occm/config').respond({useVolumeViewAsDefault: true});
            routingResolverService.resolveEntryPoint(j.nocHttp);
            j.$httpBackend.flush();
            expect(j.$state.go).toHaveBeenCalledWith('volume-view.volumes');
            expect(j.nocLocalStorage.VOLUME_VIEW.save).toHaveBeenCalledWith(true);
        });

        it('should kick the user to classic view if he is not allowed to go to volume view', function () {
            jest.spyOn(routingResolverService, 'isVolumeViewAllowed').mockReturnValue(j.$q.reject());
            routingResolverService.resolveEntryPoint(j.nocHttp);
            j.$timeout.flush();
            expect(j.$state.go).toHaveBeenCalledWith('main');
        });
    });
});
