describe('ServerUpgradeService', () => {
    var j;    beforeEach(() => {        j = magnifispec.setup();    });
    let ServerUpgradeService, callback;

    beforeEach(inject((_ServerUpgradeService_) => {
        ServerUpgradeService = _ServerUpgradeService_;
        advanceTo(moment('2017-01-01').toDate());

        j.$rootScope.about = {buildTimestamp: 0};
        callback = jest.fn().mockReturnValue(j.$q.when('callback promise value'));
    }));

    describe('pollForServerUpgrade', () => {
        it('should call the callback when new server is up', () => {
            j.$httpBackend.expectGET(j.nocBaseUrl + '/occm/system/about').respond(200, {buildTimestamp: 1});

            ServerUpgradeService.pollForServerUpgrade(callback);
            j.$httpBackend.flush();

            expect(callback).toHaveBeenCalled();
        });

        it('should wait until new server is up', () => {
            j.$httpBackend.expectGET(j.nocBaseUrl + '/occm/system/about').respond(200, {buildTimestamp: 0});

            ServerUpgradeService.pollForServerUpgrade(callback);
            j.$httpBackend.flush();

            expect(callback).not.toHaveBeenCalled();

            j.$httpBackend.expectGET(j.nocBaseUrl + '/occm/system/about').respond(200, {buildTimestamp: 1});
            j.$timeout.flush();
            j.$httpBackend.flush();

            expect(callback).toHaveBeenCalled();
        });

        it('should handle no response from server as limbo and wait more', () => {
            j.$httpBackend.expectGET(j.nocBaseUrl + '/occm/system/about').respond(400, j.errorResponse);

            ServerUpgradeService.pollForServerUpgrade(callback);
            j.$httpBackend.flush();

            expect(callback).not.toHaveBeenCalled();

            j.$httpBackend.expectGET(j.nocBaseUrl + '/occm/system/about').respond(200, {buildTimestamp: 1});
            j.$timeout.flush();
            j.$httpBackend.flush();

            expect(callback).toHaveBeenCalled();
        });

        it('should call callback anyway after threshold time is up', () => {
            j.$httpBackend.expectGET(j.nocBaseUrl + '/occm/system/about').respond(400, j.errorResponse);
            const promise = ServerUpgradeService.pollForServerUpgrade(callback);

            j.$httpBackend.flush();
            expect(callback).not.toHaveBeenCalled();

            //Now it should give up and call the callback:
            advanceTo(moment('2017-01-01 00:10').toDate());
            j.$httpBackend.expectGET(j.nocBaseUrl + '/occm/system/about').respond(400, j.errorResponse);
            j.$timeout.flush();
            j.$httpBackend.flush();

            expect(callback).toHaveBeenCalled();
            expect(promise).toBeResolvedWith('callback promise value');
        });

        it('should use location.reload as default callback in case no callback is passed', () => {
            ServerUpgradeService.pollForServerUpgrade();

            expect(j.$log.debug).toHaveBeenCalledWith('pollForServerUpgrade called with no callback, using location.reload as default.');
        });
    });
});
