describe.skip('nocUIMain spec', () => {
    'use strict';
    var j;
    beforeEach(() => {
        j = magnifispec.setup();
    });

    let _nocUIMain, _nocUIFactory, $document, $rootScope, $timeout;
    let mockedFoundBlockObject, mockedFoundLoadingObject;

    const mockWindow = {location: {reload: _.noop}, localStorage: {}, angular: {}};

    beforeEach(angular.mock.module(($provide) => {
        mockedFoundBlockObject = {
            show: jest.fn(),
            data: jest.fn(),
            hide: jest.fn(),
            css: jest.fn()
        };
        mockedFoundLoadingObject = {
            show: jest.fn(),
            data: jest.fn(),
            hide: jest.fn(),
            css: jest.fn()
        };

        $document = {
            find: (str) => {
                if (str === '#block') return mockedFoundBlockObject;
                else return mockedFoundLoadingObject;
            },
            on: jest.fn(),
            off: jest.fn()
        };

        $provide.value('$window', mockWindow);
        $provide.value('$document', $document);
    }));

    beforeEach(inject((nocUIMain, nocUIFactory, _$rootScope_, _$timeout_) => {
        _nocUIMain = nocUIMain;
        _nocUIFactory = nocUIFactory;
        $rootScope = _$rootScope_;
        $timeout = _$timeout_;
    }));

    describe('Handling form validity', () => {
        it('should add the class visited to the body when calling handleFormValidity', () => {
            const body = $('body');
            body.removeClass('visited');
            _nocUIMain.handleFormValidity();
            expect(body.hasClass('visited')).toEqual(true);
        });

        it('should remove the class visited from every element when calling clearFormValidity', () => {
            const body = $('body');
            body.addClass('visited');
            const div = $('div');
            body.append(div);
            div.addClass('visited');
            _nocUIMain.clearFormValidity();
            expect(body.hasClass('visited')).toEqual(false);
            expect(div.hasClass('visited')).toEqual(false);
        });
    });

    describe('Reload silently', () => {
        it('should unbind before unload and call window reload', inject(($window) => {
            jest.spyOn(window, '$');
            jest.spyOn($window.location, 'reload').mockImplementation(() => {});
            _nocUIMain.reloadSilently();

            expect(window.$).toHaveBeenCalledWith('body');
            expect(window.$).toHaveBeenCalledWith($window);
            expect($window.location.reload).toHaveBeenCalled();
        }));

        it('should set $window.location to the root if backToRoot is passed in', inject(($window) => {
            $window.location.href = 'http://root:22/something#test';
            jest.spyOn(window, '$');
            _nocUIMain.reloadSilently(true);

            expect($window.location.href).toEqual('http://root:22/something');
        }));
    });

    describe('BlockUI', () => {
        it('should show the block div if called once', () => {
            _nocUIMain.block();
            expect(mockedFoundBlockObject.show).toHaveBeenCalled();
        });

        it('should return a function which decreases the loading count by one when called for the first time', () => {
            const unblock = _nocUIMain.block();
            expect(mockedFoundBlockObject.show.mock.calls.length).toEqual(1);
            expect(mockedFoundBlockObject.hide.mock.calls.length).toEqual(0);
            expect(mockedFoundLoadingObject.show.mock.calls.length).toEqual(1);
            expect(mockedFoundLoadingObject.hide.mock.calls.length).toEqual(0);
            unblock();
            unblock();
            expect(mockedFoundBlockObject.show.mock.calls.length).toEqual(1);
            expect(mockedFoundBlockObject.hide.mock.calls.length).toEqual(1);
            expect(mockedFoundLoadingObject.show.mock.calls.length).toEqual(1);
            expect(mockedFoundLoadingObject.hide.mock.calls.length).toEqual(1);
            _nocUIMain.loading();
            unblock();
            expect(mockedFoundBlockObject.show.mock.calls.length).toEqual(1);
            expect(mockedFoundBlockObject.hide.mock.calls.length).toEqual(1);
            expect(mockedFoundLoadingObject.show.mock.calls.length).toEqual(2);
            expect(mockedFoundLoadingObject.hide.mock.calls.length).toEqual(1);
        });

        it('should hide the block div if unblock is called the same number of times as blocked', () => {
            const unblock1 = _nocUIMain.block();
            const unblock2 = _nocUIMain.block();
            const unblock3 = _nocUIMain.block();
            unblock1();
            unblock2();
            unblock3();
            expect(mockedFoundBlockObject.show.mock.calls.length).toEqual(1);
            expect(mockedFoundBlockObject.hide.mock.calls.length).toEqual(1);
            expect(mockedFoundLoadingObject.show.mock.calls.length).toEqual(1);
            expect(mockedFoundLoadingObject.hide.mock.calls.length).toEqual(1);
        });

        it('should show the block div if unblock is called once less than blocked', () => {
            const unblock1 = _nocUIMain.block();
            const unblock2 = _nocUIMain.block();
            _nocUIMain.block();
            unblock1();
            unblock2();
            expect(mockedFoundBlockObject.show.mock.calls.length).toEqual(1);
            expect(mockedFoundBlockObject.hide.mock.calls.length).toEqual(0);
            expect(mockedFoundLoadingObject.show.mock.calls.length).toEqual(1);
            expect(mockedFoundLoadingObject.hide.mock.calls.length).toEqual(0);
        });

        it('should prevent events from being called on document when blocked', () => {
            _nocUIMain.block();
            expect($document.on).toHaveBeenCalledWith('mousedown mouseup keydown keypress keyup touchstart touchend touchmove', false);
        });

        it('should enable events from being called on document when unblocked', () => {
            const unblock = _nocUIMain.block();
            unblock();
            expect($document.off).toHaveBeenCalledWith('mousedown mouseup keydown keypress keyup touchstart touchend touchmove');
        });

        it('should set the background color of the block div to transparent if the appropriate parameter is passed, and set it back in unblock', () => {
            const unblock = _nocUIMain.block('class1', true);
            expect(mockedFoundBlockObject.css).toHaveBeenCalledWith('background-color', 'transparent');

            unblock();
            expect(mostRecentCall(mockedFoundBlockObject.css)).toEqual(['background-color', '']);
        });

        it('unblock should stop loading component without 500ms timeout', () => {
            const unblock = _nocUIMain.block('class1', true);
            expect(mockedFoundBlockObject.show.mock.calls.length).toEqual(1);
            expect(mockedFoundLoadingObject.show.mock.calls.length).toEqual(1);
            unblock();
            expect(mockedFoundBlockObject.hide.mock.calls.length).toEqual(1);
            expect(mockedFoundLoadingObject.hide.mock.calls.length).toEqual(1); //Hide call was made with no timeout
            $timeout.flush(500);
            expect(mockedFoundLoadingObject.hide.mock.calls.length).toEqual(1); //Hide call stays the same after timeout
        });
    });

    describe('LoadingUI', () => {
        it('should show the loading div if called once', () => {
            _nocUIMain.loading();
            expect(mockedFoundLoadingObject.show).toHaveBeenCalled();
        });

        it('should not be able to hide a show until after 500 milliseconds', () => {
            const unloading = _nocUIMain.loading();
            expect(mockedFoundLoadingObject.show.mock.calls.length).toEqual(1); //First call to loading show show once
            expect(mockedFoundLoadingObject.hide.mock.calls.length).toEqual(0); //First call to loading should not hide
            unloading();
            expect(mockedFoundLoadingObject.hide.mock.calls.length).toEqual(0); //First call to unloading should not hide yet
            $timeout.flush(500);
            expect(mockedFoundLoadingObject.hide.mock.calls.length).toEqual(1); //After timeout should have automatically hidden
        });

        it('should return a function which decreases the loading count by one when called for the first time', () => {
            const unloading = _nocUIMain.loading();
            $timeout.flush(500);
            expect(mockedFoundLoadingObject.show.mock.calls.length).toEqual(1); //First call to loading show show once
            expect(mockedFoundLoadingObject.hide.mock.calls.length).toEqual(0); //First call to loading should not hide
            unloading();
            unloading();
            expect(mockedFoundLoadingObject.hide.mock.calls.length).toEqual(1); //To successive calls to the same unloading only make one hide
            expect(mockedFoundLoadingObject.hide.mock.calls.length).toEqual(1);
            _nocUIMain.loading();
            unloading();
            expect(mockedFoundLoadingObject.show.mock.calls.length).toEqual(2); //New call to loading adds another show
            expect(mockedFoundLoadingObject.hide.mock.calls.length).toEqual(1); //Call to old unloading makes no change
        });

        it('should hide the loading div if unloading is called the same number of times as loading', () => {
            const unloading = [];
            unloading.push(_nocUIMain.loading());
            unloading.push(_nocUIMain.loading());
            unloading.push(_nocUIMain.loading());
            $timeout.flush(500);
            unloading[0]();
            unloading[1]();
            unloading[2]();
            expect(mockedFoundLoadingObject.show.mock.calls.length).toEqual(1);
            expect(mockedFoundLoadingObject.hide.mock.calls.length).toEqual(1);
        });

        it('should show the loading div if unloading is called once less than loading', () => {
            const unloading = [];
            unloading.push(_nocUIMain.loading());
            unloading.push(_nocUIMain.loading());
            unloading.push(_nocUIMain.loading());
            $timeout.flush(500);
            unloading[0]();
            unloading[1]();
            expect(mockedFoundLoadingObject.show.mock.calls.length).toEqual(1);
            expect(mockedFoundLoadingObject.hide.mock.calls.length).toEqual(0);
        });
    });

    describe('logout', () => {
        beforeEach(inject(($httpBackend, nocBaseUrl, nocUIMain, $window) => {
            $window.location.href = 'http://root:22/something#test';
            $httpBackend.expectPOST(nocBaseUrl + '/auth/logout').respond(204);
            jest.spyOn(nocUIMain, 'reloadSilently').mockImplementation(() => {});
            $rootScope.supportServices = {};
        }));

        it('should reload silently after successful logout', inject(($httpBackend, nocUIMain) => {
            nocUIMain.logout();
            $timeout.flush();
            $httpBackend.flush();

            expect(nocUIMain.reloadSilently).toHaveBeenCalled();
        }));

        it('should delay logout if kindly asked for', inject(($httpBackend, nocUIMain) => {
            nocUIMain.logout(500);

            //Do nothing until 500 millis passed
            $timeout.flush(300);
            expect(nocUIMain.reloadSilently).not.toHaveBeenCalled();
            $httpBackend.verifyNoOutstandingRequest();

            //Ok. now is a good time...
            $timeout.flush(200);
            $httpBackend.flush();

            expect(nocUIMain.reloadSilently).toHaveBeenCalled();
        }));
    });

    describe('formValidate', () => {
        let nocUIInstance, resolved;
        beforeEach(() => {
            nocUIInstance = _nocUIFactory.spawn('testClass', $rootScope);
            resolved = false;
        });

        it('should resolve to true if the scope has been destroyed', () => {
            nocUIInstance.kill();
            nocUIInstance.formValidate(_.constant(false))
                .then(() => {
                    resolved = true;
                });
            $timeout.flush();
            expect(resolved).toEqual(true);
        });

        describe('behavior when valid or invalid', () => {
            beforeEach(() => {
                resolved = 'neutral value';
                jest.spyOn(_nocUIMain, 'handleFormValidity').mockImplementation(() => {});
            });

            it('should return a rejected promise if the validation function returns false and call handleFormValidity', () => {
                const promise = nocUIInstance.formValidate(_.constant(false));
                promise.then(() => {
                    resolved = true;
                }, () => {
                    resolved = false;
                });
                expect(resolved).toEqual('neutral value');
                $timeout.flush();
                expect(resolved).toEqual(false);
                expect(_nocUIMain.handleFormValidity).toHaveBeenCalled();
            });

            it('should return a resolved promise if the validation function returns true and not call handleFormValidity', () => {
                const promise = nocUIInstance.formValidate(_.constant(true));
                promise.then(() => {
                    resolved = true;
                }, () => {
                    resolved = false;
                });
                expect(resolved).toEqual('neutral value');
                $timeout.flush();
                expect(resolved).toEqual(true);
                expect(_nocUIMain.handleFormValidity).not.toHaveBeenCalled();
            });
        });
    });
});
