describe("uiGridFilterService", function () {
    "use strict";

    var j;    beforeEach(() => {        j = magnifispec.setup();    });
    var uiGridFilterService, scope;

    beforeEach(inject(function (_uiGridFilterService_) {
        uiGridFilterService = _uiGridFilterService_;
        scope = j.$rootScope.$new();
        scope.gridOptions = {
            columnDefs: [
                {
                    visible: true,
                    displayName: "Source",
                    field: "source.clusterName",
                    disabled: true,
                    pinnedLeft: true,
                    default: true,
                    minWidth: 120,
                    _filterBy: ["source.clusterName", "source.volumeName", "source.workingEnvironmentStatus"],
                    cellTemplate: "<div class='ui-grid-cell-contents'><div title='{{COL_FIELD}} ({{::row.entity.source.workingEnvironmentStatus}})'>{{COL_FIELD}} <span style='font-style: italic'>({{::row.entity.source.workingEnvironmentStatus}})</span></div><div><strong title='{{::row.entity.source.volumeName}}'>{{::row.entity.source.volumeName}}</strong></div></div>"
                },
                {
                    visible: false,
                    displayName: "Source SVM Name",
                    field: "source.svmName",
                    minWidth: 120
                },
                {
                    visible: true,
                    displayName: "Destination",
                    field: "destination.clusterName",
                    disabled: false,
                    pinnedLeft: true,
                    default: true,
                    minWidth: 120,
                    _filterBy: ["destination.clusterName", "destination.volumeName", "destination.workingEnvironmentStatus"],
                    cellTemplate: "<div class='ui-grid-cell-contents'><div title='{{COL_FIELD}} ({{::row.entity.destination.workingEnvironmentStatus}})'>{{COL_FIELD}} <span style='font-style: italic'>({{::row.entity.destination.workingEnvironmentStatus}})</span></div><div><strong title='{{::row.entity.destination.volumeName}}'>{{::row.entity.destination.volumeName}}</strong></div></div>"
                },
                {
                    visible: false,
                    displayName: "Target SVM Name",
                    field: "destination.svmName",
                    minWidth: 170
                },
                {
                    visible: true,
                    displayName: "Mirror State",
                    disabled: true,
                    field: "mirrorState",
                    default: false,
                    minWidth: 170
                }
            ]
        };
    }));
    describe("contains() returns", function () {
        /*eslint-disable lodash/prefer-lodash-method*/
        var mockedRelationship = {
            "source": {
                "clusterName": "hili2912900",
                "svmName": "svm_hili2912900",
                "nodeName": null,
                "volumeName": "hilivol2912900"
            },
            "destination": {
                "clusterName": "Itay6th",
                "svmName": "svm_Itay6th",
                "nodeName": "Itay6th-01",
                "volumeName": "hilivol2912900_copy"
            },
            "mirrorState": "SnapMirrored",
            "totalTransferSize": {
                "size": 1449984,
                "unit": "Byte"
            }
        };

        it("true if search field is empty", function () {
            expect(uiGridFilterService.includes(scope, mockedRelationship, "")).toEqual(true);
            expect(uiGridFilterService.includes(scope, mockedRelationship, null)).toEqual(true);
        });

        it("true if any of the filtered fields is the same as the search term", function () {
            expect(uiGridFilterService.includes(scope, mockedRelationship, "SnapMirrored")).toEqual(true);
        });

        it("true if any of the filtered fields contains the search term", function () {
            expect(uiGridFilterService.includes(scope, mockedRelationship, "snapmirr")).toEqual(true);
        });

        it("true if any of the filtered fields contains the search term - ignore casing", function () {
            expect(uiGridFilterService.includes(scope, mockedRelationship, "SNAPMIRROR")).toEqual(true);
        });

        it("true if any of the filtered fields defined by _filterBy contains the search term", function () {
            expect(uiGridFilterService.includes(scope, mockedRelationship, "hilivol2912900_copy")).toEqual(true);
        });

        it("true with non default column definitions", function () {
            scope.gridOptions.columnDefs = [{
                visible: true,
                displayName: "Total Transfer Size",
                field: "totalTransferSize.size"
            }];
            expect(uiGridFilterService.includes(scope, mockedRelationship, "1449984")).toEqual(true);
        });

        it("false if none of the fields contain the search value", function () {
            expect(uiGridFilterService.includes(scope, mockedRelationship, "blalba")).toEqual(false);
        });

        it("false even if non visible column contains the text", function () {
            expect(uiGridFilterService.includes(scope, mockedRelationship, "svm_hili2912900")).toEqual(false);
        });

        it("false when given term that exceeds cell range", function () {
            //sub string of Itay6thsnapmirrored
            expect(uiGridFilterService.includes(scope, mockedRelationship, "6thsnapmirr")).toEqual(false);
        });

        it("false when given term with undefined substring", function () {
            expect(uiGridFilterService.includes(scope, mockedRelationship, "fine")).toEqual(false);
        });
        /*eslint-enable lodash/prefer-lodash-method*/
    });


    describe("installFilter marks rows on search when", function () {
        var rowsProcessor;
        var rows;

        beforeEach(function () {
            uiGridFilterService.installFilter({
                grid: {
                    refresh: function () {
                        rowsProcessor(rows);
                    },
                    registerRowsProcessor: function (processor) {
                        rowsProcessor = processor;
                    }
                }
            }, scope);
        });

        it("no row matchs", function () {
            rows = [{entity:{source: {clusterName:"abcd"},mirrorState:"xyz"}}];
            scope.search = {term:"nothinglikethis"};

            scope.doFiltering();

            expect(scope.visibleRowsCount).toBe(0);
            expect(rows[0].visible).toBe(false);
        });
        it("one row matchs", function () {
            rows = [{entity:{source: {clusterName:"abcd"},mirrorState:"xyz"}}];
            scope.search = {term:"bc"};

            scope.doFiltering();

            expect(scope.visibleRowsCount).toBe(1);
            expect(rows[0].visible).toBe(true);
        });
        it("one row matchs out of few", function () {
            rows = [{entity:{source: {clusterName:"abcd"},mirrorState:"xyz"}},{entity:{source: {clusterName:"1234"},mirrorState:"098"}}];
            scope.search = {term:"bc"};

            scope.doFiltering();

            expect(scope.visibleRowsCount).toBe(1);
            expect(rows[0].visible).toBe(true);
            expect(rows[1].visible).toBe(false);
        });
    });
});

