describe('Version util', () => {
    "use strict";
    var j;    beforeEach(() => {        j = magnifispec.setup();    });

    describe('cotVersionDisplay() should', () => {
        function assertDisplay(orig,display){
            it(orig + ' - as a util', inject((versionUtil) => expect(versionUtil.cotVersionDisplay(orig)).toBe(display)));
            it(orig + ' - as a filter', inject(($filter) => expect($filter('cotVersion')(orig)).toBe(display)));
        }

        describe('should remove the template part', () => {
            assertDisplay('ONTAP-8.3.1P1.T4','ONTAP-8.3.1P1');
            assertDisplay('ONTAP-8.3.1.T6','ONTAP-8.3.1');
            assertDisplay('ONTAP-8.3.1.T6.ha','ONTAP-8.3.1');
            assertDisplay('ONTAP-8.3.1.T32','ONTAP-8.3.1');
            assertDisplay('ONTAP-8.3.1.ha','ONTAP-8.3.1');
            assertDisplay('ONTAP-9.0X17.T1.azure','ONTAP-9.0X17');
        });

        describe('should not do anything', () => {
            assertDisplay('ONTAP-8.3.1P1','ONTAP-8.3.1P1');
            assertDisplay('ONTAP-8.3.1','ONTAP-8.3.1');
            assertDisplay('a', 'a');
            assertDisplay('', '');
            assertDisplay(undefined, undefined);
        });
    });

    describe('compare() should', () => {
        function assertCompare(ver1, ver2, result) {
            it(ver1 + ' and ' + ver2, inject((versionUtil) => {
                expect(versionUtil.compare(ver1, ver2)).toBe(result);
            }));
        }

        describe('work with clean versions comparing', () => {
            assertCompare('1', '1',0);
            assertCompare('1', '2',-1);
            assertCompare('2', '1',1);

            assertCompare('1.2', '1.2',0);
            assertCompare('2.1', '1.2',1);
            assertCompare('1.2', '1.1',1);
            assertCompare('1.2', '2.1',-1);
            assertCompare('1.1', '1.2',-1);

            assertCompare('1.2.3', '1.2.3',0);
            assertCompare('1.2.4', '1.2.3',1);
            assertCompare('1.3.3', '1.2.3',1);
            assertCompare('2.2.3', '1.2.3',1);
            assertCompare('1.2.3','1.2.4',-1);
            assertCompare('1.2.3','1.3.3',-1);
            assertCompare('1.2.3','2.2.3',-1);

            assertCompare('0.0.3', '0.0.3',0);
            assertCompare('0.0.4', '0.0.3',1);
            assertCompare('0.0.3', '0.0.4',-1);
        });

        describe('ignore string around the numbers comparing', () => {
            assertCompare('1.t1', '1',0);
            assertCompare('1.t1', '2',-1);
            assertCompare('2.t1', '1',1);

            assertCompare('1.2', '1.2.rc',0);
            assertCompare('2.1', '1.2.rc',1);
            assertCompare('1.2', '1.1.rc',1);
            assertCompare('1.2', '2.1.rc',-1);
            assertCompare('1.1', '1.2.rc',-1);

            assertCompare('something_1.2.3', 'xxx 1.2.3',0);
            assertCompare('something_1.2.4', 'xxx 1.2.3',1);
            assertCompare('something_1.3.3', 'xxx 1.2.3',1);
            assertCompare('something_2.2.3', 'xxx 1.2.3',1);
            assertCompare('something_1.2.3', 'xxx 1.2.4',-1);
            assertCompare('something_1.2.3', 'xxx 1.3.3',-1);
            assertCompare('something_1.2.3', 'xxx 2.2.3',-1);

            assertCompare('0.0.3x', '0.0.3y',0);
            assertCompare('0.0.4x', '0.0.3y',1);
            assertCompare('0.0.3x', '0.0.4y',-1);

            assertCompare('ONTAP-8.3.2X17.T1', 'ONTAP-8.3.2X17.T2',0);
            assertCompare('ONTAP-8.4.2X17.T1', 'ONTAP-8.3.2X17.T2',1);
            assertCompare('ONTAP-8.4.2X17.T1', 'ONTAP-8.6.2X17.T2',-1);
        });
    });
});
