describe("working environment statuses", function () {
    "use strict";
    var j;    beforeEach(() => {        j = magnifispec.setup();    });
    var workingEnvironmentStatuses;
    beforeEach(inject(function (_workingEnvironmentStatuses_) {
        workingEnvironmentStatuses = _workingEnvironmentStatuses_;
    }));

    describe('getAllowedOperations', () => {
        it('should return the correct data for combination of on and VSA_AWS', () => {
            expect(workingEnvironmentStatuses.getAllowedOperations('VSA_AWS', 'on')).toEqual({
                'access-timeline-from-gallery': false,
                'add-capacity': true,
                'aggregate-add-capacity': true,
                'aggregate-add-disks': true,
                'aggregate-delete': true,
                'aggregate-info': true,
                'change-tier': true,
                'change-instance-type': true,
                'change-license': true,
                'change-network-utilization': true,
                'change-ontap-image': true,
                'cifs-setup': true,
                'clone-volume': true,
                'configuration-backups': true,
                'configure-k8s': true,
                'cost': true,
                'delete': true,
                'delete-volume': true,
                'edit-tags': true,
                'edit-volume': true,
                'enable-capacity-tiering': true,
                'launch-system-manager': true,
                'monitor-ha': true,
                'no-actions-in-gallery': false,
                'no-quick-navigation-in-gallery': false,
                'ransomware-protection': true,
                'refresh': true,
                'register-support': true,
                'replicate': true,
                'replication-status': true,
                'resources': true,
                'resources-menu': true,
                'return-to-wizard': false,
                'set-password': true,
                'show-failure-message': false,
                'shutdown-vsa': true,
                'snapshot-volume': true,
                'sync-to-s3': true,
                'sync-to-s3-indication-in-gallery': true,
                'tiering-level': true,
                'turn-off': true,
                'turn-on': false,
                'update-credentials-from-gallery': false,
                'upgrade-mediator': true,
                'view-aggregates': true,
                'view-information': true,
                'view-instances': true,
                'view-instances-data': true,
                'view-svm': true,
                'writing-speed': true,
                'remove': false
            });
        });

        it('should return the right values for remove operation', () => {
            expect(workingEnvironmentStatuses.getAllowedOperations('VSA_AWS', 'on')['remove']).toEqual(false);
            expect(workingEnvironmentStatuses.getAllowedOperations('VSA_AWS', 'failed')['remove']).toEqual(false);
            expect(workingEnvironmentStatuses.getAllowedOperations('VSA_AWS', 'failed-to-create')['remove']).toEqual(true);
            expect(workingEnvironmentStatuses.getAllowedOperations('VSA_AWS', 'not-found')['remove']).toEqual(true);
            expect(workingEnvironmentStatuses.getAllowedOperations('ON_PREM', 'on')['remove']).toEqual(true);
            expect(workingEnvironmentStatuses.getAllowedOperations('ON_PREM', 'failed')['remove']).toEqual(true);
        });

        it('operations should be sorted alphabetically (for developers sanity only, no user value)', () => {
            let actions = workingEnvironmentStatuses.getAllowedOperations('VSA_AWS', 'on');
            delete actions['remove']; //'remove' is added later programmatic

            expect(_.keys(actions)).toEqual(_.keys(actions).sort());
        });
    });

    describe("isRelationshipOperationAllowed", function () {
        describe("break-relationship", function () {
            it("should return false if both the source and target are not in the right state", function () {
                expect(workingEnvironmentStatuses.isRelationshipOperationAllowed("break-relationship", "off", "off", "async_mirror")).toEqual(false);
            });

            it("should return false if only the source working environments is in the right state", function () {
                expect(workingEnvironmentStatuses.isRelationshipOperationAllowed("break-relationship", "on", "off", "async_mirror")).toEqual(false);
            });

            it("should return true if only the target working environments is in the right state", function () {
                expect(workingEnvironmentStatuses.isRelationshipOperationAllowed("break-relationship", "off", "on", "async_mirror")).toEqual(true);
            });

            it("should return true if both of the working environments are in the right state", function () {
                expect(workingEnvironmentStatuses.isRelationshipOperationAllowed("break-relationship", "on", "on", "async_mirror")).toEqual(true);
            });

            it("should allow break when the policy type is not one of the supported types", function () {
                expect(workingEnvironmentStatuses.isRelationshipOperationAllowed("break-relationship", "on", "on", "mirror_vault")).toEqual(true);
            });

            it("should not allow break when the policy type is not one of the supported types", function () {
                expect(workingEnvironmentStatuses.isRelationshipOperationAllowed("break-relationship", "on", "on", "vault")).toEqual(false);
            });
        });

        describe("resync-relationship", function () {
            it("should return false if both the source and target are not in the right state", function () {
                expect(workingEnvironmentStatuses.isRelationshipOperationAllowed("resync-relationship", "off", "off", "async_mirror")).toEqual(false);
            });

            it("should return false if only the source working environments is in the right state", function () {
                expect(workingEnvironmentStatuses.isRelationshipOperationAllowed("resync-relationship", "on", "off", "async_mirror")).toEqual(false);
            });

            it("should return false if only the target working environments is in the right state", function () {
                expect(workingEnvironmentStatuses.isRelationshipOperationAllowed("resync-relationship", "off", "on", "async_mirror")).toEqual(false);
            });

            it("should return true if both of the working environments are in the right state", function () {
                expect(workingEnvironmentStatuses.isRelationshipOperationAllowed("resync-relationship", "on", "on", "async_mirror")).toEqual(true);
            });

            it("should work properly even when the status is not in lower case", function () {
                expect(workingEnvironmentStatuses.isRelationshipOperationAllowed("resync-relationship", "on", "on", "async_mirror")).toEqual(true);
            });

            it("should not allow resync when the policy type is not one of the supported types", function () {
                expect(workingEnvironmentStatuses.isRelationshipOperationAllowed("resync-relationship", "on", "on", "vault")).toEqual(false);
            });

            it("should allow resync when the policy type is not one of the supported types", function () {
                expect(workingEnvironmentStatuses.isRelationshipOperationAllowed("resync-relationship", "on", "on", "mirror_vault")).toEqual(true);
            });
        });

        describe("delete-relationship", function () {
            it("should return true both are off", function () {
                expect(workingEnvironmentStatuses.isRelationshipOperationAllowed("delete-relationship", "off", "off", "async_mirror")).toEqual(false);
            });

            it("should return true target is off", function () {
                expect(workingEnvironmentStatuses.isRelationshipOperationAllowed("delete-relationship", "on", "off", "async_mirror")).toEqual(false);
            });

            it("should return true if source is off", function () {
                expect(workingEnvironmentStatuses.isRelationshipOperationAllowed("delete-relationship", "off", "on", "async_mirror")).toEqual(false);
            });

            it("should return true if both of the working environments are in the right state", function () {
                expect(workingEnvironmentStatuses.isRelationshipOperationAllowed("resync-relationship", "on", "on", "async_mirror")).toEqual(true);
            });
        });



        describe("isReplicationAllowed", function () {
            var sourceWe, destinationWe;
            beforeEach(function () {
                sourceWe = {name: "sourceWe", publicId: "sourceWe", status: {status: "on"}, workingEnvironmentType:"VSA", _typeAndProvider: "VSA_AWS"};
                destinationWe = {name: "destinationWe", publicId: "destinationWe", status: {status: "on"}, workingEnvironmentType:"VSA", _typeAndProvider: "VSA_AWS", ontapClusterProperties: {ontapVersion: "8.3"}};
            });

            it("should allow replication from VSA to VSA", function () {
                expect(workingEnvironmentStatuses.isReplicationAllowed(sourceWe, destinationWe)).toEqual(true);
            });

            it("should allow replication from VSA to ON_PREM", function () {
                destinationWe.workingEnvironmentType = "ON_PREM";
                expect(workingEnvironmentStatuses.isReplicationAllowed(sourceWe, destinationWe)).toEqual(true);
            });

            it("should allow replication from ON_PREM to VSA", function () {
                sourceWe.workingEnvironmentType = "ON_PREM";
                expect(workingEnvironmentStatuses.isReplicationAllowed(sourceWe, destinationWe)).toEqual(true);
            });

            it("should allow replication from ON_PREM to ON_PREM", function () {
                sourceWe.workingEnvironmentType = "ON_PREM";
                destinationWe.workingEnvironmentType = "ON_PREM";
                expect(workingEnvironmentStatuses.isReplicationAllowed(sourceWe, destinationWe)).toEqual(true);
            });

            it("should not allow replication when destination is not 'on'", function () {
                destinationWe.status = "off";
                expect(workingEnvironmentStatuses.isReplicationAllowed(sourceWe, destinationWe)).toEqual(false);
                destinationWe.status = "Starting";
                expect(workingEnvironmentStatuses.isReplicationAllowed(sourceWe, destinationWe)).toEqual(false);
                destinationWe.status = "not-found";
                expect(workingEnvironmentStatuses.isReplicationAllowed(sourceWe, destinationWe)).toEqual(false);
            });


            it("should not allow replication when sourceWe is not 'on'", function () {
                sourceWe.status = "off";
                expect(workingEnvironmentStatuses.isReplicationAllowed(sourceWe, destinationWe)).toEqual(false);
                sourceWe.status = "stopping";
                expect(workingEnvironmentStatuses.isReplicationAllowed(sourceWe, destinationWe)).toEqual(false);
                sourceWe.status = "failed";
                expect(workingEnvironmentStatuses.isReplicationAllowed(sourceWe, destinationWe)).toEqual(false);
            });

            it("should not allow replication if the target ontap version is lower than 8.3", function () {
                destinationWe.ontapClusterProperties.ontapVersion = "8.2.TC1";
                expect(workingEnvironmentStatuses.isReplicationAllowed(sourceWe, destinationWe)).toEqual(false);
            });
        });
    });
});

