describe('nocAdvancedAllocationService', () => {
    'use strict';
    var j;
    beforeEach(() => {
        j = magnifispec.setup();
    });
    let nocAdvancedAllocationService, aggregatesResource;
    let targetWorkingEnvironment, scope;
    beforeEach(inject((_nocAdvancedAllocationService_, _aggregatesResource_) => {
        nocAdvancedAllocationService = _nocAdvancedAllocationService_;
        aggregatesResource = _aggregatesResource_;
        targetWorkingEnvironment = {publicId: 1, name: 'we1', _pathPrefix: 'vsa/'};
        scope = {};
    }));

    describe('getAggregates for Azure', function () {
        beforeEach(function () {
            targetWorkingEnvironment._pathPrefix = '/azure/vsa/';
            targetWorkingEnvironment._typeAndProvider = 'VSA_Azure';
        });
        it('should call the right url for azure', function () {
            j.$httpBackend.expectGET(j.nocBaseUrl + '/azure/vsa/aggregates/1').respond(200, []);
            nocAdvancedAllocationService.getAggregates(j.nocHttp, j.pidgeon, targetWorkingEnvironment, scope);
            j.$httpBackend.flush();
        });
    });

    describe('getAggregates', () => {
        const responseAggr = [
            {name: 'c', disks: [], volumes: []},
            {name: 'a', disks: [], volumes: []},
            {name: 'b', disks: [], volumes: []}
        ];

        it('should bring the aggregates upon success, initialize and sort them', () => {
            jest.spyOn(aggregatesResource, 'getAggregates').mockReturnValue(j.$q.when(responseAggr));
            jest.spyOn(aggregatesResource, 'initializeAggregate').mockImplementation(() => {
            });
            expect(nocAdvancedAllocationService.getAggregates(j.nocHttp, j.pidgeon, targetWorkingEnvironment))
                .toBeResolvedWith([
                    {name: 'a', disks: [], volumes: []},
                    {name: 'b', disks: [], volumes: []},
                    {name: 'c', disks: [], volumes: []}
                ]);
            expect(aggregatesResource.initializeAggregate.mock.calls.length).toEqual(3);
        });

        it('should display pidgeon when no aggregates available', () => {
            jest.spyOn(aggregatesResource, 'getAggregates').mockReturnValue(j.$q.when([]));
            expect(nocAdvancedAllocationService.getAggregates(j.nocHttp, j.pidgeon, targetWorkingEnvironment, scope))
                .toBeRejected();
            expect(j.pidgeon.error).toHaveBeenCalledWith('No aggregates other than root aggregate were found in working environment we1.', null, 'aggregates');
        });

        it('should work upon error', function () {
            j.$httpBackend.expectGET(j.nocBaseUrl + '/vsa/aggregates?workingEnvironmentId=1').respond(400, {message: 'generic failure message'});
            nocAdvancedAllocationService.getAggregates(j.nocHttp, j.pidgeon, targetWorkingEnvironment, scope);
            j.$httpBackend.flush();
            expect(j.pidgeon.error).toHaveBeenCalled();
            expect(scope.aggregates).not.toEqual(responseAggr);
        });

        it('should call the callback function upon success if supplied', function () {
            var callback = jest.fn();
            j.$httpBackend.expectGET(j.nocBaseUrl + '/vsa/aggregates?workingEnvironmentId=1').respond(200, responseAggr);
            nocAdvancedAllocationService.getAggregates(j.nocHttp, j.pidgeon, targetWorkingEnvironment, scope)
                .then(callback);
            j.$httpBackend.flush();
            expect(callback).toHaveBeenCalled();
        });
    });

    describe('getSVMs', function () {
        var responseSvms = {
            svms: [
                {someValue: 'someData2'}
            ]
        };
        it('should work upon success', function () {
            j.$httpBackend.expectGET(j.nocBaseUrl + '/vsa/working-environments/1?fields=svms').respond(200, responseSvms);
            nocAdvancedAllocationService.getSVMs(j.nocHttp, targetWorkingEnvironment, scope);
            j.$httpBackend.flush();
            expect(scope.destinationSvms).toEqual(responseSvms.svms);
        });

        it('should work upon error', function () {
            j.$httpBackend.resetExpectations();
            j.$httpBackend.expectGET(j.nocBaseUrl + '/vsa/working-environments/1?fields=svms').respond(400, {message: 'generic failure message'});
            nocAdvancedAllocationService.getSVMs(j.nocHttp, targetWorkingEnvironment, scope);
            j.$httpBackend.flush();
            expect(j.pidgeon.error).toHaveBeenCalled();
            expect(scope.destinationSvms).not.toEqual(responseSvms);
        });

        it('should call the callback function upon success if supplied', function () {
            var callback = jest.fn();
            j.$httpBackend.expectGET(j.nocBaseUrl + '/vsa/working-environments/1?fields=svms').respond(200, responseSvms);
            nocAdvancedAllocationService.getSVMs(j.nocHttp, targetWorkingEnvironment, scope)
                .then(callback);
            j.$httpBackend.flush();
            expect(callback).toHaveBeenCalled();
        });

        afterEach(function () {
            expect(j.nocUI.unloading).toHaveBeenCalled();
        });
    });
});
