describe('Settings Controller', () => {
    'use strict';

    var j;    beforeEach(() => {        j = magnifispec.setup();    });
    let configValuesUrl, originalConfig;
    beforeEach(() => {
        configValuesUrl = j.nocBaseUrl + '/occm/config';
        jest.spyOn(j.nocNavigateAway, 'promptOnLeaveBeforeDiscardingChanges').mockImplementation(() => {});
        magnifispec.mockStandaloneController('SettingsController', {});
        originalConfig = {
            maxAggregateFreeSpacePercentage: 50,
            s3EbsRatio: 10,
            maxVolumeGrowSizePercentage: 1000,
            proxyUri: 'http://www.proxy.com:8080',
            autoVsaCapacityManagement: true,
            proxyUserName: '',
            proxyPassword: '',
            proxyDomain: '',
            autoUpgrade: false,
            autoOntapUpgrade: false
        };
        j.$httpBackend.expectGET(configValuesUrl).respond(originalConfig);
    });

    describe('Initialization', () => {
        it('should get the values and populates the scope', () => {
            expect(j.nocUI.loading).toHaveBeenCalled();
            j.$httpBackend.flush();
            expect(j.scope.values.maxAggregateFreeSpacePercentage).toEqual(50);
            expect(j.scope.values.s3EbsRatio).toEqual(10);
            expect(j.scope.values.maxVolumeGrowSizePercentage).toEqual(1000);
            expect(j.scope.values.autoVsaCapacityManagement).toEqual(true);
            expect(j.scope.values.proxyUri).toEqual('http://www.proxy.com:8080');
            expect(j.scope.values).toEqual(j.scope.originalValues);
            expect(j.nocUI.unloading).toHaveBeenCalled();
        });

        it('should fail gracefully when error occurs while fetching data', () => {
            expect(j.nocUI.loading).toHaveBeenCalled();
            j.$httpBackend.resetExpectations();
            j.$httpBackend.expectGET(configValuesUrl).respond(400, j.errorResponse);
            j.$httpBackend.flush();

            expect(j.pidgeon.error).toHaveBeenCalled();

            expect(j.nocUI.unloading).toHaveBeenCalled();
        });
    });

    describe('saving changes', () => {
        beforeEach(() => {
            j.scope.values = _.cloneDeep(originalConfig);
            j.scope.originalValues = _.cloneDeep(originalConfig);
            j.scope.settingsForm = {
                $valid: true,
                $setPristine: _.noop
            };
        });

        it('should send 5 keys and values', () => {
            j.scope.values.maxAggregateFreeSpacePercentage = 20;
            j.scope.values.s3EbsRatio = 20;
            j.scope.values.maxVolumeGrowSizePercentage = 980;
            j.scope.values.autoVsaCapacityManagementViewValue = 'manual';
            j.scope.values.proxyUri = '   http://secondProxy.com:90   '; //Should trim the proxy.

            j.$httpBackend.expectPUT(configValuesUrl, {
                maxAggregateFreeSpacePercentage: 20,
                s3EbsRatio: 20,
                maxVolumeGrowSizePercentage: 980,
                proxyUri: 'http://secondProxy.com:90',
                autoVsaCapacityManagement: false}).respond(200);
            j.scope.saveChanges();
            j.$httpBackend.flush();
            expect(j.pidgeon.success).toHaveBeenCalledWith('Setting changes saved.');
            expect(j.nocUI.block).toHaveBeenCalled();
            expect(j.nocUI.unblock).toHaveBeenCalled();
        });

        it('should send request only for a changed value', () => {
            j.$httpBackend.flush();
            j.scope.values.maxAggregateFreeSpacePercentage = 30;
            j.scope.values.proxyUri = null;
            j.scope.originalValues.proxyUri = null;
            j.scope.values.autoVsaCapacityManagementViewValue = 'auto';
            j.$httpBackend.expectPUT(configValuesUrl, {maxAggregateFreeSpacePercentage: 30}).respond(200);
            j.scope.saveChanges();
            j.$httpBackend.flush();
            expect(j.pidgeon.success).toHaveBeenCalledWith('Setting changes saved.');
            expect(j.nocUI.block).toHaveBeenCalled();
            expect(j.nocUI.unblock).toHaveBeenCalled();
        });

        it('should not send requests if a value did not change', () => {
            j.scope.values.autoVsaCapacityManagementViewValue = 'auto';
            j.$httpBackend.flush();
            j.scope.saveChanges();
            j.$httpBackend.verifyNoOutstandingExpectation();
        });

        it('should fail gracefully', () => {
            j.$httpBackend.flush();
            j.scope.values.maxAggregateFreeSpacePercentage = 20;
            j.scope.values.autoVsaCapacityManagementViewValue = 'auto';
            j.$httpBackend.expectPUT(configValuesUrl, {maxAggregateFreeSpacePercentage: 20}).respond(400, j.errorResponse);
            j.scope.saveChanges();
            j.$httpBackend.flush();
            expect(j.pidgeon.error).toHaveBeenCalled();
            expect(j.nocUI.block).toHaveBeenCalled();
            expect(j.nocUI.unblock).toHaveBeenCalled();
        });

        it('should validate the form', () => {
            j.scope.settingsForm = {$valid: false};
            j.scope.saveChanges();
            expect(j.nocUI.handleFormValidity).toHaveBeenCalledWith(j.scope.settingsForm, j.scope);
        });
    });

    describe('isDirty', () => {
        it('should detect dirty like a boss', () => {
            j.scope.values = {bla: 1, blu: 2};
            j.scope.originalValues = {bla: 2, blu: 3};
            expect(j.scope.isDirty()).toEqual(true);
        });
    });

    describe('discard changes dialog', () => {
        it('should register to discard changes service', () => {
            expect(j.nocNavigateAway.promptOnLeaveBeforeDiscardingChanges).toHaveBeenCalledWith(j.scope);
        });
    });
});
