describe.skip('Tabs Controller', () => {
    var j;
    beforeEach(() => {
        j = magnifispec.setup();
    });
    let mockedTenants = [{publicId: 1, name: "Tenant 1"}, {publicId: "angry", name: "angry"}];

    beforeEach(() => {
        j.$rootScope.checkAndSetState = jest.fn();
        j.$rootScope.currentUser = {
            roleId: "Role-2",
            tenantId: 1
        };
    });

    mockedTenants = [{publicId: 1, name: "Tenant 1"}, {publicId: "angry", name: "angry"}];
    describe("Constructed with no tabs", () => {
        let mockedLayoutOptions = {
            basePath: "main",
            tabs: []
        };

        it("should throw an exception", () =>
            expect(() =>
                magnifispec.mockController("TabsController", {
                    layoutOptions: mockedLayoutOptions,
                    tenants: mockedTenants
                })
                    .toThrowError("Unable to initialize tabs controller with no tabs!")
            )
        );
    });

    describe("Constructed with tabs", () => {
        let mockedLayoutOptions = {
            basePath: "main",
            tabs: [{tabName: "tab1"}, {tabName: "tab2"}]
        };

        beforeEach(() => {
            let scope = j.$rootScope.$new();
            jest.spyOn(scope, "$on").mockImplementation(() => {});
            magnifispec.mockController("TabsController", {
                layoutOptions: mockedLayoutOptions,
                $scope: scope,
                tenants: mockedTenants
            });
        });

        it("should watch on stateChangeStart with handleStateChange", () =>
            expect(j.scope.$on)
                .toHaveBeenCalledWith("$stateChangeStart", j.scope.handleStateChange)
        );

        describe("Handle state change", () => {
            it("should immediately if the first portion of the toState is not the base path from layout options", () => {
                jest.spyOn(j.scope, "setCurrentTab").mockImplementation(() => {});

                j.scope.handleStateChange(null, {name: "kuki"});

                expect(j.scope.setCurrentTab)
                    .not.toHaveBeenCalled();
            });

            it("should attempt to set current tab if the base path matches", () => {
                jest.spyOn(j.scope, "setCurrentTab").mockReturnValue(true);
                j.scope.handleStateChange(null, {name: "main.tab1"});

                expect(j.scope.setCurrentTab)
                    .toHaveBeenCalled();
            });

            it("should prevent default on the passed in event if the name of the state doesnt match a tab", inject(function ($state) {
                jest.spyOn($state, "go").mockImplementation(() => {});
                jest.spyOn($state, "get").mockReturnValue({abstract: false});
                jest.spyOn(j.scope, "setCurrentTab").mockReturnValue(false);

                let event = {preventDefault: jest.fn()};

                j.scope.handleStateChange(event, {name: "main.tab1"});

                expect(event.preventDefault)
                    .toHaveBeenCalled();
            }));

            it("should call $state go with the first tab in the tabs list if setCurrentTab fails", inject(function ($state) {

                jest.spyOn($state, "get").mockReturnValue({abstract: false});
                jest.spyOn(j.scope, "setCurrentTab").mockReturnValue(false);

                j.scope.handleStateChange(null, {name: "main.tab1"});

                expect(j.$rootScope.checkAndSetState)
                    .toHaveBeenCalledWith("main.tab1");
            }));
        });

        describe("Set current tab", () => {
            it("should set the current tab if found and true", () => {
                let returnVal = j.scope.setCurrentTab("tab2");

                expect(returnVal)
                    .toEqual(true);

                expect(j.scope.currentTab)
                    .toEqual(mockedLayoutOptions.tabs[1]);
            });

            it("should not set the current tab if not found, and false", () => {
                let returnVal = j.scope.setCurrentTab("nothere");

                expect(returnVal)
                    .toEqual(false);

                expect(j.scope.currentTab).toBeUndefined();
            });
        });
    });
});
