describe.skip('nocAuditText directive', () => {
    var j;
    beforeEach(() => {
        j = magnifispec.setup();
    });
    let generateAuditText = (scopeAttributes, selector) => magnifispec.digest('<noc-audit-text data="summary"/>', scopeAttributes, selector);

    it('should format audit group with parameters, marking them with audit-text-value class', () => {
        let value = generateAuditText({summary: {actionName: 'Login', userName: 'Manfred'}}, '.audit-text-value');
        expect(value.text()).toBe('Manfred');
    });

    it('should return a default text when no template found', () => {
        let missing = generateAuditText({summary: {actionName: 'anActionWithNoTemplate'}}, '.audit-text-missing-template');

        expect(missing.text()).toMatch('anActionWithNoTemplate');
        expect(missing.attr('title')).toMatch(/.+template.*/i); //say something with template....
    });

    it('should return a default text when no template found (no action name)', () => {
        let missing = generateAuditText({summary: {}}, '.audit-text-missing-template');

        expect(missing.text()).toMatch('Missing action name');
        expect(missing.attr('title')).toMatch(/.*action name.*/i); //say something with action name....
    });

    it('should return a default text when no template found (no summary)', () => {
        let missing = generateAuditText({}, '.audit-text-missing-template');

        expect(missing.text()).toMatch('Missing action name');
        expect(missing.attr('title')).toMatch(/.*action name.*/i); //say something with action name....
    });

    describe('All audit templates', () =>
        it('should have one span element with the action name bolded (.audit-text-action-name css class)', inject((auditTemplateFinder) => {
            _.forEach(auditTemplateFinder.templates, (template) => {
                //For some reason jasmine-jquery fails to parse one of the template that contains angular expression with {{}} and | so we're removing it since we don't care about it
                template = _.replace(template, /[{}|]/g, '');

                const actionNameSpan = $(`<div>${template}</div>`).find('span.audit-text-action-name');

                //Add actionName to the expected and real values to see them in the test results
                expect(actionNameSpan.length).toBeGreaterThan(0);
                expect(actionNameSpan.text()).not.toBeEmpty();
            });
        }))
    );
});
