describe('ChangeLicenseOrInstanceController', () => {
    var j;    beforeEach(() => {        j = magnifispec.setup();    });

    const licenses = [
        {
            "licenseType": "cot-explore-paygo",
            "name": "Cloud Volumes ONTAP Explore",
            "description": "Suitable for smaller capacity applications (Up to 2TB of underlying AWS storage)",
            "capacityLimit": "2.0 TB",
            "instanceTypes": [
                {
                    "instanceType": "m3.xlarge",
                    "supportsEncryption": false,
                    "supportedFeatures": ["four-nics"]
                }
            ]
        },
        {
            "licenseType": "cot-standard-paygo",
            "name": "Cloud Volumes ONTAP Standard",
            "description": "Flexible performance and larger capacity for a wider range of applications (Up to 10TB of underlying AWS storage)",
            "capacityLimit": "10.0 TB",
            "instanceTypes": [
                {
                    "instanceType": "m3.2xlarge",
                    "supportsEncryption": false,
                    "supportedFeatures": []
                },
                {
                    "instanceType": "r3.xlarge",
                    "supportsEncryption": true,
                    "supportedFeatures": ["network-utilization"]
                }
            ]
        },
        {
            "licenseType": "cot-premium-paygo",
            "name": "Cloud Volumes ONTAP Premium",
            "description": "Flexible performance and expanded capacity for a wider range of applications. Supports up to 368 TB",
            "capacityLimit": "368.0 TB",
            "instanceTypes": [
                {
                    "instanceType": "r3.2xlarge",
                    "supportsEncryption": true
                }
            ]
        }
    ];

    beforeEach(inject((MetadataResource) => {
        jest.spyOn(j.workingEnvironmentResource, 'getLicenseAndInstanceTypes').mockReturnValue(j.$q.when(licenses));
        jest.spyOn(MetadataResource, 'legacyInstanceTypes').mockReturnValue(j.$q.when([]));

        magnifispec.mockTabsController('ChangeLicenseOrInstanceController', {}, {
            selectedWorkingEnvironment: {
                _pathPrefix: '/vsa/',
                publicId: '1',
                workingEnvironmentType: 'VSA',
                status: {status: 'ON'},
                providerProperties: {numOfNics: 6},
                _nameForInstance: 'instance'
            },
            vsaResponse: {
                instanceType: 'm3.2xlarge',
                cluster: {licenseType: {name: 'Cloud Volumes ONTAP Standard'}},
                vsaEncryptionResponse: {ontapEncryption: false}
            },
            showMonitoring: jest.fn(),
            changeLicenseSelectObjects: {}
        });

        magnifispec.processPromise();
    }));

    describe('initialize', () => {
        beforeEach(function() {
            jest.spyOn(j.scope, 'getLegacyTypes').mockReturnValue(j.$q.when([]));
            magnifispec.processPromise();
        });

        it("should update the local VSA response to the vsa response when exists", function () {
            expect(j.scope.localVsaResponse).toEqual({
                instanceType: 'm3.2xlarge',
                cluster: {licenseType: {name: "Cloud Volumes ONTAP Standard"}},
                vsaEncryptionResponse: {ontapEncryption: false}
            });
        });

        it("should update the local VSA response to the off response when needed", function () {
            j.scope.vsaResponse = null;
            j.scope.offVsaResponse = {instanceType: 'r3.xlarge', licenseType: "Cloud Volumes ONTAP Standard"};
            j.scope.initialize();
            expect(j.scope.localVsaResponse).toEqual({instanceType: 'r3.xlarge', licenseType: "Cloud Volumes ONTAP Standard"});
        });

        it("should go back to monitoring if there is no VSA response yet", function () {
            j.scope.vsaResponse = null;
            j.scope.offVsaResponse = null;
            j.scope.initialize();
            expect(j.scope.showMonitoring).toHaveBeenCalled();
        });

        describe('Lifs handling in Azure', () => {
            beforeEach(() => {
                j.scope.selectedWorkingEnvironment._isAzure = true;
                jest.spyOn(j.scope, 'checkOptionToAddLifs').mockImplementation(() => {});
                jest.spyOn(j.scope, 'disableInstancesAndLicensesWithLessThan8Nics').mockImplementation(() => {});
            });

            describe('case configuration is not singleNetworkInterface', () => {
                beforeEach(() => {
                    j.scope.selectedWorkingEnvironment.providerProperties.singleNetworkInterface = false;
                });

                it('should disable instances and licenses with less than eight nics if more than 4 LIFs exist for VSAs in Azure', () => {
                    j.scope.initialize();
                    magnifispec.processPromise();

                    expect(j.scope.disableInstancesAndLicensesWithLessThan8Nics).toHaveBeenCalled();
                });

                it('should NOT disable instances and licenses with less than eight nics if only 4 LIFs exist for VSAs in Azure', () => {
                    j.scope.selectedWorkingEnvironment.providerProperties.numOfNics = 4;
                    jest.spyOn(j.workingEnvironmentResource, 'getNumberOfAvailableIpsInSubnet').mockReturnValue(j.$q.when({availableIps: 5}));

                    j.scope.initialize();
                    magnifispec.processPromise();

                    expect(j.scope.disableInstancesAndLicensesWithLessThan8Nics).not.toHaveBeenCalled();
                    expect(j.workingEnvironmentResource.getNumberOfAvailableIpsInSubnet).toHaveBeenCalled();
                });

                it('should check the option to add lifs if VSA is off and only 4 LIFs exist for VSAs in Azure', () => {
                    jest.spyOn(j.workingEnvironmentResource, 'getNumberOfAvailableIpsInSubnet').mockReturnValue(j.$q.when({availableIps: 5}));
                    j.scope.selectedWorkingEnvironment.providerProperties.numOfNics = 4;
                    j.scope.selectedWorkingEnvironment.status.status = 'off';

                    j.scope.initialize();
                    magnifispec.processPromise();

                    expect(j.scope.checkOptionToAddLifs).toHaveBeenCalled();
                });

                it('should check the option to add lifs if there are no available IPs and only 4 LIFs exist for VSAs in Azure', () => {
                    jest.spyOn(j.workingEnvironmentResource, 'getNumberOfAvailableIpsInSubnet').mockReturnValue(j.$q.when({availableIps: 0}));
                    j.scope.selectedWorkingEnvironment.providerProperties.numOfNics = 4;

                    j.scope.initialize();
                    magnifispec.processPromise();

                    expect(j.scope.checkOptionToAddLifs).toHaveBeenCalled();
                });
            });

            describe('case configuration is singleNetworkInterface', () => {
                beforeEach(() => {
                    j.scope.selectedWorkingEnvironment.providerProperties.singleNetworkInterface = true;
                });

                it('should not disable instances and licenses with less than eight nics if more than 4 LIFs exist for VSAs in Azure', () => {
                    j.scope.initialize();
                    magnifispec.processPromise();

                    expect(j.scope.disableInstancesAndLicensesWithLessThan8Nics).not.toHaveBeenCalled();
                });

                it('should NOT disable instances and licenses with less than eight nics if only 4 LIFs exist for VSAs in Azure', () => {
                    j.scope.selectedWorkingEnvironment.providerProperties.numOfNics = 4;
                    jest.spyOn(j.workingEnvironmentResource, 'getNumberOfAvailableIpsInSubnet').mockReturnValue(j.$q.when({availableIps: 5}));

                    j.scope.initialize();
                    magnifispec.processPromise();

                    expect(j.scope.disableInstancesAndLicensesWithLessThan8Nics).not.toHaveBeenCalled();
                    expect(j.workingEnvironmentResource.getNumberOfAvailableIpsInSubnet).not.toHaveBeenCalled();
                });

                it('should not check the option to add lifs if VSA is off and only 4 LIFs exist for VSAs in Azure', () => {
                    jest.spyOn(j.workingEnvironmentResource, 'getNumberOfAvailableIpsInSubnet').mockReturnValue(j.$q.when({availableIps: 5}));
                    j.scope.selectedWorkingEnvironment.providerProperties.numOfNics = 4;
                    j.scope.selectedWorkingEnvironment.status.status = 'off';

                    j.scope.initialize();
                    magnifispec.processPromise();

                    expect(j.scope.checkOptionToAddLifs).not.toHaveBeenCalled();
                });

                it('should not check the option to add lifs if there are no available IPs and only 4 LIFs exist for VSAs in Azure', () => {
                    jest.spyOn(j.workingEnvironmentResource, 'getNumberOfAvailableIpsInSubnet').mockReturnValue(j.$q.when({availableIps: 0}));
                    j.scope.selectedWorkingEnvironment.providerProperties.numOfNics = 4;

                    j.scope.initialize();
                    magnifispec.processPromise();

                    expect(j.scope.checkOptionToAddLifs).not.toHaveBeenCalled();
                });
            });
        });

        describe("http handling", function () {
            it("should get the license and instance types and update the licenses and select the right license and instance upon success", function () {
                expect(j.scope.changeLicenseSelectObjects).toEqual({
                    selectedLicense : j.scope.licenses[1],
                    selectedInstanceType: {
                        instanceType: 'r3.xlarge',
                        supportsEncryption: true,
                        supportedFeatures: ["network-utilization"]
                    }
                });
                expect(j.scope.changeLicenseSelectObjects.selectedLicense.instanceTypes).toEqual([{
                    instanceType: "r3.xlarge",
                    supportsEncryption: true,
                    supportedFeatures: ["network-utilization"]
                }]);
            });
        });
    });

    describe('disableInstancesAndLicensesWithLessThan8Nics', () => {
        it('should add a disabled flag to instances that don\'t support 8 nics', () => {
            j.scope.disableInstancesAndLicensesWithLessThan8Nics();
            expect(j.scope.licenses[0].instanceTypes[0].disabled).toEqual(true);
            expect(j.scope.licenses[1].instanceTypes[0].disabled).toEqual(false);
            expect(j.scope.licenses[0]._disabled).toEqual(true);
            expect(j.scope.licenses[1]._disabled).toEqual(false);
        });
    });

    describe('change license and instance type', () => {
        beforeEach(function () {
            jest.spyOn(j.scope, 'getLegacyTypes').mockReturnValue(j.$q.when([]));
            magnifispec.processPromise();
            j.scope.changeLicenseSelectObjects.selectedLicense = j.scope.licenses[1];
            j.scope.changeLicenseSelectObjects.selectedInstanceType = j.scope.changeLicenseSelectObjects.selectedLicense.instanceTypes[0];
        });

        it("should send the right request and update the working environment state to updating upon success", function () {
            jest.spyOn(j.workingEnvironmentResource, 'updateWorkingEnvironmentStatus').mockImplementation(() => {});

            j.scope.originalLicenseName = "something else";
            j.scope.originalInstanceType = "something else";

            j.$httpBackend.expectPUT(j.nocBaseUrl + '/vsa/working-environments/1/license-instance-type', {
                licenseType: j.scope.changeLicenseSelectObjects.selectedLicense.licenseType,
                instanceType: j.scope.changeLicenseSelectObjects.selectedInstanceType.instanceType
            }).respond(200,{});
            j.scope.onConfirm();
            j.$httpBackend.flush();
            expect(j.workingEnvironmentResource.updateWorkingEnvironmentStatus).toHaveBeenCalledWith(j.scope.selectedWorkingEnvironment, "updating");
            expect(j.scope.showMonitoring).toHaveBeenCalled();
        });

        it("should send the right request and handle error state correctly", function () {
            j.scope.originalLicenseName = "something else";
            j.scope.originalInstanceType = "something else";

            j.$httpBackend.expectPUT(j.nocBaseUrl + '/vsa/working-environments/1/license-instance-type', {
                licenseType: j.scope.changeLicenseSelectObjects.selectedLicense.licenseType,
                instanceType: j.scope.changeLicenseSelectObjects.selectedInstanceType.instanceType}).respond(400, j.errorResponse);
            j.scope.onConfirm();
            j.$httpBackend.flush();
            expect(j.pidgeon.error).toHaveBeenCalled();
        });
    });

    describe('disableDueToOntapEncryption', () => {
        it('should disable non supporting instances, disable licenses with no supporting instances and add appropriate disable messages', () => {
            j.scope.localVsaResponse.vsaEncryptionResponse.ontapEncryption = true;
            j.scope.licenses = [{
                licenseType: 'cot-explore-paygo',
                instanceTypes: [
                    {
                        instanceType: 'm3.xlarge',
                        supportsEncryption: false
                    }
                ]
            }, {
                licenseType: 'cot-standard-paygo',
                instanceTypes: [
                    {
                        instanceType: 'm3.2xlarge',
                        supportsEncryption: false
                    },
                    {
                        instanceType: 'r3.xlarge',
                        supportsEncryption: true
                    }
                ]
            }, {
                licenseType: 'cot-premium-paygo',
                instanceTypes: [
                    {
                        instanceType: 'r3.2xlarge',
                        supportsEncryption: true
                    }
                ]
            }];

            j.scope.disableDueToOntapEncryption();

            expect(j.scope.licenses).toEqual([{
                licenseType: 'cot-explore-paygo',
                instanceTypes: [{
                    instanceType: 'm3.xlarge',
                    supportsEncryption: false,
                    disabled: true,
                    disabledText: 'This instance type does not support ONTAP Encryption'
                }],
                _disabled: true,
                _disableReason: 'This license does not support ONTAP Encryption'
            }, {
                licenseType: 'cot-standard-paygo',
                instanceTypes: [{
                    instanceType: 'm3.2xlarge',
                    supportsEncryption: false,
                    disabled: true,
                    disabledText: 'This instance type does not support ONTAP Encryption'
                }, {
                    instanceType: 'r3.xlarge',
                    supportsEncryption: true,
                    disabled: false
                }],
                _disabled: false
            }, {
                licenseType: 'cot-premium-paygo',
                instanceTypes: [{
                    instanceType: 'r3.2xlarge',
                    supportsEncryption: true,
                    disabled: false
                }],
                _disabled: false
            }]);
        });
    });

    describe('selectDefaultInstanceType', () => {
        let getDefaultInstanceSpy;

        beforeEach(inject((ManifestMapper) => {
            getDefaultInstanceSpy = jest.spyOn(ManifestMapper, 'getDefaultInstance');
        }));

        it('should return manifest default if it is not disabled', () => {
            getDefaultInstanceSpy.mockReturnValue('r3.xlarge');
            j.scope.changeLicenseSelectObjects.selectedLicense = {
                'licenseType': 'cot-standard-paygo',
                'instanceTypes': [
                    {'instanceType': 'm3.2xlarge'},
                    {'instanceType': 'r3.xlarge'}
                ]
            };

            j.scope.selectDefaultInstanceType();
            expect(j.scope.changeLicenseSelectObjects.selectedInstanceType).toEqual({'instanceType': 'r3.xlarge'});
        });

        it('should not return manifest default if it is disabled', () => {
            getDefaultInstanceSpy.mockReturnValue('r3.xlarge');
            j.scope.changeLicenseSelectObjects.selectedLicense = {
                'licenseType': 'cot-standard-paygo',
                'instanceTypes': [
                    {'instanceType': 'm3.2xlarge'},
                    {'instanceType': 'r3.xlarge', disabled: true}
                ]
            };

            j.scope.selectDefaultInstanceType();
            expect(j.scope.changeLicenseSelectObjects.selectedInstanceType).toEqual({'instanceType': 'm3.2xlarge'});
        });

        it('when no manifest default it should return the first instance which is not disabled and not legacy', () => {
            jest.spyOn(j.scope, 'isInstanceLegacy').mockImplementation(function (instanceType) {
                return instanceType === 'm2.large' || instanceType === 'm3.large';
            });

            j.scope.changeLicenseSelectObjects.selectedLicense = {
                'licenseType': 'cot-standard-paygo',
                'instanceTypes': [
                    {'instanceType': 'm1.large', disabled: true}, //Disabled
                    {'instanceType': 'm2.large'},                 //Legacy
                    {'instanceType': 'm3.large', disabled: true}, //Disabled & Legacy
                    {'instanceType': 'm4.large'}                  //OK
                ]
            };

            j.scope.selectDefaultInstanceType();
            expect(j.scope.changeLicenseSelectObjects.selectedInstanceType).toEqual({'instanceType': 'm4.large'});
        });

        it('should return the first non disabled instance if all instances are legacy', () => {
            jest.spyOn(j.scope, 'isInstanceLegacy').mockReturnValue(true);

            j.scope.changeLicenseSelectObjects.selectedLicense = {
                'licenseType': 'cot-standard-paygo',
                'instanceTypes': [
                    {'instanceType': 'm1.large', disabled: true},
                    {'instanceType': 'm2.large'}
                ]
            };

            j.scope.selectDefaultInstanceType();
            expect(j.scope.changeLicenseSelectObjects.selectedInstanceType).toEqual({'instanceType': 'm2.large'});
        });
    });
});
