describe.skip('Encryption setup controller spec', () => {
    let m;
    beforeEach(() => {
        m = magnifispec.setup();
    });
    const j = vsaResponse = {
            vsaEncryptionResponse: {
                ontapEncryptionCertificates: [
                    {name: 'a', certificateType: 'client'},
                    {name: 'b', certificateType: 'server_ca'},
                    {name: 'c', certificateType: 'server_ca'}
                ],
                keyManagers: [
                    {address: '1.1.1.1', status: 'available'},
                    {address: '2.2.2.2', status: 'not_responding'}
                ]
            },
            id: 'encryptedVsaId1'
        };
    let nocEncryptionResourcesService, nocEncryptionResourcesServiceSpy;

    beforeEach(inject((_nocEncryptionResourcesService_) => {
        nocEncryptionResourcesService = _nocEncryptionResourcesService_;
        nocEncryptionResourcesServiceSpy = jest.spyOn(nocEncryptionResourcesService, 'getOccmCertificate').mockImplementation(
            magnifispec.mockPromise({_pathPrefix: '/vsa/', publicId: 'initialized_value', endDate: 'tomorrow'}));
        magnifispec.mockTabsController('CotEncryptionSettingsController', {}, {
            selectedWorkingEnvironment: {_pathPrefix: '/vsa/', publicId: '1', workingEnvironmentType: 'VSA'},
            vsaResponse,
            clientCertificate: {name: 'a', certificateType: 'client'},
            serverCertificate: {name: 'b', certificateType: 'server_ca'}
        });
        magnifispec.processPromise();
    }));

    describe('initialize', () => {
        it('should initialize', () => {
            expect(j.scope.occmCertificate).toBeDefined();
        });
    });

    describe('getOccmCertificate', () => {
        it('should work upon success with certificate', () => {
            nocEncryptionResourcesServiceSpy.mockImplementation(
                magnifispec.mockPromise({_pathPrefix: '/vsa/', publicId: 'certificate_object1', _expirationState: 'mocked_state'}));

            j.scope.getOccmCertificate();
            magnifispec.processPromise();

            expect(j.scope.occmCertificate).toEqual({_pathPrefix: '/vsa/', publicId: 'certificate_object1', _expirationState: 'mocked_state'});
        });

        it('should work upon success without certificate', () => {
            nocEncryptionResourcesServiceSpy.mockImplementation(
                magnifispec.mockPromise(null, true)
            );

            j.scope.getOccmCertificate();
            magnifispec.processPromise();

            expect(j.scope.occmCertificate).toBeNull();
        });

        it('should work upon error', () => {
            nocEncryptionResourcesServiceSpy.mockImplementation(
                magnifispec.mockPromise()
            );

            j.scope.getOccmCertificate();
            magnifispec.processPromise();

            expect(j.scope.occmCertificate.publicId).toEqual('initialized_value');
        });
    });

    describe('viewDetails', () => {
        it('should add spaces and open a dialog', () => {
            magnifispec.fakeDialog();

            j.scope.viewDetails({directIssuerName: 'a, b, c', subjectName: 'd, e, f'});

            expect(j.nocDialog.dialog).toHaveBeenCalledWith('Certificate details', 'assets/templates/dialogs/certificate-details.html', {
                certificate: {directIssuerName: 'a, b, c', subjectName: 'd, e, f'}
            });
        });
    });

    describe('openUpdateKeyManagersCaCertificateDialog', () => {
        describe('should open dialog upon success', () => {
            beforeEach(() => {
                jest.spyOn(nocEncryptionResourcesService, 'getKeyManagersCaCertificatesWithOptions').mockImplementation(
                    magnifispec.mockPromise({
                        data: [{id: 1, certificate: 'one'}, {id: 2, certificate: 'two'}],
                        select2options: 'keyManagersCaCertificatesOptionsMock'
                    }));
                jest.spyOn(j.scope, 'updateKeyManagerCaCertificate').mockImplementation(() => {});
            });

            it('and perform update upon confirm', () => {
                j.scope.serverCertificate = {certificate: 'one'};
                magnifispec.mockConfirm(true);

                j.scope.openUpdateKeyManagersCaCertificateDialog();
                magnifispec.processPromise();

                expect(j.nocDialog.confirm).toHaveBeenCalledWith('Update Key Manager CA Certificate',
                    'assets/templates/dialogs/update-key-managers-ca-certificate.html',
                    {
                        keyManagersCaCertificates: [{id: 2, certificate: 'two'}],
                        keyManagersCaCertificateOptions: 'keyManagersCaCertificatesOptionsMock',
                        selectedKeyManagerCaCertificate: {},
                        okClicked: j.scope.okClicked
                    }
                );
                expect(j.scope.updateKeyManagerCaCertificate).toHaveBeenCalled();
            });

            it('and not perform update upon cancel', () => {
                magnifispec.mockConfirm(false);

                j.scope.openUpdateKeyManagersCaCertificateDialog();
                magnifispec.processPromise();

                expect(j.scope.updateKeyManagerCaCertificate).not.toHaveBeenCalled();
            });

            it('should work with no serverCertificate', () => {
                j.scope.serverCertificate = undefined;
                magnifispec.mockConfirm(true);

                j.scope.openUpdateKeyManagersCaCertificateDialog();
                magnifispec.processPromise();

                expect(j.nocDialog.confirm).toHaveBeenCalledWith('Update Key Manager CA Certificate',
                    'assets/templates/dialogs/update-key-managers-ca-certificate.html',
                    {
                        keyManagersCaCertificates: [{id: 1, certificate: 'one'}, {id: 2, certificate: 'two'}],
                        keyManagersCaCertificateOptions: 'keyManagersCaCertificatesOptionsMock',
                        selectedKeyManagerCaCertificate: {},
                        okClicked: j.scope.okClicked
                    }
                );
                expect(j.scope.updateKeyManagerCaCertificate).toHaveBeenCalled();
            });
        });
        it('should show pidgeon upon error', () => {
            jest.spyOn(nocEncryptionResourcesService, 'getKeyManagersCaCertificatesWithOptions').mockImplementation(
                magnifispec.mockPromise()
            );
            jest.spyOn(j.nocDialog, 'confirm').mockImplementation(() => {});

            j.scope.openUpdateKeyManagersCaCertificateDialog();
            magnifispec.processPromise();

            expect(j.nocDialog.confirm).not.toHaveBeenCalled();
        });
    });

    describe('okClicked', () => {
        it('should call modalOK if form is valid', () => {
            const myObj = {
                updateKeyManagersCaCertificateForm: {$valid: true},
                okClicked: j.scope.okClicked,
                modalOk: jest.fn()
            };
            myObj.okClicked();
            expect(myObj.modalOk).toHaveBeenCalled();
        });

        it('should handleFormValidity if form is not valid', () => {
            const myObj = {
                updateKeyManagersCaCertificateForm: {$valid: false},
                okClicked: j.scope.okClicked
            };
            myObj.okClicked();
            expect(j.nocUI.handleFormValidity).toHaveBeenCalledWith(myObj.updateKeyManagersCaCertificateForm, j.scope);
        });
    });

    describe('updateKeyManagerCaCertificate', () => {
        it('should work upon success', () => {
            jest.spyOn(nocEncryptionResourcesService, 'formatCertificate').mockImplementation(() => {});
            j.$httpBackend.expectPOST(j.nocBaseUrl + '/vsa/encryption/encryptedVsaId1/update-key-manager-ca-certificate', {keyManagerCaCertificateId: 'value1'}).respond(200, 'newCaCertificate');

            j.scope.updateKeyManagerCaCertificate('value1');
            j.$httpBackend.flush();

            expect(nocEncryptionResourcesService.formatCertificate).toHaveBeenCalledWith('newCaCertificate');
            expect(j.scope.serverCertificate).toEqual('newCaCertificate');
            expect(j.pidgeon.inProgress).toHaveBeenCalledWith('Key Managers CA Certificate update started.', true);
        });

        it('should work upon error', () => {
            j.$httpBackend.expectPOST(j.nocBaseUrl + '/vsa/encryption/encryptedVsaId1/update-key-manager-ca-certificate', {keyManagerCaCertificateId: 'value1'}).respond(400, j.errorResponse);

            j.scope.updateKeyManagerCaCertificate('value1');
            j.$httpBackend.flush();

            expect(j.scope.serverCertificate).toEqual({name: 'b', certificateType: 'server_ca'});
            expect(j.pidgeon.error).toHaveBeenCalled();
        });

        afterEach(() => {
            expect(j.pidgeon.clear).toHaveBeenCalled();
            expect(j.nocUI.loading).toHaveBeenCalled();
            expect(j.nocUI.unloading).toHaveBeenCalled();
        });
    });

    describe('deleteKeyManager', () => {
        it('should prompt to the uer to confirm deletion', () => {
            magnifispec.mockConfirm(true);
            j.scope.deleteKeyManager({name: 'keyManager1', address: '11.22.33.44'});
            expect(j.nocDialog.confirm).toHaveBeenCalledWith('Are you sure you want to delete Key Manager 11.22.33.44 (keyManager1)?');
        });

        it('should add the key manager name only when available', () => {
            magnifispec.mockConfirm(true);
            j.scope.deleteKeyManager({address: '11.22.33.44'});
            expect(j.nocDialog.confirm).toHaveBeenCalledWith('Are you sure you want to delete Key Manager 11.22.33.44?');
        });

        it('should block the ui and make a post request to delete and on success unblock', () => {
            magnifispec.mockConfirm(true);
            j.$httpBackend.expectDELETE('/occm/api/vsa/encryption/encryptedVsaId1/key-managers/11.22.33.44').respond(200);

            j.scope.deleteKeyManager({name: 'keyManager1', address: '11.22.33.44'});
            magnifispec.processPromise();

            expect(j.nocUI.block.mock.calls.length).toEqual(1);
            j.$httpBackend.flush();
            expect(j.nocUI.unblock.mock.calls.length).toEqual(1);
        });

        it('should block the ui and make a post request to delete and on failure unblock and push an alert', () => {
            magnifispec.mockConfirm(true);
            j.$httpBackend.expectDELETE('/occm/api/vsa/encryption/encryptedVsaId1/key-managers/11.22.33.44').respond(400, j.errorResponse);

            j.scope.deleteKeyManager({name: 'keyManager1', address: '11.22.33.44'});
            magnifispec.processPromise();

            expect(j.nocUI.block.mock.calls.length).toEqual(1);
            j.$httpBackend.flush();
            expect(j.nocUI.unblock.mock.calls.length).toEqual(1);
            expect(j.pidgeon.error).toHaveBeenCalled();
        });

        it('should do nothing upon canceling a delete', () => {
            magnifispec.mockConfirm(false);
            j.scope.deleteKeyManager({name: 'keyManager1', address: '11.22.33.44'});
            expect(j.nocUI.block.mock.calls.length).toEqual(0);
            expect(j.nocUI.unblock.mock.calls.length).toEqual(0);
        });

        it('should not try to delete an the last key manager', () => {
            jest.spyOn(j.nocDialog, 'confirm').mockImplementation(() => {});
            j.scope.vsaResponse.vsaEncryptionResponse.keyManagers = [{name: 'lastKeyManager', address: '11.22.33.44'}];
            j.scope.deleteKeyManager({name: 'keyManager1', address: '11.22.33.44'});
            expect(j.nocDialog.confirm).not.toHaveBeenCalled();
        });
    });

    describe('updateClientCertificate', () => {
        it('should display appropriate message when no OCCM certificate is configured', () => {
            jest.spyOn(j.nocDialog, 'alert').mockImplementation(() => {});
            j.scope.occmCertificate = null;
            j.scope.updateClientCertificate();
            expect(j.nocDialog.alert).toHaveBeenCalledWith('Cloud Manager could not renew the Cloud Volumes ONTAP certificate because Cloud Manager is not configured to be an intermediate CA. Cloud Manager Admin can configure this in Encryption Setup.');
        });

        it('should display appropriate message when OCCM certificate is expired', () => {
            jest.spyOn(j.nocDialog, 'alert').mockImplementation(() => {});
            j.scope.occmCertificate = {_expirationState: {state: 'certificate-expired'}};
            j.scope.updateClientCertificate();
            expect(j.nocDialog.alert).toHaveBeenCalledWith('Cloud Manager could not renew the Cloud Volumes ONTAP certificate because the Cloud Manager intermediate CA certificate has expired. The Cloud Manager Admin must renew the intermediate CA certificate from the Encryption Setup page.');
        });

        describe('should open a dialog box', () => {
            beforeEach(() => {
                j.scope.occmCertificate = {_expirationState: {state: 'certificate-valid-long'}};
            });

            it('should do nothing upon cancel', () => {
                magnifispec.mockConfirm(false);
                j.scope.updateClientCertificate();
            });

            describe('call API', () => {
                it('should work upon success', () => {
                    jest.spyOn(nocEncryptionResourcesService, 'formatCertificate').mockImplementation(() => {});
                    magnifispec.mockConfirm(true);

                    j.$httpBackend.expectPOST(j.nocBaseUrl + '/vsa/encryption/encryptedVsaId1/update-client-certificate').respond(200, 'newClientCertificate');

                    j.scope.updateClientCertificate();
                    j.$httpBackend.flush();

                    expect(nocEncryptionResourcesService.formatCertificate).toHaveBeenCalledWith('newClientCertificate');
                    expect(j.scope.clientCertificate).toEqual('newClientCertificate');
                    expect(j.pidgeon.inProgress).toHaveBeenCalledWith('ONTAP certificate renew started.', true);
                });

                it('should work upon error', () => {
                    magnifispec.mockConfirm(true);

                    j.$httpBackend.expectPOST(j.nocBaseUrl + '/vsa/encryption/encryptedVsaId1/update-client-certificate').respond(400, j.errorResponse);

                    j.scope.updateClientCertificate();
                    j.$httpBackend.flush();

                    expect(j.scope.clientCertificate).toEqual({name: 'a', certificateType: 'client'});
                    expect(j.pidgeon.error).toHaveBeenCalled();
                });

                afterEach(() => {
                    expect(j.nocDialog.confirm).toHaveBeenCalledWith('Are you sure that you want to generate a new client certificate for this Cloud Volumes ONTAP system?');
                    expect(j.pidgeon.clear).toHaveBeenCalledWith('generating new ONTAP client certificate');
                    expect(j.nocUI.loading).toHaveBeenCalled();
                    expect(j.nocUI.unloading).toHaveBeenCalled();
                });
            });
        });
    });

    describe('showAddKeyManagerDialog', () => {
        it('should alert when more key managers count exceeded limit', () => {
            j.scope.vsaResponse = {vsaEncryptionResponse: {keyManagers: ['a', 'b', 'c', 'd']}};
            jest.spyOn(j.nocDialog, 'alert').mockImplementation(() => {});
            j.scope.showAddKeyManagerDialog();
            expect(j.nocDialog.alert).toHaveBeenCalledWith('You cannot configure Cloud Volumes ONTAP to use more than 4 key managers.');
        });

        it('should open the addKeyManagerDialog when it is possible to add key managers', () => {
            const vsaResponse = {vsaEncryptionResponse: {keyManagers: ['a', 'b', 'c']}};
            j.scope.vsaResponse = vsaResponse;
            jest.spyOn(j.nocDialog, 'dialog').mockImplementation(() => {});
            j.scope.showAddKeyManagerDialog();
            expect(j.nocDialog.dialog).toHaveBeenCalledWith('Add a Key Manager', 'assets/templates/dialogs/add-key-manager.html', {
                vsaResponse,
                selectedWorkingEnvironment: j.scope.selectedWorkingEnvironment,
                width: 415
            });
        });
    });
});
