describe('ShutdownVSAController', () => {
    'use strict';
    var j;    beforeEach(() => {        j = magnifispec.setup();    });
    beforeEach(() => {
        magnifispec.mockTabsController('ShutdownVSAController', {}, {
            selectedWorkingEnvironment: {_pathPrefix: '/vsa/', publicId: 1, _originalSchedules: []},
            showMonitoring: jest.fn()
        });
        jest.spyOn(j.scope, 'initialize').mockImplementation(() => {});
        magnifispec.processPromise();
    });

    describe('fixDaysValues', () => {
        it('should add 7 to any start day that is smaller than stop day', () => {
            const weekend = {name: 'Weekend', schedule: [{'stop': {'day': 6, 'hour': 8, 'minute': 0}, 'start': {'day': 1, 'hour': 8, 'minute': 0}}], 'enabled': true};
            j.scope.fixDaysValues(weekend);
            expect(weekend).toEqual({name: 'Weekend', schedule: [{'stop': {'day': 6, 'hour': 8, 'minute': 0}, 'start': {'day': 8, 'hour': 8, 'minute': 0}}], 'enabled': true});
        });

        it('should replace any 7 with 0 since server treats sunday as 7 while java script treats it as 0', () => {
            const weekdays = {name: 'Weekdays', schedule: [{'stop': {'day': 7, 'hour': 8, 'minute': 0}, 'start': {'day': 7, 'hour': 10, 'minute': 0}}], 'enabled': true};
            j.scope.fixDaysValues(weekdays);
            expect(weekdays).toEqual({name: 'Weekdays', schedule: [{'stop': {'day': 0, 'hour': 8, 'minute': 0}, 'start': {'day': 0, 'hour': 10, 'minute': 0}}], 'enabled': true});
        });
    });

    describe('buildDaysList', () => {
        it('should build a array of days', () => {
            const weekdays = {
                name: 'Weekdays',
                schedule: [{stop: {day: 1, hour: 20, minute: 0}, start: {day: 2, hour: 8, minute: 0}}, {
                    stop: {
                        day: 2,
                        hour: 20,
                        minute: 0
                    }, start: {day: 3, hour: 8, minute: 0}
                }, {stop: {day: 3, hour: 20, minute: 0}, start: {day: 4, hour: 8, minute: 0}}, {
                    stop: {
                        day: 4,
                        hour: 20,
                        minute: 0
                    }, start: {day: 5, hour: 8, minute: 0}
                }, {stop: {day: 5, hour: 20, minute: 0}, start: {day: 7, hour: 8, minute: 0}}],
                enabled: true
            };
            expect(j.scope.buildDaysList(weekdays)).toEqual(['Mon', 'Tue', 'Wed', 'Thu', 'Fri']);
        });
    });

    describe('initializeSchedules', () => {
        it('should create a copy array and fill the right values for the _maxPeriod and _shutdownPeriod for weekend', () => {
            const weekend = {name: 'Weekend', schedule: [{'stop': {'day': 6, 'hour': 8, 'minute': 0}, 'start': {'day': 1, 'hour': 8, 'minute': 0}}], 'enabled': true};
            const result = j.scope.initializeSchedules([weekend]);
            expect(result[0]._maxPeriod).toEqual(48);
            expect(result[0]._shutdownPeriod).toEqual(48);
        });

        it('should create a copy array and fill the right values for the _maxPeriod and _shutdownPeriod for weekdays', () => {
            const weekdays = {
                name: 'Weekdays',
                schedule: [
                    {'stop': {'day': 1, 'hour': 20, 'minute': 0}, 'start': {'day': 2, 'hour': 8, 'minute': 0}},
                    {'stop': {'day': 2, 'hour': 20, 'minute': 0}, 'start': {'day': 3, 'hour': 8, 'minute': 0}},
                    {'stop': {'day': 3, 'hour': 20, 'minute': 0}, 'start': {'day': 4, 'hour': 8, 'minute': 0}},
                    {'stop': {'day': 4, 'hour': 20, 'minute': 0}, 'start': {'day': 5, 'hour': 8, 'minute': 0}},
                    {'stop': {'day': 5, 'hour': 20, 'minute': 0}, 'start': {'day': 6, 'hour': 8, 'minute': 0}}
                ],
                'enabled': true};
            const result = j.scope.initializeSchedules([weekdays]);
            expect(result[0]._maxPeriod).toEqual(24);
            expect(result[0]._shutdownPeriod).toEqual(12);
        });
    });

    describe('saveSchedules', () => {
        beforeEach(() => {
            jest.spyOn(j.scope, 'buildSchedulesRequest').mockReturnValue('Schedules request 1');
            j.scope.schedules = ['s1', 's2'];
            jest.spyOn(j.scope, 'isValid').mockReturnValue(true);
        });

        it('should move to volumes upon success', () => {
            jest.spyOn(j.workingEnvironmentResource, 'updateSchedules').mockReturnValue(j.$q.when());

            j.scope.saveSchedules();
            magnifispec.processPromise();

            expect(j.scope.showMonitoring).toHaveBeenCalled();
            expect(j.scope.buildSchedulesRequest).toHaveBeenCalled();
            expect(j.scope.selectedWorkingEnvironment._originalSchedules).toEqual('Schedules request 1');
        });
    });

    describe('fixInvalidShutdownPeriod', () => {
        let schedule;
        beforeEach(() => {
            schedule = {_maxPeriod: 48};
        });

        it('should round the period to 1 if smaller than one', () => {
            expect(j.scope.fixInvalidShutdownPeriod(schedule, 0.1)).toEqual(1);
        });

        it('should round the period to an integer when not', () => {
            expect(j.scope.fixInvalidShutdownPeriod(schedule, 45.1)).toEqual(45);
        });

        it('should limit it to maxPeriod if larger', () => {
            expect(j.scope.fixInvalidShutdownPeriod(schedule, 49)).toEqual(48);
        });
    });
});
