describe('WorkingEnvironmentInformationController', () => {
    var j;    beforeEach(() => {        j = magnifispec.setup();    });

    beforeEach(() => {
        magnifispec.mockTabsController('WorkingEnvironmentInformationController', {}, {
            updateSelectedWorkingEnvironment: jest.fn().mockReturnValue(j.$q.when()),
            selectedWorkingEnvironment: {
                publicId: '1',
                workingEnvironmentType: 'VSA',
                ontapClusterProperties: 'someProperties!'
            },
            vsaResponse: {
                cluster: {lifs: [{name: 'lif1', lifType: 'Lala'}, {name: 'lif2', lifType: 'Cluster'}]}
            }
        });

        magnifispec.processPromise();
    });

    describe('initializeController', () => {
        it('should create clusterResponse for ON_PREM', () => {
            j.scope.selectedWorkingEnvironment.workingEnvironmentType = 'ON_PREM';

            j.scope.initializeController();

            expect(j.scope.clusterResponse).toEqual('someProperties!');
        });
    });

    describe('removeClusterLif', () => {
        it('should remove lifs with Cluster type if exists and not modify the original array', () => {
            const lifs = [{lifType: 'a'}, {lifType: 'Cluster'}, {lifType: 'b'}, {lifType: 'c'}, {lifType: 'd'}];
            const lifsWithoutCluster = j.scope.removeClusterLif(lifs);

            expect(lifs).toEqual([{lifType: 'a'}, {lifType: 'Cluster'}, {lifType: 'b'}, {lifType: 'c'}, {lifType: 'd'}]);
            expect(lifsWithoutCluster).toEqual([{lifType: 'a'}, {lifType: 'b'}, {lifType: 'c'}, {lifType: 'd'}]);
        });

        it('should not remove lifs with Cluster type if does not exist', () => {
            const lifs = [{lifType: 'a'}, {lifType: 'b'}, {lifType: 'c'}, {lifType: 'd'}];
            const lifsWithoutCluster = j.scope.removeClusterLif(lifs);

            expect(lifsWithoutCluster).toEqual([{lifType: 'a'}, {lifType: 'b'}, {lifType: 'c'}, {lifType: 'd'}]);
        });

        it('should not remove lifs with Cluster type for an empty array', () => {
            const lifsWithoutCluster = j.scope.removeClusterLif([]);

            expect(lifsWithoutCluster).toEqual([]);
        });
    });

    describe('showEditRouteTablesDialog', () => {
        it('should open the edit route tables dialog', () => {
            jest.spyOn(j.nocDialog, 'dialog').mockImplementation(() => {});
            j.scope.showEditRouteTablesDialog();
            expect(j.nocDialog.dialog).toHaveBeenCalledWith('Route Tables', 'assets/templates/tabs/working-environments/ha-edit-route-tables-dialog.html', jasmine.anything());
        });
    });
});
