describe("SVMOffControllerSpec", () => {
    var j;    beforeEach(() => {        j = magnifispec.setup();    });

    let workingEnvironment;
    let serverResponse;
    let vsaResponse;

    beforeEach(() => {
        workingEnvironment = {
        _pathPrefix:"/vsa/",
        _cloudProviderName: "AWS",
        _allowedOperations: {
            resources: true
        },
        publicId: "someId1",
        name: "bestWorkEnvEver",
        status: {
            status: "off"
        },
        workingEnvironmentType: "VSA",
        _stateInfo: {
            subTitle: ""
        }
    };
        serverResponse = {someResponse: "someValue", actionsRequired: ["some action required"]};
        vsaResponse = {status: {}, amiId: "amitami"};
        j.$httpBackend.expectGET(j.nocBaseUrl + '/vsa/working-environments/someId1?fields=status,encryptionProperties,ontapClusterProperties.fields(upgradeVersions),activeActions,supportedFeatures,k8sProperties,fpolicyProperties,awsProperties').respond(vsaResponse);
        j.$httpBackend.expectGET(j.nocBaseUrl + '/vsa/working-environments/someId1?fields=actionsRequired').respond(serverResponse);
        magnifispec.mockTabsController("MonitoringSVMOffController", {}, {
            monitoringInitialized: true,
            selectedWorkingEnvironment: workingEnvironment,
            setActionRequiredFor: _.noop
        });
        jest.spyOn(j.workingEnvironmentResource,"initWorkingEnvironment").mockImplementation(() => {});
        j.$httpBackend.flush();
    });

    describe("getOffSvmResponse", () => {
        beforeEach(() => {
            j.scope.selectedWorkingEnvironment = workingEnvironment;
        });
        it("should get offVsaResponse upon success", () => {
            j.$rootScope.$broadcast(j.workingEnvironmentResource.VSA_GET_EVENT2,{workingEnvironment: {publicId: "1", vpcId:"1"},vsaResponse: {state: 'ok', activeActions: null}});
            expect(j.scope.offVsaResponse).toEqual({state: 'ok', activeActions: null});
        });

        describe("should get offVsaResponse upon error ", () => {
            it("when status is off", () => {
                j.$rootScope.$broadcast(j.workingEnvironmentResource.VSA_GET_ERROR_EVENT,{workingEnvironment: {publicId: "1", vpcId:"1"},error: j.errorResponse});
                expect(j.pidgeon.error).toHaveBeenCalledWith(j.pidgeon.serverErrorPrefix + "retrieving Cloud Volumes ONTAP information for working environment bestWorkEnvEver", j.errorResponse, "vsaInformation");
            });

            it("when status is deleting", () => {
                j.scope.selectedWorkingEnvironment.status = {status: "deleting"};
                j.$rootScope.$broadcast(j.workingEnvironmentResource.VSA_GET_ERROR_EVENT,{workingEnvironment: {publicId: "1", vpcId:"1"},error: j.errorResponse});
                expect(j.pidgeon.error).not.toHaveBeenCalled();
            });

            it("when status is deleted and stop polling vsa", () => {
                jest.spyOn(j.scope,"vsaPollingStopper").mockImplementation(() => {});
                j.scope.selectedWorkingEnvironment.status = {status: "deleted"};
                j.$rootScope.$broadcast(j.workingEnvironmentResource.VSA_GET_ERROR_EVENT,{workingEnvironment: {publicId: "1", vpcId:"1"},error: j.errorResponse});
                expect(j.pidgeon.error).not.toHaveBeenCalled();
            });
        });
    });

    describe("getActionsRequired", () => {
        beforeEach(() => {
            expect(j.nocUI.loading).toHaveBeenCalled();
            j.scope.selectedWorkingEnvironment = workingEnvironment;
        });

        it("Should work upon success", () => {
            j.$httpBackend.expectGET(j.nocBaseUrl + '/vsa/working-environments/someId1?fields=actionsRequired').respond(serverResponse);
            j.scope.setActionRequiredFor = jest.fn();
            j.scope.getActionsRequired();
            j.$httpBackend.flush();
            expect(j.scope.setActionRequiredFor).toHaveBeenCalledWith("someId1", false, {"someId1": serverResponse.actionsRequired});
        });

        it("Should work upon error", () => {
            j.$httpBackend.expectGET(j.nocBaseUrl + '/vsa/working-environments/someId1?fields=actionsRequired').respond(400, j.errorResponse);
            j.scope.getActionsRequired();
            j.$httpBackend.flush();
            expect(j.pidgeon.error).toHaveBeenCalled();
        });

        afterEach(() => {
            expect(j.nocUI.unloading).toHaveBeenCalled();
        });
    });
});

describe("SVMOffControllerSpec", () => {
    var j;    beforeEach(() => {        j = magnifispec.setup();    });

    const workingEnvironment = {
        _allowedOperations: {
            resources: true
        },
        publicId: "someId1",
        name: "bestWorkEnvEver",
        status: "deleting",
        workingEnvironmentType: "VSA",
        _stateInfo: {
            subTitle: ""
        }
    };

    beforeEach(() => {
        magnifispec.mockTabsController("MonitoringSVMOffController", {}, {
            monitoringInitialized: true,
            selectedWorkingEnvironment: workingEnvironment
        });
    });

    describe("getActionsRequired", () => {
        beforeEach(() => {
            jest.spyOn(j.scope, "getActionsRequired").mockImplementation(() => {});
        });

        it("Should not be called", () => {
            expect(j.scope.getActionsRequired).not.toHaveBeenCalled();
        });
    });
});