describe('costMonthlyBreakdownComponent', () => {
    var j;
    beforeEach(() => {
        j = magnifispec.setup();
    });

    const mockAnnualData = [
        {
            start: '2018-01-01',
            end: '2018-01-31',
            costByService: [],
            total: {amount: 5}
        }, {
            start: '2018-02-01',
            end: '2018-02-28',
            costByService: [
                {
                    name: 'S3',
                    total: {amount: 5}
                },
                {
                    name: 'EC2',
                    total: {amount: 2},
                    costByUsage: [{name: 'Node: m4.2xlarge', cost: {amount: 2}}]
                }
            ],
            total: {amount: 7}
        }, {
            start: '2018-03-01',
            end: '2018-03-15',
            costByService: [
                {
                    name: 'EBS',
                    total: {amount: 7},
                    costByUsage: [{name: 'Other', cost: {amount: 2}}, {name: 'st1', cost: {amount: 3}}, {name: 'Snapshots', cost: {amount: 1}}, {name: 'IOPS', cost: {amount: 1}}]
                },
                {
                    name: 'EC2',
                    total: {amount: 2},
                    costByUsage: []
                },
                {
                    name: 'DataTransfer',
                    total: {amount: 4},
                    costByUsage: []
                }, {
                    name: 'S3',
                    total: {amount: 1},
                    costByUsage: []
                }
            ],
            total: {amount: 14}
        }];
    let controller;

    beforeEach(inject(($componentController) => {
        controller = $componentController('costMonthlyBreakdownComponent', null, {annualData: mockAnnualData});
    }));

    describe('init', () => {
        it('should initialze values and call onChangeMonthSelection', () => {
            jest.spyOn(controller, 'onChangeMonthSelection').mockImplementation(() => {});
            controller.$onInit();

            expect(controller.selectedIndex).toEqual(0);
            expect(controller.selectOptions).toEqual([{value: '2018-03-01', label: 'March 2018'}, {value: '2018-02-01', label: 'February 2018'}, {value: '2018-01-01', label: 'January 2018'}]);
            expect(controller.selectedOption).toEqual({value: '2018-03-01', label: 'March 2018'});
            expect(controller.onChangeMonthSelection).toHaveBeenCalled();
        });
    });

    describe('onChangeMonthSelection', () => {
        beforeEach(() => {
            controller.$onInit();
        });
        it('should set values according to the default selected month (current month)', () => {
            expect(controller.selectedMonthName).toEqual('March');
            expect(controller.daysInMonth).toEqual(14);
            expect(controller.monthData).toEqual(mockAnnualData[2]);
            expect(controller.monthData.costByService[0].costByUsage).toEqual([{name: 'IOPS', cost: {amount: 1}}, {name: 'st1', cost: {amount: 3}}, {name: 'Snapshots', cost: {amount: 1}}, {name: 'Other', cost: {amount: 2}}]);
            expect(controller.costDataForChart).toEqual([{name: 'Compute', y: 2}, {name: 'EBS Storage', y: 7}, {name: 'S3 Cost', y: 1}, {name: 'Data Transfer', y: 4}]);
        });
        it('should set values according to a selected month with partial costByService', () => {
            controller.selectedOption = {value: '2018-02-01'};
            controller.onChangeMonthSelection();
            expect(controller.selectedMonthName).toEqual('February');
            expect(controller.daysInMonth).toEqual(27);
            expect(controller.monthData).toEqual(mockAnnualData[1]);
            expect(controller.costDataForChart).toEqual([{name: 'Compute', y: 2}, {name: 'EBS Storage', y: 0}, {name: 'S3 Cost', y: 5}, {name: 'Data Transfer', y: 0}]);
        });

        it('should return the correct category index', () => {
            expect(controller.getCategoryIndex('EC2')).toEqual(1);
            expect(controller.getCategoryIndex('EBS')).toEqual(0);
            expect(controller.getCategoryIndex('S3')).toEqual(3);
            expect(controller.getCategoryIndex('DataTransfer')).toEqual(2);
        });
    });
});
