describe('edit volume controller', () => {
    'use strict';

    const mockedSnapshots = [
        {
            name: 'snap1',
            description: 'snap1 desc',
            schedules: [
                {
                    frequency: 'weekly',
                    retention: '6'
                },
                {
                    frequency: 'monthly',
                    retention: '2'
                }
            ]
        },
        {
            name: 'snap2',
            description: 'snap2 desc',
            schedules: [
                {
                    frequency: 'weekly',
                    retention: '6'
                },
                {
                    frequency: 'monthly',
                    retention: '2'
                }
            ]
        },
        {
            name: 'default-1weekly',
            description: 'snap2 desc',
            schedules: [
                {
                    frequency: 'weekly',
                    retention: '6'
                },
                {
                    frequency: 'monthly',
                    retention: '2'
                }
            ]
        }
    ];

    var j;    beforeEach(() => {        j = magnifispec.setup();    });
    let usageProfiles;
    const volumes = [{name: 'vol1', svmName: 'svm1'}];

    beforeEach(inject((_usageProfiles_) => {
        // Test with feature flag on
        j.$httpBackend.expectGET(j.nocBaseUrl + '/vsa/working-environments/1?fields=snapshotPolicies').respond({snapshotPolicies: mockedSnapshots});
        usageProfiles = _usageProfiles_;

        const defaultExportPolicyIpsSpy = jest.fn().mockReturnValue(['1.1.1.1/24']);
        magnifispec.mockTabsController('EditVolumeController', {}, {
            volumes,
            updateSelectedWorkingEnvironment: jest.fn().mockReturnValue(j.$q.when()),
            selectedWorkingEnvironment: {
                tenantId: 1,
                publicId: '1',
                name: 'we1',
                workingEnvironmentType: 'VSA',
                providerVolumeType: 'gp2',
                _pathPrefix: '/vsa/'
            },
            showMonitoring: jest.fn(),
            getDefaultExportPolicyIps: defaultExportPolicyIpsSpy,
            selectedVolume: {
                name: 'vol1',
                exportPolicyInfo: {policyType: 'none'},
                size: {size: 100, unit: 'GB'},
                svmName: 'svm1',
                usageProfile: usageProfiles.fromFlags(true, true, false),
                snapshotPolicyName: 'none',
                shareInfo: [{
                    accessControlList: [{
                        users: ['Everyone'],
                        permission: 'ful_control'
                    }],
                    shareName: 'share1'
                }]
            }
        });
        magnifispec.processPromise();
        j.$httpBackend.flush();

        j.scope.showMonitoring = jest.fn();
    }));

    describe("send update volume request to the server", function () {
        beforeEach(function () {
            jest.spyOn(j.scope, "generateRequestBody").mockReturnValue({snapshotPolicyName : "none"});
            j.scope.editVolumeForm = {$valid: true};
        });

        it('on normal scenario VSA', () => {
            jest.spyOn(j.scope, 'isDirty').mockReturnValue(true);
            j.scope.selectedWorkingEnvironment = {
                _pathPrefix: '/vsa/',
                publicId: 'we_123',
                workingEnvironmentType: 'VSA',
                providerVolumeType: 'gp2'
            };
            j.$httpBackend.expectPUT(j.nocBaseUrl + j.nocUtil.workingEnvironmentTypeToRequestPathPrefix('VSA', 'AWS') + 'volumes/we_123/svm1/vol1', {
                originalVolumeInfo: {
                    shareInfo: [{
                        accessControlList: [{
                            users: ['Everyone'],
                            permission: 'ful_control'
                        }], shareName: 'share1'
                    }],
                    exportPolicyInfo: {policyType: 'none'}
                },
                snapshotPolicyName: 'none'
            }).respond(200);
            j.scope.saveAddVolume();
            j.$httpBackend.flush();

            expect(j.scope.showMonitoring).toHaveBeenCalledWith({
                _pathPrefix: '/vsa/',
                publicId: 'we_123',
                workingEnvironmentType: 'VSA',
                providerVolumeType: 'gp2'
            });
        });

        it('on normal scenario on prem', () => {
            jest.spyOn(j.scope, 'isDirty').mockReturnValue(true);
            j.scope.selectedWorkingEnvironment = {
                _pathPrefix: '/onprem/',
                publicId: 'we_123',
                workingEnvironmentType: 'ON_PREM',
                providerVolumeType: 'gp2'
            };
            j.$httpBackend.expectPUT(j.nocBaseUrl + j.nocUtil.workingEnvironmentTypeToRequestPathPrefix('ON_PREM') + 'volumes/we_123/svm1/vol1', {
                'originalVolumeInfo': {
                    'shareInfo': [{
                        'accessControlList': [{
                            'users': ['Everyone'],
                            'permission': 'ful_control'
                        }], 'shareName': 'share1'
                    }],
                    exportPolicyInfo: {policyType: 'none'}
                }, 'snapshotPolicyName': 'none'
            }).respond(200);
            j.scope.saveAddVolume();
            j.$httpBackend.flush();

            expect(j.scope.showMonitoring).toHaveBeenCalledWith({
                _pathPrefix: '/onprem/',
                publicId: 'we_123',
                workingEnvironmentType: 'ON_PREM',
                providerVolumeType: 'gp2'
            });
        });

        it('with server error', () => {
            jest.spyOn(j.scope, 'isDirty').mockReturnValue(true);
            j.scope.selectedWorkingEnvironment = {
                _pathPrefix: '/vsa/',
                publicId: 'we_123',
                workingEnvironmentType: 'VSA',
                providerVolumeType: 'gp2'
            };
            j.$httpBackend.expectPUT(j.nocBaseUrl + j.nocUtil.workingEnvironmentTypeToRequestPathPrefix('VSA', 'AWS') + 'volumes/we_123/svm1/vol1', {
                'originalVolumeInfo': {
                    'shareInfo': [{
                        'accessControlList': [{
                            'users': ['Everyone'],
                            'permission': 'ful_control'
                        }], 'shareName': 'share1'
                    }],
                    exportPolicyInfo: {policyType: 'none'}
                }, 'snapshotPolicyName': 'none'
            }).respond(500, j.errorResponse);
            j.scope.saveAddVolume();
            j.$httpBackend.flush();

            expect(j.pidgeon.error).toHaveBeenCalled();
            expect(j.scope.showMonitoring).not.toHaveBeenCalled();
        });

        it("fail on validation", function () {
            j.scope.editVolumeForm = {$valid: false};
            j.scope.saveAddVolume();

            expect(j.nocUI.handleFormValidity).toHaveBeenCalledWith(j.scope.addVolumeForm, j.scope);
            expect(j.scope.showMonitoring).not.toHaveBeenCalled();
        });

        it("should go back to gallery if the form is not dirty", function () {
            jest.spyOn(j.scope, "isDirty").mockReturnValue(false);
            jest.spyOn(j.scope, "cancelAddVolume").mockImplementation(() => {});
            j.scope.saveAddVolume();

            expect(j.$httpBackend.flush).toThrowError(j.backendFlushFailMessage);
            expect(j.scope.cancelAddVolume).toHaveBeenCalled();
        });

    });

    describe("cancelAddVolume", function () {
        var workingEnvironment = {_pathPrefix:"/vsa/",publicId: "we1"};
        beforeEach(function () {
            j.scope.showMonitoring = jest.fn();
        });
        it("should call showMonitoring", function () {
            j.scope.cancelAddVolume(workingEnvironment);
            expect(j.scope.showMonitoring).toHaveBeenCalledWith(workingEnvironment);
        });

    });

    describe("getSnapshotPolicy ", function () {
        it("should remove default-1weekly", function(){
            var item = _.find(j.scope.snapshotPolicies, {name: 'default-1weekly'});
            expect(item).not.toBeDefined();
            expect(j.scope.snapshotPolicies.length).toEqual(2);
        });

        it("should return the snapshot policy with the given name ByName", function () {
            j.scope.snapshotPolicies = [
                {name: "a"},
                {name: "b"}
            ];
            expect(j.scope.getSnapshotPolicyByName("a")).toEqual({name: "a"});
        });
    });

    describe("isDirty", function () {
        beforeEach(function () {
            j.scope.volumeRequest = {name: "a", usageProfile: usageProfiles.default};
            j.scope.originalEditVolumeRequest = _.cloneDeep(j.scope.volumeRequest);
        });

        it("should be true if the request changed", function () {
            j.scope.volumeRequest.name = "b";

            expect(j.scope.isDirty()).toBeTruthy();
        });

        it("should be false if the request didn't change", function () {
            expect(j.scope.isDirty()).toBeFalsy();
        });
    });

    describe("initialize controller", function () {
        var workingEnvironment;

        beforeEach(function () {
            workingEnvironment = {_pathPrefix:"/vsa/", publicId: 1, name: "we1", svmName: "svm1"};
            jest.spyOn(j.scope,"resolveVolumeBySvmNameAndVolumeName").mockImplementation(() => {});
            jest.spyOn(j.scope,"generateEditVolumeRequest").mockReturnValue({cifsProtocolInfo: "NFS"});
            jest.spyOn(j.scope,"fetchSnapshotPolicy").mockImplementation(() => {});
            j.$state.params = {selectedVolumeSVM: "svm1", selectedVolumeName: "vol1"};
        });
        it("should set the selected working environment to the given working environment and call the right methods for initialization", function () {
            j.scope.initializeController(workingEnvironment);
            j.$rootScope.$apply();

            expect(j.scope.resolveVolumeBySvmNameAndVolumeName).toHaveBeenCalledWith("svm1", "vol1");
        });

    });

    describe("selectShareInfo", function () {
        it("should call findShareInfoByName and buildCifsProtocolInfo with the right shareInfo ", function () {
            j.scope.selectedVolume.shareInfo = {shareName : "shareName", accessControlList: []};
            jest.spyOn(j.scope, "findShareInfoByName").mockReturnValue({name: "this is my share info"});
            jest.spyOn(j.scope, "buildCifsProtocolInfo").mockImplementation(() => {});
            j.scope.selectShareInfo();
            expect(j.scope.findShareInfoByName).toHaveBeenCalledWith("share1");
            expect(j.scope.buildCifsProtocolInfo).toHaveBeenCalledWith({name: "this is my share info"});

        });
    });

    describe("buildCifsProtocolInfo", function () {
        var shareInfo;
        beforeEach(function () {
            shareInfo = {
                shareName: "shareName",
                accessControlList: [{users: ["user1", "user2"], permission: "full_control"}]
            };

        });
        it("should return the right cifsProtocolInfo when there is only one permission", function () {

            expect(j.scope.buildCifsProtocolInfo(shareInfo)).toEqual({
                accessControl: {
                    permission: "full_control",
                    users: ["user1", "user2"],
                    _users: "user1;user2"
                },
                shareName: "shareName",
                shouldShowPermissions: true
            });
        });

        it("should return the right cifsProtocolInfo when there is more than one permission", function () {
            shareInfo.accessControlList.push({users: ["user3", "user4"], permission: "update"});
            expect(j.scope.buildCifsProtocolInfo(shareInfo)).toEqual({
                accessControl: {
                    permission: "full_control",
                    _users: "user1;user2",
                    users: ["user1", "user2"]
                },
                shareName: "shareName",
                shouldShowPermissions: false
            });
        });

        it("should build the right object even when the access control array is empty", function () {
            shareInfo.accessControlList = [];
            expect(j.scope.buildCifsProtocolInfo(shareInfo)).toEqual({
                    accessControl: {
                        permission: "",
                        _users: "",
                        users: []
                    },
                    shareName: "shareName",
                    shouldShowPermissions: true
                });
        });
    });

    describe("resolveVolumeBySvmNameAndVolumeName", function () {
        beforeEach(function () {
            j.scope.volumes = [{name:"vol1", svmName:"svm1"}];
            jest.spyOn(j.scope,"cancelAddVolume").mockImplementation(() => {});
            j.scope.selectedVolume = undefined;
        });
        it("should cancel navigation if there are no volumes on the scope", function () {
            j.scope.volumes = undefined;
            j.scope.resolveVolumeBySvmNameAndVolumeName("svm1","vol1");
            expect(j.scope.selectedVolume).toBeUndefined();
            expect(j.scope.cancelAddVolume).toHaveBeenCalled();
        });

        it("should set the selected volume if found in the list of volumes", function () {
            j.scope.resolveVolumeBySvmNameAndVolumeName("svm1","vol1");
            expect(j.scope.selectedVolume).toEqual({name:"vol1", svmName:"svm1"});
            expect(j.scope.cancelAddVolume).not.toHaveBeenCalled();
        });

        it("should cancel navigation if the required volume is not found in the volumes list", function () {
            j.scope.resolveVolumeBySvmNameAndVolumeName("svm1","vol2");
            expect(j.scope.selectedVolume).toBeUndefined();
            expect(j.scope.cancelAddVolume).toHaveBeenCalled();

        });
    });

    describe("buildUpdatedShareInfo", function () {
        it("should build the right permissions if there is only one permission in the list", function () {
            jest.spyOn(j.scope,"findShareInfoByName").mockReturnValue({shareName:"shareName", accessControlList:[{permission:"custom", users:["user1"]}]});
            expect(j.scope.buildUpdatedShareInfo("shareName","anotherPermission",["user1", "user2"])).toEqual({shareName: "shareName", accessControlList:[{permission:"anotherPermission", users: ["user1", "user2"]}]});
        });

        it("should use the original permissions if there is more than one in the list", function () {
            jest.spyOn(j.scope,"findShareInfoByName").mockReturnValue({shareName:"shareName", accessControlList:[{permission:"custom", users:["user1"]},{permission:"custom2", users:["user12"]}]});
            expect(j.scope.buildUpdatedShareInfo("shareName","anotherPermission",["user1", "user2"])).toEqual({shareName: "shareName", accessControlList:[{permission:"custom", users:["user1"]},{permission:"custom2", users:["user12"]}]});
        });
        it("should build the right object even when the original access list is empty", function () {
            jest.spyOn(j.scope,"findShareInfoByName").mockReturnValue({shareName:"shareName", accessControlList:[]});
            expect(j.scope.buildUpdatedShareInfo("shareName","anotherPermission",["user1", "user2"])).toEqual({shareName: "shareName", accessControlList:[{permission:"anotherPermission", users:["user1", "user2"]}]});
        });
    });

    describe("shareInfoOptions", function () {
        beforeEach(function () {
            jest.spyOn(j.nocUtil,"formatSelect2").mockImplementation(() => {});
        });
        it("should build the right format for 1 permission", function () {
            jest.spyOn(j.scope,"findShareInfoByName").mockReturnValue({shareName:"shareName", accessControlList:[{permission:"custom", users:["user1"]}]});
            j.scope.shareInfoOptions.formatResult({id:"1"});
            expect(j.nocUtil.formatSelect2).toHaveBeenCalledWith("shareName","shareName","","1 permission");
        });
        it("should build the right format for more than one permission", function () {
            jest.spyOn(j.scope,"findShareInfoByName").mockReturnValue({shareName:"shareName", accessControlList:[{permission:"custom", users:["user1"]},{permission:"custom2", users:["user12"]}]});
            j.scope.shareInfoOptions.formatResult({id:"1"});
            expect(j.nocUtil.formatSelect2).toHaveBeenCalledWith("shareName","shareName","","2 permissions");
        });
    });

    describe("generateEditVolumeRequest", function () {
        var volume;
        beforeEach(function () {
            volume = {
                name: "vol1",
                size: {size: 100, unit: "GB"},
                svmName: "svm1",
                thinProvisioning: true,
                deduplication: true,
                compression: false,
                snapshotPolicy: "none",
                shareInfo: [{
                    accessControlList: [{
                        users: ["Everyone"],
                        permission: "ful_control"
                    }],
                    shareName: "share1"
                }],
                providerVolumeType: 'gp2'
            };

            jest.spyOn(j.scope, "buildCifsProtocolInfo").mockReturnValue("shareInfo");
            jest.spyOn(usageProfiles, "fromFlags").mockReturnValue("usageProfiles");
        });

        it("should build the right edit request when there is no export policy, but there is share info", function () {
            expect(j.scope.generateEditVolumeRequest(volume)).toEqual({
                name : 'vol1',
                sizeGb : 100,
                workingEnvironmentId : '1',
                svmName : 'svm1',
                aggregateName : undefined,
                exportPolicyInfo : undefined,
                shareInfo : 'shareInfo',
                usageProfile : "usageProfiles",
                snapshotPolicyName : "none",
                providerVolumeType: 'gp2'
            });

            expect(j.scope.buildCifsProtocolInfo).toHaveBeenCalledWith({
                accessControlList: [{
                    users: ["Everyone"],
                    permission: "ful_control"
                }],
                shareName:"share1"
            });
            expect(usageProfiles.fromFlags).toHaveBeenCalledWith(true,true,false);
        });

        it("should return the right edit request when there is an export policy with no ips, and no share info", function () {
            volume.shareInfo = [];
            volume.exportPolicyInfo = {policyType : "policyType", ips : []};
            expect(j.scope.generateEditVolumeRequest(volume)).toEqual({
                name : 'vol1',
                sizeGb : 100,
                workingEnvironmentId : '1',
                svmName : 'svm1',
                aggregateName : undefined,
                exportPolicyInfo : {policyType: "policyType", ips: [], _ips: ""},
                shareInfo : undefined,
                usageProfile : "usageProfiles",
                snapshotPolicyName : "none",
                providerVolumeType: 'gp2'
            });
        });

        it("should return the right edit request when there is an export policy with ips, and no share info", function () {
            volume.shareInfo = [];
            volume.exportPolicyInfo = {policyType : "policyType", ips : ["1.1.1.1", "2.2.2.2"]};
            expect(j.scope.generateEditVolumeRequest(volume)).toEqual({
                name : 'vol1',
                sizeGb : 100,
                workingEnvironmentId : '1',
                svmName : 'svm1',
                aggregateName : undefined,
                exportPolicyInfo : {policyType: "policyType", ips: ["1.1.1.1", "2.2.2.2"], _ips: "1.1.1.1, 2.2.2.2"},
                shareInfo : undefined,
                usageProfile : "usageProfiles",
                snapshotPolicyName : "none",
                providerVolumeType: 'gp2'
            });
        });
    });
});
