describe("HaEditRouteTablesDialogController", function () {
    "use strict";

    var j;    beforeEach(() => {        j = magnifispec.setup();    });

    beforeEach(function () {
        j.$httpBackend.expectGET(j.nocBaseUrl + "/aws/ha/working-environments/1?fields=haProperties").respond({haProperties: {routeTables: ['routeTable1','routeTable2']}});
        j.$httpBackend.expectGET(j.nocBaseUrl + "/aws/ha/metadata/vpcs?region=region").respond([{vpcId: 'vpcId', subnets: [{subnetId: 'subnet1',cidr:'0.0.0.0/0'},{subnetId: 'subnet2',cidr:'0.0.0.0/1'}]}]);
        j.$httpBackend.expectGET(j.nocBaseUrl + "/aws/ha/metadata/route-tables?region=region&vpcId=vpcId").respond([
            {id:'routeTable1', subnets:['subnet1'], tags: [{key: 'Name', value: 'routeTable1Name'}]},
            {id:'routeTable2', subnets:['subnet2','subnet1'], tags: [{key: 'Name', value: 'routeTable2Name'}, {key: 'A', value: 'B'}]}
        ]);
        magnifispec.mockTabsController('HaEditRouteTablesDialogController', {}, {selectedWorkingEnvironment: {publicId: 1, awsProperties: {regionName: 'region', vpc: {vpcId: 'vpcId'}}}, mainScreenNocHttp : j.nocHttp});
        j.$httpBackend.flush();
    });

    describe("initialize", function(){

        it("should initialize the route tables", function(){
            expect(j.scope.wizardController.routeTables).toEqual(
                [{
                    id: 'routeTable1',
                    subnets:['subnet1'],
                    _subnets: [{subnetId:'subnet1',cidr:'0.0.0.0/0'}],
                    tags: [{key: 'Name', value: 'routeTable1Name'}],
                    name: 'routeTable1Name',
                    expanded: false,
                    selected: true
                }, {
                    id: 'routeTable2',
                    subnets:['subnet2','subnet1'],
                    _subnets: [{subnetId:'subnet1',cidr:'0.0.0.0/0'},{subnetId: 'subnet2',cidr:'0.0.0.0/1'}], // sorted by cidr
                    tags: [{key: 'A', value: 'B'}, {key: 'Name', value: 'routeTable2Name'}], // sorted by key
                    name: 'routeTable2Name',
                    expanded: false,
                    selected: true
                }]
            );
            expect(j.scope.controllerData).toEqual({ selectAll: true }); // all route tables were selected by id
        });

    });

    describe('isValid', () => {
        it('should return true if form is valid', () => {
            j.scope.routingTablesForm = {$valid: true};
            expect(j.scope.isValid()).toEqual(true);

            j.scope.routingTablesForm = {$valid: false};
            expect(j.scope.isValid()).toEqual(false);
        });
    });

    describe("saveRouteTables", function(){
        it('should not save if form is not valid', () => {
            j.scope.mainScreenNocHttp = {put: jest.fn()};
            jest.spyOn(j.scope, 'isValid').mockReturnValue(false);
            j.scope.saveRouteTables();

            expect(j.scope.userClickedNext).toEqual(true);
            expect(j.scope.mainScreenNocHttp.put).not.toHaveBeenCalled();
        });

        it("should save the route tables and close the dialog", function(){
            jest.spyOn(j.scope, 'isValid').mockReturnValue(true);
            j.scope.mainScreenNocHttp.put = jest.fn().mockReturnValue(j.$q.when());
            j.scope.closeDialog = jest.fn();

            j.scope.saveRouteTables();

            magnifispec.processPromise(); // initiate .then()
            expect(j.scope.mainScreenNocHttp.put).toHaveBeenCalledWith(
                {url: '/aws/ha/working-environments/1/route-tables',data: {routeTableIds: ['routeTable1','routeTable2']}},
                "saving route tables", j.scope.mainScreenNocHttp.LOADING
            );
            expect(j.scope.closeDialog).toHaveBeenCalled();
            expect(j.scope.userClickedNext).toEqual(true);
        });

    });
});