describe('SupportRegistrationFormController', () => {
    'use strict';

    let j;
    beforeEach(() => {
        j = magnifispec.setup();
    });
    describe('Successful initialization', () => {
        beforeEach(() => {
            j.$rootScope.currentUser = {roleId: 'Role-1'};
            j.$httpBackend.expectGET(j.nocBaseUrl + '/vsa/working-environments/1?fields=supportRegistrationInformation').respond({supportRegistrationInformation: [{supportRegistrationStatus: 'registered'}, {supportRegistrationStatus: 'deregistered'}]});
            j.$httpBackend.expectGET(j.nocBaseUrl + '/occm/config').respond({ipaManualUrl: 'http://support.com'});
            j.$httpBackend.expectGET(j.nocBaseUrl + '/accounts').respond({who: 'me'});
            j.$httpBackend.expectGET(j.nocBaseUrl + '/vsa/working-environments/1/support-registration').respond({hey: 'you'});
            magnifispec.mockTabsController('SupportRegistrationFormController', {}, {
                showMonitoring: jest.fn(),
                selectedTenant: {publicId: 1},
                selectedWorkingEnvironment: {_pathPrefix: '/vsa/', publicId: 1}
            });
            j.$httpBackend.flush();
        });

        it('should set the registrationStatusMessage and nssUserName on the scope', () => {
            expect(j.scope.registrationStatusMessages).toEqual(['registered', 'not-registered']);
            expect(j.scope.isNotRegistered).toEqual(true);
            expect(j.scope.manualRegistrationUrl).toEqual('http://support.com');
        });
    });

    describe('manualRegistrationPopup', () => {
        beforeEach(() => {
            magnifispec.mockTabsController('SupportRegistrationFormController', {}, {
                selectedTenant: {publicId: 1},
                selectedWorkingEnvironment: {_isAWS: false, _isAzure: false, isHA: false},
                vsaResponse: {
                    cluster: {
                        clusterName: 'clusterName',
                        serialNumbers: 'serialNumber',
                        nodes: [{
                            name: 'node1',
                            cloudProviderId: 'instanceId1',
                            serialNumber: 'serialNumber1'
                        }, {name: 'node2', cloudProviderId: 'instanceId2', serialNumber: 'serialNumber2'}]
                    },
                    accountId: 'accountId',
                    instanceId: 'instanceId',
                    subscriptionId: 'subscriptionId',
                    deploymentId: 'deploymentId'
                }
            });
            jest.spyOn(j.nocDialog, 'dialog').mockImplementation(() => {
            });

            j.scope.vsaResponse.cluster.serialNumbers = 'serialNumber';
            j.scope.vsaResponse.accountId = 'accountId';
        });

        function expectQueryToEqual(expectedQuery) {
            j.scope.manualRegistrationPopup();
            expect(j.nocDialog.dialog).toHaveBeenCalledWith('NetApp Support Registration', 'assets/templates/dialogs/support-registration-manual-register-popup.html', jasmine.any(Object)); // Third arg is dialog args
            const dialogArgs = mostRecentCall(j.nocDialog.dialog);
            const actualQuery = dialogArgs[2].query;
            expect(actualQuery).toEqual(expectedQuery);
        }

        it('should open a dialog with a link that sends registration info in url for Azure', () => {
            j.scope.selectedWorkingEnvironment._isAzure = true;
            expectQueryToEqual('?instanceId=deploymentId&serialNumber=serialNumber&accountId=subscriptionId&hc=-1174119051');
        });

        it('should open a dialog with a link that sends registration info in url for AWS', () => {
            j.scope.selectedWorkingEnvironment._isAWS = true;
            expectQueryToEqual('?instanceId=instanceId&serialNumber=serialNumber&accountId=accountId&hc=2118370267');
        });

        it('should open a dialog with a link that sends registration info in url for HA', () => {
            j.scope.selectedWorkingEnvironment._isAWS = true;
            j.scope.selectedWorkingEnvironment.isHA = true;
            expectQueryToEqual('?instanceId=instanceId1&serialNumber=serialNumber1&accountId=accountId&hc=1348967366&instanceId=instanceId2&serialNumber=serialNumber2');
        });
    });
});
