describe('CIFS Setup Controller', () => {
    'use strict';

    var j;    beforeEach(() => {        j = magnifispec.setup();    });
    beforeEach(() => {
        magnifispec.mockTabsController('CifsConfigurationsController',
            {$stateParams: {workingEnvironmentId: 1}},
            {selectedWorkingEnvironment: {name: 'we1', workingEnvironmentType: 'VSA', _pathPrefix: '/vsa/'}});
    });

    describe('initialization', () => {
        beforeEach(() => {
            j.$httpBackend.expectGET('/occm/api/vsa/working-environments/1/cifs').respond(200, []);
        });

        it('set initial controller state', () => {
            j.$httpBackend.flush();
            expect(j.nocUI.block).toHaveBeenCalled();
            expect(j.nocUI.loading).toHaveBeenCalled();
            expect(j.scope.state).toEqual(j.scope.basic);
            expect(j.scope.mode).toEqual(j.scope.modes.create);
            expect(j.nocUI.unblock).toHaveBeenCalled();
            expect(j.nocUI.unloading).toHaveBeenCalled();
        });

        describe('finishing', () => {
            beforeEach(() => {
                j.scope.showMonitoring = jest.fn();
            });

            it('happy path', () => {
                j.$httpBackend.expectPOST('/occm/api/vsa/working-environments/1/cifs', {
                    'dnsDomain': '',
                    'activeDirectoryDomain': '',
                    'ipAddresses': ['1.1.1.1'],
                    'netBIOS': 'we1',
                    'organizationalUnit': 'CN=Computers',
                    'activeDirectoryUsername': 'administrator',
                    'activeDirectoryPassword': ''
                }).respond(200);

                j.scope.forms.cifsForm = {$valid: true};

                j.scope.request.ip1 = '1.1.1.1';

                j.scope.finish();
                j.$httpBackend.flush();

                expect(j.scope.showMonitoring).toHaveBeenCalledWith();
            });

            it('failed request', () => {
                j.scope.request.ip1 = '1.1.1.1';
                j.scope.request.ip2 = '1.1.1.2';

                j.$httpBackend.expectPOST('/occm/api/vsa/working-environments/1/cifs', {
                    'dnsDomain': '',
                    'activeDirectoryDomain': '',
                    'ipAddresses': ['1.1.1.1', '1.1.1.2'],
                    'netBIOS': 'we1',
                    'organizationalUnit': 'CN=Computers',
                    'activeDirectoryUsername': 'administrator',
                    'activeDirectoryPassword': ''
                }).respond(400, j.errorResponse);

                j.scope.forms.cifsForm = {$valid: true};

                j.scope.finish();
                j.$httpBackend.flush();
                expect(j.pidgeon.clear).toHaveBeenCalledWith('cifs');
                expect(j.nocUI.handleFormValidity).toHaveBeenCalled();
                expect(j.nocUI.block).toHaveBeenCalled();
                expect(j.pidgeon.error).toHaveBeenCalled();
                expect(j.nocUI.unblock).toHaveBeenCalled();
            });

            it('failed validation', () => {
                jest.spyOn(j.scope, 'switchState').mockImplementation(() => {});
                j.scope.request.ip1 = '1.1.1.1';
                j.scope.request.ip2 = '1.1.1.2';

                j.scope.forms.cifsForm = {$valid: false};
                j.scope.forms.cifsForm[j.scope.advanced] = {$valid: false};
                j.scope.forms.cifsForm[j.scope.basic] = {$valid: true};

                j.scope.finish();

                expect(j.scope.switchState).toHaveBeenCalledWith(j.scope.advanced);

                expect(j.pidgeon.clear).toHaveBeenCalledWith('cifs');
                expect(j.nocUI.handleFormValidity).toHaveBeenCalled();
                expect(j.pidgeon.error).not.toHaveBeenCalledWith('', j.errorResponse, 'cifs');
            });
        });

        describe('binding', () => {
            beforeEach(() => {
                j.$httpBackend.flush();
            });

            it('should bind active directory domain to dns domain and unbind', () => {
                expect(j.scope.request.dnsDomain).toEqual('');
                j.scope.request.activeDirectoryDomain = 'domain1';
                j.scope.$digest();
                expect(j.scope.request.dnsDomain).toEqual('domain1');

                j.scope.bindDnsAndAD.status = false;
                j.scope.bindingChanged();
                j.scope.$digest();
                j.scope.request.dnsDomain = 'dnsDomain1';

                j.scope.request.activeDirectoryDomain = 'black';
                j.scope.$digest();
                expect(j.scope.request.dnsDomain).toEqual('dnsDomain1');
            });
        });
    });
});
