describe('CreateNewAggregateDiskSizeAndIopsStepController', () => {
    'use strict';

    var j;    beforeEach(() => {        j = magnifispec.setup();    });
    const providerVolumeTypes = {
        gp2: {
            default: true,
            description: 'General purpose SSD volume that balances price and performance.',
            display: 'General Purpose SSD (gp2)',
            name: 'gp2',
            shortName: 'gp2',
            diskName: 'gp2',
            supportedInHa: true,
            supportedSizes: [
                {size: 100, unit: 'GB', _identifier: '100 GB', maxDisksAllow: {numOfDisks: 6, reason: null}},
                {size: 500, unit: 'GB', _identifier: '500 GB', maxDisksAllow: {numOfDisks: 6, reason: null}},
                {size: 1, unit: 'TB', _identifier: '1 TB', maxDisksAllow: {numOfDisks: 4, reason: null}},
                {size: 2, unit: 'TB', _identifier: '2 TB', maxDisksAllow: {numOfDisks: 4, reason: null}},
                {size: 4, unit: 'TB', _identifier: '4 TB', maxDisksAllow: {numOfDisks: 2, reason: null}},
                {size: 8, unit: 'TB', _identifier: '8 TB', maxDisksAllow: {numOfDisks: 1, reason: null}}
            ]
        },
        st1: {
            default: false,
            name: 'st1',
            shortName: 'st1',
            diskName: 'st1',
            supportedSizes: [
                {size: 1, unit: 'TB', _identifier: '1 TB', maxDisksAllow: {numOfDisks: 6, reason: null}},
                {size: 2, unit: 'TB', _identifier: '2 TB', maxDisksAllow: {numOfDisks: 4, reason: null}},
                {size: 4, unit: 'TB', _identifier: '4 TB', maxDisksAllow: {numOfDisks: 0, reason: 'server limit'}},
                {size: 8, unit: 'TB', _identifier: '8 TB', maxDisksAllow: {numOfDisks: 0, reason: 'server limit'}}
            ]
        },
        sc1: {
            default: false,
            name: 'st1',
            shortName: 'st1',
            diskName: 'st1',
            supportedSizes: [
                {size: 4, unit: 'TB', _identifier: '4 TB', maxDisksAllow: {numOfDisks: 0, reason: 'server limit'}},
                {size: 8, unit: 'TB', _identifier: '8 TB', maxDisksAllow: {numOfDisks: 0, reason: 'server limit'}}
            ]
        }
    };

    beforeEach(() => {
        j.$httpBackend.expectGET(/config$/).respond(200, {ebsSizeToPiopsRatio: 50});

        magnifispec.mockWizardStepController('CreateNewAggregateDiskSizeAndIopsStepController', {
        }, {
            possibleTiers: {
                gp2: {
                    default: true,
                    description: 'General purpose SSD volume that balances price and performance.',
                    display: 'General Purpose SSD (gp2)',
                    name: 'gp2',
                    shortName: 'gp2',
                    diskName: 'gp2',
                    supportedInHa: true,
                    supportedSizes: [
                        {size: 100, unit: 'GB', _identifier: '100 GB', maxDisksAllow: {numOfDisks: 6, reason: null}},
                        {size: 500, unit: 'GB', _identifier: '500 GB', maxDisksAllow: {numOfDisks: 6, reason: null}},
                        {size: 1, unit: 'TB', _identifier: '1 TB', maxDisksAllow: {numOfDisks: 4, reason: null}},
                        {size: 2, unit: 'TB', _identifier: '2 TB', maxDisksAllow: {numOfDisks: 4, reason: null}},
                        {size: 4, unit: 'TB', _identifier: '4 TB', maxDisksAllow: {numOfDisks: 2, reason: null}},
                        {size: 8, unit: 'TB', _identifier: '8 TB', maxDisksAllow: {numOfDisks: 1, reason: null}}
                    ]
                },
                io1: {
                    default: true,
                    description: 'For critical applications that require the highest performance at a higher cost.',
                    display: 'Provisioned IOPS SSD (io1)',
                    name: 'io1',
                    shortName: 'io1',
                    diskName: 'io1',
                    supportedInHa: true,
                    supportedSizes: [
                        {size: 100, unit: 'GB', _identifier: '100 GB', maxDisksAllow: {numOfDisks: 6, reason: null}},
                        {size: 500, unit: 'GB', _identifier: '500 GB', maxDisksAllow: {numOfDisks: 6, reason: null}},
                        {size: 1, unit: 'TB', _identifier: '1 TB', maxDisksAllow: {numOfDisks: 4, reason: null}},
                        {size: 2, unit: 'TB', _identifier: '2 TB', maxDisksAllow: {numOfDisks: 4, reason: null}},
                        {size: 4, unit: 'TB', _identifier: '4 TB', maxDisksAllow: {numOfDisks: 2, reason: null}},
                        {size: 8, unit: 'TB', _identifier: '8 TB', maxDisksAllow: {numOfDisks: 1, reason: null}}
                    ]
                },
                st1: {
                    default: false,
                    name: 'st1',
                    shortName: 'st1',
                    diskName: 'st1',
                    supportedSizes: [
                        {size: 1, unit: 'TB', _identifier: '1 TB', maxDisksAllow: {numOfDisks: 6, reason: null}},
                        {size: 2, unit: 'TB', _identifier: '2 TB', maxDisksAllow: {numOfDisks: 4, reason: null}},
                        {size: 4, unit: 'TB', _identifier: '4 TB', maxDisksAllow: {numOfDisks: 0, reason: 'server limit'}},
                        {size: 8, unit: 'TB', _identifier: '8 TB', maxDisksAllow: {numOfDisks: 0, reason: 'server limit'}}
                    ]
                }
            },

            providerVolumeSize: {size: 1, unit: 'TB', _identifier: '1 TB', maxDisksAllow: {numOfDisks: 4, reason: null}},
            selectedTier: {
                display: 'General Purpose SSD (gp2)',
                name: 'gp2',
                diskName: 'gp2',
                supportedSizes: [
                    {size: 100, unit: 'GB', _identifier: '100 GB', maxDisksAllow: {numOfDisks: 6, reason: null}},
                    {size: 500, unit: 'GB', _identifier: '500 GB', maxDisksAllow: {numOfDisks: 6, reason: null}},
                    {size: 1, unit: 'TB', _identifier: '1 TB', maxDisksAllow: {numOfDisks: 4, reason: null}},
                    {size: 2, unit: 'TB', _identifier: '2 TB', maxDisksAllow: {numOfDisks: 4, reason: null}},
                    {size: 4, unit: 'TB', _identifier: '4 TB', maxDisksAllow: {numOfDisks: 2, reason: null}},
                    {size: 8, unit: 'TB', _identifier: '8 TB', maxDisksAllow: {numOfDisks: 1, reason: null}}
                ]
            },
            selectedWorkingEnvironment: {
                publicId: 1,
                name: 'we1',
                _cloudProviderName: 'AWS',
                capacityFeatures: {
                    supportsMixedAggregates: true,
                    providerVolumesType: [],
                    defaultProviderVolumeType: {
                        size: {
                            size: 1099511627776,
                            unit: 'Byte'
                        },
                        diskType: 'gp2'
                    }
                },
                _providerVolumeTypes: providerVolumeTypes
            },
            addAggregateRequest: {}
        });
    });

    describe('createAggregate', () => {
        let aggregatesResourceSpy;

        beforeEach(inject((aggregatesResource) => {
            j.scope.wizardController.exit = jest.fn();
            j.scope.wizardController.isValid = jest.fn().mockReturnValue(true);
            aggregatesResourceSpy = jest.spyOn(aggregatesResource, 'createAggregate').mockReturnValue(j.$q.when({}));

            j.scope.wizardController.addAggregateRequest = {
                workingEnvironmentId: 1,
                name: 'aggrName',
                numberOfDisks: 1
            };
        }));

        it('should be able to send add aggregate request', () => {
            magnifispec.mockConfirm(true);

            j.scope.createAggregate();
            magnifispec.processPromise();

            expect(j.scope.wizardController.wizardCompletedSucssesfully).toEqual(true);
            expect(j.scope.exit).toHaveBeenCalled();
            expect(j.nocDialog.confirm).toHaveBeenCalled();
            expect(aggregatesResourceSpy).toHaveBeenCalledWith(jasmine.any(Object), jasmine.any(Object), {
                name: 'aggrName',
                workingEnvironmentId: 1,
                numberOfDisks: 1,
                diskSize: {
                    size: 1,
                    unit: 'TB',
                    _identifier: '1 TB',
                    maxDisksAllow: {numOfDisks: 4, reason: null}
                },
                providerVolumeType: 'gp2',
                capacityTier: undefined,
                iops: null
            });
        });

        it('should show correct pidgeon', () => {
            magnifispec.mockConfirm(true);

            j.scope.createAggregate();
            magnifispec.processPromise();

            expect(j.pidgeon.inProgress).toHaveBeenCalledWith('Aggregate creation in progress.', true);
        });

        it('should validate the form', () => {
            jest.spyOn(j.nocDialog, 'confirm').mockImplementation(() => {});
            j.scope.wizardController.isValid.mockReturnValue(false);

            j.scope.createAggregate();

            expect(j.nocUI.handleErrorsInWizard).toHaveBeenCalledWith(true);
            expect(j.nocDialog.confirm).not.toHaveBeenCalled();
        });

        it('should not send post request if confirm was false', () => {
            magnifispec.mockConfirm(false);

            j.scope.createAggregate();

            expect(aggregatesResourceSpy).not.toHaveBeenCalled();
            expect(j.nocDialog.confirm).toHaveBeenCalled();
        });

        it('should not send iops if the selected tier disk name is not io1', () => {
            j.scope.wizardController.iops = 1234;
            magnifispec.mockConfirm(true);

            j.scope.createAggregate();
            magnifispec.processPromise();

            expect(j.scope.wizardController.addAggregateRequest.iops).toEqual(null);
        });

        it('should send iops only if the selected tier disk name is io1', () => {
            j.scope.wizardController.selectedTier = {name: 'io1+S3', diskName: 'io1', display: 'Provisioned IOPS SSD (io1) + S3'};
            j.scope.wizardController.iops = 1234;
            magnifispec.mockConfirm(true);

            j.scope.createAggregate();
            magnifispec.processPromise();

            expect(j.scope.wizardController.addAggregateRequest.iops).toEqual(1234);
        });

        it('should create tier with S3 correctly', inject((nocApprovalMessageBuilder) => {
            j.scope.wizardController.provider = 'AWS';
            jest.spyOn(nocApprovalMessageBuilder, 'buildTitle').mockReturnValue('HTML code');

            magnifispec.mockConfirm(true);
            j.scope.wizardController.addAggregateRequest.name = 'aggrWithS3';
            j.scope.wizardController.selectedTier = {name: 'gp2', diskName: 'gp2'};
            j.scope.wizardController.tieringEnabled = true;

            j.scope.createAggregate();
            magnifispec.processPromise();

            expect(j.nocDialog.confirm).toHaveBeenCalledWith('', 'assets/templates/dialogs/purchase-confirm-content.html', {html: 'HTML code'});
            expect(nocApprovalMessageBuilder.buildTitle).toHaveBeenCalledWith({
                provider: 'AWS',
                numOfDisks: 1,
                diskSize: {size: 1, unit: 'TB'},
                diskType: 'General Purpose SSD (gp2)',
                isHA: undefined
            });

            expect(j.scope.addAggregateRequest).toEqual({
                name: 'aggrWithS3',
                workingEnvironmentId: 1,
                numberOfDisks: 1,
                diskSize: jasmine.objectContaining({size: 1, unit: 'TB', _identifier: '1 TB'}),
                providerVolumeType: 'gp2',
                capacityTier: 'S3',
                iops: null
            });
        }));
    });

    describe('initializeController', () => {
        it('should call getConfig and add the ebsSizeToPiopsRatio to the scope', () => {
            jest.spyOn(j.scope, 'onDiskSizeChange').mockImplementation(() => {});

            jest.spyOn(j.configService, 'getConfig').mockReturnValue(j.$q.when({ebsSizeToPiopsRatio: 50}));
            j.scope.initializeController();
            j.$httpBackend.flush();

            expect(j.scope.ebsSizeToPiopsRatio).toEqual(50);
            expect(j.scope.onDiskSizeChange).toHaveBeenCalled();
        });
    });

    describe('onDiskSizeChange', () => {
        let constants;
        beforeEach(inject((Constants) => {
            constants = Constants;
        }));

        it('should convert the providerVolumeSize to GB if in TB', () => {
            j.scope.wizardController.providerVolumeSize = {size: 1, unit: 'TB', maxDisksAllow: {numOfDisks: 5, reason: null}};

            j.scope.onDiskSizeChange();

            expect(j.scope.wizardController.providerVolumeSizeGB).toEqual(1024);
        });

        it('should leave the providerVolumeSize in GB if already in GB', () => {
            j.scope.wizardController.providerVolumeSize = {size: 100, unit: 'GB', maxDisksAllow: {numOfDisks: 5, reason: null}};

            j.scope.onDiskSizeChange();

            expect(j.scope.wizardController.providerVolumeSizeGB).toEqual(100);
        });

        it('should calculate maxIopsValue', () => {
            j.scope.wizardController.providerVolumeSize = {size: 100, unit: 'GB', maxDisksAllow: {numOfDisks: 5, reason: null}};
            j.scope.ebsSizeToPiopsRatio = 50;
            constants.awsIopsLimitPerVolume = 20000;

            j.scope.onDiskSizeChange();

            expect(j.scope.maxIopsValue).toEqual(5000);
        });

        it('should create the range properly and select first in range', () => {
            j.scope.wizardController.providerVolumeSize = {size: 100, unit: 'GB', maxDisksAllow: {numOfDisks: 5, reason: null}};
            j.scope.onDiskSizeChange();

            expect(j.scope.numberOfDisksOptions).toEqual([1, 2, 3, 4, 5]);
            expect(j.scope.wizardController.addAggregateRequest.numberOfDisks).toEqual(1);
        });
    });
});
