describe('CreateNewAggregateTieringStepController', () => {
    'use strict';

    var j;    beforeEach(() => {        j = magnifispec.setup();    });
    const providerVolumeTypes = {
        gp2: {
            default: true,
            description: 'General purpose SSD volume that balances price and performance.',
            display: 'General Purpose SSD (gp2)',
            name: 'gp2',
            shortName: 'gp2',
            diskName: 'gp2',
            supportedInHa: true,
            supportedSizes: [
                {size: 100, unit: 'GB', _identifier: '100 GB', maxDisksAllow: {numOfDisks: 6, reason: null}},
                {size: 500, unit: 'GB', _identifier: '500 GB', maxDisksAllow: {numOfDisks: 6, reason: null}},
                {size: 1, unit: 'TB', _identifier: '1 TB', maxDisksAllow: {numOfDisks: 4, reason: null}},
                {size: 2, unit: 'TB', _identifier: '2 TB', maxDisksAllow: {numOfDisks: 4, reason: null}},
                {size: 4, unit: 'TB', _identifier: '4 TB', maxDisksAllow: {numOfDisks: 2, reason: null}},
                {size: 8, unit: 'TB', _identifier: '8 TB', maxDisksAllow: {numOfDisks: 1, reason: null}}
            ]
        },
        'gp2+S3': {
            default: true,
            description: 'A tiered storage configuration that stores data on SSDs and volume Snapshot copies in an S3 bucket to reduce storage costs.',
            display: 'General Purpose SSD (gp2) and S3',
            name: 'gp2+S3',
            shortName: 'gp2+S3',
            diskName: 'gp2',
            supportedInHa: true,
            supportedSizes: [
                {size: 100, unit: 'GB', _identifier: '100 GB', maxDisksAllow: {numOfDisks: 6, reason: null}},
                {size: 500, unit: 'GB', _identifier: '500 GB', maxDisksAllow: {numOfDisks: 6, reason: null}},
                {size: 1, unit: 'TB', _identifier: '1 TB', maxDisksAllow: {numOfDisks: 4, reason: null}},
                {size: 2, unit: 'TB', _identifier: '2 TB', maxDisksAllow: {numOfDisks: 4, reason: null}},
                {size: 4, unit: 'TB', _identifier: '4 TB', maxDisksAllow: {numOfDisks: 2, reason: null}},
                {size: 8, unit: 'TB', _identifier: '8 TB', maxDisksAllow: {numOfDisks: 1, reason: null}}
            ]
        },
        st1: {
            default: false,
            name: 'st1',
            shortName: 'st1',
            diskName: 'st1',
            supportedSizes: [
                {size: 1, unit: 'TB', _identifier: '1 TB', maxDisksAllow: {numOfDisks: 6, reason: null}},
                {size: 2, unit: 'TB', _identifier: '2 TB', maxDisksAllow: {numOfDisks: 4, reason: null}},
                {size: 4, unit: 'TB', _identifier: '4 TB', maxDisksAllow: {numOfDisks: 0, reason: 'server limit'}},
                {size: 8, unit: 'TB', _identifier: '8 TB', maxDisksAllow: {numOfDisks: 0, reason: 'server limit'}}
            ]
        },
        sc1: {
            default: false,
            name: 'st1',
            shortName: 'st1',
            diskName: 'st1',
            supportedSizes: [
                {size: 4, unit: 'TB', _identifier: '4 TB', maxDisksAllow: {numOfDisks: 0, reason: 'server limit'}},
                {size: 8, unit: 'TB', _identifier: '8 TB', maxDisksAllow: {numOfDisks: 0, reason: 'server limit'}}
            ]
        }
    };

    beforeEach(() => {
        magnifispec.mockWizardStepController('CreateNewAggregateTieringStepController', {}, {
            selectedWorkingEnvironment: {
                publicId: 1,
                name: 'we1',
                _cloudProviderName: 'AWS',
                _objectStorageDisplayName: 'S3',
                capacityFeatures: {
                    supportsMixedAggregates: true,
                    supportedCapacityTiers: {forceCompositeVersion: true},
                    providerVolumesType: [],
                    defaultProviderVolumeType: {
                        size: {
                            size: 1099511627776,
                            unit: 'Byte'
                        },
                        diskType: 'gp2'
                    }
                },
                _providerVolumeTypes: providerVolumeTypes
            },
            addAggregateRequest: {},
            setWizardStep: jest.fn(),
            path: {tiering: 'path.to.tiering.step'}
        });
    });

    describe('initialize controller', () => {
        it('should initialize for the first time', () => {
            j.scope.wizardController.selectedTier = null;
            jest.spyOn(j.scope, 'tierSelected').mockImplementation(() => {});

            j.scope.initializeController();

            expect(j.scope.possibleTiers).toEqual({
                gp2: {
                    default: true,
                    description: 'General purpose SSD volume that balances price and performance.',
                    display: 'General Purpose SSD (gp2)',
                    name: 'gp2',
                    shortName: 'gp2',
                    diskName: 'gp2',
                    supportedInHa: true,
                    supportedSizes: [
                        {size: 100, unit: 'GB', _identifier: '100 GB', maxDisksAllow: {numOfDisks: 6, reason: null}},
                        {size: 500, unit: 'GB', _identifier: '500 GB', maxDisksAllow: {numOfDisks: 6, reason: null}},
                        {size: 1, unit: 'TB', _identifier: '1 TB', maxDisksAllow: {numOfDisks: 4, reason: null}},
                        {size: 2, unit: 'TB', _identifier: '2 TB', maxDisksAllow: {numOfDisks: 4, reason: null}},
                        {size: 4, unit: 'TB', _identifier: '4 TB', maxDisksAllow: {numOfDisks: 2, reason: null}},
                        {size: 8, unit: 'TB', _identifier: '8 TB', maxDisksAllow: {numOfDisks: 1, reason: null}}
                    ]
                },
                'gp2+S3': {
                    default: true,
                    description: 'A tiered storage configuration that stores data on SSDs and volume Snapshot copies in an S3 bucket to reduce storage costs.',
                    display: 'General Purpose SSD (gp2) and S3',
                    name: 'gp2+S3',
                    shortName: 'gp2+S3',
                    diskName: 'gp2',
                    supportedInHa: true,
                    supportedSizes: [
                        {size: 100, unit: 'GB', _identifier: '100 GB', maxDisksAllow: {numOfDisks: 6, reason: null}},
                        {size: 500, unit: 'GB', _identifier: '500 GB', maxDisksAllow: {numOfDisks: 6, reason: null}},
                        {size: 1, unit: 'TB', _identifier: '1 TB', maxDisksAllow: {numOfDisks: 4, reason: null}},
                        {size: 2, unit: 'TB', _identifier: '2 TB', maxDisksAllow: {numOfDisks: 4, reason: null}},
                        {size: 4, unit: 'TB', _identifier: '4 TB', maxDisksAllow: {numOfDisks: 2, reason: null}},
                        {size: 8, unit: 'TB', _identifier: '8 TB', maxDisksAllow: {numOfDisks: 1, reason: null}}
                    ]
                },
                st1: {
                    default: false,
                    name: 'st1',
                    shortName: 'st1',
                    diskName: 'st1',
                    supportedSizes: [
                        {size: 1, unit: 'TB', _identifier: '1 TB', maxDisksAllow: {numOfDisks: 6, reason: null}},
                        {size: 2, unit: 'TB', _identifier: '2 TB', maxDisksAllow: {numOfDisks: 4, reason: null}}
                    ]
                }
            });
            expect(j.scope.tierSelected).toHaveBeenCalledWith(j.scope.possibleTiers['gp2']);
        });

        it('should construct tier correctly in first initialization', () => {
            j.scope.wizardController.selectedTier = null;
            jest.spyOn(j.scope, 'tierSelected').mockImplementation(() => {});
            j.scope.wizardController.selectedWorkingEnvironment.capacityFeatures.defaultProviderVolumeType.capacityTier = 'S3';

            j.scope.initializeController();

            expect(j.scope.tierSelected).toHaveBeenCalledWith(j.scope.possibleTiers['gp2']);
        });

        it('should remember the previous information after initializeController', () => {
            j.scope.wizardController.selectedTier = {name: 'gp2+S3', diskName: 'gp2'};
            j.scope.wizardController.providerVolumeSize = {size: 4, unit: 'TB', _identifier: '4 TB', maxDisksAllow: {numOfDisks: 2, reason: null}};
            j.scope.wizardController.addAggregateRequest.numberOfDisks = 2;

            j.scope.initializeController();

            expect(j.scope.wizardController.providerVolumeSize).toEqual(
                {size: 4, unit: 'TB', _identifier: '4 TB', maxDisksAllow: {numOfDisks: 2, reason: null}}
            );
            expect(j.scope.wizardController.addAggregateRequest.numberOfDisks).toEqual(2);
        });

    });

    describe('disabledTooltipForTier', () => {
        beforeEach(() => {
            j.scope.selectedWorkingEnvironment._isAWS = true;
        });
        it('should return tier disabledMessage if exists', () => {
            expect(j.scope.disabledTooltipForTier({name: 'gp2', disabledMessage: 'reason1'})).toEqual('reason1');
        });
        it('should return "S3 tiering is not available" if selected tier doesnt support capacity tiering', () => {
            expect(j.scope.disabledTooltipForTier({name: 'gp2'})).toEqual('S3 tiering is not available for this disk type.');
        });
        it('should return nothing if disk supports tiering and no disabled message exists', () => {
            expect(j.scope.disabledTooltipForTier({name: 'gp2', supportedCapacityTiers: ['S3']})).not.toBeDefined();
        });
    });

    describe('tierSelected', () => {
        beforeEach(() => {
            j.scope.selectedWorkingEnvironment.capacityFeatures.defaultProviderVolumeType.size = {size: 1, unit: 'TB'};
        });

        it('should set the tier and selectedDiskTypeDisplay', () => {
            j.scope.tierSelected(j.scope.possibleTiers['gp2']);

            expect(j.scope.wizardController.selectedTier).toEqual(j.scope.possibleTiers['gp2']);
            expect(j.scope.wizardController.setWizardStep).toHaveBeenCalledWith('path.to.tiering.step');
        });

        it('should initialize disk size to working environment default if found', () => {
            const tier = {
                supportedSizes: [
                    {size: 500, unit: 'GB', _identifier: '500 GB', maxDisksAllow: {numOfDisks: 6, reason: null}},
                    {size: 1, unit: 'TB', _identifier: '1 TB', maxDisksAllow: {numOfDisks: 6, reason: null}},
                    {size: 2, unit: 'TB', _identifier: '2 TB', maxDisksAllow: {numOfDisks: 4, reason: null}}
                ]
            };

            j.scope.tierSelected(tier);

            expect(j.scope.wizardController.setWizardStep).toHaveBeenCalledWith('path.to.tiering.step');
            expect(j.scope.wizardController.providerVolumeSize).toEqual(
                {size: 1, unit: 'TB', _identifier: '1 TB', maxDisksAllow: {numOfDisks: 6, reason: null}}
            );
        });

        it('should initialize disk size to first in list if working environment default is not found', () => {
            const tier = {
                supportedSizes: [
                    {size: 2, unit: 'TB', maxDisksAllow: 1, _identifier: '2 TB'},
                    {size: 4, unit: 'TB', maxDisksAllow: 1, _identifier: '4 TB'},
                    {size: 8, unit: 'TB', maxDisksAllow: 1, _identifier: '8 TB'}
                ]
            };

            j.scope.tierSelected(tier);

            expect(j.scope.wizardController.providerVolumeSize).toEqual(
                {size: 2, unit: 'TB', maxDisksAllow: 1, _identifier: '2 TB'}
            );
        });
    });
});
