describe('CreateNewVolumeDetailsStepController', () => {
    'use strict';

    var j;    beforeEach(() => {        j = magnifispec.setup();    });
    let usageProfiles;
    let getWorkingEnvironmentByPublicIdSpy;

    beforeEach(inject((_usageProfiles_) => {
        getWorkingEnvironmentByPublicIdSpy = jest.spyOn(j.workingEnvironmentResource, 'getWorkingEnvironmentByPublicId').mockReturnValue(j.$q.when({
                tenantId: 1,
                publicId: '1',
                name: 'we1',
                workingEnvironmentType: 'VSA',
                _cloudProviderName: 'AWS',
                _typeAndProvider: 'VSA_AWS',
                providerVolumeType: 'gp2',
                _pathPrefix: '/vsa/',
                awsProperties: {vpc: {cidrBlock: '1.1.1.1/24'}},
                capacityFeatures: {
                    defaultProviderVolumeType: {diskType: 'gp2'},
                    supportedCapacityTiers: {
                        capacityTiersDisableReasons: [],
                        compositeSupported: true,
                        forceCompositeVersion: true
                    }
                },
                _providerVolumeTypes: {
                    'gp2': {
                        name: 'gp2',
                        shortName: 'gp2',
                        diskName: 'gp2',
                        display: 'General Purpose SSD (gp2)',
                        description: 'General purpose SSD volume that balances price and performance.',
                        supportedInHa: true,
                        'default': true
                    }
                }
            }));
        j.$httpBackend.expectGET(j.nocBaseUrl + '/vsa/working-environments/1?fields=snapshotPolicies').respond({snapshotPolicies: 'policies'});

        magnifispec.mockWizardStepController('CreateNewVolumeDetailsStepController', {}, {});
        j.$httpBackend.flush();

        usageProfiles = _usageProfiles_;
    }));

    describe('next', () => {
        beforeEach(() => {
            j.scope.isValid = jest.fn().mockReturnValue(true);
            j.scope.wizardController.next = jest.fn();
        });

        it('should validate the wizard', () => {
            j.scope.isValid.mockReturnValue(false);

            j.scope.next();

            expect(j.nocUI.handleErrorsInWizard).toHaveBeenCalledWith(true);
        });

        it('should call create the volume for singleStepVolumeWizard', () => {
            jest.spyOn(j.scope, 'startCreateVolumeProcess').mockImplementation(() => {});
            j.scope.singleStepVolumeWizard = true;

            j.scope.next();

            expect(j.scope.startCreateVolumeProcess).toHaveBeenCalled();
        });

        it('should continue to CIFS step if needed', () => {
            j.scope.needCifsConfiguration = true;

            j.scope.next();

            expect(j.scope.wizardController.next).toHaveBeenCalledWith('.cifs');
        });

        it('should continue to CIFS step if needed, even for singleStepVolumeWizard condition', () => {
            j.scope.singleStepVolumeWizard = true;
            j.scope.needCifsConfiguration = true;

            j.scope.next();

            expect(j.scope.wizardController.next).toHaveBeenCalledWith('.cifs');
        });

        it('should continue to usage profile step if needed', () => {
            j.scope.needCifsConfiguration = false;

            j.scope.next();

            expect(j.scope.wizardController.next).toHaveBeenCalledWith('.usageProfile');
        });
    });

    describe('startCreateVolumeProcess', () => {
        it('should call appropriate function for VSA', () => {
            jest.spyOn(j.scope, 'createVolumeOnVsa').mockImplementation(() => {});
            j.scope.wizardController.selectedWorkingEnvironment.workingEnvironmentType = 'VSA';

            j.scope.startCreateVolumeProcess();

            expect(j.scope.createVolumeOnVsa).toHaveBeenCalled();
        });

        it('should call appropriate function for ON_PREM', () => {
            jest.spyOn(j.scope, 'createNewVolume').mockImplementation(() => {});
            j.scope.wizardController.selectedWorkingEnvironment.workingEnvironmentType = 'ON_PREM';

            j.scope.startCreateVolumeProcess();

            expect(j.scope.createNewVolume).toHaveBeenCalled();
        });
    });

    describe('createVolumeOnVsa', () => {
        beforeEach(() => {
            j.scope.volumeProtocol = {type: 'NFS'};
            j.scope.wizardController.volumeRequest.name = 'vol1';
            jest.spyOn(j.scope, 'createNewVolume').mockImplementation(() => {});
            j.scope.wizardController.selectedTier = {name: 'gp2', diskName: 'gp2', display: 'General Purpose SSD (gp2)'};
        });

        it('should update volumeRequest with selected tier', () => {
            j.scope.createVolumeOnVsa();

            expect(j.scope.wizardController.volumeRequest.providerVolumeType).toEqual('gp2');
        });

        it('should update volumeRequest with appropriate disk type for tier', () => {
            j.scope.wizardController.selectedTier = {name: 'gp2+S3', diskName: 'gp2', display: 'General Purpose SSD (gp2) + S3'};

            j.scope.createVolumeOnVsa();

            expect(j.scope.wizardController.volumeRequest.providerVolumeType).toEqual('gp2');
        });

        it('should ask for quote for the volume and do nothing if quote fails', () => {
            jest.spyOn(j.nocDialog, 'confirm').mockImplementation(() => {});
            j.$httpBackend.expectPOST(j.nocBaseUrl + '/vsa/volumes/quote').respond(400, {numOfDisks: 4});

            j.scope.createVolumeOnVsa();
            j.$httpBackend.flush();

            expect(j.scope.createNewVolume).not.toHaveBeenCalled();
            expect(j.nocDialog.confirm).not.toHaveBeenCalled();
        });

        it('it should ask for quote and create the volume in automatic mode', () => {
            jest.spyOn(j.nocDialog, 'confirm').mockImplementation(() => {});
            const quoteResponse = {
                numOfDisks: 4,
                autoVsaCapacityManagement: true
            };
            j.$httpBackend.expectPOST(j.nocBaseUrl + '/vsa/volumes/quote').respond(200, quoteResponse);

            j.scope.createVolumeOnVsa();
            j.$httpBackend.flush();

            expect(j.scope.createNewVolume).toHaveBeenCalledWith(quoteResponse);
            expect(j.nocDialog.confirm).not.toHaveBeenCalled();
        });

        it('should not send iops if the selected tier disk name is not io1', () => {
            j.scope.wizardController.iops = 1234;

            j.scope.createVolumeOnVsa();

            expect(j.scope.wizardController.volumeRequest.iops).toEqual(null);
        });

        it('should send iops only if the selected tier disk name is io1', () => {
            j.scope.wizardController.selectedTier = {name: 'io1+S3', diskName: 'io1', display: 'Provisioned IOPS SSD (io1) + S3'};
            j.scope.wizardController.iops = 1234;

            j.scope.createVolumeOnVsa();

            expect(j.scope.wizardController.volumeRequest.iops).toEqual(1234);
        });

        describe('should ask for quote, then ask for approval if needed', () => {
            const quoteResponse = {numOfDisks: 4};
            let askForQuoteSpy, getUserApprovalForNewVolumeIfNeededSpy;

            beforeEach(inject((volumesResource) => {
                j.scope.wizardController.selectedDiskTypeDisplay = j.scope.wizardController.selectedTier.display;
                askForQuoteSpy = jest.spyOn(volumesResource, 'askForQuote').mockReturnValue(j.$q.when(quoteResponse));
                getUserApprovalForNewVolumeIfNeededSpy = jest.spyOn(volumesResource, 'getUserApprovalForNewVolumeIfNeeded').mockImplementation((...args) => j.$q.when(args[0]));
            }));

            it('and create volume upon confirm', () => {
                j.scope.createVolumeOnVsa();
                magnifispec.processPromise();

                expect(askForQuoteSpy).toHaveBeenCalledWith(jasmine.any(Object), '/vsa/', {
                    name: 'vol1',
                    workingEnvironmentId: '1',
                    svmName: undefined,
                    exportPolicyInfo: {policyType: 'custom', ips: ['1.1.1.1/24'], _ips: '1.1.1.1/24'},
                    snapshotPolicyName: undefined,
                    iops: null,
                    providerVolumeType: 'gp2',
                    capacityTier: undefined,
                    tieringPolicy: undefined,
                    verifyNameUniqueness: true,
                    enableThinProvisioning: true,
                    enableDeduplication: true,
                    enableCompression: true
                }, 'creating volume');
                expect(getUserApprovalForNewVolumeIfNeededSpy).toHaveBeenCalledWith(quoteResponse, 'AWS', 'General Purpose SSD (gp2)', undefined, 'vol1');
                expect(j.scope.createNewVolume).toHaveBeenCalledWith(quoteResponse);
            });
        });
    });

    describe('createNewVolume', () => {
        beforeEach(() => {
            j.scope.wizardController.exit = jest.fn();
            jest.spyOn(j.scope, 'generateRequestBody').mockImplementation((quoteResponse) => {
                return 'generated request for quote: ' + quoteResponse;
            });
        });

        it('case for automatic mode VSA', () => {
            j.$httpBackend.expect('POST', j.nocBaseUrl + '/vsa/volumes?createAggregateIfNotFound=true', (data) => {
                expect(data).toEqual('generated request for quote: quote for VSA');

                return true;
            }).respond(200);

            j.scope.createNewVolume('quote for VSA');
            j.$httpBackend.flush();

            expect(j.scope.generateRequestBody).toHaveBeenCalledWith('quote for VSA', 'NFS');
        });

        it('case for manual mode VSA (create on aggregate)', () => {
            j.$state.params.aggregateName = 'aggr2';
            j.$httpBackend.expect('POST', j.nocBaseUrl + '/vsa/volumes', (data) => {
                expect(data).toEqual('generated request for quote: quote for VSA on aggr');

                return true;
            }).respond(200);

            j.scope.createNewVolume('quote for VSA on aggr');
            j.$httpBackend.flush();

            expect(j.scope.generateRequestBody).toHaveBeenCalledWith('quote for VSA on aggr', 'NFS');
        });

        it('case for ON_PREM', () => {
            j.scope.wizardController.selectedWorkingEnvironment.workingEnvironmentType = 'ON_PREM';
            j.$httpBackend.expect('POST', j.nocBaseUrl + '/vsa/volumes', (data) => {
                expect(data).toEqual('generated request for quote: undefined');

                return true;
            }).respond(200);

            j.scope.createNewVolume();
            j.$httpBackend.flush();

            expect(j.scope.generateRequestBody).toHaveBeenCalledWith(undefined, 'NFS');
        });

        afterEach(() => {
            expect(j.scope.wizardController.wizardCompletedSucssesfully).toEqual(true);
            expect(j.scope.wizardController.exit).toHaveBeenCalled();
        });
    });

    describe('getSelectedWorkingEnvironment', () => {
        it('should get the correct working environment from the server', () => {
            j.nocUI.loading.mockClear();
            j.nocUI.unloading.mockClear();
            getWorkingEnvironmentByPublicIdSpy.mockReturnValue(j.$q.when('we'));
            j.$state.params.type = 'VSA';
            j.$state.params.provider = 'AWS';

            expect(j.scope.getSelectedWorkingEnvironment('id1')).toBeResolved();

            expect(j.scope.wizardController.selectedWorkingEnvironment).toEqual('we');
            expect(getWorkingEnvironmentByPublicIdSpy).toHaveBeenCalledWith(jasmine.any(Object), 'id1', '/vsa/', 'svmName,capacityFeatures,cloudSyncProperties,ontapClusterProperties,awsProperties');
            expect(j.nocUI.loading.mock.calls.length).toEqual(1);
            expect(j.nocUI.unloading.mock.calls.length).toEqual(1);
        });
    });

    describe('syncToS3StatusForAddNewVolume', () => {
        const workingEnvironment = {};

        it('should return "ENABLED" if synced and data broker status is COMPLETE', () => {
            workingEnvironment.cloudSyncProperties = {
                isSynced: true,
                status: {status: 'COMPLETE'}
            };

            expect(j.scope.syncToS3StatusForAddNewVolume(workingEnvironment)).toEqual('ENABLED');
        });

        it('should return "DISABLED" if data broker status is not COMPLETE', () => {
            workingEnvironment.cloudSyncProperties = {
                isSynced: true,
                status: {status: 'FAILED'}
            };

            expect(j.scope.syncToS3StatusForAddNewVolume(workingEnvironment)).toEqual('DISABLED');
        });

        it('should return "NOT_AVAILABLE" if we is not synced to s3', () => {
            workingEnvironment.cloudSyncProperties = {
                isSynced: false
            };

            expect(j.scope.syncToS3StatusForAddNewVolume(workingEnvironment)).toEqual('NOT_AVAILABLE');
        });
    });

    describe('initializeAdvancedAllocation', () => {
        beforeEach(inject((nocAdvancedAllocationService) => {
            jest.spyOn(nocAdvancedAllocationService, 'getAggregates').mockReturnValue(j.$q.when([{name: 'aggr1'}, {name: 'aggr2'}]));
            jest.spyOn(nocAdvancedAllocationService, 'getSVMs').mockReturnValue(j.$q.when());
            j.scope.destinationSvms = [{name: 'svm1'}, {name: 'svm2'}];
        }));

        it('should initialize aggregates and aggregate name', () => {
            expect(j.scope.initializeAdvancedAllocation()).toBeResolved();

            expect(j.scope.aggregates).toEqual([{name: 'aggr1'}, {name: 'aggr2'}]);
            expect(j.scope.wizardController.volumeRequest.aggregateName).toEqual('aggr1');
        });

        it('should initialize svm name if only one exists', () => {
            j.scope.destinationSvms = [{name: 'svm1'}];

            expect(j.scope.initializeAdvancedAllocation()).toBeResolved();

            expect(j.scope.wizardController.volumeRequest.svmName).toEqual('svm1');
        });

        it('should not initialize svm name if more than one exist', () => {
            expect(j.scope.initializeAdvancedAllocation()).toBeResolved();

            expect(j.scope.wizardController.volumeRequest.svmName).not.toBeDefined();
        });
    });

    describe('initialize controller', () => {
        beforeEach(() => {
            j.scope.wizardController.selectedWorkingEnvironment = {
                _pathPrefix: '/vsa/',
                publicId: 1,
                name: 'we1',
                svmName: 'svm1',
                awsProperties: {vpc: {cidrBlock: '1.1.1.1/24'}},
                capacityFeatures: {
                    defaultProviderVolumeType: {diskType: 'gp2'},
                    supportedCapacityTiers: {
                        capacityTiersDisableReasons: [],
                        compositeSupported: true,
                        forceCompositeVersion: true
                    }
                },
                _providerVolumeTypes: {
                    'gp2': {
                        name: 'gp2',
                        shortName: 'gp2',
                        diskName: 'gp2',
                        display: 'General Purpose SSD (gp2)',
                        description: 'General purpose SSD volume that balances price and performance.',
                        supportedInHa: true,
                        supportedCapacityTiers: ['S3'],
                        'default': true
                    },
                    'st1': {
                        name: 'st1',
                        shortName: 'st1',
                        diskName: 'st1',
                        display: 'Throughput Optimized HDD (st1)',
                        description: 'Low cost HDD volume designed for frequently accessed',
                        supportedInHa: true,
                        supportedCapacityTiers: ['S3']
                    }
                }
            };
            j.scope.volumeProtocol = 'NFS';
            j.scope.wizardController.title = 'base wizard title';
        });

        it('should build the right add request for ON PREM', () => {
            j.scope.wizardController.selectedWorkingEnvironment.workingEnvironmentType = 'ON_PREM';
            j.scope.wizardController.selectedWorkingEnvironment.capacityFeatures = null;

            j.scope.initializeController();

            expect(j.scope.singleStepVolumeWizard).toEqual(true);
            expect(j.scope.volumeRequest).toEqual({
                workingEnvironmentId: 1,
                svmName: 'svm1',
                usageProfile: usageProfiles.default,
                exportPolicyInfo: {policyType: 'none', ips: [], _ips: ''},
                shareInfo: {
                    accessControl: {_users: 'Everyone;', users: ['Everyone'], permission: 'full_control'},
                    shareName: ''
                }
            });
        });

        it('should build the right add request for VSA_AWS', () => {
            j.scope.wizardController.selectedWorkingEnvironment.workingEnvironmentType = 'VSA';
            j.scope.wizardController.selectedWorkingEnvironment._typeAndProvider = 'VSA_AWS';

            j.scope.initializeController();

            expect(j.scope.wizardController.selectedTier).toEqual({
                name: 'gp2',
                shortName: 'gp2',
                diskName: 'gp2',
                display: 'General Purpose SSD (gp2)',
                description: 'General purpose SSD volume that balances price and performance.',
                supportedInHa: true,
                supportedCapacityTiers: ['S3'],
                default: true
            });
            expect(j.scope.wizardController.tieringEnabled).toEqual(true);
            expect(j.scope.wizardController.selectedDiskTypeDisplay).toEqual('General Purpose SSD (gp2)');
            expect(j.scope.singleStepVolumeWizard).toEqual(false);
            expect(j.scope.volumeRequest).toEqual({
                workingEnvironmentId: 1,
                svmName: 'svm1',
                usageProfile: usageProfiles.default,
                exportPolicyInfo: {policyType: 'custom', ips: ['1.1.1.1/24'], _ips: '1.1.1.1/24'},
                shareInfo: {
                    accessControl: {_users: 'Everyone;', users: ['Everyone'], permission: 'full_control'},
                    shareName: ''
                }
            });
        });

        it('should init correctly for VSA_AWS with tiering', () => {
            j.scope.wizardController.selectedWorkingEnvironment.workingEnvironmentType = 'VSA';
            j.scope.wizardController.selectedWorkingEnvironment.capacityFeatures.defaultProviderVolumeType = {
                size: {size: 1, unit: 'TB'},
                diskType: 'gp2',
                capacityTier: 'S3'
            };
            j.scope.initializeController();

            expect(j.scope.wizardController.selectedTier).toEqual({
                name: 'gp2',
                shortName: 'gp2',
                diskName: 'gp2',
                display: 'General Purpose SSD (gp2)',
                description: 'General purpose SSD volume that balances price and performance.',
                supportedInHa: true,
                supportedCapacityTiers: ['S3'],
                default: true
            });
            expect(j.scope.wizardController.selectedDiskTypeDisplay).toEqual('General Purpose SSD (gp2)');
            expect(j.scope.wizardController.tieringEnabled).toEqual(true);
        });

        it('should build the right add request for VSA_Azure', () => {
            j.scope.wizardController.selectedWorkingEnvironment.workingEnvironmentType = 'VSA';
            j.scope.wizardController.selectedWorkingEnvironment._typeAndProvider = 'VSA_Azure';
            j.scope.wizardController.selectedWorkingEnvironment._cloudProviderName = 'Azure';
            j.scope.wizardController.selectedWorkingEnvironment.providerProperties = {vnetCidr: '1.2.3.4/66'};
            j.scope.wizardController.selectedWorkingEnvironment.capacityFeatures.defaultProviderVolumeType.diskType = 'Premium_LRS';
            j.scope.wizardController.selectedWorkingEnvironment._providerVolumeTypes = {
                'Premium_LRS': {
                    name: 'Premium_LRS',
                    shortName: 'Prm',
                    diskName: 'Premium_LRS',
                    display: 'Premium Storage (SSD)',
                    description: 'Higher performance at a higher cost.',
                    'default': true
                }
            };

            j.scope.initializeController();

            expect(j.scope.singleStepVolumeWizard).toEqual(false);
            expect(j.scope.volumeRequest).toEqual({
                workingEnvironmentId: 1,
                svmName: 'svm1',
                usageProfile: usageProfiles.default,
                exportPolicyInfo: {policyType: 'custom', ips: ['1.2.3.4/66'], _ips: '1.2.3.4/66'},
                shareInfo: {
                    accessControl: {_users: 'Everyone;', users: ['Everyone'], permission: 'full_control'},
                    shareName: ''
                }
            });
        });

        it('should call initialize advanced allocation for ON_PREM', () => {
            jest.spyOn(j.scope, 'initializeAdvancedAllocation').mockReturnValue(j.$q.when());
            j.scope.wizardController.selectedWorkingEnvironment.workingEnvironmentType = 'ON_PREM';
            j.scope.wizardController.selectedWorkingEnvironment._typeAndProvider = 'ON_PREM';

            j.scope.initializeController();

            expect(j.scope.initializeAdvancedAllocation).toHaveBeenCalled();
        });

        it('should build the model for create volume on aggregate', () => {
            j.$state.params.aggregateName = 'aggr1';
            j.$state.params.providerVolumeType = 'gp2';
            j.$state.params.iops = '999';

            j.scope.initializeController();

            expect(j.scope.wizardController.selectedTier).toEqual({
                name: 'gp2',
                shortName: 'gp2',
                diskName: 'gp2',
                display: 'General Purpose SSD (gp2)',
                description: 'General purpose SSD volume that balances price and performance.',
                supportedInHa: true,
                supportedCapacityTiers: ['S3'],
                default: true
            });
            expect(j.scope.wizardController.title).toEqual('base wizard title on aggr1');
            expect(j.scope.volumeRequest.aggregateName).toEqual('aggr1');
            expect(j.scope.wizardController.iops).toEqual(999);
            expect(j.scope.singleStepVolumeWizard).toBeFalsy();
        });

        it('should disable "No Storage Efficiency" in case of creating volume on aggregate whose tier is +S3', () => {
            j.$state.params.aggregateName = 'aggr1';
            j.$state.params.providerVolumeType = 'gp2';
            j.$state.params.capacityTier = 'S3';

            j.scope.initializeController();

            expect(j.scope.disabledUsageProfile).toEqual('No Storage Efficiency');
        });

        describe('syncToS3', () => {
            it('should mark the checkbox for synced VSA', () => {
                j.scope.wizardController.selectedWorkingEnvironment.workingEnvironmentType = 'VSA';
                j.$httpBackend.expectGET(j.nocBaseUrl + '/vsa/working-environments/1?fields=snapshotPolicies').respond({snapshotPolicies: 'policies'});
                jest.spyOn(j.scope, 'syncToS3StatusForAddNewVolume').mockReturnValue('ENABLED');

                j.scope.initializeController();
                magnifispec.processPromise();

                expect(j.scope.syncToS3Status).toEqual('ENABLED');
                expect(j.scope.volumeRequest.syncToS3).toEqual(true);
            });

            it('should not mark the checkbox for unsynced VSA', () => {
                j.scope.wizardController.selectedWorkingEnvironment.workingEnvironmentType = 'VSA';
                j.$httpBackend.expectGET(j.nocBaseUrl + '/vsa/working-environments/1?fields=snapshotPolicies').respond({snapshotPolicies: 'policies'});
                jest.spyOn(j.scope, 'syncToS3StatusForAddNewVolume').mockReturnValue('NOT_AVAILABLE');

                j.scope.initializeController();
                magnifispec.processPromise();

                expect(j.scope.syncToS3Status).toEqual('NOT_AVAILABLE');
                expect(j.scope.volumeRequest.syncToS3).toBeFalsy();
            });

            it('should not mark the checkbox for synced ON_PREM', () => {
                j.scope.wizardController.selectedWorkingEnvironment.workingEnvironmentType = 'ON_PREM';
                j.scope.wizardController.selectedWorkingEnvironment._pathPrefix = 'onprem/';
                jest.spyOn(j.scope, 'initializeAdvancedAllocation').mockImplementation(() => {});
                j.$httpBackend.expectGET(j.nocBaseUrl + '/onprem/working-environments/1?fields=snapshotPolicies').respond({snapshotPolicies: 'policies'});
                jest.spyOn(j.scope, 'syncToS3StatusForAddNewVolume').mockReturnValue('ENABLED');

                j.scope.initializeController();
                magnifispec.processPromise();

                expect(j.scope.syncToS3Status).toEqual('NO_ACCESS');
                expect(j.scope.volumeRequest.syncToS3).toEqual(false);
            });

            it('should not mark the checkbox for unsynced ON_PREM', () => {
                j.scope.wizardController.selectedWorkingEnvironment.workingEnvironmentType = 'ON_PREM';
                j.scope.wizardController.selectedWorkingEnvironment._pathPrefix = 'onprem/';
                jest.spyOn(j.scope, 'initializeAdvancedAllocation').mockImplementation(() => {});
                j.$httpBackend.expectGET(j.nocBaseUrl + '/onprem/working-environments/1?fields=snapshotPolicies').respond({snapshotPolicies: 'policies'});
                jest.spyOn(j.scope, 'syncToS3StatusForAddNewVolume').mockReturnValue('NOT_AVAILABLE');

                j.scope.initializeController();
                magnifispec.processPromise();

                expect(j.scope.syncToS3Status).toEqual('NOT_AVAILABLE');
                expect(j.scope.volumeRequest.syncToS3).toBeFalsy();
            });
        });
    });
});
