describe.skip('setup wizard steps', () => {
    const j = magnifispec.setup(),
        routePrefix = '/occm/api',
        validateRoute = routePrefix + '/occm/setup/validate',
        ourHeaders = {
            'Accept': 'application/json',
            'Content-Type': 'application/json;charset=utf-8',
            'If-Modified-Since': '0',
            'Cache-Control': 'no-cache',
            'Pragma': 'no-cache'
        };

    describe('SetupWizardRestorePreviousController', () => {
        beforeEach(() => {
            magnifispec.mockWizardStepController('SetupWizardRestorePreviousController', {}, {wizardController: {}});
        });

        it('should call API and act upon success', () => {
            j.$httpBackend.expectPOST(j.nocBaseUrl + '/occm/setup/restore').respond(200, {});
            j.$httpBackend.expectPOST(j.nocBaseUrl + '/auth/login').respond(200);
            j.scope.isValid = jest.fn().mockReturnValue(true);

            j.scope.restore();
            j.$httpBackend.flush();

            expect(j.nocUI.reloadSilently).toHaveBeenCalledWith(true);
        });

        it('should handle errors in wizard', () => {
            j.scope.isValid = jest.fn().mockReturnValue(false);

            j.scope.restore();

            expect(j.nocUI.handleErrorsInWizard).toHaveBeenCalled();
        });
    });

    describe('proxy step', () => {
        beforeEach(() => {
            magnifispec.mockWizardStepController('SetupWizardProxyController', {}, {setupRequest: {proxyUrl: {}}, wizardController: {}});
            j.$state.go = jest.fn();
        });

        describe('not valid cases', () => {
            it('should not advance if form not valid', () => {
                j.scope.isValid = jest.fn().mockReturnValue(false);

                j.scope.next();

                expect(j.scope.isValid).toHaveBeenCalled();
                expect(j.nocUI.handleErrorsInWizard).toHaveBeenCalled();
            });

            it('should validate through the server', () => {
                j.scope.isValid = jest.fn().mockReturnValue(true);
                j.scope.setupRequest.proxyUrl.uri = 'http://bla';
                j.$httpBackend.expectPOST(validateRoute + '/proxy', j.scope.setupRequest.proxyUrl).respond(400, j.errorResponse);

                j.scope.next();
                expect(j.nocUI.block).toHaveBeenCalled();
                j.$httpBackend.flush();

                expect(j.pidgeon.error).toHaveBeenCalled();
                expect(j.nocUI.unblock).toHaveBeenCalled();
            });
        });

        describe('valid cases', () => {
            const proxyNextStepName = '.company';

            it('should just go if empty uri', () => {
                j.scope.isValid = jest.fn().mockReturnValue(true);
                j.scope.setupRequest.proxyUrl.uri = '';

                j.scope.next();

                j.$httpBackend.expectPOST(validateRoute + '/connectivity').respond(204);

                expect(j.nocUI.block).toHaveBeenCalled();
                j.$httpBackend.flush();

                expect(j.nocUI.unblock).toHaveBeenCalled();
                expect(j.scope.isValid).toHaveBeenCalled();
                expect(j.nocUI.clearFormValidity).toHaveBeenCalled();
                expect(j.$state.go).toHaveBeenCalledWith(proxyNextStepName);
            });

            it('validate through server if not empty uri', () => {
                j.scope.isValid = jest.fn().mockReturnValue(true);
                j.scope.setupRequest.proxyUrl.uri = 'http://bla';
                j.$httpBackend.expectPOST(validateRoute + '/proxy', j.scope.setupRequest.proxyUrl).respond(200);

                j.scope.next();
                expect(j.nocUI.block).toHaveBeenCalled();
                j.$httpBackend.flush();

                expect(j.$state.go).toHaveBeenCalledWith(proxyNextStepName);
                expect(j.nocUI.unblock).toHaveBeenCalled();
            });
        });
    });

    describe('simple step', () => {
        beforeEach(() => {
            magnifispec.mockWizardStepController('SetupWizardSimpleController', {}, {wizardController: {}});
        });
    });

    describe('tenant final step', () => {
        const setupRequest = {
            proxyUrl: {},
            userRequest: {
                firstName: 'Abraham',
                lastName: 'Lincoln',
                email: 'bla@bla.com',
                password: '1234'
            },
            tenantRequest: {
                name: 'Washington',
                description: 'Washington'
            }
        };

        beforeEach(() => {
            magnifispec.mockWizardStepController('SetupWizardReviewController', {}, {setupRequest, wizardController: {}});
            j.$state.go = jest.fn();
        });

        describe('providers should return the right string', () => {
            beforeEach(() => {
                j.scope.setupRequest = {
                    userRequest: {
                    }
                };
            });

            it('for case where none', () => {
                expect(j.scope.providers()).toEqual('N/A');
            });
        });

        describe('not valid cases', () => {
            it('should not advance if form not valid', () => {
                j.scope.isValid = jest.fn().mockReturnValue(false);

                j.scope.finish();

                expect(j.scope.isValid).toHaveBeenCalled();
                expect(j.nocUI.handleErrorsInWizard).toHaveBeenCalled();
            });

            describe('cases when form is valid but problems occur in http', () => {
                let AuthResource;
                beforeEach(inject((_AuthResource_) => {
                    AuthResource = _AuthResource_;
                    j.scope.isValid = jest.fn().mockReturnValue(true);
                    j.scope.setupRequest.proxyUrl.uri = 'http://bla';
                    ourHeaders['Setup-Proxy'] = j.scope.setupRequest.proxyUrl.uri;
                    j.scope.finish();
                    expect(j.pidgeon.clear).toHaveBeenCalledWith('finish');
                }));

                it('should handle validate through server and fail setup', () => {
                    jest.spyOn(AuthResource, 'login').mockImplementation(() => {});
                    j.$httpBackend.expectPOST(validateRoute + '/tenant', j.scope.setupRequest.tenantRequest, ourHeaders).respond(200);
                    j.$httpBackend.expectPOST(validateRoute + '/user', j.scope.setupRequest.userRequest, ourHeaders).respond(200);
                    j.$httpBackend.expectPOST(validateRoute + '/proxy', j.scope.setupRequest.proxyUrl).respond(200);
                    j.$httpBackend.expectPOST(routePrefix + '/occm/setup/init', j.scope.setupRequest).respond(400, j.errorResponse);
                    j.$httpBackend.flush();
                    expect(j.pidgeon.error).toHaveBeenCalledWith(j.pidgeon.serverErrorPrefix + 'setting up Cloud Manager', j.errorResponse, 'setting up Cloud Manager');
                    expect(AuthResource.login).not.toHaveBeenCalled();
                });

                it('should handle validate through server and fail login ', () => {
                    j.$httpBackend.expectPOST(validateRoute + '/tenant', j.scope.setupRequest.tenantRequest, ourHeaders).respond(200);
                    j.$httpBackend.expectPOST(validateRoute + '/user', j.scope.setupRequest.userRequest, ourHeaders).respond(200);
                    j.$httpBackend.expectPOST(validateRoute + '/proxy', j.scope.setupRequest.proxyUrl).respond(200);
                    j.$httpBackend.expectPOST(routePrefix + '/occm/setup/init', j.scope.setupRequest).respond(200, {executed: false});
                    j.$httpBackend.expectPOST(routePrefix + '/auth/login', {
                        email: j.scope.setupRequest.userRequest.email,
                        password: j.scope.setupRequest.userRequest.password
                    }).respond(400, j.errorResponse);
                    j.$httpBackend.flush();
                    expect(j.pidgeon.error).toHaveBeenCalledWith(j.pidgeon.serverErrorPrefix + 'logging in', j.errorResponse, 'finish');
                });
            });
        });

        describe('valid cases', () => {
            it('validate through server and go', () => {
                j.scope.isValid = jest.fn().mockReturnValue(true);
                j.scope.setupRequest.proxyUrl.uri = 'http://bla';
                ourHeaders['Setup-Proxy'] = j.scope.setupRequest.proxyUrl.uri;
                j.$httpBackend.expectPOST(validateRoute + '/tenant', j.scope.setupRequest.tenantRequest, ourHeaders).respond(200);
                j.$httpBackend.expectPOST(validateRoute + '/user', j.scope.setupRequest.userRequest, ourHeaders).respond(200);
                j.$httpBackend.expectPOST(validateRoute + '/proxy', j.scope.setupRequest.proxyUrl).respond(200);
                j.$httpBackend.expectPOST(routePrefix + '/occm/setup/init', j.scope.setupRequest).respond(200, {executed: false});
                j.$httpBackend.expectPOST(routePrefix + '/auth/login', {
                    email: j.scope.setupRequest.userRequest.email,
                    password: j.scope.setupRequest.userRequest.password
                }).respond(200);

                j.scope.finish();
                expect(j.pidgeon.clear).toHaveBeenCalledWith('finish');
                j.$httpBackend.flush();

                expect(j.nocUI.block.mock.calls.length).toEqual(5);
                expect(j.nocUI.unblock.mock.calls.length).toEqual(5);
                expect(j.nocUI.reloadSilently).toHaveBeenCalledWith(true);
            });
        });

        it('should poll the about if OCCM is upgrading in the background', () => {
            j.scope.isValid = jest.fn().mockReturnValue(true);
            j.scope.setupRequest.proxyUrl.uri = 'http://bla';
            j.$rootScope.about = {buildTimestamp: 555555555};
            ourHeaders['Setup-Proxy'] = j.scope.setupRequest.proxyUrl.uri;
            j.$httpBackend.expectPOST(validateRoute + '/tenant', j.scope.setupRequest.tenantRequest, ourHeaders).respond(200);
            j.$httpBackend.expectPOST(validateRoute + '/user', j.scope.setupRequest.userRequest, ourHeaders).respond(200);
            j.$httpBackend.expectPOST(validateRoute + '/proxy', j.scope.setupRequest.proxyUrl).respond(200);
            j.$httpBackend.expectPOST(routePrefix + '/occm/setup/init', j.scope.setupRequest).respond(200, {upgradeToVersion: '333'});
            j.$httpBackend.expectGET(routePrefix + '/occm/system/about').respond(200, {buildTimestamp: 555555555});
            j.scope.finish();
            j.$httpBackend.flush();

            //At this stage the UI already got one 'about' response with 'old' buildTimestamp. UI scheduled another
            //call to 'about' in 1000ms.
            //Now the test will handle the second call to 'about', with 'new' buildTimestamp, and flush the timeout,
            //and expect that the UI will login to OCCM:
            j.$httpBackend.expectGET(routePrefix + '/occm/system/about').respond(200, {buildTimestamp: 666666666});
            j.$httpBackend.expectPOST(routePrefix + '/auth/login', {
                email: j.scope.setupRequest.userRequest.email,
                password: j.scope.setupRequest.userRequest.password
            }).respond(200);

            j.$timeout.flush(1000);
            j.$httpBackend.flush();
        });
    });
});
