describe('SnapMirrorDestinationVolumeController', () => {
    'use strict';

    var j;    beforeEach(() => {        j = magnifispec.setup();    });
    let nocAdvancedAllocationService;

    beforeEach(inject((_nocAdvancedAllocationService_) => {
        nocAdvancedAllocationService = _nocAdvancedAllocationService_;
        j.$httpBackend.expectGET(j.nocBaseUrl + '/vsa/aggregates?workingEnvironmentId=value2').respond(200, [{name: 'aggrAggr0'}, {name: 'aggrAggr1'}]);
        magnifispec.mockWizardStepController('SnapMirrorDestinationVolumeController', {}, {
            targetWorkingEnvironment: {
                name: 'value1',
                publicId: 'value2',
                svmName: 'svmName',
                _pathPrefix: '/vsa/',
                _providerVolumeTypes: {gp2: {name: 'gp2', shortName: 'gp2', display: ''}, st1: {name: 'st1', shortName: 'st1', display: ''}},
                capacityFeatures: {defaultProviderVolumeType: {diskType: 'st1', capacityTier: 'S3'}, supportedCapacityTiers: {capacityTiersDisableReasons: []}}
            },
            selectedVolume: {
                size: {},
                usedSize: {},
                thinProvisioning: true
            },
            path: {destinationVolume: 'path.to.this.step'},
            setWizardStep: jest.fn()
        });
        j.$httpBackend.flush();
    }));

    describe('initialize', () => {
        it('should work, and not auto select aggregate and svm when more than one available', () => {
            jest.spyOn(nocAdvancedAllocationService, 'getAggregates').mockReturnValue(j.$q.when());
            jest.spyOn(nocAdvancedAllocationService, 'getSVMs').mockReturnValue(j.$q.when());

            j.scope.initialize();
            magnifispec.processPromise();

            expect(nocAdvancedAllocationService.getAggregates).toHaveBeenCalled();
            expect(nocAdvancedAllocationService.getSVMs).not.toHaveBeenCalled();
            expect(j.scope.wizardController.selectedDestinationAggregateName).toEqual('');
            expect(j.scope.wizardController.destinationSvmName).not.toBeDefined();
        });

        it('should auto select aggregate and svm if only one available - case ON_PREM', () => {
            j.scope.targetWorkingEnvironment.workingEnvironmentType = 'ON_PREM';
            jest.spyOn(nocAdvancedAllocationService, 'getAggregates').mockReturnValue(j.$q.when([{name: 'only_one1'}]));
            jest.spyOn(nocAdvancedAllocationService, 'getSVMs').mockReturnValue(j.$q.when());
            j.scope.destinationSvms = [{name: 'only_one2'}];

            j.scope.initialize();
            magnifispec.processPromise();

            expect(j.scope.wizardController.selectedDestinationAggregateName).toEqual('only_one1');
            expect(j.scope.wizardController.destinationSvmName).toEqual('only_one2');
        });

        it('should not auto select aggregate if only one available - case VSA', () => {
            j.scope.targetWorkingEnvironment.workingEnvironmentType = 'VSA';
            j.scope.selectedVolume.providerVolumeType = 'gp2';
            jest.spyOn(nocAdvancedAllocationService, 'getAggregates').mockReturnValue(j.$q.when([{
                name: 'only_one1',
                providerVolumes: [{diskType: 'gp2'}]
            }]));

            j.scope.initialize();
            magnifispec.processPromise();

            j.$timeout.flush();
            expect(j.scope.wizardController.selectedDestinationAggregateName).toEqual('');
        });

        describe('initializing tier related features when destination is VSA', () => {
            beforeEach(() => {
                jest.spyOn(j.scope, 'initializeAggregatesForDisplay').mockImplementation(() => {});
                j.scope.targetWorkingEnvironment.workingEnvironmentType = 'VSA';
                j.scope.targetWorkingEnvironment._isAWS = true;
                jest.spyOn(nocAdvancedAllocationService, 'getAggregates').mockReturnValue(j.$q.when([{
                    name: 'only_one1',
                    providerVolumes: [{diskType: 'gp2'}]
                }]));
            });

            it('should preselect the target volume disk type', () => {
                j.scope.selectedVolume.providerVolumeType = 'gp2';

                j.scope.initialize();
                magnifispec.processPromise();

                 expect(j.scope.wizardController.selectedTier.name).toEqual('st1');
                 expect(j.scope.wizardController.tieringEnabled).toEqual(false);
            });

            afterEach(() => {
                expect(j.scope.initializeAggregatesForDisplay).toHaveBeenCalled();
                expect(j.scope.possibleTiers).toEqual({
                    gp2: {name: 'gp2', shortName: 'gp2', display: ''},
                    st1: {name: 'st1', shortName: 'st1', display: ''}
                });
            });
        });

        it('should initialize destinationVolumeIcon', () => {
            jest.spyOn(j.scope, 'initDestinationVolumeIcon').mockImplementation(() => {});
            jest.spyOn(nocAdvancedAllocationService, 'getAggregates').mockReturnValue(j.$q.when([]));

            j.scope.initialize();
            magnifispec.processPromise();

            expect(j.scope.initDestinationVolumeIcon).toHaveBeenCalled();
        });
    });

    describe('initializeAggregatesForDisplay', () => {
        it('should create aggregates list when appropriate aggregate found', () => {
            j.scope.originalAggregates = [
                {name: 'aggr1', value: 'aggr1', _tier: 'gp2', _providerVolumeType: 'gp2'},
                {name: 'aggr2', value: 'aggr2', _tier: 'gp2+S3', _providerVolumeType: 'gp2'}
            ];
            j.scope.wizardController.selectedTier = {name: 'gp2'};

            j.scope.initializeAggregatesForDisplay();
            j.$timeout.flush();

            expect(j.scope.wizardController.selectedDestinationAggregateName).toEqual('');
            expect(j.scope.aggregatesWithStaticOption).toEqual([
                {name: 'Automatically select the best aggregate', value: ''},
                {name: 'aggr1', value: 'aggr1', _tier: 'gp2', _providerVolumeType: 'gp2'},
                {name: 'aggr2', value: 'aggr2', _tier: 'gp2+S3', _providerVolumeType: 'gp2'}
            ]);
        });

        it('should create aggregates list when no appropriate aggregate found', () => {
            j.scope.originalAggregates = [
                {name: 'aggr1', value: 'aggr1', _tier: 'gp2', _providerVolumeType: 'gp2'},
                {name: 'aggr2', value: 'aggr2', _tier: 'gp2+S3', _providerVolumeType: 'gp2', capacityTier: 'S3'}
            ];
            j.scope.wizardController.selectedTier = {name: 'sc1'};

            j.scope.initializeAggregatesForDisplay();
            j.$timeout.flush();

            expect(j.scope.wizardController.selectedDestinationAggregateName).toEqual('');
            expect(j.scope.aggregatesWithStaticOption).toEqual([
                {name: 'Create new aggregate of tier sc1', value: ''},
                {name: 'aggr1', value: 'aggr1', _tier: 'gp2', _providerVolumeType: 'gp2'},
                {name: 'aggr2', value: 'aggr2', _tier: 'gp2+S3', _providerVolumeType: 'gp2', capacityTier: 'S3'}
            ]);
        });
    });

    describe('initDestinationVolumeIcon', () => {
        it('case AWS:gp2', () => {
            j.scope.targetWorkingEnvironment._isAWS = true;
            j.scope.wizardController.selectedTier = {name: 'gp2', shortName: 'gp2'};
            j.scope.wizardController.wizardController.tieringEnabled = false;

            j.scope.initDestinationVolumeIcon();

            expect(j.scope.wizardController._destinationIconName).toEqual('volume-gp2');
        });

        it('case AWS:gp2+S3', () => {
            j.scope.targetWorkingEnvironment._isAWS = true;
            j.scope.wizardController.selectedTier = {name: 'gp2', shortName: 'gp2'};
            j.scope.wizardController.wizardController.tieringEnabled = true;
            j.scope.wizardController.targetProviderCapacityTierName = 'S3';

            j.scope.initDestinationVolumeIcon();

            expect(j.scope.wizardController._destinationIconName).toEqual('volume-gp2+s3');
        });

        it('case ON_PREM', () => {
            j.scope.wizardController.selectedTier = undefined;
            j.scope.wizardController.wizardController.tieringEnabled = undefined;

            j.scope.initDestinationVolumeIcon();

            expect(j.scope.wizardController._destinationIconName).toEqual('resource-volume');
        });

        it('case Azure:prm', () => {
            j.scope.wizardController.selectedTier = {name: 'Premium_LRS', shortName: 'prm'};
            j.scope.wizardController.wizardController.tieringEnabled = undefined;

            j.scope.initDestinationVolumeIcon();

            expect(j.scope.wizardController._destinationIconName).toEqual('volume-prm');
        });
    });

    describe('tierSelected', () => {
        it('should remove check if selected disktype doesn\'t support tiering', () => {
            j.scope.wizardController.selectedTier = {name: 'st1', supportedCapacityTiers: ['S3']};
            j.scope.wizardController.tieringEnabled = true;
            j.scope.tierSelected({name: 'io1'});

            expect(j.scope.wizardController.selectedTier).toEqual({name: 'io1'});
            expect(j.scope.wizardController.tieringEnabled).toEqual(false);
        });

        it('should keep checkbox checked if selected disktype supports tiering', () => {
            j.scope.wizardController.targetProviderCapacityTierName = 'S3';
            j.scope.wizardController.selectedTier = {name: 'st1', supportedCapacityTiers: ['S3']};
            j.scope.wizardController.tieringEnabled = true;
            j.scope.tierSelected({name: 'gp2', supportedCapacityTiers: ['S3']});

            expect(j.scope.wizardController.selectedTier).toEqual({name: 'gp2', supportedCapacityTiers: ['S3']});
            expect(j.scope.wizardController.tieringEnabled).toEqual(true);
        });

        it('should set the tier and init aggregates list and selection', () => {
            jest.spyOn(j.scope, 'initializeAggregatesForDisplay').mockImplementation(() => {});

            j.scope.tierSelected('newTier');

            expect(j.scope.wizardController.selectedTier).toEqual('newTier');
            expect(j.scope.initializeAggregatesForDisplay).toHaveBeenCalled();
        });

        it('should set the wizard step back to current step', () => {
            j.scope.tierSelected('newTier');

            expect(j.scope.setWizardStep).toHaveBeenCalledWith('path.to.this.step');
        });
    });

    describe('disabledTooltipForTier', () => {
        it('should return the tier disabled message if exists', () => {
            expect(j.scope.disabledTooltipForTier({disabledMessage: 'lalala'})).toEqual('lalala');
            j.scope.wizardController.targetProviderCapacityTierName = 'S3';
            expect(j.scope.disabledTooltipForTier({supportedCapacityTiers: 'S3'})).toEqual(undefined);
            expect(j.scope.disabledTooltipForTier({supportedCapacityTiers: undefined})).toEqual('S3 tiering is not available for this disk type.');
        });
    });

    it('should cleanup after itself on $destroy', () => {
        j.scope.wizardController.advancedMode = true;
        j.scope.wizardController.selectedDestinationAggregateName = 'aggr1';
        j.scope.wizardController.destinationSvmName = 'svm1';

        j.$rootScope.$broadcast('$destroy');

        expect(j.scope.wizardController.advancedMode).not.toBeDefined();
        expect(j.scope.wizardController.selectedDestinationAggregateName).not.toBeDefined();
        expect(j.scope.wizardController.destinationSvmName).not.toBeDefined();
    });

    describe('prepare request', () => {
        it('should return the right values', () => {
            const volumeRequest = {
                workingEnvironmentId: 'value2',
                svmName: 'svmName',
                name: 'someName',
                size: {size: 2, unit: 'GB'},
                initialSize: {size: 333, unit: 'GB'},
                providerVolumeType: 'gp2',
                capacityTier: undefined,
                isReplicationFlow: true,
                snapshotPolicyName: 'none',
                exportPolicyInfo: {
                    policyType: 'none',
                    ips: []
                },
                usageProfile: {
                    flags: {
                        enableThinProvisioning: 'value10',
                        enableDeduplication: false,
                        enableCompression: false
                    }
                }
            };

            j.scope.wizardController.targetVolumeName = 'someName';
            j.scope.wizardController.selectedTier = {name: 'gp2', diskName: 'gp2'};
            j.scope.wizardController.selectedVolume = {
                size: {size: 2, unit: 'GB'},
                usedSize: {size: 333, unit: 'GB'},
                thinProvisioning: 'value10'
            };
            j.scope.prepareRequest();
            expect(j.scope.volumeRequest).toEqual(volumeRequest);
        });

        it('should return the right values for composite tier, and use thin provisioning for composite tier, even if source is thick', () => {
            j.scope.wizardController.selectedVolume = {
                thinProvisioning: false
            };

            j.scope.wizardController.targetProviderCapacityTierName = 'S3';
            const volumeRequest = {
                providerVolumeType: 'gp2',
                capacityTier: 'S3',
                usageProfile: {
                    flags: {
                        enableThinProvisioning: true,
                        enableDeduplication: false,
                        enableCompression: false
                    }
                }
            };

            j.scope.wizardController.selectedTier = {name: 'gp2', diskName: 'gp2'};
            j.scope.wizardController.tieringEnabled = true;
            j.scope.prepareRequest();
            expect(j.scope.volumeRequest).toEqual(jasmine.objectContaining(volumeRequest));
        });
    });

    describe('askForQuote', () => {
        let errorFunction;
        beforeEach(() => {
            jest.spyOn(j.scope, 'prepareRequest').mockImplementation(() => {});
            j.scope.volumeRequest = {
                usageProfile: {flags: 'lala'},
                exportPolicyInfo: {policyType: 'notNone', ips: [], _ips: ''}
            };
            j.scope.wizardController.quoteResponse = undefined;
            errorFunction = jest.fn();
        });

        describe('should call quote API for target VSA with automatic aggregate selection', () => {
            it('should update quoteResponse upon success', () => {
                j.$httpBackend.expectPOST(j.nocBaseUrl + j.nocUtil.workingEnvironmentTypeToRequestPathPrefix('VSA', 'AWS') + 'volumes/quote').respond(400, {
                    numOfDisks: 4,
                    aggregateName: 'value12'
                });
                j.scope.askForQuote().catch(errorFunction);
                j.$httpBackend.flush();
                expect(errorFunction).toHaveBeenCalled();
            });

            it('with server error', () => {
                j.$httpBackend.expectPOST(j.nocBaseUrl + j.nocUtil.workingEnvironmentTypeToRequestPathPrefix('VSA', 'AWS') + 'volumes/quote').respond(500, j.errorResponse);
                j.scope.askForQuote().catch(errorFunction);
                j.$httpBackend.flush();
                expect(errorFunction).toHaveBeenCalled();
            });
        });

        describe('should call quote API for target VSA with specific aggregate', () => {
            beforeEach(() => {
                j.scope.wizardController.selectedDestinationAggregateName = 'value11';
            });

            it('should work upon success', () => {
                j.$httpBackend.expectPOST(j.nocBaseUrl + j.nocUtil.workingEnvironmentTypeToRequestPathPrefix('VSA', 'AWS') + 'volumes/quote').respond(400, {
                    numOfDisks: 3,
                    aggregateName: 'value11'
                });
                j.scope.askForQuote().catch(errorFunction);
                j.$httpBackend.flush();
                expect(errorFunction).toHaveBeenCalled();
            });

            it('with server error', () => {
                j.$httpBackend.expectPOST(j.nocBaseUrl + j.nocUtil.workingEnvironmentTypeToRequestPathPrefix('VSA', 'AWS') + 'volumes/quote').respond(500, j.errorResponse);
                j.scope.askForQuote().catch(errorFunction);
                j.$httpBackend.flush();
                expect(errorFunction).toHaveBeenCalled();
            });
        });
    });

    describe('next', () => {
        describe('case when form is not valid', () => {
            it('should validate the form', () => {
                j.scope.isValid = jest.fn().mockReturnValue(false);

                j.scope.next();

                expect(j.nocUI.handleErrorsInWizard).toHaveBeenCalledWith(true);
            });
        });
        describe('case when form is valid', () => {
            beforeEach(() => {
                j.scope.isValid = jest.fn().mockReturnValue(true);
                j.$state.go = jest.fn();
            });

            describe('case target is VSA: should call API', () => {
                beforeEach(() => {
                    jest.spyOn(j.scope, 'askForQuote');
                    j.scope.targetWorkingEnvironment.workingEnvironmentType = 'VSA';
                    j.scope.wizardController.selectedTier = {};
                });

                it('should update quoteResponse upon success', () => {
                    j.$httpBackend.expectPOST(j.nocBaseUrl + j.nocUtil.workingEnvironmentTypeToRequestPathPrefix('VSA', 'AWS') + 'volumes/quote').respond(200, {
                        numOfDisks: 4,
                        aggregateName: 'value12'
                    });
                    j.scope.next();
                    j.$httpBackend.flush();

                    expect(j.scope.wizardController.quoteResponse.numOfDisks).toEqual(4);
                    expect(j.scope.wizardController.quoteResponse.aggregateName).toEqual('value12');
                    expect(j.$state.go).toHaveBeenCalledWith('.maxTransferRate');
                });

                it('should work upon error', () => {
                    j.$httpBackend.expectPOST(j.nocBaseUrl + j.nocUtil.workingEnvironmentTypeToRequestPathPrefix('VSA', 'AWS') + 'volumes/quote').respond(500, j.errorResponse);
                    j.scope.next();
                    j.$httpBackend.flush();

                    expect(j.pidgeon.error).toHaveBeenCalled();
                    expect(j.scope.wizardController.quoteResponse).not.toBeDefined();
                });
            });

            describe('case target is on_prem', () => {
                it('should call state.go', () => {
                    jest.spyOn(j.scope, 'askForQuote').mockImplementation(() => {});
                    j.scope.targetWorkingEnvironment.workingEnvironmentType = 'ON_PREM';
                    j.scope.next();
                    expect(j.scope.askForQuote).not.toHaveBeenCalled();
                    expect(j.$state.go).toHaveBeenCalledWith('.maxTransferRate');
                });
            });
        });
    });
});
