describe('Init Step Controller', () => {
    'use strict';

    var j;    beforeEach(() => {        j = magnifispec.setup();    });
    let request1, request2;
    let wEs;

    beforeEach(() => {
        wEs = [{
            publicId: 'VsaWorkingEnvironment-weId1',
            tenantId: 'Tenant-1',
            name: 'we1',
            workingEnvironmentType: 'ON_PREM'
        }, {
            publicId: 'VsaWorkingEnvironment-weId2',
            tenantId: 'Tenant-1',
            name: 'we2',
            workingEnvironmentType: 'ON_PREM'
        }, {
            publicId: 'VsaWorkingEnvironment-weId3',
            tenantId: 'Tenant-1',
            name: 'we3',
            workingEnvironmentType: 'ON_PREM'
        }];
        request1 = j.$httpBackend.expectGET(j.nocBaseUrl + '/onprem/working-environments/VsaWorkingEnvironment-weId1?fields=status,svmName,capacityFeatures,cloudSyncProperties');
        request2 = j.$httpBackend.expectGET(j.nocBaseUrl + '/onprem/working-environments/VsaWorkingEnvironment-weId3?fields=status,svmName,capacityFeatures');
        j.$state.params = {sourceId: 'VsaWorkingEnvironment-weId1', targetId: 'VsaWorkingEnvironment-weId3', sourceType: 'ON_PREM', sourceProvider: 'AWS', targetType: 'ON_PREM', targetProvider: 'AWS'};
        magnifispec.mockWizardStepController('SnapMirrorInitController', {}, {
            selectedTenant: {publicId: 'value2'},
            exit: jest.fn(),
            pushError: jest.fn(),
            sourceWorkingEnvironment: {workingEnvironmentType: 'ON_PREM'},
            targetWorkingEnvironment: {workingEnvironmentType: 'ON_PREM'}
        });
    });

    describe('handleLifs', () => {
        it('case A: when peering exist', () => {
            const response = {
                interClusterLifs: [
                    {address: 'ip1', anotherField: 'value1', isPeered: true},
                    {address: 'ip2', anotherField: 'value2', isPeered: false},
                    {address: 'ip3', anotherField: 'value3', isPeered: true}
                ],
                peerInterClusterLifs: [
                    {address: 'ip4', anotherField: 'value4', isPeered: true}
                ]
            };

            j.scope.handleLifs(response);
            expect(j.scope.wizardController.nextStep).toEqual({init: '.volumes'});
            expect(j.scope.wizardController.selectedLifsAddresses).toEqual({
                sourceLifs: ['ip1', 'ip3'],
                destinationLifs: ['ip4']
            });
        });

        it('Case B: one LIF in each side', () => {
            const response = {
                interClusterLifs: [
                    {address: 'ip1', anotherField: 'value1', peered: false}
                ],
                peerInterClusterLifs: [
                    {address: 'ip4', anotherField: 'value4', peered: true}
                ]
            };

            j.scope.handleLifs(response);
            expect(j.scope.wizardController.nextStep).toEqual({init: '.volumes'});
            expect(j.scope.wizardController.selectedLifsAddresses).toEqual({sourceLifs: ['ip1'], destinationLifs: ['ip4']});
        });

        it('Case C: one LIF at the source, many in the destination', () => {
            const response = {
                interClusterLifs: [
                    {address: 'ip1', anotherField: 'value1', peered: false}
                ],
                peerInterClusterLifs: [
                    {address: 'ip4', anotherField: 'value4', peered: false},
                    {address: 'ip5', anotherField: 'value5', peered: false},
                    {address: 'ip6', anotherField: 'value6', peered: false}
                ]
            };

            j.scope.handleLifs(response);
            expect(j.scope.wizardController.nextStep).toEqual({init: '.destinationLifs', destinationLifs: '.volumes'});
            expect(j.scope.wizardController.lifs.destinationLifs).toEqual(response.peerInterClusterLifs);
            expect(j.scope.wizardController.selectedLifsAddresses.sourceLifs).toEqual(['ip1']);
            expect(j.scope.wizardController.selectedLifsAddresses.destinationLifs).toBe(undefined);
        });

        it('case D: when no peering, multi LIFs on both sides', () => {
            const response = {
                interClusterLifs: [
                    {address: 'ip1', anotherField: 'value1'},
                    {address: 'ip2', anotherField: 'value2'},
                    {address: 'ip3', anotherField: 'value3'}
                ],
                peerInterClusterLifs: [
                    {address: 'ip4', anotherField: 'value4'},
                    {address: 'ip5', anotherField: 'value5'},
                    {address: 'ip6', anotherField: 'value6'}
                ]
            };

            j.scope.handleLifs(response);
            expect(j.scope.wizardController.nextStep).toEqual({init: '.sourceLifs', destinationLifs: '.volumes', sourceLifs: '.destinationLifs'});
            expect(j.scope.wizardController.lifs.sourceLifs).toEqual(response.interClusterLifs);
            expect(j.scope.wizardController.lifs.destinationLifs).toEqual(response.peerInterClusterLifs);
            expect(j.scope.wizardController.selectedLifsAddresses.sourceLifs).toBe(undefined);
            expect(j.scope.wizardController.selectedLifsAddresses.destinationLifs).toBe(undefined);
        });

        it('case E: when no peering, multi LIFs on source, one on destination', () => {
            const response = {
                interClusterLifs: [
                    {address: 'ip1', anotherField: 'value1'},
                    {address: 'ip2', anotherField: 'value2'},
                    {address: 'ip3', anotherField: 'value3'}
                ],
                peerInterClusterLifs: [
                    {address: 'ip4', anotherField: 'value4'}
                ]
            };

            j.scope.handleLifs(response);
            expect(j.scope.wizardController.nextStep).toEqual({init: '.sourceLifs', sourceLifs: '.volumes'});
            expect(j.scope.wizardController.lifs.sourceLifs).toEqual(response.interClusterLifs);
            expect(j.scope.wizardController.selectedLifsAddresses.sourceLifs).toBe(undefined);
            expect(j.scope.wizardController.selectedLifsAddresses.destinationLifs).toEqual(['ip4']);
        });

        it('case F: no side is OnPrem', () => {
            j.scope.wizardController.sourceWorkingEnvironment.workingEnvironmentType = 'VSA';
            j.scope.wizardController.targetWorkingEnvironment.workingEnvironmentType = 'VSA';

            const response = {
                interClusterLifs: [
                    {address: 'ip1', anotherField: 'value1'},
                    {address: 'ip2', anotherField: 'value2'},
                    {address: 'ip3', anotherField: 'value3'}
                ],
                peerInterClusterLifs: [
                    {address: 'ip4', anotherField: 'value4'},
                    {address: 'ip5', anotherField: 'value5'},
                    {address: 'ip6', anotherField: 'value6'}
                ]
            };

            j.scope.handleLifs(response);
            expect(j.scope.wizardController.nextStep).toEqual({init: '.volumes'});
            expect(j.scope.wizardController.lifs.sourceLifs).toEqual(response.interClusterLifs);
            expect(j.scope.wizardController.selectedLifsAddresses.sourceLifs).toEqual(['ip1', 'ip2', 'ip3']);
            expect(j.scope.wizardController.selectedLifsAddresses.destinationLifs).toEqual(['ip4', 'ip5', 'ip6']);
        });
    });

    describe('getInterclusterLifs', () => {
        //The getInterclusterLifs is called from the initialize, not need to call it from here explicitly, just check that it worked properly.
        beforeEach(() => {
            request1.respond(200, wEs[0]);
            request2.respond(200, wEs[2]);
        });

        it('should call handleLifs upon success', () => {
            j.$httpBackend.expectGET(j.nocBaseUrl + '/replication/intercluster-lifs?peerWorkingEnvironmentId=VsaWorkingEnvironment-weId3&workingEnvironmentId=VsaWorkingEnvironment-weId1').respond(200, {res: 'sourceResponse'});

            jest.spyOn(j.scope, 'handleLifs').mockImplementation(() => {});
            jest.spyOn(j.scope, 'next').mockImplementation(() => {});

            j.$httpBackend.flush();
            expect(j.scope.handleLifs).toHaveBeenCalled();
            expect(j.scope.handleLifs.mock.calls[0][0]).toEqual({res: 'sourceResponse'});
            expect(j.scope.next).toHaveBeenCalled();
        });

        it('should call pidgeon upon failure', () => {
            j.$httpBackend.expectGET(j.nocBaseUrl + '/replication/intercluster-lifs?peerWorkingEnvironmentId=VsaWorkingEnvironment-weId3&workingEnvironmentId=VsaWorkingEnvironment-weId1').respond(400, j.errorResponse);
            j.$httpBackend.flush();

            expect(j.scope.pushError).toHaveBeenCalledWith(j.pidgeon.serverErrorPrefix + 'retrieving LIFs for Replication: ' + j.errorResponse.messageForDisplay(), j.errorResponse, 'DataFabricController', 'replicationWizard');
            expect(j.scope.exit.mock.calls.length).toEqual(1); //One in the init process.
        });
    });

    describe('initialize', () => {
        it('should work upon success', () => {
            jest.spyOn(j.scope, 'getInterclusterLifs').mockImplementation(() => {});
            request1.respond(200, {res: 'sourceResponse', workingEnvironmentType: 'ON_PREM'});
            request2.respond(200, {res: 'bla', workingEnvironmentType: 'ON_PREM'});

            j.$httpBackend.flush();
            expect(j.scope.getInterclusterLifs).toHaveBeenCalled();
            expect(j.scope.wizardController.sourceWorkingEnvironment.res).toEqual('sourceResponse');
            expect(j.scope.wizardController.targetWorkingEnvironment.res).toEqual('bla');
        });

        it('should work upon error', () => {
            jest.spyOn(j.scope, 'getInterclusterLifs').mockImplementation(() => {});
            request1.respond(400, j.errorResponse);
            request2.respond(400, j.errorResponse);
            j.$httpBackend.flush();
            expect(j.scope.pushError).toHaveBeenCalledWith(j.pidgeon.serverErrorPrefix + 'retrieving working environments details for replication: ' + j.errorResponse.messageForDisplay(), j.errorResponse, 'DataFabricController', 'replicationWizard');
            expect(j.scope.getInterclusterLifs).not.toHaveBeenCalled();
            expect(j.scope.exit.mock.calls.length).toEqual(1); //One in the init process.
        });

        afterEach(() => {
            expect(j.pidgeon.clear).toHaveBeenCalledWith('init');
            expect(j.nocUI.loading).toHaveBeenCalled();
            expect(j.nocUI.unloading).toHaveBeenCalled();
        });
    });

    describe('initialize that was canceled in the middle', () => {
        it('should exit immediately if the path prefix can not be calculated, which can happen upon cancel', () => {
            j.$state.params = {sourceType: '', sourceProvider: '', targetType: '', targetProvider: ''};
            jest.spyOn(j.workingEnvironmentResource, 'getWorkingEnvironmentByPublicId').mockImplementation(() => {});
            j.scope.initialize();
            expect(j.workingEnvironmentResource.getWorkingEnvironmentByPublicId).not.toHaveBeenCalled();
            expect(j.scope.exit).toHaveBeenCalled();
        });
    });

    describe('next', () => {
        it('should go to the right place according to nextStep map', () => {
            j.$state.go = jest.fn();
            j.scope.wizardController.nextStep = {init: '.nextStepName'};
            j.scope.next();
            expect(j.$state.go).toHaveBeenCalledWith('snap-mirror-wizard.init.nextStepName');
        });
    });
});
