describe("SnapMirrorPolicyController", function () {
    "use strict";

    var policies = [
        {isDefault: false, isSupported: true, reasons: [],_displayName: "DPDefault", name: "DPDefault", type: "async_mirror", description: "Default policy for DP relationship", rules: [{snapMirrorLabel: "sm_created", retention: 1}]},
        {isDefault: true, isSupported: true, reasons: [],_displayName: "Mirror all snapshots", name: "MirrorAllSnapshots", type: "async_mirror", description: "Asynchronous SnapMirror policy for mirroring all snapshots and latest active file system", rules: [{snapMirrorLabel: "sm_created", retention: 1}, {snapMirrorLabel: "all_source_snapshots", retention: 1}]},
        {isDefault: false, isSupported: true, reasons: [], name: "MirrorAndVault", type: "mirror_vault", description: "A unified Asynchronous SnapMirror and SnapVault policy for mirroring the latest active file system and daily and weekly snapshots", rules: [{snapMirrorLabel: "sm_created", retention: 1}, {snapMirrorLabel: "daily", retention: 7}]},
        {isDefault: false, isSupported: false, reason: "unsupported version", name: "XDPDefault", type: "vault", description: "Default policy for XDP relationship with daily and weekly rules", rules: [{snapMirrorLabel: "daily", retention: 7}]}
    ];

    var nocPoliciesService;
    var j;    beforeEach(() => {        j = magnifispec.setup();    });
    beforeEach(inject(function (_nocPoliciesService_) {
        nocPoliciesService = _nocPoliciesService_;
        jest.spyOn(nocPoliciesService, "getPolicies").mockReturnValue(j.$q.when(policies));
        jest.spyOn(nocPoliciesService, "getSnapshotLabels").mockReturnValue(j.$q.when(policies));
        magnifispec.mockWizardStepController("SnapMirrorPolicyController", {}, {
            targetWorkingEnvironment: {name: "targetWE", publicId: "targetWEId"},
            sourceWorkingEnvironment: {name: "sourceWE", publicId: "sourceWEId"},
            selectedVolume: {svmName: "svmName", name: "volName"},
            next: jest.fn()
        });
        magnifispec.processPromise();
    }));

    describe("initialize", function () {
        it("should call nocPoliciesService getPolicies and getSnapshotLabels", function () {
            expect(nocPoliciesService.getPolicies).toHaveBeenCalled();
            expect(nocPoliciesService.getSnapshotLabels).toHaveBeenCalled();

            expect(j.scope.policies).toEqual([policies[0], policies[2], policies[3]]);
            expect(j.scope.defaultPolicies).toEqual([policies[1]]);

            var callsForGetPoliciesExceptNocHttp = nocPoliciesService.getPolicies.mock.calls[0].splice(-2);
            expect(callsForGetPoliciesExceptNocHttp).toEqual(['sourceWEId', 'targetWEId']);
            var callsForGetSnapshotLabelsExceptNocHttp = nocPoliciesService.getSnapshotLabels.mock.calls[0].splice(-4);
            expect(callsForGetSnapshotLabelsExceptNocHttp).toEqual([policies, 'sourceWEId', 'svmName', 'volName']);
        });
    });

    describe("select policy", function () {
        it("should do nothing if the policy is not supported", function () {
            j.$state.$current = {data: {name : "policies"}};
            j.scope.selectPolicy(policies[3]);
            expect(j.scope.wizardController.next).not.toHaveBeenCalled();
        });

        it("should set the policy in wizard controller to the given policy", function () {
            j.$state.$current = {data: {name : "schedules"}};
            j.scope.selectPolicy(policies[2]);
            expect(j.scope.wizardController.policy).toEqual(policies[2]);
        });

        it("should move to the next step if the current step is different then the next step", function () {
            j.$state.$current = {data: {name : "policies"}};
            j.scope.selectPolicy(policies[2]);
            expect(j.scope.wizardController.next).toHaveBeenCalledWith(".schedules");
        });
    });

    describe("backing up data", function () {
        it("should backup data and clear fields upon destroy", function () {
            j.scope.wizardController.policy = policies[2];
            j.$rootScope.$broadcast('$destroy');
            expect(j.parentScope.backupStepData).toHaveBeenCalledWith("policy", {
                policy : policies[2]
            });
            expect(j.scope.wizardController.policy).toBeNull();
        });

        it("should restore backed up data upon creation ", function () {
            expect(j.parentScope.getStepData).toHaveBeenCalledWith("policy");
        });

    });

});

