describe('Review Step Controller', () => {
    'use strict';

    var j;    beforeEach(() => {        j = magnifispec.setup();    });
    let replicationRequest, replicationVolumeVSA, replicationVolumeOnPrem, requestVSA, requestOnPrem, constants;

    beforeEach(inject(function (Constants) {
        constants = Constants;
        magnifispec.mockWizardStepController("SnapMirrorReviewController", {}, {
            isValid: _.noop,
            targetVolumeName: "someName",
            selectedVolume: {size: {size: 20, unit: 'GB'}},
            targetWorkingEnvironment: {_pathPrefix:"/vsa/", _cloudProviderName: "AWS", publicId: "id555", workingEnvironmentType: "VSA", _typeAndProvider: "VSA_AWS", providerVolumeType: "gp2"},
            selectedLifsAddresses: {"sourceLifs": ["ip1", "ip2"], "destinationLifs": ["ip3", "ip4"]},
            selectedTier: {name: 'gp2', diskName: 'gp2', display: 'General Purpose SSD (gp2)'},
            quoteResponse: {numOfDisks: 88, aggregateName: "value9"}
        });

        replicationRequest = {
            sourceWorkingEnvironmentId: "value1",
            destinationWorkingEnvironmentId: "value5",
            sourceInterclusterLifIps: ["ip1", "ip2"],
            destinationInterclusterLifIps: ["ip3", "ip4"],
            policyName: "policy1",
            scheduleName: "value7",
            maxTransferRate: undefined
        };

        replicationVolumeVSA = {
            sourceSvmName: "value4",
            sourceVolumeName: "value3",
            destinationVolumeName: "value6",

            destinationAggregateName: "value9",
            destinationProviderVolumeType: 'gp2',
            destinationCapacityTier: undefined,
            numOfDisksApprovedToAdd: 88,
            advancedMode: undefined
        };

        replicationVolumeOnPrem = {
            sourceSvmName: "value4",
            sourceVolumeName: "value3",
            destinationVolumeName: "value6",

            destinationAggregateName: "value13",
            destinationSvmName: "value14"
        };

        requestVSA = {
            replicationRequest: replicationRequest,
            replicationVolume: replicationVolumeVSA
        };

        requestOnPrem = {
            replicationRequest: replicationRequest,
            replicationVolume: replicationVolumeOnPrem
        };
    }));

    describe('showPopUp', () => {
        beforeEach(() => {
            j.scope.wizardController.volumeRequest = {name: 'vol1'};
        });

        it('Should show popup', () => {
            magnifispec.mockConfirm(false);

            j.scope.showPopUp();
            magnifispec.processPromise();

            expect(j.nocDialog.confirm).toHaveBeenCalledWith('', 'assets/templates/dialogs/add-volume-approval-content.html', {
                html: jasmine.any(Object),
                error: {
                    numOfDisks: 88,
                    aggregateName: 'value9',
                    provider: 'AWS',
                    isHA: undefined,
                    diskType: 'General Purpose SSD (gp2)'
                },
                volumeName: 'vol1', hideHint: true, show: true, okButtonOnly: true
            });
        });

        it('should display half of the disks for HA', () => {
            j.scope.targetWorkingEnvironment.isHA = true;
            magnifispec.mockConfirm(false);

            j.scope.showPopUp();
            magnifispec.processPromise();

            expect(j.nocDialog.confirm).toHaveBeenCalledWith('', 'assets/templates/dialogs/add-volume-approval-content.html', {
                html: jasmine.any(Object),
                error: {
                    numOfDisks: 44,
                    aggregateName: 'value9',
                    provider: 'AWS',
                    isHA: true,
                    diskType: 'General Purpose SSD (gp2)'
                },
                volumeName: 'vol1', hideHint: true, show: true, okButtonOnly: true
            });
        });
    });

    describe("buildSnapMirrorRequest", function () {
        beforeEach(function () {
            j.scope.wizardController.sourceWorkingEnvironment = {_pathPrefix:"/vsa/",publicId: "value1", workingEnvironmentType: "value2"};
            j.scope.wizardController.selectedVolume = {name: "value3", svmName: "value4"};
            j.scope.wizardController.targetWorkingEnvironment = {_pathPrefix:"/vsa/",publicId: "value5"};
            j.scope.wizardController.targetVolumeName = "value6";
            j.scope.wizardController.selectedSchedule = {name: "value7"};
            j.scope.wizardController.selectedDestinationAggregateName = "value13";
            j.scope.wizardController.destinationSvmName = "value14";
            j.scope.wizardController.policy = {name:"policy1"};
        });

        it("should return the right values for target VSA in simple mode", function () {
            j.scope.targetWorkingEnvironment.workingEnvironmentType = "VSA";
            expect(j.scope.buildSnapMirrorRequest()).toEqual(requestVSA);
        });

        it('should return the right values for target VSA in simple mode when selecting composite tier as destination', () => {
            j.scope.targetWorkingEnvironment.workingEnvironmentType = 'VSA';
            j.scope.wizardController.selectedTier = {name: 'gp2', diskName: 'gp2'};
            j.scope.wizardController.tieringEnabled = true;
            j.scope.wizardController.targetProviderCapacityTierName = 'S3';
            requestVSA.replicationVolume.destinationCapacityTier = 'S3';

            expect(j.scope.buildSnapMirrorRequest()).toEqual(requestVSA);
        });

        it("should return the right values for target VSA in advanced mode", function () {
            j.scope.targetWorkingEnvironment.workingEnvironmentType = "VSA";
            j.scope.wizardController.advancedMode = true;
            requestVSA.replicationVolume.advancedMode = true;
            expect(j.scope.buildSnapMirrorRequest()).toEqual(requestVSA);
        });

        it("should return the right values for target ON_PREM", function () {
            j.scope.targetWorkingEnvironment.workingEnvironmentType = "ON_PREM";
            expect(j.scope.buildSnapMirrorRequest()).toEqual(requestOnPrem);
        });

        it("Should return empty string for one time schedule", function () {
            j.scope.wizardController.selectedSchedule = {name: constants.emptyReplicationScheduleDescription};
            j.scope.targetWorkingEnvironment.workingEnvironmentType = "VSA";
            var oneTimeScheduleRequest = requestVSA;
            oneTimeScheduleRequest.replicationRequest.scheduleName = null;

            expect(j.scope.buildSnapMirrorRequest()).toEqual(oneTimeScheduleRequest);
        });

        it("Should return maxTransferRate if defined", function () {
            j.scope.wizardController.maxTransferRate = 123;
            j.scope.targetWorkingEnvironment.workingEnvironmentType = "VSA";
            var maxTransferRateRequest = requestVSA;
            maxTransferRateRequest.replicationRequest.maxTransferRate = 123;

            expect(j.scope.buildSnapMirrorRequest()).toEqual(maxTransferRateRequest);
        });
    });

    describe("approveAndSnapMirror", function () {
        beforeEach(function () {
            jest.spyOn(j.scope, "isValid").mockReturnValue(true);
            j.scope.setPathToPreviousOrRoot = jest.fn();
        });

        it("should work upon success", function () {
            jest.spyOn(j.scope, "buildSnapMirrorRequest").mockReturnValue(requestVSA);
            j.$httpBackend.expectPOST(j.nocBaseUrl + "/replication/vsa", requestVSA).respond(200, {});
            j.scope.approveAndSnapMirror();
            j.$httpBackend.flush();

            expect(j.pidgeon.inProgress).toHaveBeenCalledWith("Replication started.", true);
            expect(j.scope.setPathToPreviousOrRoot).toHaveBeenCalled();
        });

        it("should work upon error", function () {
            jest.spyOn(j.scope, "buildSnapMirrorRequest").mockReturnValue(requestVSA);
            j.$httpBackend.expectPOST(j.nocBaseUrl + "/replication/vsa", requestVSA).respond(400, j.errorResponse);
            j.scope.approveAndSnapMirror();
            j.$httpBackend.flush();

            expect(j.pidgeon.error).toHaveBeenCalled();
            expect(j.scope.setPathToPreviousOrRoot).not.toHaveBeenCalled();
        });

        it("should work for onPrem", function () {
            j.scope.targetWorkingEnvironment.workingEnvironmentType = "ON_PREM";
            jest.spyOn(j.scope, "buildSnapMirrorRequest").mockReturnValue(requestOnPrem);
            j.$httpBackend.expectPOST(j.nocBaseUrl + "/replication/onprem", requestOnPrem).respond(200, {});
            j.scope.approveAndSnapMirror();
            j.$httpBackend.flush();

            expect(j.pidgeon.inProgress).toHaveBeenCalledWith("Replication started.", true);
            expect(j.scope.setPathToPreviousOrRoot).toHaveBeenCalled();
        });

        afterEach(function () {
            expect(j.nocUI.block).toHaveBeenCalled();
            expect(j.scope.buildSnapMirrorRequest).toHaveBeenCalled();
            expect(j.nocUI.unblock).toHaveBeenCalled();
        });
    });

    describe("validity check", function () {
        it("should call handle form validity if the form is not valid", function () {
            jest.spyOn(j.scope, "isValid").mockReturnValue(false);
            jest.spyOn(j.scope, "buildSnapMirrorRequest").mockImplementation(() => {});

            j.scope.approveAndSnapMirror();
            expect(j.nocUI.handleErrorsInWizard).toHaveBeenCalled();
            expect(j.scope.buildSnapMirrorRequest).not.toHaveBeenCalled();
        });
    });

    it("should cleanup after itself on $destory", function () {
        j.scope.wizardController.agreeResources = "true";
        j.$rootScope.$broadcast('$destroy');
        expect(j.scope.wizardController.agreeResources).toBe(null);
    });
});
