describe("Cloud Volumes ONTAP upgrade step controllers", function(){
    "use strict";

    var j;    beforeEach(() => {        j = magnifispec.setup();    });

    describe("CloudOntapUpgradeEulaController", function () {
        beforeEach(function(){
            magnifispec.mockWizardStepController("CloudOntapUpgradeEulaController", {}, {virginStuff: {stateName: "test"}});
        });

        it("should set eulaAgree to false every time it comes back to this state", function(){
            j.scope.$broadcast("$stateChangeSuccess", {name: "test"});

            expect(j.scope.eulaAgree).toEqual(false);
        });
    });

    describe("CloudOntapUpgradeVersionController", function () {
        var resourceSpy;
        var versionsAvailable = [{image: "image1"},{image: "image2"}];
        var workingEnvironment = {publicId: 1, ontapClusterProperties: {ontapVersion: "1.1.1", upgradeVersions: versionsAvailable}};

        beforeEach(function(){
            j.$state.params = {workingEnvironmentId: "1", type:"VSA", provider:"AWS"};
            resourceSpy = jest.spyOn(j.workingEnvironmentResource, "getVSAByPublicId").mockImplementation(magnifispec.mockPromise(workingEnvironment));
            magnifispec.mockWizardStepController("CloudOntapUpgradeVersionController", {}, {virginStuff: {stateName: "test"}});
        });

        it("should set releaseNotesRead to false every time it comes back to this state", function(){
            j.scope.$broadcast("$stateChangeSuccess", {name: "test"});

            expect(j.scope.releaseNotesRead).toEqual(false);
        });

        describe("setShowOlderVersions", function(){
           it("should set showOlderVersions to the given value", function(){
               j.scope.setShowOlderVersions("test");
               expect(j.scope.showOlderVersions).toEqual("test");
           });
        });

        describe("selectVersion", function(){
            it("should set the version in the parent controller, and go to the same state", function(){
                jest.spyOn(j.$state, "go").mockImplementation(() => {});
                j.scope.selectVersion("test");
                expect(j.parentScope.selectedVersion).toEqual("test");
                expect(j.$state.go).toHaveBeenCalledWith("test");
            });
        });

        describe("initialize", function(){
            var vsaResponse = {publicId: 1, ontapClusterProperties: {ontapVersion: "1.1.1", upgradeVersions: [{ image : 'image1' }, { image : 'image2' }]}, _pathPrefix: '/vsa/'};
            beforeEach(function(){
                j.workingEnvironmentResource.getVSAByPublicId.mockClear();
            });
            it("should resolve the working environment from the $state", function(){
                resourceSpy.and.callThrough();
                j.$httpBackend.expectGET(j.nocBaseUrl + "/vsa/working-environments/1?fields=status,ontapClusterProperties.fields(upgradeVersions),activeActions,supportedFeatures,k8sProperties").respond(vsaResponse);
                j.scope.initialize();
                j.$httpBackend.flush();
            });

            it("should get the vsaResponse and cloudOntapUpdateVersions after working environment resolves", function(){
                resourceSpy.mockImplementation(magnifispec.mockPromise(workingEnvironment));
                j.scope.initialize();
                magnifispec.processPromise();

                expect(j.parentScope.versionsAvailable).toEqual(versionsAvailable);
                expect(j.parentScope.workingEnvironment).toEqual(vsaResponse);
                expect(j.scope.currentVersion).toEqual(vsaResponse.ontapClusterProperties.ontapVersion);
            });
        });
    });

    describe("CloudOntapReviewController", function () {

        var workingEnvironment = {_pathPrefix:"/vsa/",publicId: 1};
        var version = {imageVersion: "1.1.1"};
        var payload = {updateType: "OCCM_PROVIDED", updateParameter: "1.1.1"};
        var isValidSpy;

        beforeEach(function(){
            magnifispec.mockWizardStepController("CloudOntapReviewController", {}, {
                exit: jest.fn()
            });
            isValidSpy = j.scope.isValid = jest.fn().mockReturnValue(true);
        });

        describe("upgrade method", function(){
            it("should validate the form", function () {
                isValidSpy.mockReturnValue(false);
                j.scope.upgrade(workingEnvironment, version);
                expect(j.nocUI.handleErrorsInWizard).toHaveBeenCalled();
            });

            it("should do everything it needs to do in success case", function(){
                j.$httpBackend.expectPOST(j.nocBaseUrl + "/vsa/working-environments/1/update-image", payload).respond(200);

                j.scope.upgrade(workingEnvironment, version);

                j.$httpBackend.flush();

                expect(j.pidgeon.success).toHaveBeenCalledWith("Working environment upgrade successfully triggered.");
                expect(j.scope.exit).toHaveBeenCalled();
            });

            it("should do everything it needs to do in failure case", function(){
                j.$httpBackend.expectPOST(j.nocBaseUrl + "/vsa/working-environments/1/update-image", payload).respond(400, j.errorResponse);
                j.scope.upgrade(workingEnvironment, version);

                j.$httpBackend.flush();

                expect(j.pidgeon.error).toHaveBeenCalled();
            });
        });
    });
});

