describe('EditAwsEncryptionKeysDialogController', () => {
    'use strict';
    let j;

    beforeEach(() => {
        j = magnifispec.setup();
    });

    beforeEach(() => {
        magnifispec.mockConfirmController('EditAwsEncryptionKeysDialogController', {}, {
            wizardController: {
                awsEncryptionKeys: magnifispec.awsEncryptionKeysObject.awsEncryptionKeys,
                awsEncryptionKey: magnifispec.awsEncryptionKeysObject.awsEncryptionKey,
                path: {encryption: ''}
            }
        });
    });

    describe('updateSelection', function () {
        const defaultKeyId = 'c1bb46f0-f6c9-4b60-b3cd-03bda6948a18';
        const newKeyId = 'y4ui6lf9-4081-4b6a-rtyg-baf579e1706e';
        it('Should update the selected key when chosen with a different key', () => {
            expect(_.find(j.scope.wizardController.awsEncryptionKeys, {keyId: defaultKeyId}).chosen).toEqual(true);
            j.scope.updateSelection(newKeyId);
            expect(_.find(j.scope.wizardController.awsEncryptionKeys, {keyId: defaultKeyId}).chosen).toEqual(false);
            expect(_.find(j.scope.wizardController.awsEncryptionKeys, {keyId: newKeyId}).chosen).toEqual(true);
            j.scope.updateSelection(defaultKeyId);
            expect(_.find(j.scope.wizardController.awsEncryptionKeys, {keyId: defaultKeyId}).chosen).toEqual(true);
            expect(_.find(j.scope.wizardController.awsEncryptionKeys, {keyId: newKeyId}).chosen).toEqual(false);
        });

        it('Should not update the selected key when chosen with the same key', function () {
            expect(_.find(j.scope.wizardController.awsEncryptionKeys, {keyId: defaultKeyId}).chosen).toEqual(true);
            j.scope.updateSelection(defaultKeyId);
            expect(_.find(j.scope.wizardController.awsEncryptionKeys, {keyId: defaultKeyId}).chosen).toEqual(true);
            expect(_.find(j.scope.wizardController.awsEncryptionKeys, {keyId: newKeyId}).chosen).toEqual(false);
        });
    });

    describe('saveEditAwsEncryptionKeys', function () {
        const defaultKeyId = 'c1bb46f0-f6c9-4b60-b3cd-03bda6948a18';
        const newKeyId = 'y4ui6lf9-4081-4b6a-rtyg-baf579e1706e';
        beforeEach(() => {
            j.scope.closeDialog = _.noop;
            j.scope.editAwsEncryptionKeys = {$valid: true};
            j.$state.go = jest.fn();
        });
        it('Should update the awsEncryptionKey field when saved with a different key', () => {
            expect(j.scope.wizardController.awsEncryptionKey.keyId).toEqual(defaultKeyId);
            j.scope.updateSelection(newKeyId);
            expect(j.scope.wizardController.awsEncryptionKey.keyId).toEqual(defaultKeyId);
            j.scope.saveEditAwsEncryptionKeys();
            expect(j.scope.wizardController.awsEncryptionKey.keyId).toEqual(newKeyId);
            j.scope.updateSelection(defaultKeyId);
            expect(j.scope.wizardController.awsEncryptionKey.keyId).toEqual(newKeyId);
            j.scope.saveEditAwsEncryptionKeys();
            expect(j.scope.wizardController.awsEncryptionKey.keyId).toEqual(defaultKeyId);
        });

        it('Should not update the awsEncryptionKey field when chosen with the same key', () => {
            expect(j.scope.wizardController.awsEncryptionKey.keyId).toEqual(defaultKeyId);
            j.scope.updateSelection(defaultKeyId);
            expect(j.scope.wizardController.awsEncryptionKey.keyId).toEqual(defaultKeyId);
            j.scope.saveEditAwsEncryptionKeys();
            expect(j.scope.wizardController.awsEncryptionKey.keyId).toEqual(defaultKeyId);
        });
    });
});