describe('BaseCreateVpc', () => {
    'use strict';

    var j;    beforeEach(() => {        j = magnifispec.setup();    });
    let $onDestroyFunction;

    beforeEach(inject((BaseCreateVpc, _MetadataResource_) => {
        magnifispec.initializeManifestWithPermutations();
        j.scope = {
            wizardController: {
                clearStepData: jest.fn(),
                getStepData: jest.fn(),
                backupStepData: jest.fn(),
                networkRequirements: {
                    node: 2
                }
            },
            getKeyPairs: _.noop,
            $on: (name, callback) => {
                if (name === '$destroy') $onDestroyFunction = callback;
            },
            $broadcast: (name) => {
                if (name === '$destroy') $onDestroyFunction();
            }
        };
        BaseCreateVpc.call(j.scope, j.pidgeon, j.nocUI, j.nocHttp);
        jest.spyOn(_MetadataResource_, 'getKeyPairsForAllRegions').mockReturnValue(j.$q.when({regionToKeyPairs: {
            'eu-central-1': ['a', 'b'],
            'eu-west-1': ['a', 'b'],
            'ap-southeast-1': ['a', 'b'],
            'us-west-2': ['a', 'b'],
            'us-west-1': ['a', 'b']
        }}));

        jest.spyOn(_MetadataResource_, 'getNetworkRequirements').mockReturnValue(j.$q.when('network-requirements'));
    }));

    describe('initialize', () => {
        beforeEach(() => {
            j.scope.wizardController.workingEnvironmentType = 'a';
            j.scope.wizardController.workingEnvironmentTypeAndProvider = {type: 'VSA_AWS', providerName: 'AWS'};

            jest.spyOn(j.scope, 'clearAll').mockImplementation(() => {});
            jest.spyOn(j.scope, 'getRegions').mockReturnValue(j.$q.when('getRegionsPromise'));
        });

        it('should initialize everything', () => {
            j.scope.wizardController.vpcBeenHereBefore = 'HA';

            expect(j.scope.initialize()).toBeResolvedWith('getRegionsPromise');

            expect(j.scope.wizardController.vpcBeenHereBefore).toEqual('a');
            expect(j.scope.clearAll).toHaveBeenCalled();

            magnifispec.processPromise();

            expect(j.scope.wizardController.networkRequirements).toEqual('network-requirements');
            expect(j.scope.getRegions).toHaveBeenCalled();
        });

        it('shouldn\'t really do anything, and return rejected promise if we initialized with this kind of working environment', () => {
            j.scope.wizardController.vpcBeenHereBefore = 'a';

            expect(j.scope.initialize()).toBeRejected();

            expect(j.scope.clearAll).not.toHaveBeenCalled();
        });

        it('should initialize HA parameters when creating HA working environment', () => {
            j.scope.wizardController.workingEnvironmentType = 'HA';

            j.scope.initialize();

            expect(j.scope.wizardController.haParameters).toEqual({});
        });

        it('should restore backed up data upon initialization ', () => {
            j.scope.initialize();
            expect(j.scope.wizardController.getStepData).toHaveBeenCalledWith('vpc-base');
        });
    });

    describe('clearAllButRegionsAndVpcs', () => {
        it('should set all createVpcSelectObjects except region access and key and set state to vpc', inject(($state) => {
            _.set(j.scope.wizardController, 'createVpcSelectObjects.region', {name: 'r1'});
            j.scope.wizardController.createVpcSelectObjects.vpc = {name: 'v1'};
            j.scope.wizardController.createVpcSelectObjects.subnet = {name: 's1'};
            j.scope.wizardController.createVpcSelectObjects.access = {name: 'password'};
            j.scope.wizardController.createVpcSelectObjects.key = {name: 'key1'};
            _.set(j.scope.wizardController, 'path.vpc', 'vpc');
            jest.spyOn(j.scope, 'getVPCs').mockImplementation(() => {});
            jest.spyOn($state, 'go').mockImplementation(() => {});
            j.scope.clearAllButRegionsAndVpcs();
            expect($state.go).toHaveBeenCalledWith('vpc');
            expect(j.scope.wizardController.createVpcSelectObjects.region.name).toEqual('r1');
            expect(j.scope.wizardController.createVpcSelectObjects.vpc).toBeUndefined();
            expect(j.scope.wizardController.createVpcSelectObjects.subnet).toBeUndefined();
            expect(j.scope.wizardController.createVpcSelectObjects.access.name).toEqual('password');
            expect(j.scope.wizardController.createVpcSelectObjects.key.name).toEqual('key1');
        }));
    });

    describe('clearAll', () => {
        it('should clear all regions', () => {
            j.scope.wizardController.vpcsWithSubnets = 'value1';
            _.set(j.scope.wizardController, 'createVpcSelectObjects.region', 'value3');
            j.scope.wizardController.createVpcSelectObjects.vpc = 'value4';
            j.scope.wizardController.createVpcSelectObjects.subnet = 'value5';
            j.scope.wizardController.createVpcSelectObjects.securityGroup = 'value6';
            j.scope.clearAll();
            expect(j.scope.wizardController.vpcsWithSubnets).toBeNull();
            expect(j.scope.wizardController.createVpcSelectObjects.region).toBeUndefined();
            expect(j.scope.wizardController.createVpcSelectObjects.vpc).toBeUndefined();
            expect(j.scope.wizardController.createVpcSelectObjects.subnet).toBeUndefined();
            expect(j.scope.wizardController.createVpcSelectObjects.securityGroup).toBeUndefined();
        });
    });

    describe('getRegions', () => {
        beforeEach(() => {
            jest.spyOn(j.scope, 'handleRegionChange').mockReturnValue(j.$q.when('handleRegionChangePromise'));
            j.scope.wizardController.createVpcSelectObjects = {};
        });

        it('should update the vsa regions upon success call to getVSARegions', () => {
            j.scope.selectedOntapVersion = '1';

            expect(j.scope.getRegions()).toBeResolvedWith('handleRegionChangePromise');

            expect(j.scope.regions).toEqual(
                [
                    {name: 'EU', code: 'eu-central-1', location: 'Frankfurt'},
                    {name: 'EU', code: 'eu-west-1', location: 'Ireland'},
                    {name: 'Asia Pacific', code: 'ap-southeast-1', location: 'Singapore'},
                    {name: 'US West', code: 'us-west-2', location: 'Oregon'},
                    {name: 'US West', code: 'us-west-1', location: 'N. California'}
                ]
            );
            expect(j.scope.handleRegionChange).toHaveBeenCalled();
        });

        it('should set the selected region to the first region when there is no OCCM detection', () => {
            j.scope.getRegions();
            magnifispec.processPromise();

            expect(j.scope.wizardController.createVpcSelectObjects.region).toEqual(j.scope.regions[0]);
            expect(j.scope.handleRegionChange).toHaveBeenCalled();
        });

        it('should set the selected region to the OCCM region when there is OCCM detection', () => {
            j.scope.wizardController.instancePlacementDefaultForWizard = {region: 'us-west-1'};

            j.scope.getRegions();
            magnifispec.processPromise();

            expect(j.scope.wizardController.createVpcSelectObjects.region.code).toEqual(j.scope.wizardController.instancePlacementDefaultForWizard.region);
            expect(j.scope.handleRegionChange).toHaveBeenCalled();
        });

        it('should go to the first region if the instance placement is not in the valid vsa regions', () => {
            j.scope.wizardController.instancePlacement = {region: 'any-other-region'};

            j.scope.getRegions();
            magnifispec.processPromise();

            expect(j.scope.wizardController.createVpcSelectObjects.region).toEqual(j.scope.regions[0]);
            expect(j.scope.handleRegionChange).toHaveBeenCalled();
        });

        it('should add an alert upon error call to getVSARegions', () => {
            j.scope.selectedOntapVersion = '1';
            magnifispec.initializeManifestWithNoPermutations(); //Empty manifest
            expect(j.scope.getRegions()).toBeRejected();

            expect(j.pidgeon.error).toHaveBeenCalledWith('Unable to retrieve list of regions', null, 'regions');
            expect(j.scope.handleRegionChange).not.toHaveBeenCalled();
        });

        afterEach(() => {
            expect(j.pidgeon.clear).toHaveBeenCalledWith('regions');
        });
    });

    describe('getVPCs', () => {
        beforeEach(() => {
            j.scope.wizardController.workingEnvironmentTypeAndProvider = {type: 'VSA', providerName: 'AWS'};
            j.scope.wizardController.pathPrefix = '/vsa/';
            j.scope.wizardController.createVpcSelectObjects = {region: {code: 'region1'}};
            jest.spyOn(j.scope, 'selectVPC').mockImplementation(() => {});
        });

        it('should update the vpcsWithSubnets upon success call to vpc_to_subnets', () => {
            const vpcsToSubnets = [{tags: [{key: 'Name', value: 'myName'}], subnets: [{cidr: '1.1.1.1/1'}], securityGroups: []}, {tags: [], subnets: [], securityGroups: []}];
            j.$httpBackend.expectGET(j.nocBaseUrl + '/vsa/metadata/vpcs?region=region1').respond(200, vpcsToSubnets);
            j.scope.getVPCs();

            j.$httpBackend.flush();
            expect(j.nocUI.loading).toHaveBeenCalled();
            expect(j.pidgeon.clear).toHaveBeenCalledWith('vpcsWithSubnets');
            expect(j.scope.wizardController.vpcsWithSubnets).toEqual([_.assignIn(vpcsToSubnets[0], {
                _hasTags: true,
                _name: 'myName',
                securityGroups: []
            }), _.assignIn(vpcsToSubnets[1], {
                _hasTags: false,
                _name: null,
                securityGroups: []
            })]);
        });

        it('should add an alert upon error call to vpc to subnets', () => {
            //Just wait so all the initialization requests will be sent before starting the test
            j.$httpBackend.expectGET(j.nocBaseUrl + '/vsa/metadata/vpcs?region=region1').respond(400, j.errorResponse);
            j.scope.getVPCs();

            j.$httpBackend.flush();
            expect(j.nocUI.loading).toHaveBeenCalled();
            expect(j.pidgeon.error).toHaveBeenCalledWith(j.pidgeon.serverErrorPrefix + 'retrieving list of VPCs with subnets', j.errorResponse, 'vpcsWithSubnets');
        });

        it('should call selectVPC if there is instancePlacement with a vpcId and the vpc is in the list', () => {
            j.scope.wizardController.workingEnvironmentType = 'VSA';
            j.scope.wizardController.instancePlacementDefaultForWizard = {vpcId: 1};
            const vpcsToSubnets = [{vpcId: 1, subnets: [{subnetId: '1a'}], securityGroups: []}, {vpcId: 2, tags: [], subnets: [], securityGroups: []}];
            j.$httpBackend.expectGET(j.nocBaseUrl + '/vsa/metadata/vpcs?region=region1').respond(200, vpcsToSubnets);
            j.scope.getVPCs();

            j.$httpBackend.flush();
            expect(j.nocUI.loading).toHaveBeenCalled();
            expect(j.pidgeon.clear).toHaveBeenCalledWith('vpcsWithSubnets');
            expect(j.scope.wizardController.createVpcSelectObjects.vpc.vpcId).toEqual(1);
            expect(j.scope.selectVPC).toHaveBeenCalled();
        });

        it('should not call selectVPC even if there is instancePlacement with a vpcId but the vpc is not in the list', () => {
            j.scope.wizardController.workingEnvironmentType = 'VSA';
            j.scope.wizardController.instancePlacement = {vpcId: 2};
            const vpcsToSubnets = [{subnets: [{subnetId: 1}], securityGroups: []}, {tags: [], subnets: [], securityGroups: []}];
            j.$httpBackend.expectGET(j.nocBaseUrl + '/vsa/metadata/vpcs?region=region1').respond(200, vpcsToSubnets);
            j.scope.getVPCs();

            j.$httpBackend.flush();
            expect(j.nocUI.loading).toHaveBeenCalled();
            expect(j.pidgeon.clear).toHaveBeenCalledWith('vpcsWithSubnets');
            expect(j.scope.wizardController.selectedVpcId).not.toBeDefined();
            expect(j.scope.selectVPC).not.toHaveBeenCalledWith();
        });

        it('should set cancelled on the previous VPC config', () => {
            j.scope.lastVPCConfig = {_: {}};
            const prev = j.scope.lastVPCConfig;
            j.scope.getVPCs();
            expect(prev._.cancelled).toEqual(true);
        });

        it('should not process the success of the previous request if there was already a successive request', () => {
            const vpcsToSubnets = [{tags: ['I have a tag'], subnets: [{cidr: '1.1.1.1/1'}], securityGroups: []}, {tags: [], subnets: [], securityGroups: []}];
            j.$httpBackend.expectGET(j.nocBaseUrl + '/vsa/metadata/vpcs?region=region1').respond(200, vpcsToSubnets);
            j.$httpBackend.expectGET(j.nocBaseUrl + '/vsa/metadata/vpcs?region=region1').respond(200, vpcsToSubnets);

            j.scope.getVPCs();
            j.scope.getVPCs();
            j.$httpBackend.flush();
            expect(j.pidgeon.clear.mock.calls.length).toEqual(2);
        });

        it('should not process the error of the previous request if there was already a successive request', () => {
            j.$httpBackend.expectGET(j.nocBaseUrl + '/vsa/metadata/vpcs?region=region1').respond(400, {});
            j.$httpBackend.expectGET(j.nocBaseUrl + '/vsa/metadata/vpcs?region=region1').respond(400, {});

            j.scope.getVPCs();
            j.scope.getVPCs();
            j.$httpBackend.flush();
            expect(j.pidgeon.error.mock.calls.length).toEqual(1);
        });
    });

    describe('selectVpc', () => {
        beforeEach(inject(($state) => {
            j.scope.wizardController.vpcsWithSubnets = [
                {
                    vpcId: 'vpc1',
                    subnets: [{cidr: '1.1.1.1/2', availableIps: 1}, {cidr: '2.2.2.2/2', availableIps: 1}],
                    securityGroups: [{securityGroupId: 'sg1', description: '', name: 'sg1'}, {securityGroupId: 'sg2', description: '', name: 'sg2'}]
                },
                {
                    vpcId: 'vpc2',
                    subnets: [{cidr: '3.3.3.3/3', availableIps: 2}],
                    securityGroups: [{securityGroupId: 'sg1', description: '', name: 'sg1'}]
                },
                {
                    vpcId: 'vpc3',
                    subnets: [{cidr: '3.3.3.3/3', availableIps: 1}, {subnetId: 2, cidr: '4.4.4.4/4'}],
                    securityGroups: [{securityGroupId: 'sg1', description: '', name: 'sg1'}, {securityGroupId: 'sg2', description: '', name: 'sg2'}]
                },
                {
                    vpcId: 'vpc4',
                    subnets: []
                }
            ];
            j.scope.wizardController.createVpcSelectObjects = {};
            j.scope.wizardController.createVpcSelectObjects.vpc = {vpcId: 'vpc1', subnets: [{cidr: '1.1.1.1/2'}, {cidr: '2.2.2.2/2'}], securityGroups: [{securityGroupId: 'sg1', description: '', name: 'sg1'}, {securityGroupId: 'sg2', description: '', name: 'sg2'}]};
            j.scope.wizardController.createVpcSelectObjects.subnet = j.scope.wizardController.createVpcSelectObjects.vpc.subnets[0];
            j.scope.wizardController.createVpcSelectObjects.securityGroup = j.scope.wizardController.createVpcSelectObjects.vpc.securityGroups[0];
            j.scope.wizardController.useExistingSecurityGroup = true;
            _.set(j.scope.wizardController, 'path.vpc', 'vpc');
            jest.spyOn($state, 'go').mockImplementation(() => {});
        }));

        it('should do nothing if the given vpc has no subnets, although it is a different vpc', inject(($state) => {
            j.scope.wizardController.createVpcSelectObjects.vpc = _.find(j.scope.wizardController.vpcsWithSubnets, {vpcId: 'vpc4'});
            j.scope.selectVPC();
            expect($state.go).toHaveBeenCalledWith('vpc');
            expect(j.scope.wizardController.createVpcSelectObjects.vpc).toBeUndefined();
            expect(j.scope.wizardController.createVpcSelectObjects.subnet).toBeUndefined();
            expect(j.scope.wizardController.createVpcSelectObjects.securityGroup).toBeUndefined();
        }));

        it('should update the selected vpc, subnet and subnet name if the given vpc is different than the selected and has subnets', () => {
            j.scope.wizardController.createVpcSelectObjects.vpc = _.find(j.scope.wizardController.vpcsWithSubnets, {vpcId: 'vpc2'});

            j.scope.selectVPC();
            expect(j.scope.wizardController.createVpcSelectObjects.vpc).toEqual({vpcId: 'vpc2', subnets: [{cidr: '3.3.3.3/3', availableIps: 2}], securityGroups: [{securityGroupId: 'sg1', description: '', name: 'sg1'}]});
            expect(j.scope.wizardController.createVpcSelectObjects.subnet).toEqual({cidr: '3.3.3.3/3', availableIps: 2});
            expect(j.scope.wizardController.createVpcSelectObjects.securityGroup).toEqual({securityGroupId: 'sg1', description: '', name: 'sg1'});
        });

        it('should select no subnet if the vpc has more than one subnet', () => {
            j.scope.wizardController.createVpcSelectObjects.vpc = _.find(j.scope.wizardController.vpcsWithSubnets, {vpcId: 'vpc3'});
            j.scope.selectVPC();
            expect(j.scope.wizardController.createVpcSelectObjects.vpc).toEqual(j.scope.wizardController.vpcsWithSubnets[2]);
            expect(j.scope.wizardController.createVpcSelectObjects.subnet).toBeUndefined();
            expect(j.scope.wizardController.createVpcSelectObjects.securityGroup).toEqual({securityGroupId: 'sg1', description: '', name: 'sg1'});
        });

        it('should select no subnet if the the only subnet has not enough ips available', () => {
            j.scope.wizardController.vpcsWithSubnets = [
                {
                    vpcId: 'vpc1',
                    subnets: [{cidr: '1.1.1.1/2', availableIps: 1}],
                    securityGroups: [{securityGroupId: 'sg1', description: '', name: 'sg1'}, {securityGroupId: 'sg2', description: '', name: 'sg2'}]
                },
                {
                    vpcId: 'vpc2',
                    subnets: [{cidr: '3.3.3.3/3', availableIps: 1}],
                    securityGroups: [{securityGroupId: 'sg1', description: '', name: 'sg1'}]
                }
            ];
            j.scope.wizardController.createVpcSelectObjects.vpc = _.find(j.scope.wizardController.vpcsWithSubnets, {vpcId: 'vpc1'});
            j.scope.wizardController.workingEnvironmentType = 'VSA';
            j.scope.selectVPC();
            expect(j.scope.wizardController.createVpcSelectObjects.vpc).toEqual(j.scope.wizardController.vpcsWithSubnets[0]);
            expect(j.scope.wizardController.createVpcSelectObjects.subnet).toEqual({cidr: '1.1.1.1/2', availableIps: 1});
            expect(j.scope.wizardController.createVpcSelectObjects.securityGroup).toEqual({securityGroupId: 'sg1', description: '', name: 'sg1'});
        });

        it('should set the subnet to the subnet in instancePlacement if instancePlacement is present and its subnet is available', () => {
            j.scope.wizardController.workingEnvironmentType = 'VSA';
            j.scope.wizardController.instancePlacementDefaultForWizard = {subnetId: 2};
            const occmSubnet = {subnetId: 2, cidr: '4.4.4.4/4'};
            j.scope.wizardController.createVpcSelectObjects.vpc = _.find(j.scope.wizardController.vpcsWithSubnets, {vpcId: 'vpc3'});
            j.scope.selectVPC();
            expect(j.scope.wizardController.createVpcSelectObjects.vpc).toEqual(j.scope.wizardController.vpcsWithSubnets[2]);
            expect(j.scope.wizardController.createVpcSelectObjects.subnet).toEqual(occmSubnet);
            expect(j.scope.wizardController.createVpcSelectObjects.securityGroup).toEqual({securityGroupId: 'sg1', description: '', name: 'sg1'});
        });

        it('should not set the subnet to the subnet in instancePlacement if instancePlacement present and its subnet is not available', () => {
            j.scope.wizardController.workingEnvironmentType = 'VSA';
            j.scope.wizardController.instancePlacementDefaultForWizard = {subnetId: 'notFamiliarId'};
            j.scope.wizardController.createVpcSelectObjects.vpc = _.find(j.scope.wizardController.vpcsWithSubnets, {vpcId: 'vpc3'});
            j.scope.selectVPC();
            expect(j.scope.wizardController.createVpcSelectObjects.vpc).toEqual(j.scope.wizardController.vpcsWithSubnets[2]);
            expect(j.scope.wizardController.createVpcSelectObjects.subnet).toBeUndefined();
        });

        it('should not set the subnet to the subnet in instancePlacement if it does not have enough ips available', () => {
            j.scope.wizardController.workingEnvironmentType = 'VSA';
            j.scope.wizardController.instancePlacement = {subnetId: 2};
            j.scope.wizardController.vpcsWithSubnets[2].subnets[1] = {subnetId: 2, cidr: '4.4.4.4/4', availableIps: 1};
            j.scope.wizardController.createVpcSelectObjects.vpc = _.find(j.scope.wizardController.vpcsWithSubnets, {vpcId: 'vpc3'});
            j.scope.selectVPC();
            expect(j.scope.wizardController.createVpcSelectObjects.vpc).toEqual(j.scope.wizardController.vpcsWithSubnets[2]);
            expect(j.scope.wizardController.createVpcSelectObjects.subnet).toBeUndefined();
        });

        it('should call clearFormValidity to prevent the checkbox from being red', () => {
            j.scope.wizardController.createVpcSelectObjects.vpc = _.find(j.scope.wizardController.vpcsWithSubnets, {vpcId: 'vpc3'});
            j.scope.selectVPC();
            expect(j.nocUI.clearFormValidity).toHaveBeenCalled();
        });
    });

    describe('handleRegionChange', () => {
        beforeEach(() => {
            j.scope.wizardController.createVpcSelectObjects = {};

            jest.spyOn(j.scope, 'getVPCs').mockReturnValue(j.$q.when('getVPCsPromise'));
            jest.spyOn(j.scope, 'clearAllButRegionsAndVpcs').mockImplementation(() => {});

            j.scope.regions = ['region1', 'region2'];
        });

        it('should do nothing if not passed with new region', () => {
            expect(j.scope.handleRegionChange()).toBeRejected();

            expect(j.scope.getVPCs).not.toHaveBeenCalled();
        });

        it('should clear the selected vpc and subnet and clear license and version data if the region is defined', () => {
            j.scope.lastSelectedRegion = {code: 'region2'};
            expect(j.scope.handleRegionChange({code: 'region1'})).toBeResolvedWith('getVPCsPromise');
            j.scope.getKeyPairs = jest.fn();

            expect(j.scope.wizardController.createVpcSelectObjects.region).toEqual({code: 'region1'});
            expect(j.scope.wizardController.createVpcSelectObjects.vpc).toBeUndefined();
            expect(j.scope.wizardController.createVpcSelectObjects.subnet).toBeUndefined();
            expect(j.scope.wizardController.createVpcSelectObjects.securityGroup).toBeUndefined();
            expect(j.scope.wizardController.selectedRegion).toEqual('region1');
            expect(j.scope.wizardController.clearStepData).toHaveBeenCalledWith('license');
            expect(j.scope.getVPCs).toHaveBeenCalled();
        });

        it('should get key pairs for HA', () => {
            j.scope.lastSelectedRegion = 'region2';
            j.scope.getKeyPairs = jest.fn();
            j.scope.wizardController.workingEnvironmentType = 'HA';

            j.scope.handleRegionChange('region1');

            expect(j.scope.getKeyPairs).toHaveBeenCalled();
        });
    });

    describe('backing up data', () => {
        it('should backup data and clear fields upon destroy', () => {
            const createVpcSelectObjects = {
                region: 'Afula',
                vpc: 'VPC1',
                subnet: 'subnet1',
                securityGroup: 'value123'
            };
            j.scope.wizardController.createVpcSelectObjects = createVpcSelectObjects;
            j.scope.wizardController.networkRequirements = 'req1';
            j.scope.wizardController.selectedRegion = 'region-code-1';
            j.scope.regions = ['region-code-1', 'region-code-2', 'region-code-3'];

            j.scope.$broadcast('$destroy');
            expect(j.scope.wizardController.backupStepData).toHaveBeenCalledWith('vpc-base', {
                createVpcSelectObjects,
                networkRequirements: 'req1',
                selectedRegion: 'region-code-1',
                regions: ['region-code-1', 'region-code-2', 'region-code-3']
            });

            expect(j.scope.wizardController.networkRequirements).toBeUndefined();
            expect(j.scope.wizardController.selectedRegion).toBeUndefined();
        });
    });
});
