describe('Wizard CIFS Setup Controller', () => {
    'use strict';

    var j;    beforeEach(() => {        j = magnifispec.setup();    });

    beforeEach(() => {
        magnifispec.mockTabsController('CreateCIFSStepController', {},
            {
                wizardController: {
                    workingEnvironmentName: 'We1',
                    backupStepData: jest.fn()
                },
                getStepData: jest.fn()
            });
    });

    describe('initialization', () => {
        beforeEach(() => {
            j.$state.go = jest.fn();
        });

        it('check initial state', () => {
            expect(j.scope.state).toEqual(j.scope.basic);
            expect(j.scope.mode).toEqual(j.scope.modes.create);
        });

        it('happy path', () => {
            j.scope.forms.cifsForm = {$valid: true};
            expect(j.scope.wizardController.cifsRequest).not.toBeDefined();
            j.scope.request.ip1 = '1.1.1.1';

            j.scope.next();
            expect(j.nocUI.clearFormValidity).toHaveBeenCalled();
            expect(j.$state.go).toHaveBeenCalledWith('.tiering-policy');
        });

        it('failed validation', () => {
            j.scope.request.ip1 = '1.1.1.1';
            j.scope.request.ip2 = '1.1.1.2';
            j.scope.state = j.scope.basic;

            j.scope.forms.cifsForm = {$valid: false};
            j.scope.forms.cifsForm[j.scope.advanced] = {$valid: true};

            j.scope.next();

            expect(j.nocUI.handleFormValidity).toHaveBeenCalled();
            expect(j.$state.go).not.toHaveBeenCalled();
        });

        it('failed validation of the advanced part', () => {
            j.scope.request.ip1 = '1.1.1.1';
            j.scope.request.ip2 = '1.1.1.2';
            j.scope.state = j.scope.basic;

            j.scope.forms.cifsForm = {$valid: false};
            j.scope.forms.cifsForm[j.scope.advanced] = {$valid: false};

            j.scope.next();

            expect(j.scope.state).toEqual(j.scope.advanced);
            expect(j.nocUI.handleFormValidity).toHaveBeenCalled();
            expect(j.$state.go).not.toHaveBeenCalled();
        });

        it('should bind active directory domain to dns domain and unbind', () => {
            expect(j.scope.request.dnsDomain).toEqual('');
            j.scope.request.activeDirectoryDomain = 'domain1';
            j.scope.$digest();
            expect(j.scope.request.dnsDomain).toEqual('domain1');

            j.scope.bindDnsAndAD = false;
            j.scope.$digest();
            j.scope.request.dnsDomain = 'dns1';

            j.scope.request.activeDirectoryDomain = 'black';
            j.scope.$digest();
            expect(j.scope.request.dnsDomain).toEqual('dns1');
        });
    });
});
