describe('VirginCreateWorkingEnvironmentDiskSizeStepController', () => {
    'use strict';

    var j;    beforeEach(() => {        j = magnifispec.setup();    });

    beforeEach(() => {
        magnifispec.mockWizardStepController('VirginCreateWorkingEnvironmentDiskSizeStepController', {}, {
            setWizardStep: jest.fn(),
            clearStepData: jest.fn()
        });
    });

    describe('initialize', () => {
        it('should call getConfig and add the ebsSizeToPiopsRatio to the scope', () => {
            j.$httpBackend.expectGET(/config$/).respond(200, {ebsSizeToPiopsRatio: 50});
            jest.spyOn(j.scope, 'onDiskSizeChange').mockImplementation(() => {});

            magnifispec.mockDeferredPromise(true, j.configService, 'getConfig', {ebsSizeToPiopsRatio: 50});
            j.scope.initialize();
            j.$httpBackend.flush();

            expect(j.scope.ebsSizeToPiopsRatio).toEqual(50);
            expect(j.scope.onDiskSizeChange).toHaveBeenCalled();
        });
    });

    describe('onDiskSizeChange', () => {
        let constants;
        beforeEach(inject((Constants) => {
            constants = Constants;
        }));

        it('should convert the providerVolumeSize to GB if in TB', () => {
            j.scope.wizardController.providerVolumeSize = {size: 1, unit: 'TB'};

            j.scope.onDiskSizeChange();

            expect(j.scope.wizardController.providerVolumeSizeGB).toEqual(1024);
        });

        it('should leave the providerVolumeSize in GB if already in GB', () => {
            j.scope.wizardController.providerVolumeSize = {size: 100, unit: 'GB'};

            j.scope.onDiskSizeChange();

            expect(j.scope.wizardController.providerVolumeSizeGB).toEqual(100);
        });

        it('should calculate maxIopsValue', () => {
            j.scope.wizardController.providerVolumeSize = {size: 100, unit: 'GB'};
            j.scope.ebsSizeToPiopsRatio = 50;
            constants.awsIopsLimitPerVolume = 20000;

            j.scope.onDiskSizeChange();

            expect(j.scope.maxIopsValue).toEqual(5000);
        });
    });
});
