describe.skip('VirginCreateWorkingEnvironmentHaRouteTablesStepController', () => {
    var j;
    beforeEach(() => {
        j = magnifispec.setup();
    });
    let MetadataResource;

    beforeEach(inject((_MetadataResource_) => {
        MetadataResource = _MetadataResource_;
        jest.spyOn(MetadataResource, 'getRouteTables').mockReturnValue(j.$q.when());
        magnifispec.mockWizardStepController('VirginCreateWorkingEnvironmentHaRouteTablesStepController', {}, {
            createVpcSelectObjects: {
                region: {code: 'region1'},
                vpc: {
                    vpcId: 'vpcId1',
                    subnets: [
                        {subnetId: 'node1SubnetId', cidr: '2.2.2.2/0'},
                        {subnetId: 'node2SubnetId', cidr: '1.1.1.1/0'},
                        {subnetId: 'nada', cidr: '0.0.0.0/0'}
                    ]
                },
                subnet: {subnetId: 'node1SubnetId'},
                node2Subnet: {subnetId: 'node2SubnetId', cidr: '1.1.1.1/0'}
            },
            haParameters: {
                dataFloatingIP: '1.1.1.1',
                dataFloatingIP2: '1.1.1.2',
                clusterFloatingIP: '1.1.1.3'
            },
            path: {},
            next: jest.fn()
        });

        j.scope.setWizardStep = jest.fn();
    }));

    describe('validateFloatingIPAddresses ', () => {
        it('should call resource function with correct parameters', () => {
            jest.spyOn(MetadataResource, 'validateFloatingIPAddresses').mockImplementation(() => {
            });
            j.scope.wizardController.routeTables = [{id: 'id1', selected: false}, {id: 'id2', selected: true}];

            j.scope.validateFloatingIPAddresses();

            expect(MetadataResource.validateFloatingIPAddresses).toHaveBeenCalledWith(jasmine.any(Object), 'region1', 'vpcId1', ['id2'], {
                clusterFloatingIP: '1.1.1.3',
                dataFloatingIP: '1.1.1.1',
                dataFloatingIP2: '1.1.1.2',
                svmFloatingIP: undefined
            }, undefined);
        });
    });

    describe('next', () => {
        beforeEach(() => {
            jest.spyOn(j.scope, 'validateFloatingIPAddresses').mockImplementation(() => {
            });
        });

        it('should validate everything and move to next step', () => {
            j.nocUI.formValidate.mockReturnValue(j.$q.when());

            j.scope.next();
            magnifispec.processPromise();

            expect(j.scope.validateFloatingIPAddresses).toHaveBeenCalled();
            expect(j.scope.wizardController.next).toHaveBeenCalledWith('.encryption');
        });

        it('should not move to next step if form is not valid', () => {
            j.nocUI.formValidate.mockReturnValue(j.$q.reject());

            j.scope.next();
            magnifispec.processPromise();

            expect(j.scope.validateFloatingIPAddresses).not.toHaveBeenCalled();
            expect(j.scope.wizardController.next).not.toHaveBeenCalled();
        });

        afterEach(() => {
            expect(j.scope.userClickedNext).toEqual(true);
        });
    });

    describe('getRouteTables', () => {
        beforeEach(() => {
            jest.spyOn(j.scope, 'onSelectionChange').mockImplementation(() => {
            });
            MetadataResource.getRouteTables.mockReturnValue(j.$q.when([
                {main: false, subnets: ['node1SubnetId']}, //Should be selected because it has a node subnet
                {main: true, subnets: ['node1SubnetId', 'node2SubnetId']}, //Should be selected because it's the main route table
                {main: false, subnets: ['nada']} //Should not be selected
            ]));
        });

        it('should initialize the route tables, with _subnet sorted by cidr, and select the required route tables', () => {
            j.scope.getRouteTables();
            magnifispec.processPromise();

            expect(j.scope.wizardController.routeTables).toEqual([
                {
                    main: false,
                    subnets: ['node1SubnetId'],
                    _subnets: [{subnetId: 'node1SubnetId', cidr: '2.2.2.2/0'}],
                    selected: true
                },
                {
                    main: true,
                    subnets: ['node1SubnetId', 'node2SubnetId'],
                    _subnets: [{subnetId: 'node2SubnetId', cidr: '1.1.1.1/0'}, {
                        subnetId: 'node1SubnetId',
                        cidr: '2.2.2.2/0'
                    }],
                    selected: true
                },
                {main: false, subnets: ['nada'], _subnets: [{subnetId: 'nada', cidr: '0.0.0.0/0'}], selected: false}
            ]);

            expect(j.scope.onSelectionChange).toHaveBeenCalled();
            expect(mostRecentCall(MetadataResource.getRouteTables)).toMatch('region1', 'vpc1');
        });

        it('should do nothing if routeTables already exist', () => {
            j.scope.wizardController.routeTables = 'some_tables';

            j.scope.getRouteTables();

            expect(j.scope.wizardController.routeTables).toEqual('some_tables');
            expect(MetadataResource.getRouteTables.mock.calls.length).toEqual(1); //Initialization only.
        });
    });
});
