describe.skip("VirginCreateWorkingEnvironmentHaVpcStepController", function () {
    "use strict";

    var j;
    beforeEach(() => {
        j = magnifispec.setup();
    });
    beforeEach(function(){
        magnifispec.mockWizardStepController("VirginCreateWorkingEnvironmentHaVpcStepController", {}, {path: {}});
        jest.spyOn(j.$state, "go").mockImplementation(() => {});
        j.vpcsUrl = j.nocBaseUrl + "/vsa/metadata/vpcs";
    });


    describe("getAvailabilityZones", function () {
        it("should get the availability zones", function () {
            _.set(j.scope.wizardController, "createVpcSelectObjects.vpc",
                {subnets: [{id: 'a', availabilityZone: 'Florentine'}, {id: 'b', availabilityZone: 'Ramat Aviv'}, {id: 'c', availabilityZone: 'Florentine'}]});
            j.scope.getAvailabilityZones();
            expect(j.scope.wizardController.availabilityZones).toEqual(['Florentine', 'Ramat Aviv']);
        });
    });

    describe("clearAvailabilityZones", function () {
        it("should clear the availability zones", function () {
            j.scope.wizardController.node1AvailabilityZone = "1";
            j.scope.wizardController.node2AvailabilityZone = "2";
            j.scope.wizardController.mediatorAvailabilityZone = "3";

            j.scope.clearAvailabilityZones();

            expect(j.scope.wizardController.node1AvailabilityZone).toBeNull();
            expect(j.scope.wizardController.node2AvailabilityZone).toBeNull();
            expect(j.scope.wizardController.mediatorAvailabilityZone).toBeNull();
        });
    });

    describe("next", function () {
        it("should go to next step", function () {
            j.scope.wizardController.next = jest.fn();
            j.scope.next();
            j.$timeout.flush();
            expect(j.scope.wizardController.next).toHaveBeenCalledWith(".ha-public-key");
        });
    });

    describe("subnet selection validation", function () {
        beforeEach(function () {
            j.scope.wizardController.networkRequirements = {
                "node": 8,
                "mediator": 1
            };

            _.set(j.scope.wizardController, "createVpcSelectObjects.vpc", {
                subnets: [
                    {subnetId: 'a', availabilityZone: 'Florentine', availableIps: 9},
                    {subnetId: 'b', availabilityZone: 'Ramat Aviv', availableIps: 9},
                    {subnetId: 'c', availabilityZone: 'Florentine', availableIps: 2}
                ]
            });
        });

        it("should return false if no network requirements are defined", function () {
            j.scope.wizardController.networkRequirements = undefined;
            expect(j.scope.validateSubnetSelection()).toEqual(false);
            expect(j.pidgeon.error).not.toHaveBeenCalled();
        });

        it("should return true if the selection has enough available IPs", function () {
            _.assignIn(j.scope.wizardController.createVpcSelectObjects,
            {
                subnet: {subnetId: "b"},
                node2Subnet: {subnetId: "a"},
                mediatorSubnet: {subnetId: "a"}
            });

            expect(j.scope.validateSubnetSelection()).toEqual(true);
            expect(j.pidgeon.error).not.toHaveBeenCalled();
        });

        it("should return false if the selection has enough available IPs", function () {
            _.assignIn(j.scope.wizardController.createVpcSelectObjects,
                {
                    subnet: {subnetId: "a"},
                    node2Subnet: {subnetId: "a"},
                    mediatorSubnet: {subnetId: "a"}
                });

            expect(j.scope.validateSubnetSelection()).toEqual(false);
            expect(j.pidgeon.error).toHaveBeenCalled();
        });
    });

    describe("backing up data", function () {
        it("should backup data and clear fields upon destroy", function () {
            var createVpcSelectObjects = {
                node2Subnet: {subnetId: "a"},
                mediatorSubnet: {subnetId: "a"},
                node1AvailabilityZone: "Montifiory",
                node2AvailabilityZone:"Lev HaIr",
                mediatorAvailabilityZone: "Rova Hatzofatim"
            };
            j.scope.wizardController.createVpcSelectObjects = createVpcSelectObjects;
            j.scope.wizardController.backupStepData = jest.fn();
            j.scope.wizardController.vpcBeenHereBefore = "HA";
            j.scope.wizardController.availabilityZones = ['kuki', 'puki'];

            j.$rootScope.$broadcast('$destroy');

            expect(mostRecentCall(j.scope.wizardController.backupStepData)).toMatch("ha-vpc", {
                createVpcSelectObjects: createVpcSelectObjects,
                availabilityZones: ['kuki', 'puki'],
                vpcBeenHereBefore: "HA"
            });
            expect(j.scope.wizardController.createVpcSelectObjects).toBeUndefined();
            expect(j.scope.wizardController.vpcBeenHereBefore).toBeNull();
            expect(j.scope.wizardController.availabilityZones).toBeNull();
            expect(j.scope.wizardController.networkRequirements).toBeUndefined();
        });

        it("should restore backed up data upon creation ", function () {
            expect(j.parentScope.getStepData).toHaveBeenCalledWith("ha-vpc");
        });
    });
});