describe('VirginCreateWorkingEnvironmentKeyManagersStepController', () => {
    'use strict';

    var j;    beforeEach(() => {        j = magnifispec.setup();    });

    beforeEach(function () {
        j.$httpBackend.expectGET(j.nocBaseUrl+ '/occm/config').respond({certificateValidityPeriod: 365});
        magnifispec.mockWizardStepController("VirginCreateWorkingEnvironmentKeyManagersStepController", {}, {
            keyManagers: [{publicId: "pid2"}],
            keyManagersCaCertificates:  [{publicId: "pid1"}],
            currentStateName: "key-managers"
        });
    });

    describe("initialize", function () {
        it("Should initialize correctly", function () {
            j.$httpBackend.flush();

            expect(j.scope.wizardController.ontapEncryption.keyManagers).toEqual([{publicId: "pid2"}]);
            expect(j.scope.wizardController.keyManagersSelectedMap).toEqual({"pid2" : true});
            expect(j.scope.wizardController.ontapEncryption.keyManagerCaCertificateId).toEqual("pid1");
            expect(j.scope.certificateValidityPeriodDays).toEqual(365);
            expect(j.scope.certificateValidityPeriod).toEqual('a year');
        });

        it("should pidgeon upon error", function () {
            j.$httpBackend.resetExpectations();
            j.$httpBackend.expectGET(j.nocBaseUrl+ '/occm/config').respond(400, j.errorResponse);

            j.$httpBackend.flush();

            expect(j.pidgeon.error).toHaveBeenCalled();
        });

        afterEach(function () {
            expect(j.pidgeon.clear).toHaveBeenCalledWith("create-key-managers");
        });
    });

    describe("initialize2 - the road not taken", function () {
        beforeEach(function () {
            j.scope.wizardController.ontapEncryption = {keyManagers: []};
            j.scope.wizardController.keyManagersSelectedMap = {};
        });

        it("should not select key manager automatically if there is more than one", function () {
            j.scope.wizardController.keyManagers =  [{name: 'a', publicId: 1}, {name: 'b', publicId: 2}];
            j.scope.initialize();
            expect(j.scope.wizardController.ontapEncryption.keyManagers).toEqual([]);
            expect(j.scope.wizardController.keyManagersSelectedMap).toEqual({});
        });

        it("should not select key manager CA certificate automatically if there is more than one", function () {
            j.scope.wizardController.keyManagersCaCertificates = [{publicId: "pid1"}, {publicId: "pid2"}];
            j.scope.initialize();
            expect(j.scope.wizardController.ontapEncryption.keyManagerCaCertificateId).toBeUndefined();
        });

        it('Should recreate from failed request successfully', () => {
            j.scope.wizardController.keyManagers = [{publicId: 'km1'}, {publicId: 'km2'}];
            j.scope.wizardController.keyManagersCaCertificates = [{publicId: 'kmcac1'}, {publicId: 'kmcac2'}];
            j.scope.wizardController.failedRequestType = 'VSA';
            j.scope.wizardController.failedRequest = {
                ontapEncryptionParameters: {
                    keyManagerCaCertificateId: 'kmcac2',
                    keyManagerIds: ['km2']
                }
            };

            j.scope.initialize();

            expect(j.scope.wizardController.ontapEncryption.keyManagers).toEqual([{publicId: 'km2'}]);
            expect(j.scope.wizardController.keyManagersSelectedMap).toEqual({'km2': true});
            expect(j.scope.wizardController.ontapEncryption.keyManagerCaCertificateId).toEqual('kmcac2');
        });
    });

    describe("keyManagerClicked", function () {
        beforeEach(function () {
            j.scope.wizardController.ontapEncryption.keyManagers = [];
        });
        it("should mark key manager as selected and add it to the list when clicked", function () {
            j.scope.keyManagerClicked({publicId: "keyManager1"});
            expect(j.scope.wizardController.ontapEncryption.keyManagers).toEqual([{publicId: "keyManager1"}]);
            expect(j.scope.wizardController.keyManagersSelectedMap["keyManager1"]).toBeTruthy();
        });
        it("should mark key manager as not selected and remove it from the list when clicked", function () {
            var keyManager1 = {publicId: "keyManager1"};
            j.scope.wizardController.keyManagersSelectedMap["keyManager1"] = true;

            j.scope.wizardController.ontapEncryption.keyManagers = [keyManager1, {publicId: "keyManager2"}];
            j.scope.keyManagerClicked(keyManager1);
            expect(j.scope.wizardController.ontapEncryption.keyManagers).toEqual([{publicId: "keyManager2"}]);
            expect(j.scope.wizardController.keyManagersSelectedMap["keyManager1"]).toBeFalsy();
        });

        it("should bring the wizard back to the current step when scrolled up to here from advanced step", function () {
            j.scope.wizardController.path = {"key-managers": "path to this step"};
            j.scope.wizardController.setWizardStep = jest.fn();
            j.scope.wizardController.currentStateName = "advanced-step";

            j.scope.keyManagerClicked({publicId: "keyManager1"});

            expect(j.scope.wizardController.setWizardStep).toHaveBeenCalledWith("path to this step");
        });
    });

    describe("next", function () {
        beforeEach(function () {
            j.scope.wizardController.next = jest.fn();
        });

        describe("Should move to next step", function () {
            beforeEach(function () {
                j.scope.wizardController.ontapEncryption.keyManagers = [
                    {name: "km1", usernameField: "value1", usernameValue: "value2"},
                    {name: "km2", usernameField: "value1", usernameValue: "value2"}
                ]; //Selecting 2 key manager
            });

            it("WE type VSA", function () {
                j.scope.wizardController.workingEnvironmentType = "VSA";
                j.scope.wizardController.workingEnvironmentTypeAndProvider = {providerName : "AWS"};

                j.scope.next();
            });

            it("WE type HA", function () {
                j.scope.wizardController.workingEnvironmentType = "HA";

                j.scope.next();
            });

            it("should move to the right step for Azure we type", function () {
                j.scope.wizardController.workingEnvironmentType = "VSA";
                j.scope.wizardController.workingEnvironmentTypeAndProvider = {providerName : "Azure"};

                j.scope.next();
            });

            afterEach(() => {
                expect(j.scope.wizardController.next).toHaveBeenCalledWith('.byol');
            });
        });

        describe("Should not move to next step", function () {
            beforeEach(function () {
                j.nocDialog.alert = jest.fn();
            });

            it("if key managers were not selected", function () {
                j.scope.wizardController.ontapEncryption.keyManagers = []; //None key managers were selected

                j.scope.next();

                expect(j.nocDialog.alert).toHaveBeenCalledWith("You must select 1 to 4 Key Managers before you can proceed.");
            });

            it("Should not move to next step if too many key managers were selected", function () {
                j.scope.wizardController.ontapEncryption.keyManagers = [{name: "km1"}, {name: "km2"}, {name: "km3"}, {name: "km4"}, {name: "km5"}];

                j.scope.next();

                expect(j.nocDialog.alert).toHaveBeenCalledWith("You must select 1 to 4 Key Managers before you can proceed.");
            });

            it("Should not move to next step if the selected key managers have different FIPS fields", function () {
                j.scope.wizardController.ontapEncryption.keyManagers = [{name: "km1", usernameField:"value1"}, {name: "km2", usernameField: "value2"}];

                j.scope.next();

                expect(j.nocDialog.alert).toHaveBeenCalledWith('Key manager client authentication fields should be the same for all selected key managers.');
            });

            it("Should not move to next step if the selected key managers have different FIPS values", function () {
                j.scope.wizardController.ontapEncryption.keyManagers = [{name: "km1", usernameValue:"value1"}, {name: "km2", usernameValue: "value2"}];

                j.scope.next();

                expect(j.nocDialog.alert).toHaveBeenCalledWith('Key manager client authentication fields should be the same for all selected key managers.');
            });

            afterEach(function () {
                expect(j.scope.wizardController.next).not.toHaveBeenCalled();
            });
        });
    });

    describe("occmCertificateExpiresInCertificateValidityPeriod", function () {
        beforeEach(function () {
            j.scope.wizardController.certificate = {endDate: "some_date"};
        });
        it("should call nocUtil.certificateExpirationState: case warning", function () {
            jest.spyOn(j.nocUtil, "certificateExpirationState").mockReturnValue({state: 'certificate-warning', message: ""});
            expect(j.scope.occmCertificateExpiresInCertificateValidityPeriod()).toEqual(true);
        });

        it("should call nocUtil.certificateExpirationState: case valid-long", function () {
            jest.spyOn(j.nocUtil, "certificateExpirationState").mockReturnValue({state: 'certificate-valid-long', message: ""});
            expect(j.scope.occmCertificateExpiresInCertificateValidityPeriod()).toEqual(false);
        });
    });

    describe("backing up data", function () {
        it("should backup data and clear fields upon destroy", function () {
            j.scope.wizardController.ontapEncryption = {key1: 'value1'};
            j.scope.wizardController.keyManagersSelectedMap = {key2: 'value2'};
            j.$rootScope.$broadcast('$destroy');
            expect(j.parentScope.backupStepData).toHaveBeenCalledWith("create-key-managers", {
                ontapEncryption: {key1: 'value1'},
                keyManagersSelectedMap : {key2: 'value2'}
            });
            expect(j.scope.wizardController.ontapEncryption).toBeNull();
            expect(j.scope.wizardController.keyManagersSelectedMap).toBeNull();
        });

        it("should restore backed up data upon creation ", function () {
            expect(j.parentScope.getStepData).toHaveBeenCalledWith("create-key-managers");
        });
    });
});
