describe('VirginCreateWorkingEnvironmentLicenseStepController', () => {
    'use strict';

    var j;    beforeEach(() => {        j = magnifispec.setup();    });

    beforeEach(inject((ManifestMapper, MetadataResource) => {
        j.$rootScope.runningInIC = false;
        j.$rootScope.currentUser = {tenantId: 2};

        jest.spyOn(j.configService, 'getConfig').mockReturnValue(j.$q.when({}));
        jest.spyOn(ManifestMapper, 'getAvailableLicenses').mockReturnValue([{
            type: 'cot-explore-paygo',
            name: 'Cloud Volumes ONTAP Explore',
            description: 'Suitable for smaller capacity applications. Supports up to 2 TB of underlying AWS storage.'
        }, {
            type: 'cot-standard-paygo',
            name: 'Cloud Volumes ONTAP Standard',
            description: 'Flexible performance and larger capacity for a wider range of applications. Supports up to 10 TB of underlying AWS storage.'
        }, {
            type: 'cot-premium-paygo',
            name: 'Cloud Volumes ONTAP Premium',
            description: 'Flexible performance and expanded capacity for a wider range of applications. Supports up to 368 TB of underlying AWS storage.'
        }, {
            type: 'cot-premium-byol',
            name: 'Cloud Volumes ONTAP BYOL',
            description: 'Flexible performance and expanded capacity, with a longer term subscription. Supports up to 368 TB of underlying AWS storage.'
        }]);
        jest.spyOn(MetadataResource, 'legacyInstanceTypes').mockReturnValue(j.$q.reject());

        magnifispec.mockWizardStepController('VirginCreateWorkingEnvironmentLicenseStepController', {}, {
            workingEnvironmentTypeAndProvider: {providerName: 'AWS'},
            createVpcSelectObjects: {region: {code: 'us-west-1'}},
            currentStateName: 'license'
        });
        magnifispec.processPromise();

        j.$state.go = jest.fn();
    }));

    describe('setInstanceType', () => {
        let getDefaultInstanceSpy;

        beforeEach(inject((ManifestMapper) => {
            j.scope.wizardController.selectedLicense = {type: 'explore'};
            getDefaultInstanceSpy = jest.spyOn(ManifestMapper, 'getDefaultInstance');
        }));

        it('should set the instanceType to parameter if passed', () => {
            j.scope.setInstanceType('instance-type-1');

            expect(j.scope.wizardController.selectedInstanceType).toEqual('instance-type-1');
        });

        it('should set the instanceType to manifest default if exist when no parameter passed', () => {
            getDefaultInstanceSpy.mockReturnValue('manifestDefault');

            j.scope.setInstanceType();

            expect(j.scope.wizardController.selectedInstanceType).toEqual('manifestDefault');
        });

        it('should set the instanceType to first enabled instance from license options if not other default exists', () => {
            jest.spyOn(j.scope, 'isInstanceTypeDisabled').mockImplementation((instance) => instance === 'licenseInstance1');
            j.scope.wizardController.selectedLicense.availableInstances = ['licenseInstance1', 'licenseInstance2'];

            j.scope.setInstanceType();

            expect(j.scope.wizardController.selectedInstanceType).toEqual('licenseInstance2');
        });
    });


    describe('selectOntapVersion', () => {
        beforeEach(() => {
            jest.spyOn(j.scope, 'getLicenceOptions').mockImplementation(() => {});
            jest.spyOn(j.scope, 'selectLicense').mockImplementation(() => {});
            j.scope.wizardController.selectedLicense = {type: 'license1'};
            j.scope.ontapVersions = ['version0', 'version1'];
        });

        it('should get versions array', () => {
            j.scope.selectOntapVersion();

            expect(j.scope.wizardController.selectedOntapVersion).toEqual('version0');
            expect(j.scope.getLicenceOptions).toHaveBeenCalledWith();
            expect(j.scope.selectLicense).toHaveBeenCalledWith();
        });

        it('should bring the wizard back to the current step when scrolled up to here from advanced step', () => {
            j.scope.wizardController.path = {'license': 'path to this step'};
            j.scope.wizardController.setWizardStep = jest.fn();
            j.scope.wizardController.currentStateName = 'advanced-step';

            j.scope.selectOntapVersion();

            expect(j.scope.wizardController.setWizardStep).toHaveBeenCalledWith('path to this step');
        });
    });

    describe('getLicenceOptions', () => {
        it('should filter out BYOL', () => {
            j.scope.getLicenceOptions();
            expect(j.scope.licenseOptions).toEqual([{
                type: 'cot-explore-paygo',
                name: 'Cloud Volumes ONTAP Explore',
                description: 'Suitable for smaller capacity applications. Supports up to 2 TB of underlying AWS storage.',
                _iconName: 'explore-license'
            }, {
                type: 'cot-standard-paygo',
                name: 'Cloud Volumes ONTAP Standard',
                description: 'Flexible performance and larger capacity for a wider range of applications. Supports up to 10 TB of underlying AWS storage.',
                _iconName: 'standard-license'
            }, {
                type: 'cot-premium-paygo',
                name: 'Cloud Volumes ONTAP Premium',
                description: 'Flexible performance and expanded capacity for a wider range of applications. Supports up to 368 TB of underlying AWS storage.',
                _iconName: 'premium-license'
            }
            ]);
        });
    });

    describe('selectLicense', () => {
        beforeEach(() => {
            jest.spyOn(j.scope, 'setInstanceType').mockImplementation(() => {});
        });

        it('should keep selected license', () => {
            j.scope.wizardController.selectedLicense = 'current one';

            j.scope.selectLicense();

            expect(j.scope.wizardController.selectedLicense).toEqual('current one');
        });

        it('should select default license from options', () => {
            j.scope.licenseOptions = [
                {type: 'cot-explore-paygo', name: 'Cloud Volumes ONTAP Explore', default: false},
                {type: 'cot-standard-paygo', name: 'Cloud Volumes ONTAP Standard', default: true}
            ];

            j.scope.selectLicense();

            expect(j.scope.wizardController.selectedLicense).toEqual(
                {type: 'cot-standard-paygo', name: 'Cloud Volumes ONTAP Standard', default: true}
            );
        });

        it('should select first license from options if no one is default', () => {
            j.scope.licenseOptions = [
                {type: 'cot-explore-paygo', name: 'Cloud Volumes ONTAP Explore'},
                {type: 'cot-standard-paygo', name: 'Cloud Volumes ONTAP Standard'}
            ];

            j.scope.selectLicense();

            expect(j.scope.wizardController.selectedLicense).toEqual(
                {type: 'cot-explore-paygo', name: 'Cloud Volumes ONTAP Explore'}
            );
        });

        it('should handle no licenses at all', () => {
            j.scope.licenseOptions = [];

            j.scope.selectLicense();

            expect(j.scope.wizardController.selectedLicense).toEqual({name: 'No license available'});
        });

        it('should call setInstanceType with instance parameter if passed', () => {
            j.scope.selectLicense(null, 'parameter');

            expect(j.scope.setInstanceType).toHaveBeenCalledWith('parameter');
        });
    });

    describe('initialize', () => {
        beforeEach(() => {
            jest.spyOn(j.scope, 'getLegacyInstances').mockReturnValue(j.$q.when('legacyTypes'));
            jest.spyOn(j.scope, 'getOntapVersions').mockReturnValue(['version1', 'version2']);
            jest.spyOn(j.scope, 'selectOntapVersion').mockImplementation(() => {});
            jest.spyOn(j.scope, 'getLicenceOptions').mockImplementation(() => {});
            jest.spyOn(j.scope, 'selectLicense').mockImplementation(() => {});
        });

        it('should initialize controller', () => {
            j.scope.wizardController.selectedInstanceType = 'instance1';

            j.scope.initialize();
            magnifispec.processPromise();

            expect(j.scope.disabledInstanceMessage).toEqual('This instance type does not support ONTAP Encryption');
            expect(j.scope.legacyInstanceTypes).toEqual('legacyTypes');
            expect(j.scope.ontapVersions).toEqual(['version1', 'version2']);
            expect(j.scope.selectLicense).toHaveBeenCalledWith(null, 'instance1');
        });

        it('should select ONTAP version if not selected', () => {
            j.scope.initialize();
            magnifispec.processPromise();

            expect(j.scope.selectOntapVersion).toHaveBeenCalled();
        });

        it('should get license options for existing ONTAP version if selected', () => {
            j.scope.wizardController.selectedOntapVersion = 'version1';

            j.scope.initialize();
            magnifispec.processPromise();

            expect(j.scope.selectOntapVersion).not.toHaveBeenCalled();
            expect(j.scope.getLicenceOptions).toHaveBeenCalled();
        });

        it('should show a pidgeon if no license available for the region', () => {
            j.scope.licenseOptions = [];

            j.scope.initialize();
            magnifispec.processPromise();

            expect(j.pidgeon.error).toHaveBeenCalledWith('No licenses available for selected region', null, 'licenses');
        });

        it('should initialize tenancy in AWS', () => {
            j.scope.initialize();
            magnifispec.processPromise();

            expect(j.scope.tenancies).toEqual([
                {name: 'Shared', requestName: 'default'},
                {name: 'Dedicated', requestName: 'dedicated'}
            ]);
            expect(j.scope.disableTenancyChoice).toEqual(false);
            expect(j.scope.wizardController.tenancy).toEqual({name: 'Shared', requestName: 'default'});
        });

        it('should handle dedicated VPC', () => {
            j.scope.wizardController.createVpcSelectObjects.vpc = {tenancy: 'dedicated'};

            j.scope.initialize();
            magnifispec.processPromise();

            expect(j.scope.tenancies).toEqual([
                {name: 'Shared', requestName: 'default'},
                {name: 'Dedicated', requestName: 'dedicated'}
            ]);
            expect(j.scope.disableTenancyChoice).toEqual(true);
            expect(j.scope.wizardController.tenancy).toEqual({name: 'Dedicated', requestName: 'dedicated'});
        });

        it('should handle failedRequestType correctly', () => {
            j.scope.licenseOptions = [{
                type: 'cot-explore-paygo',
                name: 'Cloud Volumes ONTAP Explore'
            }, {
                type: 'cot-standard-paygo',
                name: 'Cloud Volumes ONTAP Standard'
            }];
            j.scope.wizardController.failedRequestType = true;
            j.scope.wizardController.failedRequest = {
                vsaMetadata: {
                    ontapVersion: 'ontap123',
                    licenseType: 'cot-explore-paygo',
                    instanceType: 'm-large-cookie'
                },
                instanceTenancy: 'dedicated'
            };
            jest.spyOn(j.scope, 'setInstanceType').mockImplementation(() => {});

            j.scope.initialize();
            magnifispec.processPromise();

            expect(j.scope.wizardController.selectedOntapVersion).toEqual('ontap123');
            expect(j.scope.selectOntapVersion).toHaveBeenCalled();
            expect(j.scope.wizardController.selectedLicense).toEqual({
                type: 'cot-explore-paygo',
                name: 'Cloud Volumes ONTAP Explore'
            });
            expect(j.scope.selectLicense).toHaveBeenCalled();
            expect(j.scope.setInstanceType).toHaveBeenCalledWith('m-large-cookie');
            expect(j.scope.tenancy).toEqual({name: 'Dedicated', requestName: 'dedicated'});
        });

        afterEach(() => {
            expect(j.pidgeon.clear).toHaveBeenCalled();
            expect(j.scope.getLegacyInstances).toHaveBeenCalledWith('AWS');
        });
    });

    describe('isLicenseDisabled', () => {
        it('should return true for licenses with no legal instances', () => {
            j.scope.wizardController.encryptionType = 'ONTAP';
            expect(j.scope.isLicenseDisabled({_supportsOntapEncryption: false})).toEqual(true);
        });
        it('should return false for licenses with legal instances', () => {
            j.scope.wizardController.encryptionType = 'ONTAP';
            expect(j.scope.isLicenseDisabled({_supportsOntapEncryption: true})).toEqual(false);
        });
        it('should return false for everything if it\'s not in ontap encryption mode', () => {
            j.scope.wizardController.encryptionType = 'NONE';
            expect(j.scope.isLicenseDisabled({_supportsOntapEncryption: false})).toEqual(false);
        });
    });

    describe('isInstanceTypeDisabled', () => {
        beforeEach(() => {
            j.scope.wizardController.selectedOntapVersion = 'ONTAP-8.3.1P1.T4';
            j.scope.encryptionEnabledInstanceTypes = {'ONTAP-8.3.1P1.T4': ['lala', 'lili']};
        });
        it('should return true if should be disabled', () => {
            j.scope.wizardController.encryptionType = 'ONTAP';
            expect(j.scope.isInstanceTypeDisabled('lulu')).toEqual(true);
        });
        it('should return false if it\'s not disabled', () => {
            j.scope.wizardController.encryptionType = 'ONTAP';
            jest.spyOn(j.ManifestMapper, 'getEncryptionEnabledInstanceTypes').mockReturnValue(['lala']);
            expect(j.scope.isInstanceTypeDisabled('lala')).toEqual(false);
        });
        it('should return false if it\'s not in ontap encryption mode', () => {
            j.scope.wizardController.encryptionType = 'NONE';
            expect(j.scope.isInstanceTypeDisabled('lulu')).toEqual(false);
        });
    });

    describe('backing up data', () => {
        it('should clear fields upon destroy', () => {
            j.scope.wizardController.selectedOntapVersion = 'value2';
            j.scope.wizardController.selectedLicense = 'value2';
            j.scope.wizardController.selectedInstanceType = 'value2';
            j.scope.wizardController.tenancy = {name: 'Shared', requestName: 'default'};

            j.$rootScope.$broadcast('$destroy');
            expect(j.parentScope.backupStepData).toHaveBeenCalledWith('license', {
                selectedOntapVersion: 'value2',
                selectedLicense: 'value2',
                selectedInstanceType: 'value2',
                tenancy: {name: 'Shared', requestName: 'default'}
            });
            expect(j.scope.wizardController.selectedOntapVersion).toBeNull();
            expect(j.scope.wizardController.selectedLicense).toBeNull();
            expect(j.scope.wizardController.selectedInstanceType).toBeNull();
            expect(j.scope.wizardController.tenancy).toBeNull();
        });

        it('should restore backed up data upon creation ', () => {
            expect(j.parentScope.getStepData).toHaveBeenCalledWith('license');
        });
    });

    describe('isInstanceLegacy', () => {
        beforeEach(() => {
            j.scope.legacyInstanceTypes = ['instanceType1', 'instanceType2'];
        });

        it('should return true for instance in legacy list', () => {
            expect(j.scope.isInstanceLegacy('instanceType2', 'licenseNotByol')).toEqual(true);
            expect(j.scope.isInstanceLegacy('instanceType3', 'licenseNotByol')).toEqual(false);
        });

        it('should return false for any license for BYOL license', () => {
            expect(j.scope.isInstanceLegacy('instanceType2', 'Cloud Volumes ONTAP BYOL')).toEqual(false);
        });
    });

    describe('next', () => {
        it('should call the right function', () => {
            jest.spyOn(j.scope, 'continueAfterLicenseOrPackagesStep').mockImplementation(() => {});

            j.scope.next();

            expect(j.scope.continueAfterLicenseOrPackagesStep).toHaveBeenCalledWith('.tiering');
        });
    });
});
