describe('VirginCreateWorkingEnvironmentVPCStepController', () => {
    'use strict';

    var j;    beforeEach(() => {        j = magnifispec.setup();    });

    beforeEach(() => {
        magnifispec.mockWizardStepController('VirginCreateWorkingEnvironmentVPCStepController');
    });

    describe("next", function () {
        beforeEach(function () {
            j.$state.go = jest.fn();
        });

        describe("should call wizardController.next to handle form validity and go to next step", function () {
            beforeEach(function () {
                j.scope.wizardController.selectedRegion = "us-west1";
                j.scope.wizardController.selectedVPC = {name: "i am not null"};
                j.scope.wizardController.next = jest.fn();
            });

            it("go to license step if creating VSA", function () {
                j.scope.wizardController.workingEnvironmentType = "VSA";
                j.scope.next();
                expect(j.scope.wizardController.next).toHaveBeenCalledWith(".encryption");
            });
        });
    });

    describe('recoverFailedRequestKey', () => {
        it('should select the key from failed request if available', () => {
            j.scope.wizardController.keyPairs = [{name: 'key1'}, {name: 'key2'}, {name: 'key3'}];
            j.scope.wizardController.failedRequest = {clusterKeyPairName: 'key2'};

            j.scope.recoverFailedRequestKey();

            expect(j.scope.wizardController.createVpcSelectObjects.key).toEqual({name: 'key2'});
            expect(j.scope.wizardController.createVpcSelectObjects.access).toEqual({name: 'Key Pair', requestValue: 'key'});
        });

        it('should fallback to default key pair if key from failed request if unavailable', () => {
            j.scope.wizardController.keyPairs = [{name: 'key1'}, {name: 'key3'}];
            j.scope.wizardController.failedRequest = {clusterKeyPairName: 'key2'};

            j.scope.recoverFailedRequestKey();

            expect(j.scope.wizardController.createVpcSelectObjects.key).toEqual({name: 'key1'});
            expect(j.scope.wizardController.createVpcSelectObjects.access).toEqual({name: 'Key Pair', requestValue: 'key'});
        });

        it('should do nothing if no key pair exists in failed request', () => {
            j.scope.wizardController.failedRequest = {};

            j.scope.recoverFailedRequestKey();

            expect(j.scope.wizardController.createVpcSelectObjects.key).not.toBeDefined();
            expect(j.scope.wizardController.createVpcSelectObjects.access).not.toEqual({
                name: 'Key',
                requestValue: 'key'
            });
        });
    });
});
