(() => {
    'use strict';

    angular.module('noc').controller('RansomwareProtectionController', function ($scope, $state, workingEnvironmentResource, volumesResource, nocHttp, pidgeon) {

        const ACTIVE_ACTION_NAME_SNAPSHOT = 'Activate Snapshot Policy';
        const ACTIVE_ACTION_NAME_FPOLICY = 'Activate FPolicy';

        $scope.initializeController = () => {
            $scope.activeActionsPollingStopper = _.noop;
            const currentActiveActions = _.get($scope.selectedWorkingEnvironment, 'activeActions');
            $scope.isActivatingSnapshots = _.includes(currentActiveActions, ACTIVE_ACTION_NAME_SNAPSHOT);
            $scope.isActivatingFpolicy = _.includes(currentActiveActions, ACTIVE_ACTION_NAME_FPOLICY);
            if ($scope.isActivatingSnapshots || $scope.isActivatingFpolicy) {
                $scope.startActiveActionsPolling();
            }
            if (!$scope.volumes) {
                volumesResource.getVolumes(nocHttp, $scope.selectedWorkingEnvironment.publicId, $scope.selectedWorkingEnvironment._pathPrefix, false)
                    .then((volumes) => {
                        $scope.volumes = volumes;
                    });
            }
        };

        $scope.$on('activateSnapshot', () => {
            $scope.isAsyncOperationError = false;
            workingEnvironmentResource.activateSnapshot(nocHttp, $scope.selectedWorkingEnvironment)
                .then(() => {
                    $scope.isActivatingSnapshots = true;
                    if ($scope.activeActionsPollingStopper === _.noop) {
                        $scope.startActiveActionsPolling();
                    }
                })
                .catch(() => {
                    $scope.isAsyncOperationError = true;
                });
        });

        $scope.$on('activateFpolicy', () => {
            $scope.isAsyncOperationError = false;
            workingEnvironmentResource.activateFpolicy(nocHttp, $scope.selectedWorkingEnvironment)
                .then(() => {
                    $scope.isActivatingFpolicy = true;
                    if ($scope.activeActionsPollingStopper === _.noop) {
                        $scope.startActiveActionsPolling();
                    }
                })
                .catch(() => {
                    $scope.isAsyncOperationError = true;
                });
        });

        $scope.$on('disableFpolicy', () => {
            workingEnvironmentResource.disableFpolicy(nocHttp, $scope.selectedWorkingEnvironment);
        });

        $scope.startActiveActionsPolling = () => {
            $scope.activeActionsPollingStopper = workingEnvironmentResource.startVsaActiveActionsPolling($scope.selectedWorkingEnvironment, 20);
        };

        $scope.$on($scope.selectedWorkingEnvironment.publicId + workingEnvironmentResource.VSA_ACTIVE_ACTIONS_GET_EVENT, (event, newActiveActions) => {
            pidgeon.clear('vsa-state');
            $scope.handleActiveActionsUpdate(newActiveActions);
        });

        $scope.$on($scope.selectedWorkingEnvironment.publicId + workingEnvironmentResource.VSA_ACTIVE_ACTIONS_ERROR_EVENT, (event, error) => {
            pidgeon.error(pidgeon.serverErrorPrefix + 'retrieving Cloud Volumes ONTAP state for working environment: ' + $scope.selectedWorkingEnvironment.name, error, 'vsa-state');
        });

        $scope.handleActiveActionsUpdate = (newActiveActions) => {
            // This means the action is done
            if ($scope.isActivatingSnapshots && !_.includes(newActiveActions, ACTIVE_ACTION_NAME_SNAPSHOT)) {
                volumesResource.getVolumes(nocHttp, $scope.selectedWorkingEnvironment.publicId, $scope.selectedWorkingEnvironment._pathPrefix, false)
                    .then((volumes) => {
                        $scope.volumes = volumes;
                    });
                $scope.isActivatingSnapshots = false;
            }
            if ($scope.isActivatingFpolicy && !_.includes(newActiveActions, ACTIVE_ACTION_NAME_FPOLICY)) {
                workingEnvironmentResource.getWorkingEnvironmentByPublicId(nocHttp, $scope.selectedWorkingEnvironment.publicId, $scope.selectedWorkingEnvironment._pathPrefix, 'fpolicyProperties')
                    .then((workingEnvironmentResponse) => {
                        $scope.selectedWorkingEnvironment.fpolicyProperties = _.get(workingEnvironmentResponse, 'fpolicyProperties');
                        $scope.isActivatingFpolicy = false;

                        if (!$scope.isActivatingSnapshots && !$scope.isActivatingFpolicy) {
                            $scope.activeActionsPollingStopper();
                        }
                    });
            }
            if (!$scope.isActivatingSnapshots && !$scope.isActivatingFpolicy) {
                $scope.activeActionsPollingStopper();
            }
        };

        $scope.$on('$destroy', () => {
            $scope.activeActionsPollingStopper();
        });

        $scope.updateSelectedWorkingEnvironment($state.params.workingEnvironmentId)
            .then($scope.initializeController, $scope.backToGallery);
    });
})();
