describe('Ransomware Protection Controller', () => {
    var j;    beforeEach(() => {        j = magnifispec.setup();    });
    let volumesResource;
    let workingEnvironment;
    let getWorkingEnvironmentByPublicIdSpy;

    beforeEach(inject((_volumesResource_) => {
        volumesResource = _volumesResource_;
        jest.spyOn(volumesResource, 'getVolumes').mockReturnValue(j.$q.when([
            {
                name: 'vol1',
                volumeType: 'rw',
                rootVolume: false,
                moving: false,
                state: 'online',
                snapshotPolicy: 'default'
            },
            {
                name: 'vol2',
                volumeType: 'rw',
                rootVolume: false,
                moving: false,
                state: 'online',
                snapshotPolicy: 'none'
            },
            {
                name: 'vol3',
                volumeType: 'rw',
                rootVolume: false,
                moving: false,
                state: 'online',
                snapshotPolicy: 'none'
            },
            {
                name: 'vol4',
                volumeType: 'rw',
                rootVolume: false,
                moving: false,
                state: 'online',
                snapshotPolicy: '111'
            }]));
        magnifispec.mockTabsController('RansomwareProtectionController', {}, {
            selectedWorkingEnvironment: {_pathPrefix: '/vsa/', publicId: 1, name: 'WE1', activeActions: []},
            updateSelectedWorkingEnvironment: j.$q.when
        });
        jest.spyOn(j.workingEnvironmentResource, 'startVsaActiveActionsPolling').mockReturnValue(jest.fn());
        jest.spyOn(j.workingEnvironmentResource, 'activateSnapshot').mockReturnValue(j.$q.when());
        magnifispec.processPromise();
    }));

    describe('initializeController', () => {
        beforeEach(() => {
            jest.spyOn(j.scope, 'startActiveActionsPolling').mockImplementation(() => {});
        });

        it('should initialize properly in case `active actions` includes none', () => {
            j.scope.initializeController();

            expect(j.scope.isActivatingSnapshots).toEqual(false);
            expect(j.scope.isActivatingFpolicy).toEqual(false);
            expect(j.scope.startActiveActionsPolling).not.toHaveBeenCalled();
        });

        it('should initialize properly in case `active actions` includes only `Activate Snapshot Policy`', () => {
            j.scope.selectedWorkingEnvironment.activeActions = ['Activate Snapshot Policy'];
            j.scope.initializeController();

            expect(j.scope.isActivatingSnapshots).toEqual(true);
            expect(j.scope.isActivatingFpolicy).toEqual(false);
            expect(j.scope.startActiveActionsPolling).toHaveBeenCalled();
        });

        it('should initialize properly in case `active actions` includes only `Activate FPolicy`', () => {
            j.scope.selectedWorkingEnvironment.activeActions = ['Activate FPolicy'];
            j.scope.initializeController();

            expect(j.scope.isActivatingSnapshots).toEqual(false);
            expect(j.scope.isActivatingFpolicy).toEqual(true);
            expect(j.scope.startActiveActionsPolling).toHaveBeenCalled();
        });

        it('should initialize properly in case `active actions` includes both', () => {
            j.scope.selectedWorkingEnvironment.activeActions = ['Activate Snapshot Policy', 'Activate FPolicy'];
            j.scope.initializeController();

            expect(j.scope.isActivatingSnapshots).toEqual(true);
            expect(j.scope.isActivatingFpolicy).toEqual(true);
            expect(j.scope.startActiveActionsPolling).toHaveBeenCalled();
        });

        afterEach(() => {
            expect(j.scope.volumes).toEqual([
                {
                    name: 'vol1',
                    volumeType: 'rw',
                    rootVolume: false,
                    moving: false,
                    state: 'online',
                    snapshotPolicy: 'default'
                },
                {
                    name: 'vol2',
                    volumeType: 'rw',
                    rootVolume: false,
                    moving: false,
                    state: 'online',
                    snapshotPolicy: 'none'
                },
                {
                    name: 'vol3',
                    volumeType: 'rw',
                    rootVolume: false,
                    moving: false,
                    state: 'online',
                    snapshotPolicy: 'none'
                },
                {
                    name: 'vol4',
                    volumeType: 'rw',
                    rootVolume: false,
                    moving: false,
                    state: 'online',
                    snapshotPolicy: '111'
                }]);
        });
    });

    describe('startActiveActionsPolling', () => {
        it('should start polling', () => {
            j.scope.startActiveActionsPolling();

            expect(j.workingEnvironmentResource.startVsaActiveActionsPolling).toHaveBeenCalledWith(j.scope.selectedWorkingEnvironment, 20);
        });
    });

    describe('on activateSnapshot broadcast', () => {
        beforeEach(() => {
            j.scope.activeActionsPollingStopper = _.noop;
            jest.spyOn(j.scope, 'startActiveActionsPolling').mockImplementation(() => {});
        });

        it('should call activateSnapshot API', () => {
            j.scope.$broadcast('activateSnapshot');
            magnifispec.processPromise();

            expect(j.workingEnvironmentResource.activateSnapshot).toHaveBeenCalled();
            expect(j.scope.isActivatingSnapshots).toEqual(true);
        });

        it('should start polling if not started yet', () => {
            j.scope.$broadcast('activateSnapshot');
            magnifispec.processPromise();

            expect(j.scope.startActiveActionsPolling).toHaveBeenCalled();
        });

        it('should not start polling if already started', () => {
            j.scope.activeActionsPollingStopper = jest.fn();
            j.scope.$broadcast('activateSnapshot');
            magnifispec.processPromise();

            expect(j.scope.startActiveActionsPolling).not.toHaveBeenCalled();
        });
    });

    //TODO: add tests for "on activateFpolicy broadcast"

    describe('on activeActions broadcast', () => {
        it('should handle active actions get broadcast', () => {
            jest.spyOn(j.scope, 'handleActiveActionsUpdate').mockImplementation(() => {});
            j.scope.$broadcast(j.scope.selectedWorkingEnvironment.publicId + j.workingEnvironmentResource.VSA_ACTIVE_ACTIONS_GET_EVENT, 'newActiveActions');

            expect(j.pidgeon.clear).toHaveBeenCalledWith('vsa-state');
            expect(j.scope.handleActiveActionsUpdate).toHaveBeenCalledWith('newActiveActions');
        });

        it('should handle active actions error broadcast', () => {
            j.scope.$broadcast(j.scope.selectedWorkingEnvironment.publicId + j.workingEnvironmentResource.VSA_ACTIVE_ACTIONS_ERROR_EVENT, 'error');

            expect(j.pidgeon.error).toHaveBeenCalledWith(j.pidgeon.serverErrorPrefix + 'retrieving Cloud Volumes ONTAP state for working environment: ' + j.scope.selectedWorkingEnvironment.name, 'error', 'vsa-state');
        });
    });

    beforeEach(inject(() => {
        workingEnvironment = {
            _pathPrefix: '/vsa/',
            publicId: '1',
            name: 'we1',
            fpolicyProperties: {status: 0}
        };
        getWorkingEnvironmentByPublicIdSpy = jest.spyOn(j.workingEnvironmentResource, 'getWorkingEnvironmentByPublicId').mockReturnValue(j.$q.when(workingEnvironment));
        magnifispec.processPromise();

        getWorkingEnvironmentByPublicIdSpy.mockReturnValue(j.$q.when({
            fpolicyProperties: {status: 1}
        }));

    }));

    describe('handleActiveActionsUpdate', () => {
        it('in case activation is completed', () => {
            j.scope.activeActionsPollingStopper = jest.fn();
            j.scope.isActivatingSnapshots = true;
            j.scope.isActivatingFpolicy = true;

            j.scope.handleActiveActionsUpdate([]);
            magnifispec.processPromise();

            // The first getVolumes is called in initializeController - so making sure it is called again
            expect(volumesResource.getVolumes.mock.calls.length).toEqual(2);
            expect(j.scope.isActivatingSnapshots).toEqual(false);
            expect(j.scope.isActivatingFpolicy).toEqual(false);
            expect(j.scope.activeActionsPollingStopper).toHaveBeenCalled();
        });

        it('in case activation is not completed', () => {
            j.scope.activeActionsPollingStopper = jest.fn();
            j.scope.isActivatingSnapshots = true;
            j.scope.isActivatingFpolicy = true;
            j.scope.handleActiveActionsUpdate(['Activate Snapshot Policy', 'Activate FPolicy']);

            // The first getVolumes is called in initializeController - so making sure it is not called again
            expect(volumesResource.getVolumes.mock.calls.length).toEqual(1);
            expect(j.scope.isActivatingSnapshots).toEqual(true);
            expect(j.scope.isActivatingFpolicy).toEqual(true);
            expect(j.scope.activeActionsPollingStopper).not.toHaveBeenCalled();
        });
    });
});
